//========================================================================================
//
//	File:				BmpPart.h
//	Release Version:	$ 1.0d11 $
//
//	Author:				Henri Lamiraux
//
//	Copyright:	 1993, 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef BMPPART_H
#define BMPPART_H

// ----- Part Layer -----

#ifndef FWPART_H
#include "FWPart.h"
#endif

// ----- OS Layer -----

#ifndef FWBITMAP_H
#include "FWBitmap.h"
#endif

#ifndef FWMENU_H
#include "FWMenu.h"
#endif

// ----- Foundation Layer -----

#ifndef FWBNDSTR_H
#include "FWBndStr.h"
#endif

//========================================================================================
//	Constants
//========================================================================================

// ----- Commands -----
const ODCommandID cHalfSize			= FW_kFirstUserCommandID;
const ODCommandID cRealSize			= cHalfSize + 1;
const ODCommandID cDoubleSize		= cRealSize + 1;
const ODCommandID cFitToFrame		= cDoubleSize + 1;

//========================================================================================
//	class CBitmapPart
//========================================================================================

class FW_CLASS_ATTR CBitmapPart : public FW_CPart
{
public:
	static const ODValueType kPartKind;
	static const ODValueType kPartUserName;

//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
public:
	CBitmapPart(ODPart* odPart);
	virtual ~ CBitmapPart();
	
	virtual void Initialize(Environment* ev);

//----------------------------------------------------------------------------------------
//	Inherited API
//
public:
	virtual void 			InternalizeContent(Environment* ev, 
								ODStorageUnit* storageUnit, 
								FW_CCloneInfo* cloneInfo);
	virtual void 			ExternalizeContent(Environment* ev, 
								ODStorageUnit* storageUnit, 
								FW_CCloneInfo* cloneInfo);
	
	virtual FW_Boolean 		DoAdjustMenus(Environment* ev, FW_CMenuBar* menuBar, FW_Boolean hasMenuFocus, FW_Boolean isRoot);
	virtual FW_Boolean		DoMenu(Environment* ev, const FW_CMenuEvent& theMenuEvent);

	virtual FW_CFrame*		NewFrame(Environment* ev, 
								 ODFrame* odFrame, 
								 FW_CPresentation* presentation,
							   	 FW_Boolean fromStorage);

	virtual FW_CWindow*		NewDocumentWindow(Environment* ev);

//----------------------------------------------------------------------------------------
//	New API
//
public:
	FW_PBitmap 				GetBitmap(Environment* ev) const;
	void 					SetBitmap(Environment* ev, FW_PBitmap bitmap);

	void					AdjustFramesSize(Environment* ev);

	void					InternalizeBitmapShape(Environment* ev, ODStorageUnit* storageUnit);

#ifdef FW_BUILD_MAC
	void					InternalizePICTData(Environment* ev, ODStorageUnit* storageUnit);
	void					InternalizePICTFile(Environment* ev, ODStorageUnit* storageUnit);
	void					MacSetBitmap(Environment* ev, FW_PlatformPict picHandle);
#endif
	
private:
	void					UseDefaultBitmap(Environment* ev);
	
//----------------------------------------------------------------------------------------
//	Data Members
//
private:
	FW_PBitmap				fBitmap;			// Bitmap object I am displaying
	FW_CString32			fAboutString;		// String for the about item
	FW_CPresentation*		fBitmapPresentation;

};
#endif
