//========================================================================================
//
//	File:				BmpPart.cpp
//	Release Version:	$ 1.0d11 $
//
//	Author:				Henri Lamiraux
//
//	Copyright:	 1993, 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "ODFBMap.hpp"

#ifndef BMPPART_H
#include "BmpPart.h"
#endif

#ifndef BMPFRAME_H
#include "BmpFrame.h"
#endif

#ifndef BMPDEF_H
#include "BmpDef.h"
#endif

#ifndef BMPSEL_H
#include "BmpSel.h"
#endif

// ----- Part Layer -----

#ifndef FWPRTITE_H
#include "FWPrtIte.h"
#endif

#ifndef FWABOUT_H
#include "FWAbout.h"
#endif

#ifndef FWPRESEN_H
#include "FWPresen.h"
#endif

#ifndef FWUTIL_H
#include "FWUtil.h"
#endif

// ----- OS Layer -----

#ifndef FWSUSINK_H
#include "FWSUSink.h"
#endif

#ifndef FWPICTUR_H
#include "FWPictur.h"
#endif

#ifndef FWEVENT_H
#include "FWEvent.h"
#endif

#ifndef FWBARRAY_H
#include "FWBArray.h"
#endif

#ifndef FWFILEAC_H
#include "FWFileAc.h"
#endif

// ----- Foundation Layer -----

#ifndef FWSTREAM_H
#include "FWStream.h"
#endif

#ifndef FWMEMORY_H
#include "FWMemory.h"
#endif

#ifndef FWSUSINK_H
#include "FWSUSink.h"
#endif

// ----- OpenDoc Includes -----

#ifndef SOM_Module_OpenDoc_StdProps_defined
#include <StdProps.xh>
#endif

#ifndef SOM_ODTranslation_xh
#include <Translt.xh>
#endif

#ifndef SOM_ODSession_xh
#include <ODSessn.xh>
#endif

// ----- Macintosh Includes -----

#if defined(FW_BUILD_MAC) && !defined(__DRAG__)
#include <Drag.h>
#endif

//========================================================================================
//	Runtime Informations
//========================================================================================

#ifdef FW_BUILD_MAC
#pragma segment odfbitmap
#endif

//========================================================================================
//	Globals
//========================================================================================

const ODValueType CBitmapPart::kPartKind 		= kODFBitmapKind;
const ODValueType CBitmapPart::kPartUserName 	= kODFBitmapEditorUserString;

//========================================================================================
//	class CBitmapPart
//========================================================================================

//----------------------------------------------------------------------------------------
//	CBitmapPart::CBitmapPart
//----------------------------------------------------------------------------------------
//	CBitmapPart constructor

CBitmapPart::CBitmapPart(ODPart* odPart) :
	FW_CPart(odPart, CBitmapPart::kPartKind, CBitmapPart::kPartUserName, FW_gInstance, kPartIconID)
{
}

//----------------------------------------------------------------------------------------
//	CBitmapPart::~CBitmapPart
//----------------------------------------------------------------------------------------
// CBitmapPart destructor

CBitmapPart::~CBitmapPart()
{
}

//----------------------------------------------------------------------------------------
//	CBitmapPart::Initialize
//----------------------------------------------------------------------------------------
//	ODF Method

void CBitmapPart::Initialize(Environment* ev)
{
	// ----- Call first inherited -----
	FW_CPart::Initialize(ev);
	
	// ----- Initialize the Archiver component -----
	FW_InitializeArchiving(FW_gInstance);

	// ----- Open the resource file for this shared library -----
	FW_CSharedLibraryResourceFile resFile;

	// ----- Initialize my menu -----
	FW_CPullDownMenu* pullDownMenu = new FW_CPullDownMenu(ev, FW_CString32("Settings"));
	pullDownMenu->AppendTextItem(ev, resFile, kMenuStrings, kHalfSize, cHalfSize);
	pullDownMenu->AppendTextItem(ev, resFile, kMenuStrings, kRealSize, cRealSize);
	pullDownMenu->AppendTextItem(ev, resFile, kMenuStrings, kDoubleSize, cDoubleSize);
	pullDownMenu->AppendSeparator(ev);
	pullDownMenu->AppendTextItem(ev, resFile, kMenuStrings, kFitToFrame, cFitToFrame);
	GetMenuBar(ev)->AdoptMenuLast(ev, pullDownMenu);
	
	// ----- Load also the string for the about item -----
	::FW_LoadStringByID(resFile, kMenuStrings, MULTISTRINGRES, kAboutString, fAboutString);
	
	// ----- Register our Presentation
	fBitmapPresentation = RegisterPresentation(ev, "Apple:Presentation:ODFBitmap", TRUE, new CBitmapSelection(ev, this));
}

//----------------------------------------------------------------------------------------
//	CBitmapPart::NewDocumentWindow
//----------------------------------------------------------------------------------------

FW_CWindow* CBitmapPart::NewDocumentWindow(Environment* ev)
{	
	FW_CRect bitmapRect;
	GetBitmap(ev)->GetBitmapBounds(bitmapRect);	// return in 72 dpi
	bitmapRect.Inset(FW_IntToFixed(-10), FW_IntToFixed(-10));
	
	FW_CRect screenBounds;
	::FW_GetMainScreenBounds(screenBounds);
	screenBounds.Inset(FW_IntToFixed(3), FW_IntToFixed(3));
	
	return new FW_CWindow(ev,
						GetODPart(ev),
					 	FW_CPart::gViewAsFrameToken,
						fBitmapPresentation->GetPresentationType(ev),
						bitmapRect.Size(),
						screenBounds.TopLeft(),
						FW_kDocumentWindow);
}

//----------------------------------------------------------------------------------------
//	CBitmapPart::InternalizeContent
//----------------------------------------------------------------------------------------
//	ODF Method

void CBitmapPart::InternalizeContent(Environment* ev, ODStorageUnit* storageUnit, FW_CCloneInfo* cloneInfo)
{
	if (storageUnit->Exists(ev, kODPropContents, this->GetPartKind(ev), 0))
	{
		InternalizeBitmapShape(ev, storageUnit);
	}
#ifdef FW_BUILD_MAC
	else if (storageUnit->Exists(ev, kODPropContents, FW_CPart::gMacPICTFileType, 0))
	{
		InternalizePICTFile(ev, storageUnit);
	}
	else if (storageUnit->Exists(ev, kODPropContents, FW_CPart::gMacPICTDataType, 0))
	{
		InternalizePICTData(ev, storageUnit);
	}
#endif
	else 
		FW_DEBUG_MESSAGE("CBitmapPart::InternalizeContent - Unkown type");
}

//---------------------------------------------------------------------------------------
//	CBitmapPart::InternalizeBitmapShape
//---------------------------------------------------------------------------------------

void CBitmapPart::InternalizeBitmapShape(Environment* ev, ODStorageUnit* storageUnit)
{
	FW_CStorageUnitSink sink(storageUnit, kODPropContents, this->GetPartKind(ev));
	if (sink.GetReadableBytes() != 0)
	{
		FW_CReadableStream stream(&sink);
		stream >> fBitmap;
	}
}

#ifdef FW_BUILD_MAC
//---------------------------------------------------------------------------------------
//	CBitmapPart::InternalizePICTData
//---------------------------------------------------------------------------------------

void CBitmapPart::InternalizePICTData(Environment* ev, ODStorageUnit* storageUnit)
{
	FW_CStorageUnitSink sink(storageUnit, kODPropContents, FW_CPart::gMacPICTDataType);
	FW_CReadableStream stream(&sink);

	unsigned long pictSize = storageUnit->GetSize(ev);;	//sink.GetLength();

	FW_CAcquireTemporarySystemHandle handle(pictSize);

	stream.Read(handle.GetPointer(), pictSize);
	
	MacSetBitmap(ev, (FW_PlatformPict)handle.GetPlatformHandle());
}
#endif

#ifdef FW_BUILD_MAC
//---------------------------------------------------------------------------------------
//	CBitmapPart::InternalizePICTFile
//---------------------------------------------------------------------------------------

void CBitmapPart::InternalizePICTFile(Environment* ev, ODStorageUnit* storageUnit)
{
	if (storageUnit->Exists(ev, kODPropContents, FW_CPart::gMachfsDataType, 0))
	{
		// Get the HFS flavor
		HFSFlavor hfsInfo;
		
		FW_CByteArray byteArray;
		storageUnit->GetValue(ev, sizeof(HFSFlavor), byteArray);
		byteArray.CopyBuffer(&hfsInfo, sizeof(HFSFlavor));
		
		// Use the FileSpec in HFS flavor to open the file's data fork
		FW_CFileSpecification fileSpec(hfsInfo.fileSpec);
		FW_CAccessPermission permission(FW_CAccessPermission::kRead, FW_CAccessPermission::kDenyWrite);
		FW_PFile pictFile(fileSpec, permission);
		FW_CFileSink pictSink(pictFile);
	
		// Read the picture data from the file
		#define kPICTHeaderSize 512
		unsigned long pictSize = pictSink.GetLength() - kPICTHeaderSize;
		
		FW_CAcquireTemporarySystemHandle tempHandle(pictSize);
		
		pictSink.SetPosition(kPICTHeaderSize);	// skip PICT file header
		pictSink.Read(tempHandle.GetPointer(), pictSize);
		
		MacSetBitmap(ev, (FW_PlatformPict)tempHandle.GetPlatformHandle());
	}	
}
#endif

//----------------------------------------------------------------------------------------
//	CBitmapPart::ExternalizeContent
//----------------------------------------------------------------------------------------
//	ODF Method

void CBitmapPart::ExternalizeContent(Environment* ev, ODStorageUnit* storageUnit, FW_CCloneInfo* cloneInfo)
{
	storageUnit->Focus(ev, kODPropContents, kODPosUndefined, CBitmapPart::kPartKind, 0, kODPosUndefined);
	storageUnit->Remove(ev);
	storageUnit->AddValue(ev, CBitmapPart::kPartKind);

	// To test if fBitmap is NULL cast it to a const void*
	if ((const void*)fBitmap != NULL)
	{
		// FW_CStorageUnitSink sink(storageUnit);
		FW_CStorageUnitSink sink(storageUnit, kODPropContents, this->GetPartKind(ev));
		FW_CWritableStream archive(&sink);
		archive << fBitmap;
	}
}

//----------------------------------------------------------------------------------------
//	CBitmapPart::DoMenu
//----------------------------------------------------------------------------------------
// ODF method

FW_Boolean CBitmapPart::DoMenu(Environment* ev, const FW_CMenuEvent& theMenuEvent)
{
	FW_Boolean result = TRUE;
	
#ifdef FW_BUILD_MAC
	if (theMenuEvent.GetCommandID(ev) == kODCommandAbout)
		::FW_About(FW_gInstance);	
	else
#endif
		result = FALSE;
		
	return result;
}

//----------------------------------------------------------------------------------------
//	CBitmapPart::DoAdjustMenus
//----------------------------------------------------------------------------------------
//	ODF Method

FW_Boolean CBitmapPart::DoAdjustMenus(Environment* ev, FW_CMenuBar* menuBar, FW_Boolean hasMenuFocus, FW_Boolean isRoot)
{
	// ----- Set the About Item -----
	if (hasMenuFocus)
		menuBar->SetItemString(ev, kODCommandAbout, fAboutString);

	return FALSE;
}

//----------------------------------------------------------------------------------------
//	CBitmapPart::NewFrame
//----------------------------------------------------------------------------------------
// ODF Method

FW_CFrame* CBitmapPart::NewFrame(Environment* ev,
								 ODFrame* odFrame, 
								 FW_CPresentation* presentation,
							   	 FW_Boolean fromStorage)
{
	FW_UNUSED(fromStorage);
	
	return new CBitmapFrame(ev, odFrame, presentation, this);
}

//----------------------------------------------------------------------------------------
//	CBitmapPart::GetBitmap
//----------------------------------------------------------------------------------------
//	Bitmap Part Method

FW_PBitmap CBitmapPart::GetBitmap(Environment* ev) const
{
	if ((const void*)fBitmap == NULL)
	{
		// ----- Fool the compiler -----
		CBitmapPart* self = (CBitmapPart*)this;
		self->UseDefaultBitmap(ev);
	}
	
	return fBitmap;
}

//----------------------------------------------------------------------------------------
//	CBitmapPart::SetBitmap
//----------------------------------------------------------------------------------------
//	Bitmap Part Method

void CBitmapPart::SetBitmap(Environment* ev, FW_PBitmap bitmap)
{
	fBitmap = bitmap;
	this->Changed(ev);
}

#ifdef FW_BUILD_MAC
//----------------------------------------------------------------------------------------
//	CBitmapPart::MacSetBitmap
//----------------------------------------------------------------------------------------
//	Set the bitmap from from a pict handle. The FW_PlatformPict should be deleted by the 
//	caller

void CBitmapPart::MacSetBitmap(Environment* ev, FW_PlatformPict picHandle)
{
	FW_PPicture picture(picHandle);			// Does not take ownership	
	fBitmap = FW_PBitmap(picture);
}
#endif

//----------------------------------------------------------------------------------------
//	CBitmapPart::AdjustFramesSize
//----------------------------------------------------------------------------------------
//	Bitmap Part Method

void CBitmapPart::AdjustFramesSize(Environment* ev)
{
	FW_CPartFrameIterator ite(this);
	for (CBitmapFrame* frame = (CBitmapFrame*)ite.First(); ite.IsNotComplete(); frame = (CBitmapFrame*)ite.Next())
	{
		frame->AdjustFrameSize(ev);
	}
}

//----------------------------------------------------------------------------------------
//	CBitmapPart::UseDefaultBitmap
//----------------------------------------------------------------------------------------
//	Bitmap Part Method

void CBitmapPart::UseDefaultBitmap(Environment* ev)
{
	FW_CSharedLibraryResourceFile resFile;
	
#ifdef FW_BUILD_MAC
	FW_PPicture picture(resFile, 2128);
	fBitmap = FW_PBitmap(picture);;
#endif
#ifdef FW_BUILD_WIN
	FW_PBitmap temp(resFile, 2128);
	fBitmap = temp;
#endif
}

