//========================================================================================
//
//	File:				BmpSel.cpp
//	Release Version:	$ 1.0d11 $
//
//	Author:				Henri Lamiraux
//
//	Copyright:	 1993, 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "ODFBMap.hpp"

#ifndef BMPSEL_H
#include "BmpSel.h"
#endif

#ifndef BMPPART_H
#include "BmpPart.h"
#endif

#ifndef BMPFRAME_H
#include "BmpFrame.h"
#endif

// ----- Part Layer -----

#ifndef FWUTIL_H
#include "FWUtil.h"
#endif

#ifndef FWITERS_H
#include "FWIters.h"
#endif

#ifndef FWPRESEN_H
#include "FWPresen.h"
#endif

// ----- OS Layer -----

#ifndef FWPICTUR_H
#include "FWPictur.h"
#endif

#ifndef FWBARRAY_H
#include "FWBArray.h"
#endif

#ifndef FWODGEOM_H
#include "FWODGeom.h"
#endif

// ----- Foundation Layer -----

#ifndef FWSUSINK_H
#include "FWSUSink.h"
#endif

#ifndef FWMEMMGR_H
#include "FWMemMgr.h"
#endif

#ifndef FWMEMHLP_H
#include "FWMemHlp.h"
#endif

// ----- OpenDoc Includes -----

#ifndef SOM_Module_OpenDoc_StdProps_defined
#include <StdProps.xh>
#endif

#ifndef SOM_ODShape_xh
#include <Shape.xh>
#endif

//========================================================================================
//	Runtime Information
//========================================================================================

#ifdef FW_BUILD_MAC
#pragma segment odfbitmap
#endif

//========================================================================================
//	class CBitmapSelection
//========================================================================================

//---------------------------------------------------------------------------------------
//	CBitmapSelection::CBitmapSelection
//---------------------------------------------------------------------------------------

CBitmapSelection::CBitmapSelection(Environment* ev, CBitmapPart* thePart) :
	FW_CSelection(ev, FALSE, FALSE),
	fBitmapPart(thePart),
	fHasSelection(FALSE)
{
	fRectShape.GetStyle()->SetPattern(FW_kAntPat);
	fRectShape.GetInk()->SetTransferMode(FW_kXOr);
	fRectShape.SetRenderVerb(FW_kFrame);
}

//---------------------------------------------------------------------------------------
//	CBitmapSelection::~CBitmapSelection
//---------------------------------------------------------------------------------------

CBitmapSelection::~CBitmapSelection()
{	
}

//---------------------------------------------------------------------------------------
//	CBitmapSelection::CloseSelection
//---------------------------------------------------------------------------------------

void CBitmapSelection::CloseSelection(Environment* ev)
{
	if (fHasSelection)
	{
		DrawAnts(ev, (CBitmapFrame*)fBitmapPart->GetActiveFrame(ev));	// Erase ants
		fHasSelection = FALSE;
	}
}

//---------------------------------------------------------------------------------------
//	CBitmapSelection::SelectAll
//---------------------------------------------------------------------------------------

void CBitmapSelection::SelectAll(Environment* ev)
{
	// ----- Close the selection if there is one -----
	CloseSelection(ev);
	
	// -----
	short width, height,rowBytes, pixelSize;
	fBitmapPart->GetBitmap(ev)->GetBitmapInfo(width, height, rowBytes, pixelSize);
	FW_SPlatformRect bounds(0, 0, width, height);
	SetSelectRect(bounds);
	
	// ----- Draw the first ants -----
	DrawAnts(ev, (CBitmapFrame*)fBitmapPart->GetActiveFrame(ev));
}

//---------------------------------------------------------------------------------------
//	CBitmapSelection::IsEmpty
//---------------------------------------------------------------------------------------

FW_Boolean CBitmapSelection::IsEmpty(Environment* ev) const
{
	return !fHasSelection;
}

//---------------------------------------------------------------------------------------
//	CBitmapSelection::DoExternalizeSelection
//---------------------------------------------------------------------------------------

void CBitmapSelection::DoExternalizeSelection(Environment* ev, 
											ODStorageUnit* destinationSU,
											FW_CCloneInfo* cloneInfo)
{
FW_UNUSED(cloneInfo);

	FW_PBitmap selectionBitmap = fBitmapPart->GetBitmap(ev)->Copy(fSelectRect);

	FW_CStorageUnitSink sink(destinationSU, kODPropContents, fBitmapPart->GetPartKind(ev));
	FW_CWritableStream archive(&sink);
	archive << selectionBitmap;

	//----- On the Mac also externalize a picture -----
#ifdef FW_BUILD_MAC
	FW_SUAddPropValue(ev, destinationSU, kODPropContents, FW_CPart::gMacPICTDataType);
	
	FW_CStorageUnitSink sink2(destinationSU, kODPropContents, FW_CPart::gMacPICTDataType);
	FW_CWritableStream stream(&sink2);
	
	FW_PPicture picture = selectionBitmap->GetAsPicture();
	
	FW_PlatformPict platformPict = picture->GetPlatformPict();

	unsigned long pictSize = FW_CMemoryManager::GetSystemHandleSize((FW_PlatformHandle)platformPict);
	
	FW_CAcquireLockedSystemHandle lockedHandle((FW_PlatformHandle)platformPict);
	stream.Write(lockedHandle.GetPointer(), pictSize);		
#endif
}

//---------------------------------------------------------------------------------------
//	CBitmapSelection::DoInternalizeSelection
//---------------------------------------------------------------------------------------

FW_Boolean CBitmapSelection::DoInternalizeSelection(Environment* ev, 
													ODStorageUnit* sourceSU,
													FW_CCloneInfo* cloneInfo)
{
	FW_UNUSED(cloneInfo);

	FW_Boolean internalized = FALSE;

	// ----- If there is a property content -----
	if (sourceSU->Exists(ev, kODPropContents, CBitmapPart::kPartKind, 0))
	{
		fBitmapPart->InternalizeBitmapShape(ev, sourceSU);		
		internalized = TRUE;
	}

#ifdef FW_BUILD_MAC
	// ----- If there is a PICT data or file content -----
	else if (sourceSU->Exists(ev, kODPropContents, FW_CPart::gMacPICTDataType, 0))		// 'PICT' in Scrap
	{
		fBitmapPart->InternalizePICTData(ev, sourceSU);
		internalized = TRUE;
	}
	else if (sourceSU->Exists(ev, kODPropContents, FW_CPart::gMacPICTFileType, 0))	// PICT file
	{
		fBitmapPart->InternalizePICTFile(ev, sourceSU);
		internalized = TRUE;
	}
#endif

	if (internalized)
	{	
		CloseSelection(ev);
		fBitmapPart->AdjustFramesSize(ev);	// will invalidate

		fBitmapPart->Changed(ev);
		FW_CPresentationFrameIterator piter(GetPresentation(ev));
		fBitmapPart->ContentUpdated(ev, piter.First(), kODUnknownUpdate);
	}
	
	return internalized;
}

//---------------------------------------------------------------------------------------
//	CBitmapSelection::CreateSelectionFrameShape
//---------------------------------------------------------------------------------------
// Return the shape of the selection. This will be added to the clipboard or
// drag and drop storage unit.

ODShape* CBitmapSelection::CreateSelectionFrameShape(Environment* ev) const
{
	ODShape* shape = NULL;
	
	// ----- we don't use the zoom factor for CreateSelectionFrameShape -----
	if (!IsEmpty(ev))
	{
		FW_CRect rect(fSelectRect);
		rect.Offset(-rect.left, -rect.top);
		
		shape = ::FW_NewODShape(ev, rect);
	}
	
	return shape;
}

//---------------------------------------------------------------------------------------
//	CBitmapSelection::MoveAnts
//---------------------------------------------------------------------------------------

void CBitmapSelection::MoveAnts(Environment* ev, CBitmapFrame* frame)
{
	ODFacet* facet = frame->GetActiveFacet(ev);
	if (facet == NULL)
		return;
		
	FW_CViewContext vc(ev, frame, facet);

	DoDrawAnts(ev, vc, frame);

	FW_PStyle style = fRectShape.GetStyle();	
	FW_PPattern pattern = style->GetPattern();
	pattern->ShiftRight();
	style->SetPattern(pattern);
	
	DoDrawAnts(ev, vc, frame);
}

//---------------------------------------------------------------------------------------
//	CBitmapSelection::DrawAnts
//---------------------------------------------------------------------------------------
// Draw ants in all facets of the given frame.

void CBitmapSelection::DrawAnts(Environment* ev, CBitmapFrame* frame)
{
	if (frame != NULL)
	{
		FW_CFrameFacetIterator iter(ev, frame);
		for (ODFacet* facet = (ODFacet*)iter.First(ev); iter.IsNotComplete(ev); facet = (ODFacet*)iter.Next(ev))
		{
			FW_CViewContext vc(ev, frame, facet);
			DoDrawAnts(ev, vc, frame);
		}
	}
}

//---------------------------------------------------------------------------------------
//	CBitmapSelection::DoDrawAnts
//---------------------------------------------------------------------------------------

void CBitmapSelection::DoDrawAnts(Environment* ev, FW_CGraphicContext& gc, CBitmapFrame* frame)
{
	FW_CRect usedRect;
	frame->CalcUsedRect(ev, usedRect);

	FW_CRect rect;
	GetSelectRect(frame->GetZoomRatio(ev), usedRect, rect);

	fRectShape.SetRectangle(rect);
	fRectShape.Render(gc);
}

//----------------------------------------------------------------------------------------
//	CBitmapSelection::CreateSelectionShape
//----------------------------------------------------------------------------------------

ODShape* CBitmapSelection::CreateSelectionShape(Environment* ev, ODFacet* facet, FW_CFrame* frame) const
{
	ODShape* selectionShape = ::FW_NewODShape(ev, FW_kZeroRect);
	
	if (!IsEmpty(ev))
	{
		FW_CRect usedRect;
		((CBitmapFrame*)frame)->CalcUsedRect(ev, usedRect);
	
		FW_CRect selectRect;
		GetSelectRect(((CBitmapFrame*)frame)->GetZoomRatio(ev), usedRect, selectRect);
	
		ODRect odSelectRect = selectRect;
		selectionShape->SetRectangle(ev, &odSelectRect);
	}
	
	return selectionShape;
}

//----------------------------------------------------------------------------------------
//	CBitmapSelection::GetSelectRect
//----------------------------------------------------------------------------------------

void CBitmapSelection::GetSelectRect(const FW_CPoint& ratio, const FW_CRect& usedRect, FW_CRect& rect) const
{
	if (fHasSelection)
	{	
		rect.Set(
			ratio.x.MultipliedByInt(fSelectRect.left),
			ratio.y.MultipliedByInt(fSelectRect.top),
			ratio.x.MultipliedByInt(fSelectRect.right),
			ratio.y.MultipliedByInt(fSelectRect.bottom));
	
		rect.Offset(usedRect.left, usedRect.top);
	}
	else
		rect.SetInt(0, 0, 0, 0);
}

//----------------------------------------------------------------------------------------
//	CBitmapSelection::SetSelectRect
//----------------------------------------------------------------------------------------

void CBitmapSelection::SetSelectRect(const FW_CPoint& ratio, const FW_CRect& usedRect, const FW_CRect& rect)
{
	
	fSelectRect.Set(
		((rect.left  - usedRect.left) / ratio.x).AsInt(),
		((rect.top  - usedRect.top) / ratio.y).AsInt(),
		((rect.right  - usedRect.left) / ratio.x).AsInt(),
		((rect.bottom - usedRect.top) / ratio.y).AsInt());
	
	fHasSelection = TRUE;
}

//----------------------------------------------------------------------------------------
//	CBitmapSelection::SetSelectRect
//----------------------------------------------------------------------------------------

void CBitmapSelection::SetSelectRect(const FW_SPlatformRect& rect)
{
	fSelectRect = rect;
	fHasSelection = TRUE;
}


