//========================================================================================
//
//	File:				RbbrBand.cpp
//	Release Version:	$ 1.0d11 $
//
//	Author:				Henri Lamiraux
//
//	Copyright:	 1993, 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "ODFBMap.hpp"

#ifndef RBBRBAND_H
#include "RbbrBand.h"
#endif

// ----- Parts Layer -----

#ifndef FWUTIL_H
#include "FWUtil.h"
#endif

#ifndef FWVIEW_H
#include "FWView.h"
#endif

// ----- OS Layer -----

#ifndef FWSTYLE_H
#include "FWStyle.h"
#endif

#ifndef FWINK_H
#include "FWInk.h"
#endif

//========================================================================================
//	Runtime Information
//========================================================================================

#ifdef FW_BUILD_MAC
#pragma segment odfbitmap
#endif

//========================================================================================
//	class CRectRubberBand
//========================================================================================

//-----------------------------------------------------------------------------------------
//	CRectRubberBand::CRectRubberBand
//-----------------------------------------------------------------------------------------

CRectRubberBand::CRectRubberBand(Environment* ev, FW_CView* view, ODFacet* facet, const FW_CRect& maxRect, const FW_CPoint& zoomRatio) :
	FW_CTracker(ev, view, facet),
	fMaxRect(maxRect),
	fZoomRatio(zoomRatio),
	fRectShape(FW_kZeroRect, FW_kFrame)
{
	FW_PInk ink(FW_kRGBBlack, FW_kRGBWhite, FW_kXOr);
	FW_PStyle style(FW_IntToFixed(1), FW_kAntPat);
	
	fRectShape.SetInk(ink);
	fRectShape.SetStyle(style);
}

//-----------------------------------------------------------------------------------------
//	CRectRubberBand::~CRectRubberBand
//-----------------------------------------------------------------------------------------

CRectRubberBand::~CRectRubberBand()
{
}

//-----------------------------------------------------------------------------------------
//	CRectRubberBand::BeginTracking
//-----------------------------------------------------------------------------------------

FW_CPoint CRectRubberBand::BeginTracking(Environment* ev, 
										 const FW_CPoint& anchorPoint)
{
	FW_CPoint curLoc = AlignOnPixel(anchorPoint);
	
	FW_CViewContext vc(ev, GetView(ev), GetFacet(ev));
	FW_CRect rect(curLoc, curLoc);
	fRectShape.SetRectangle(rect);
	fRectShape.Render(vc);
	
	return curLoc;
}

//-----------------------------------------------------------------------------------------
//	CRectRubberBand::ContinueTracking
//-----------------------------------------------------------------------------------------

FW_CPoint CRectRubberBand::ContinueTracking(Environment* ev, 
											const FW_CPoint& anchorPoint, 
											const FW_CPoint& previousPoint, 
											const FW_CPoint& currentPoint)
{
FW_UNUSED(previousPoint);
	FW_CPoint curLoc = AlignOnPixel(currentPoint);

	FW_CRect rect(anchorPoint, curLoc);
	rect.Sort();
	rect.Intersection(fMaxRect);
	
	FW_CRect shapeBounds;
	fRectShape.GetRectangle(shapeBounds);
	if (rect != shapeBounds)
	{
		FW_CViewContext vc(ev, GetView(ev), GetFacet(ev));
		
		fRectShape.Render(vc);	// Erase				
		fRectShape.SetRectangle(rect);		
		fRectShape.Render(vc);	// Draw
	}
	
	return currentPoint;
}

//-----------------------------------------------------------------------------------------
//	CRectRubberBand::EndTracking
//-----------------------------------------------------------------------------------------

FW_Boolean CRectRubberBand::EndTracking(Environment* ev, 
									const FW_CPoint& anchorPoint, 
									const FW_CPoint& lastPoint)
{
	FW_CViewContext vc(ev, GetView(ev), GetFacet(ev));
	fRectShape.Render(vc);	// Erase
	
	return anchorPoint != lastPoint;
}

//-----------------------------------------------------------------------------------------
//	CRectRubberBand::AlignOnPixel
//-----------------------------------------------------------------------------------------
//	We want, what ever the zoom factor, always have the selection rectangle on a pixel boundary

FW_CPoint CRectRubberBand::AlignOnPixel(const FW_CPoint& location)
{
	FW_CPoint result = location - fMaxRect[FW_kTopLeft];
		
	result.x = result.x / fZoomRatio.x;
	result.y = result.y / fZoomRatio.y;

	result.x.SetInt(result.x.AsInt());
	result.y.SetInt(result.y.AsInt());
	
	result.x *= fZoomRatio.x;
	result.y *= fZoomRatio.y;
		
	result += fMaxRect[FW_kTopLeft];
	
	return result;
}

