//========================================================================================
//
//	File:				DrawLink.h
//	Release Version:	$ 1.0d11 $
//
//	Author:				Henri Lamiraux
//
//	Copyright:	 1993, 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef DRAWLINK_H
#define DRAWLINK_H

// ----- Part Layer -----

#ifndef FWLINK_H
#include "FWLink.h"
#endif

#ifndef FWLNKMGR_H
#include "FWLnkMgr.h"
#endif

#ifndef FWPOINT_H
#include "FWPoint.h"
#endif

//========================================================================================
//	Forward Declarations
//========================================================================================

#if FW_LIB_EXPORT_PRAGMAS
#pragma import on
#endif
class FW_CLASS_ATTR FW_CPrivOrderedCollection;
class FW_CLASS_ATTR FW_CFrame;
class FW_CLASS_ATTR FW_CCloneInfo;
#if FW_LIB_EXPORT_PRAGMAS
#pragma import off
#endif

extern const ODPropertyName kDrawPropSourceLink;
extern const ODPropertyName kDrawPropDestLink;
extern const ODValueType kDrawLinkValue;


class FW_CLASS_ATTR CBaseShape;
class FW_CLASS_ATTR CDrawSelection;
class FW_CLASS_ATTR CDrawPart;

//========================================================================================
//	class CDrawPublishLink
//========================================================================================

class FW_CLASS_ATTR CDrawPublishLink : public FW_CPublishLink
{
  public:
	FW_DECLARE_CLASS

	CDrawPublishLink(Environment* ev, ODUpdateID updateID, 
					 FW_CPresentation* presentation,
					 CDrawSelection* drawSelection,
					 FW_Boolean saveSelection);
	virtual ~ CDrawPublishLink();
	
	virtual void		Publish(Environment* ev);
	virtual void		ExternalizeLinkContent(Environment* ev, ODStorageUnit* linkSU);
	virtual ODValueType	GetPublishFormat(Environment* ev);
	
	FW_Boolean			HasEmbeddedFrame(Environment *ev, ODFrame* odFrame);

//--- Will move to FW_CPublishLink
	virtual FW_CFrame*	GetFrameToRevealLink(Environment* ev);
	virtual void		DoSelect(Environment* ev, FW_CFrame* frame);

	void				SelectShapes(Environment* ev);

	//--- Persistency
	void				Externalize(Environment* ev, ODStorageUnit* storageUnit,
									FW_CCloneInfo* cloneInfo);
	void				AddShape(Environment* ev, CBaseShape* shape);

  private:
	void RestoreDrawSelection(Environment* ev);

  private:
	CDrawSelection*					fDrawSelection;
	FW_CPrivOrderedCollection*		fCollection;
	FW_CPrivOrderedCollection*		fSavedSelection;
};

//========================================================================================
//	class CDrawSubscribeLink
//========================================================================================

class FW_CLASS_ATTR CDrawSubscribeLink : public FW_CSubscribeLink
{
	friend class CDrawLinkManager;

  public:
	FW_DECLARE_CLASS

	CDrawSubscribeLink(Environment* ev, ODLink* odLink,
						FW_CPresentation* presentation, 
						CDrawSelection* drawSelection,
						ODLinkInfo* linkInfo);
	virtual ~ CDrawSubscribeLink();

	//--- Overrides ---
	virtual void	Subscribe(Environment* ev);
	virtual void	DoUpdateLink(Environment* ev, ODStorageUnit* linkContentSU);
	virtual void	BreakLink(Environment* ev);

  private:
	void SaveAndSelectShapes(Environment* ev);
	void RestoreDrawSelection(Environment* ev);
	void SetCollection(Environment* ev);
	void EmptyCollection(Environment* ev);
	
	//--- Persistency
	void Externalize(Environment* ev, ODStorageUnit* storageUnit,
					 FW_CCloneInfo* cloneInfo);
	void AddShape(Environment* ev, CBaseShape* shape);
	
  public:
	void SelectShapes(Environment* ev);
	void SetUpdateOffset(Environment* ev, const FW_CPoint& offset);
	
  private:
	CDrawSelection*					fDrawSelection;
	FW_CPrivOrderedCollection*		fCollection;
	FW_CPrivOrderedCollection*		fSavedSelection;
	FW_CPoint						fUpdateOffset;
};

//----------------------------------------------------------------------------------------
inline void CDrawSubscribeLink::SetUpdateOffset(Environment* ev, const FW_CPoint& offset)
{
	fUpdateOffset = offset;
}

//========================================================================================
//	class CDrawLinkManager
//========================================================================================

class FW_CLASS_ATTR CDrawLinkManager : public FW_CLinkManager
{
  public:
	FW_DECLARE_CLASS

//----------------------------------------------------------------------------------------
//	Constructors/Destructors
//
  public:
	CDrawLinkManager(Environment* ev, CDrawPart* thePart);
	virtual ~CDrawLinkManager();

//----------------------------------------------------------------------------------------
//	Override Methods
//
  public:
	virtual FW_CPublishLink* 	NewPublishLink(Environment* ev, 
											   ODUpdateID updateID, 
											   FW_CPresentation* presentation);
	virtual FW_CSubscribeLink*	NewSubscribeLink(Environment* ev, 
												 ODLink* odLink,
												 ODLinkInfo* linkInfo,
												 FW_CPresentation* presentation);

	virtual ODLinkSource*	CreateLink(Environment *ev, ODByteArray* data);

	virtual void			DoChangeLinkStatus(Environment* ev, ODFrame* odFrame);
								// formerly named LinkStatusChanged

	virtual void			DoUpdateLinks(Environment* ev,
										  ODFrame* odEmbeddedFrame,
										  ODUpdateID updateID);
								// formerly named EmbeddedFrameUpdated

	virtual void			RevealLink(Environment* ev, ODLinkSource* linkSource);

//----------------------------------------------------------------------------------------
//	Support for Undo/Redoing PasteAs
//
  public:
	virtual void UndoPasteAs(Environment* ev, CDrawSubscribeLink* link);
	virtual void RedoPasteAs(Environment* ev, CDrawSubscribeLink* link);
	virtual void CommitUndone(Environment* ev, CDrawSubscribeLink* link);

	CDrawSubscribeLink* GetLatestSubscriber(Environment* ev);

	void SelectSubscribedShapes(Environment* ev, CDrawSubscribeLink* link);
	void DeleteSubscribedShapes(Environment* ev, CDrawSubscribeLink* link);

//----------------------------------------------------------------------------------------
//	Support for Persistent Links
//
  public:
	void ExternalizeLinks(Environment* ev, ODStorageUnit* su, FW_CCloneInfo* cloneInfo);
	void InternalizeLinks(Environment* ev, ODStorageUnit* storageUnit);

	void RegisterSubscribers(Environment* ev);

  private:
	void InternalizeOneDestLink(Environment* ev, ODStorageUnit* storageUnit);
	void InternalizeOneSourceLink(Environment* ev, ODStorageUnit* storageUnit);

	ODULong GetSUValue(Environment* ev, ODStorageUnit* su, unsigned long size, void *buffer);

//----------------------------------------------------------------------------------------
//	Data Members
//
  private:
	CDrawPart*		fDrawPart;
};

#endif