//========================================================================================
//
//	File:				RectShp.cpp
//	Release Version:	$ 1.0d11 $
//
//	Author:				Henri Lamiraux
//
//	Copyright:	 1993, 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "ODFDraw.hpp"

#ifndef BASESHP_H
#include "BaseShp.h"
#endif

#ifndef BOUNDSHP_H
#include "BoundShp.h"
#endif

#ifndef RECTSHP_H
#include "RectShp.h"
#endif

#ifndef UTILS_H
#include "Utils.h"
#endif

#ifndef CONSTANT_H
#include "Constant.h"
#endif

#ifndef DRAWPART_H
#include "DrawPart.h"
#endif

#ifndef DRAWPART_H
#include "DrawPart.h"
#endif

#ifndef DRAWFRM_H
#include "DrawFrm.h"
#endif

#ifndef DRAWPRXY_H
#include "DrawPrxy.h"
#endif

#ifndef DRAWLINK_H
#include "DrawLink.h"
#endif

#ifndef DRAWCLIP_H
#include "DrawClip.h"
#endif

// ----- Part Layer -----

#ifndef FWFRMING_H
#include "FWFrming.h"
#endif

#ifndef FWUTIL_H
#include "FWUtil.h"
#endif

#ifndef FWSELECT_H
#include "FWSelect.h"
#endif

#ifndef FWITERS_H
#include "FWIters.h"
#endif

// ----- OS Layer -----

#ifndef FWEVENT_H
#include "FWEvent.h"
#endif

#ifndef FWRECSHP_H
#include "FWRecShp.h"
#endif

#ifndef FWTXTBOX_H
#include "FWTxtBox.h"
#endif

#ifndef FWLINSHP_H
#include "FWLinShp.h"
#endif

#ifndef FWOVLSHP_H
#include "FWOvlShp.h"
#endif

#ifndef FWRRCSHP_H
#include "FWRRcShp.h"
#endif

#ifndef FWODGEOM_H
#include "FWODGeom.h"
#endif

// ----- Foundation Includes -----

#ifndef FWSTREAM_H
#include "FWStream.h"
#endif

// ----- OpenDoc Includes -----

#ifndef SOM_ODTransform_xh
#include <Trnsform.xh>
#endif

//========================================================================================
// Runtime Information
//========================================================================================

#ifdef FW_BUILD_MAC
#pragma segment odfdrawshapes
#endif

//========================================================================================
// RunTime Info
//========================================================================================

FW_DEFINE_CLASS_M1(CRectShape, CBoundedShape)

FW_REGISTER_ARCHIVABLE_CLASS(LRectShape, CRectShape, CRectShape::Read, CBaseShape::Write)

//========================================================================================
// class CRectShape
//========================================================================================

//----------------------------------------------------------------------------------------
// CRectShape::CRectShape
//----------------------------------------------------------------------------------------

CRectShape::CRectShape() :
	CBoundedShape(kRectShape, kFrameOnly)
{
}

//----------------------------------------------------------------------------------------
// CRectShape::CRectShape
//----------------------------------------------------------------------------------------

CRectShape::CRectShape(unsigned short shapeType, unsigned short renderVerb) :
	CBoundedShape(shapeType, renderVerb)
{
}

//----------------------------------------------------------------------------------------
// CRectShape::CRectShape
//----------------------------------------------------------------------------------------

CRectShape::CRectShape(FW_CReadableStream& archive) :
	CBoundedShape(archive)
{	
}

//----------------------------------------------------------------------------------------
// CRectShape::~CRectShape
//----------------------------------------------------------------------------------------

CRectShape::~CRectShape()
{
}

//----------------------------------------------------------------------------------------
// CRectShape::RenderShape
//----------------------------------------------------------------------------------------

void CRectShape::RenderShape(Environment* ev, ODFacet* facet, FW_CGraphicContext& gc)
{
	FW_CRect rect(fRect);
	AdjustRectForPenSize(rect, GetPenSize());

	if (HasFillStyle())
		FW_CRectShape::RenderRect(gc, rect, FW_kFill, fFillInk, fFillStyle);
	
	if (HasFrameStyle())
		FW_CRectShape::RenderRect(gc, rect, FW_kFrame, fFrameInk, fFrameStyle);
}

//----------------------------------------------------------------------------------------
// CRectShape::HitTest
//----------------------------------------------------------------------------------------

FW_Boolean CRectShape::HitTest(Environment *ev, FW_CGraphicContext& gc, const FW_CMouseEvent& theMouseEvent) const
{
	FW_CPoint mouse = theMouseEvent.GetMousePosition(ev, FW_CMouseEvent::kFrame);
	ODFacet* facet = theMouseEvent.GetFacet(ev);
	FW_CFrame* frame = FW_CFrame::ODtoFWFrame(ev, facet->GetFrame(ev));
	frame->GetContentView(ev)->FrameToViewContent(ev, mouse);
	
	FW_CRect bounds(fRect);
	AdjustRectForPenSize(bounds, GetPenSize());

	FW_CRectShape rectShape(bounds, HasFillStyle() ? FW_kFill : FW_kFrame);
	return rectShape.HitTest(gc, mouse, FW_IntToFixed(2));
}

//----------------------------------------------------------------------------------------
// CRectShape::OutlineShape
//----------------------------------------------------------------------------------------

void CRectShape::OutlineShape(FW_CGraphicContext& gc, const FW_PInk& ink, const FW_PStyle& style, const FW_CRect& rect)
{
	FW_CRect bounds(rect);
	AdjustRectForPenSize(bounds, style->GetPenSize());
	FW_CRectShape::RenderRect(gc,
							  bounds,
							  FW_kFrame,
							  ink, style);
}

//----------------------------------------------------------------------------------------
// CRectShape::GetClipRegion
//----------------------------------------------------------------------------------------

void CRectShape::GetClipRegion(Environment *ev, ODShape* clipRegion)
{
	FW_CRect bounds(fRect);
	AdjustRectForPenSize(bounds, GetPenSize());
	
	ODRect odBounds = bounds;
	clipRegion->SetRectangle(ev, &odBounds);
	
	if (FrameOnly())
	{
		bounds.Inset(GetPenSize(), GetPenSize());
		
		FW_CAcquiredODShape aqTempShape = ::FW_NewODShape(ev, bounds);
		clipRegion->Subtract(ev, aqTempShape);
	}
}

//----------------------------------------------------------------------------------------
// CRectShape::CreateShapeOutline
//----------------------------------------------------------------------------------------

ODShape* CRectShape::CreateShapeOutline(Environment *ev)
{
	FW_CRect bounds(fRect);
	AdjustRectForPenSize(bounds, GetPenSize());

		// We don't acquire it because we return it
	ODShape* dragRgn = ::FW_NewODShape(ev, bounds);

	::FW_OutlineODShape(ev, dragRgn, GetPenSize());

	return dragRgn;
}

//----------------------------------------------------------------------------------------
// CRectShape::Read
//----------------------------------------------------------------------------------------

void* CRectShape::Read(FW_CReadableStream& archive)
{
	return new CRectShape(archive);
}

