//========================================================================================
//
//	File:				FormFrm.cpp
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "Form.hpp"

#ifndef FORMFRM_H
#include "FormFrm.h"
#endif

#ifndef FORMPRT_H
#include "FormPrt.h"
#endif

#ifndef FORMVIEW_H
#include "FormView.h"
#endif

#ifndef FORMPANE_H
#include "FormPane.h"
#endif

#ifndef FORMDEF_H
#include "FormDef.h"
#endif


// ----- Framework Layer -----

#ifndef FWUTIL_H
#include "FWUtil.h"
#endif

#ifndef FWSCROLR_H
#include "FWScrolr.h"
#endif

#ifndef FWSCLBAR_H
#include "FWSclBar.h"
#endif

#ifndef FWGROWBX_H
#include "FWGrowBx.h"
#endif

#ifndef FWPUSHBU_H
#include "FWPushBu.h"
#endif

#ifndef FWGROWBX_H
#include "FWClustr.h"
#endif

#ifndef FWABOUT_H
#include "FWAbout.h"
#endif

// ----- OS Layer -----

#ifndef FWMENU_H
#include "FWMenu.h"
#endif

#ifndef FWEVENT_H
#include "FWEvent.h"
#endif

#ifndef FWALERT_H
#include "FWAlert.h"
#endif

// ----- Graphic Includes -----

#ifndef FWRECT_H
#include "FWRect.h"
#endif

#ifndef FWTXTBOX_H
#include "FWTxtBox.h"
#endif

#ifndef FWRECSHP_H
#include "FWRecShp.h"
#endif

#ifndef FWPICSHP_H
#include <FWPicShp.h>			
#endif

// ----- Foundation Layer -----

#ifndef FWNOTIFN_H
#include <FWNotifn.h>
#endif

//========================================================================================
// Runtime Informations
//========================================================================================

#ifdef FW_BUILD_MAC
#pragma segment odfform
#endif

//========================================================================================
// CFormFrame class
//========================================================================================

//----------------------------------------------------------------------------------------
// CFormFrame::CFormFrame
//----------------------------------------------------------------------------------------
CFormFrame::CFormFrame(Environment* ev, ODFrame* odFrame, FW_CPresentation* presentation, CFormPart* part)
	: FW_CFrame(ev, odFrame, presentation, part),
	fFormPart(part),
	fGrowBox(NULL),
	fPaneView(NULL),
	fButton(NULL),
	fButtonId(0),
	fConnection(this),
	fButtonNotificationToken(0)
{
}

//----------------------------------------------------------------------------------------
// CFormFrame::~CFormFrame
//----------------------------------------------------------------------------------------
CFormFrame::~CFormFrame()
{
	FW_CInterest interest(fButton, fButtonNotificationToken);
	fConnection.RemoveInterest(interest);
}

//----------------------------------------------------------------------------------------
// CFormFrame::DoMenuEvent
//----------------------------------------------------------------------------------------
FW_Boolean CFormFrame::DoMenu(Environment* ev, const FW_CMenuEvent& theMenuEvent)	// Override
{
	FW_Boolean menuHandled = TRUE;
	
	switch (theMenuEvent.GetCommandID(ev))
	{
#ifdef FW_BUILD_MAC
			case kODCommandAbout:
				::FW_About(FW_gInstance);
				break;
#endif				
		default:
			menuHandled = FALSE;
	}
	
	return menuHandled;
}

//----------------------------------------------------------------------------------------
// CFormFrame::DoAdjustMenus
//----------------------------------------------------------------------------------------

FW_Boolean CFormFrame::DoAdjustMenus(Environment* ev, FW_CMenuBar* menuBar, 
									  FW_Boolean hasMenuFocus,
									  FW_Boolean isRoot)	// Override
{
	if (hasMenuFocus)
	{
		// Set up the About menu item
		FW_CString32 aboutString;
		fFormPart->GetResourceString(kAboutBoxStrings, kAboutMenuString, aboutString);
		menuBar->SetItemString(ev, kODCommandAbout, aboutString);
		menuBar->EnableCommand(ev, kODCommandAbout, TRUE);
	}
	
	return FALSE;
}

//----------------------------------------------------------------------------------------
// CFormFrame::DoAboutBox
//----------------------------------------------------------------------------------------

FW_Boolean CFormFrame::DoAboutBox()
{
	FW_CString32 str1, str2;

	fFormPart->GetResourceString(kAboutBoxStrings, kFormPartString, str1);
	fFormPart->GetResourceString(kAboutBoxStrings, kCreditString, str2);

	FW_CAlert::DoAlert(str1, str2, FW_kOK, FW_kNoteAlert, FW_kDefaultButton1, FALSE);

	return TRUE;
}

//----------------------------------------------------------------------------------------
// CFormFrame::CreateSubViews
//----------------------------------------------------------------------------------------
// The subviews created for this frame are:
//
//   CFormFrame 
//         |_____ fGrowBox  				(only for root frame)
//         |
//         |_____ fPaneView 				(intermediate gray view)
//                    |_____ horzSB 		(horizontal scrollbar)
//                    |_____ vertSB 		(vertical scrollbar)
//                    |
//                    |_____ contentView 	(contains the scrolling form)
//                                 |_____ fButton
//                                      
void CFormFrame::CreateSubViews(Environment* ev)
{		
		FW_CPoint sbSize = FW_CScrollBar::GetDefaultScrollBarSize();
		
		FW_CRect frameRect = GetBounds(ev);  
		
		// ----- Create the GrowBox only in root frame
		if (this->IsRoot(ev)) {
			FW_CPoint gboxLoc(frameRect.right - sbSize.x, frameRect.bottom - sbSize.x);
			fGrowBox = new FW_CGrowBox(ev, this, 0, gboxLoc);
		}
		
		// ----- Create the PaneView below the space for the pict adorner
		frameRect.Inset(sbSize.x, sbSize.x);
		frameRect.top += FW_IntToFixed(100);
		fPaneView = new CFormPane(ev, this, frameRect, fFormPart);
		
		// Compute the bounds of the content view inside the PaneView
		FW_CRect contentRect(FW_kZeroPoint, fPaneView->GetSize(ev));
		contentRect.Inset(FW_IntToFixed(20), FW_IntToFixed(20));	
			
		// ----- Create the vertical scroll bar inside the PaneView
		FW_CRect vertSbRect(contentRect.right, contentRect.top - FW_IntToFixed(1), 
							contentRect.right + sbSize.x + FW_IntToFixed(1), 
							contentRect.bottom + FW_IntToFixed(2));
		FW_CScrollBar* vertSB = new FW_CScrollBar(ev, fPaneView, 0, vertSbRect);
		
		// ----- Create the horizontal scroll bar inside the PaneView	
		FW_CRect horzSbRect(contentRect.left - FW_IntToFixed(1), contentRect.bottom, 
							contentRect.right + FW_IntToFixed(2), 
							contentRect.bottom + sbSize.y + FW_IntToFixed(1));
		FW_CScrollBar* horzSB = new FW_CScrollBar(ev, fPaneView, 0,horzSbRect);
		
		// ----- Create the content view & add a scroller
		//       (the content's extent has already been set in CFormPart::NewFrame)
		CFormView* contentView = new CFormView(ev, fPaneView, contentRect, fFormPart);
		contentView->MakeContentView(ev); 
		contentView->SetExtent(ev, fFormPart->GetPictSize());	
		contentView->AddScrollBarScroller(ev, horzSB, vertSB);
		
		// ----- Create the push button at the bottom of the content view
		FW_CDynamicString label("Join ODFWired!");
		FW_CPoint extent(contentView->GetExtent(ev));
		FW_CRect buttonRect(extent.x / FW_IntToFixed(2),
							extent.y - FW_IntToFixed(70), 
							extent.x / FW_IntToFixed(2) + FW_IntToFixed(130),
							extent.y - FW_IntToFixed(40));
		fButton = new FW_CPushButton(ev, contentView, fButtonId, buttonRect, label);
	
		fButton->SetDefault(ev, TRUE);	// default button is drawn with thick outline

		// Set up button notification ---
		fButtonNotificationToken = fButton->GetButtonPressedNotificationToken(ev);
		FW_CInterest okInterest(fButton, fButtonNotificationToken);
		fConnection.AddInterest(okInterest);

		// ----- Create the radio cluster in the content view
//		FW_CDynamicString label2("Payout");
//		fRadioCluster = new FW_CRadioCluster(ev, contentView, 1,
//									FW_CRect(ff(20), ff(150), ff(400), ff(200)),
//									label2, 2);
}

//----------------------------------------------------------------------------------------
// CFormFrame::AdjustSubViews
//----------------------------------------------------------------------------------------

void CFormFrame::AdjustSubViews(Environment* ev)
{
	FW_CPoint sbSize = FW_CScrollBar::GetDefaultScrollBarSize();

	// Get scroller from content view
	FW_CView* contentView = GetContentView(ev);
	FW_CScrollBarScroller* scroller = (FW_CScrollBarScroller*)contentView->GetScroller(ev);
	FW_ASSERT(scroller != NULL);
	FW_CScrollBar* scrollBar;

	// compute new bounds of the PaneView and ContentView which get resized
	// by the same amount as the frame
	FW_CRect frameRect = GetBounds(ev);

	if (fGrowBox)
	{
		fGrowBox->Invalidate(ev);	
		fGrowBox->SetLocation(ev, FW_CPoint(frameRect.right - sbSize.x, frameRect.bottom - sbSize.y));
		fGrowBox->Invalidate(ev);	
	}
	
	frameRect.Inset(sbSize.x, sbSize.x);
	frameRect.top += FW_IntToFixed(100);

	FW_CRect contentRect(FW_kZeroPoint, frameRect.Size());
	contentRect.Inset(FW_IntToFixed(20), FW_IntToFixed(20));	
	
	fPaneView->SetSize(ev, frameRect.Size());
	contentView->SetSize(ev, contentRect.Size());

	// adjust bounds of gadgets + invalidate old/new positions
	if (scrollBar = scroller->GetScrollBar(ev, FW_CScrollBarScroller::kVertical))
	{
		scrollBar->Invalidate(ev);	
		scrollBar->SetLocation(ev, FW_CPoint(contentRect.right, contentRect.top - FW_IntToFixed(1)));
		scrollBar->SetSize(ev, FW_CPoint(sbSize.x + FW_IntToFixed(1), 
								contentRect.bottom - contentRect.top + FW_IntToFixed(1)));
		scrollBar->Invalidate(ev);	
	}		
	if (scrollBar = scroller->GetScrollBar(ev, FW_CScrollBarScroller::kHorizontal))
	{
		scrollBar->Invalidate(ev);	
		scrollBar->SetLocation(ev, FW_CPoint(contentRect.left - FW_IntToFixed(1), contentRect.bottom));
		scrollBar->SetSize(ev, FW_CPoint(contentRect.right - contentRect.left + FW_IntToFixed(1), 
										sbSize.y + FW_IntToFixed(1)));
		scrollBar->Invalidate(ev);	
	}		

	scroller->UpdateScrollParameters(ev);
}

//----------------------------------------------------------------------------------------
// CFormFrame::FrameShapeChanged
//----------------------------------------------------------------------------------------

void CFormFrame::FrameShapeChanged(Environment* ev)
{
	FW_CFrame::FrameShapeChanged(ev);
	
	// force redraw of whole frame
	this->Invalidate(ev);
}

//----------------------------------------------------------------------------------------
// CFormFrame::Draw
//----------------------------------------------------------------------------------------

void CFormFrame::Draw(Environment *ev, ODFacet* odFacet, ODShape* invalidShape)
{
//	FW_CRect	contentRect(GetContentView(ev)->GetBounds(ev));
//	contentRect.Inset(-FW_IntToFixed(1),-FW_IntToFixed(1));

	// Set drawing contect for the CFrame
	FW_CViewContext vc(ev, this, odFacet, invalidShape);

	// Erase CFrame  draw picture
	FW_CRect invalidRect;
	vc.GetClipRect(invalidRect);
	FW_CRectShape::RenderRect(vc, invalidRect, FW_kFill, FW_kWhiteEraseInk);
	fFormPart->GetPictAdorner()->Render(vc);
}

//----------------------------------------------------------------------------------------
//	CFormFrame::HandleNotification
//----------------------------------------------------------------------------------------

void CFormFrame::HandleNotification(const FW_CNotification& notification)
{
	// Check that the notification is for the button
	if (notification.GetName() == fButtonNotificationToken) {
		const FW_CButtonPressedNotification& buttonPressedNotification =
			(FW_CButtonPressedNotification&) notification;
	
		if (buttonPressedNotification.GetButtonId() == fButtonId)
			FW_Beep();
	}
	else {
		// call parent
		FW_CFrame::HandleNotification(notification);
	}
}

