//========================================================================================
//
//	File:				FormView.cpp
//	Release Version:	$ 1.0d11 $
//
//	Author:				Laurent Delamare
//
//	Copyright:	 1993, 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "Form.hpp"

#ifndef FORMVIEW_H
#include "FormView.h"
#endif

#ifndef FORMDEF_H
#include "FormDef.h"
#endif

#ifndef FORMPART_H
#include "FormPrt.h"
#endif

// ----- Framework Layer -----

#ifndef FWUTIL_H
#include "FWUtil.h"
#endif

#ifndef FWFRAME_H
#include "FWFrame.h"
#endif

// ----- OS Layer -----

#ifndef FWMENU_H
#include "FWMenu.h"
#endif

#ifndef FWEVENT_H
#include "FWEvent.h"
#endif

#ifndef FWRECT_H
#include "FWRect.h"
#endif

#ifndef FWRECSHP_H
#include "FWRecShp.h"
#endif

#ifndef FWPICSHP_H
#include "FWPicShp.h"		
#endif

#ifndef FWALERT_H
#include "FWAlert.h"			
#endif

// ----- OpenDoc Includes -----

#ifndef SOM_Module_OpenDoc_Commands_defined
#include <CmdDefs.xh>
#endif

#ifndef SOM_Module_OpenDoc_StdProps_defined
#include <StdProps.xh>
#endif

#ifndef SOM_ODSession_xh
#include <ODSessn.xh>
#endif

#ifndef SOM_ODDispatcher_xh
#include <Disptch.xh>
#endif

//========================================================================================
// RunTime Info
//========================================================================================

#ifdef FW_BUILD_MAC
#pragma segment odfform
#endif

FW_DEFINE_CLASS_M1(CFormView, FW_CView);

//========================================================================================
// CFormView
//========================================================================================

//----------------------------------------------------------------------------------------
// CFormView::CFormView
//----------------------------------------------------------------------------------------

CFormView::CFormView(Environment* ev, FW_CView* container, FW_CRect& bounds, CFormPart* part) :
	FW_CView(ev, container, bounds),
	fFormPart(part)
{	
	//this->SetDroppable(ev, TRUE);
}

//----------------------------------------------------------------------------------------
// CFormView::~CFormView
//----------------------------------------------------------------------------------------

CFormView::~CFormView()
{
}


//----------------------------------------------------------------------------------------
// CFormView::Draw
//----------------------------------------------------------------------------------------

void CFormView::Draw(Environment *ev, ODFacet* odFacet, ODShape* invalidShape)
{
	// Set up drawing context for this view
	FW_CViewContext vc(ev, this, odFacet, invalidShape);

	// Erase with white first 
	FW_CRect invalidRect;
	vc.GetClipRect(invalidRect);
	FW_CRectShape::RenderRect(vc, invalidRect, FW_kFill, FW_kWhiteEraseInk);

	// Draw the picture
	fFormPart->GetPictShape()->Render(vc);
}

//----------------------------------------------------------------------------------------
// CFormView::DoMouseDown
//----------------------------------------------------------------------------------------
// Put up an Alert "not implemented yet" when clicking inside fake gadgets in the picture

FW_Boolean CFormView::DoMouseDown(Environment *ev, const FW_CMouseEvent& theMouseEvent)
{
	FW_CString32 name("ODFForm");
	FW_CString32 sorry("Gadget not yet implemented!");

	// Get mouse poition in Content coordinates
	FW_CPoint mouseWhere(theMouseEvent.GetMousePosition(ev, FW_CMouseEvent::kFrame));
	FW_CFrame* frame = FW_CFrame::ODtoFWFrame(ev, theMouseEvent.GetFacet(ev)->GetFrame(ev));
	frame->GetContentView(ev)->FrameToViewContent(ev, mouseWhere);
	
	// Set array of rectangles for all the fake gadgets
	FW_CRect gadgetRects[7] = {
			FW_CRect(FW_IntToFixed(290), FW_IntToFixed(90), FW_IntToFixed(480), FW_IntToFixed(200)),
			FW_CRect(FW_IntToFixed(30), FW_IntToFixed(320), FW_IntToFixed(40), FW_IntToFixed(330)),
			FW_CRect(FW_IntToFixed(30), FW_IntToFixed(395), FW_IntToFixed(180), FW_IntToFixed(440)),
			FW_CRect(FW_IntToFixed(25), FW_IntToFixed(485), FW_IntToFixed(230), FW_IntToFixed(505)),
			FW_CRect(FW_IntToFixed(25), FW_IntToFixed(550), FW_IntToFixed(280), FW_IntToFixed(570)),
			FW_CRect(FW_IntToFixed(25), FW_IntToFixed(600), FW_IntToFixed(150), FW_IntToFixed(725)),
			FW_CRect(FW_IntToFixed(25), FW_IntToFixed(755), FW_IntToFixed(70), FW_IntToFixed(785))};
	
	for (int i = 0; i < 7; i++) {
		if (gadgetRects[i].Contains(mouseWhere)) {
			FW_CAlert::DoNote(name, sorry);
			break;
		}
	}
	
	return TRUE;
}
