//========================================================================================
//
//	File:				MovieFra.h
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1993, 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef MOVIEFRA_H
#define MOVIEFRA_H

#ifndef MOVIEDEF_H
#include "MovieDef.h"
#endif

// ----- Framework Includes -----

#ifndef FWFRAME_H
#include "FWFrame.h"
#endif

//========================================================================================
// Forward declarations
//========================================================================================

#if FW_LIB_EXPORT_PRAGMAS
#pragma import on
#endif
struct FW_CRect;
class FW_CLASS_ATTR FW_CFacetContext;
class FW_CLASS_ATTR FW_CMenuEvent;
class FW_CLASS_ATTR FW_CMenuBar;
class FW_CLASS_ATTR FW_CPart;
#if FW_LIB_EXPORT_PRAGMAS
#pragma import off
#endif

class FW_CLASS_ATTR CMoviePart;
class FW_CLASS_ATTR CMovie;

//========================================================================================
// CLASS CMovieFrame
//========================================================================================

class FW_CLASS_ATTR CMovieFrame : public FW_CFrame
{
//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
public:
	CMovieFrame(Environment* ev, ODFrame* odFrame, FW_CPresentation* presentation, 
							FW_CPart* part, CMovie* theMovie);
	virtual ~CMovieFrame();

//----------------------------------------------------------------------------------------
//	Inherited API
//
public:
	virtual void 			Draw(Environment* ev, ODFacet* odFacet, ODShape* invalidShape);

	virtual void 			FacetAdded(Environment* ev, ODFacet* facet);

	virtual void 			FrameShapeChanged(Environment* ev);
	virtual void 			GeometryChanged(Environment *ev,
										ODFacet* odFacet,
										FW_Boolean clipShapeChanged,
										FW_Boolean externalTransformChanged);
								
	virtual void 			FocusStateChanged(Environment* ev, 
										ODTypeToken focus, 
										FW_Boolean newState, 
										ODFrame* newOwner);
	virtual FW_Boolean 		DoAdjustMenus(Environment* ev, 
										FW_CMenuBar* menuBar, 
										FW_Boolean hasMenuFocus, 
										FW_Boolean isRoot);
	virtual FW_Boolean		DoMenu(Environment* ev, const FW_CMenuEvent& theMenuEvent);
	
	virtual FW_Boolean 		DoMouseDown(Environment* ev, const FW_CMouseEvent& theMouseEvent);
	virtual FW_Boolean 		DoVirtualKeyDown(Environment* ev, const FW_CVirtualKeyEvent& theVirtualKeyEvent);
	
	//--- Drag and Drop ------
	virtual ODDragResult 	CanAcceptDrop(Environment *ev, ODDragItemIterator* dragInfo);
	
//----------------------------------------------------------------------------------------
//	New API
//
	CMoviePart* 			GetMoviePart(Environment* ev) const;
	CMovie* 				GetMovie(Environment* ev) const;
	
	void 					AdjustFrameSize(Environment* ev, const FW_CRect& movieBox);
	void 					AdoptNewMovie(Environment* ev, CMovie* newMovie);
	void 					UpdateMovie(Environment* ev);
	
	void 					ToggleLooping(Environment* ev, FW_Boolean looping);
	void 					ToggleController(Environment* ev, FW_Boolean showController);
	
	void 					CloseSelection(Environment* ev);
	void 					ClearSelection(Environment* ev);
	void 					SelectAll(Environment* ev);
	FW_Boolean 				IsSelectionEmpty(Environment* ev);

	void 					PasteMovieToSelection(Environment* ev, CMovie* newMovie);
	void 					PasteHandleToSelection(Environment* ev, FW_PlatformHandle newHandle, OSType handleType);

private:
	CMovie*					fMovie;
};

//----------------------------------------------------------------------------------------
// CMovieFrame::GetMoviePart
//----------------------------------------------------------------------------------------

inline CMoviePart* CMovieFrame::GetMoviePart(Environment* ev) const
{
	return (CMoviePart*)this->GetPart(ev);
}

//----------------------------------------------------------------------------------------
// CMovieFrame::GetMovie
//----------------------------------------------------------------------------------------

inline CMovie* CMovieFrame::GetMovie(Environment* ev) const
{
	return fMovie;
}
#endif
