//========================================================================================
//
//	File:				TblTrckr.h
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef TBLTRCKR_H
#define TBLTRCKR_H

#ifndef TBLPART_H
#include "TblPart.h"
#endif

// ----- ODF Includes -----

#ifndef FWTRACKR_H
#include "FWTrackr.h"
#endif

// ----- OS Includes -----

#ifndef FWLINSHP_H
#include "FWLinShp.h"
#endif

#ifndef FWRECT_H
#include "FWRect.h"
#endif

//========================================================================================
//	Forward declarations
//========================================================================================

#if FW_LIB_EXPORT_PRAGMAS
#pragma import on
#endif
class FW_CLASS_ATTR FW_CFacetContext;
#if FW_LIB_EXPORT_PRAGMAS
#pragma import off
#endif

class FW_CLASS_ATTR CTablePart;
class FW_CLASS_ATTR CTableFrame;
class FW_CLASS_ATTR CTableSelection;

//========================================================================================
//	class CTableDropTracker
//========================================================================================

class FW_CLASS_ATTR CTableDropTracker : public FW_CDropTracker
{
//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
public:
	CTableDropTracker(Environment* ev, 
				CTablePart* part,
				CTableFrame* frame,
				ODFacet* facet,
				const CCell& sourceCell);
	virtual ~CTableDropTracker();
	
//----------------------------------------------------------------------------------------
//	New API
//
public:	
	// ----- Tracking -----
	virtual FW_CPoint 	BeginTracking(Environment* ev, 
							const FW_CPoint& anchorPoint);
	virtual FW_CPoint 	ContinueTracking(Environment* ev,
							const FW_CPoint& anchorPoint, 
							const FW_CPoint& previousPoint, 
							const FW_CPoint& currentPoint);

private:
	void	Hilite(Environment* ev, const CCell& cell, ODFacet* facet);
	
//----------------------------------------------------------------------------------------
//	Data Members
//
private:
	CTableFrame*		fTableFrame;
	CTablePart*			fTablePart;
	CCell				fSourceCell;
	CCell				fCurCell;
	FW_Boolean			fHiliteOn;
};

//========================================================================================
//	class CCrossShape
//========================================================================================

class CCrossShape
{
//---------------------------------------------------------------------------------------
//	Constructors/Destructors
//
  public:
	CCrossShape(const FW_CPoint& brCell, FW_CPoint& brFrm);

//---------------------------------------------------------------------------------------
//	Public API
//
  public:
	void	SetHLineToMove();
	void	SetVLineToMove();

	// ----- Rendering -----
	void	Render(FW_CGraphicContext& graphicContext);

	// ----- Transform -----
	void	MoveShape(FW_CFixed deltaX, FW_CFixed deltaY);

//---------------------------------------------------------------------------------------
//	Data Members
//
  private:
	FW_CLineShape		vLine;
	FW_CLineShape		hLine;
};

//========================================================================================
//	class CGridLineTracker
//========================================================================================

class FW_CLASS_ATTR CGridLineTracker : public FW_CTracker
{
  public:
	CGridLineTracker(Environment* ev, 
					 FW_CView* view,
					 ODFacet* facet,
					 ETableLoc tl,
					 FW_CRect dragArea,
					 FW_CPoint borders);
	virtual ~CGridLineTracker();

//----------------------------------------------------------------------------------------
//	Required Overrides
//
  public:
	virtual FW_CPoint 	BeginTracking(Environment* ev, 
									const FW_CPoint& anchorPoint);
	virtual FW_CPoint 	ContinueTracking(Environment* ev,
									const FW_CPoint& anchorPoint, 
									const FW_CPoint& previousPoint, 
									const FW_CPoint& currentPoint);
	virtual FW_Boolean 	EndTracking(Environment* ev,
									const FW_CPoint& anchorPoint, 
									const FW_CPoint& lastPoint);

//----------------------------------------------------------------------------------------
//	Accessor Functions
//
  public:
	const FW_CPoint&	GetOldBorders();
	const FW_CPoint&	GetNewBorders();

//----------------------------------------------------------------------------------------
//	Data Members
//
  private:
	FW_Boolean			fHorizMove;		// resize horizontally?
	FW_Boolean			fVertMove;		// resize vertically?
	FW_CRect			fDragArea;		// permissible drag area
	FW_CPoint			fOldBorders;
	FW_CPoint			fNewBorders;
	CCrossShape			fCross;			// cursor
};

//========================================================================================
//	CGridLineTracker Inlines
//========================================================================================

inline const FW_CPoint& CGridLineTracker::GetOldBorders()
{ 
	return fOldBorders; 
}

inline const FW_CPoint& CGridLineTracker::GetNewBorders()
{ 
	return fNewBorders; 
}

#endif
