//========================================================================================
//
//	File:				TblProxy.cpp
//	Release Version:	$ 1.0d11 $ 
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "Table.hpp"

// ----- TablePart Includes -----

#ifndef TBLPROXY_H
#include "TblProxy.h"
#endif

#ifndef TBLPART_H
#include "TblPart.h"
#endif

#ifndef TBLFRAME_H
#include "TblFrame.h"
#endif

// ----- Framework Includes -----

#ifndef FWFRMING_H
#include "FWFrming.h"
#endif

#ifndef FWITERS_H
#include "FWIters.h"
#endif

#ifndef FWUTIL_H
#include "FWUtil.h"
#endif

#ifndef FWFCTCLP_H
#include "FWFctClp.h"
#endif

#ifndef FWPXYFRM_H
#include "FWPxyFrm.h"
#endif

// ----- OS Includes -----

#ifndef FWODGEOM_H
#include "FWODGeom.h"
#endif

#ifndef FWFXMATH_H
#include "FWFxMath.h"
#endif

//========================================================================================
// RunTime Info
//========================================================================================

#ifdef FW_BUILD_MAC
#pragma segment odfTable
#endif

//========================================================================================
//	class CTableProxy
//========================================================================================

//----------------------------------------------------------------------------------------
//	CTableProxy::CTableProxy
//----------------------------------------------------------------------------------------

CTableProxy::CTableProxy(Environment* ev, CTablePart* tablePart, FW_CPresentation* presentation) :
	FW_MProxy(ev, tablePart, presentation),
	fTablePart(tablePart),
	fCell(0, 0)
{
	SetSelectState(ev, TRUE);
}

//----------------------------------------------------------------------------------------
//	CTableProxy::~CTableProxy
//----------------------------------------------------------------------------------------

CTableProxy::~CTableProxy()
{
}

//----------------------------------------------------------------------------------------
//	CTableProxy::UsedShapeChanged
//----------------------------------------------------------------------------------------

void CTableProxy::UsedShapeChanged(Environment *ev,
								   FW_CEmbeddingFrame* embeddingFrame,
								   ODFrame* odEmbeddedFrame)
{
	FW_CRect cellRect;
	fTablePart->FindRect(fCell, cellRect);
	FW_CAcquiredODShape aqInvalidShape = ::FW_NewODShape(ev, cellRect);

	FW_CAcquiredODShape aqUsedShape(FW_CopyAndRelease(ev, odEmbeddedFrame->AcquireUsedShape(ev, NULL)));
		
	// ----- For the Table part, all embedded facets have the same externalTransform
	ODFacet *facet;
	{
		FW_CODFrameFacetIterator ite(ev, odEmbeddedFrame);
		facet = ite.First(ev);
	}
	if (facet == NULL)
		return;
	
	{
		FW_CAcquiredODTransform aqExternalTransform(facet->AcquireExternalTransform(ev, NULL));
		aqUsedShape->Transform(ev, aqExternalTransform);
		aqInvalidShape->Subtract(ev, aqUsedShape);
	}
	
	// ----- Invalidate
	embeddingFrame->Invalidate(ev, aqInvalidShape);
	
	// ----- Recalculate the clip
	FW_CFacetClipper facetClipper(ev, fTablePart);
	facetClipper.Clip(ev, GetPresentation(ev), NULL);
}

//----------------------------------------------------------------------------------------
//	CTableProxy::FrameShapeRequested
//----------------------------------------------------------------------------------------

ODShape* CTableProxy::FrameShapeRequested(Environment* ev, 
										ODFrame* odEmbeddedFrame, 
										ODShape* askedFrameShape)
{
FW_UNUSED(askedFrameShape);

	FW_CRect cellRect;
	fTablePart->FindRect(fCell, cellRect);

	cellRect.Place(FW_kZeroPoint);
	ODShape* cellShape = ::FW_NewODShape(ev, cellRect);
	
	return cellShape;
}

//----------------------------------------------------------------------------------------
//	CTableProxy::MoveEmbeddedFrames
//----------------------------------------------------------------------------------------

void CTableProxy::MoveEmbeddedFrames(Environment* ev, const CCell& destCell)
{
	FW_CRect rect;
	fTablePart->FindRect(destCell, rect);
	ChangeFrameShapes(ev, rect.Width(), rect.Height());
	ChangeExternalTransforms(ev, rect.left, rect.top);
	
	// 	[HLX] because I can't create facets with a NULL clip (bug in OpenDoc) I have to 
	//	Clip my embedded facets everytime
	FW_CFacetClipper facetClipper(ev, fTablePart);
	facetClipper.Clip(ev, GetPresentation(ev), NULL);
}

//----------------------------------------------------------------------------------------
//	CTableProxy::ChangeLinkStatus
//----------------------------------------------------------------------------------------

void CTableProxy::ChangeLinkStatus(Environment* ev, ODLinkStatus linkStatus)
{
	FW_CProxyProxyFrameIterator iter(this);
	for (FW_CProxyFrame* frame = iter.First(); iter.IsNotComplete(); frame = iter.Next())
	{
		if (linkStatus == kODInLinkSource)
		{	// if frame is already in a destination, leave its status alone
			if (frame->GetFrame(ev)->GetLinkStatus(ev) != kODInLinkDestination)
				frame->GetFrame(ev)->ChangeLinkStatus(ev, linkStatus);
		}
		else
			frame->GetFrame(ev)->ChangeLinkStatus(ev, linkStatus);
	}
}
