/*
	File:		FWDictLi.h

	Contains:	Definition of class FW_CPrivDictionaryList

	Written by:	Richard Rodseth

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <1>	10/24/94	jpa		first checked in
		 <6>	 8/17/94	jpa		Added DeleteSOMValues method to delete them
									properly [1181515]
		 <5>	  6/9/94	RR		Remove ASLM stuff
		 <4>	  6/1/94	RR		Include PlfmDef.h
		 <2>	 5/10/94	RR		Removed ASLM_COMPATIBLE/CDECL
		 <1>	  5/5/94	CG		first checked in
		 <6>	 3/15/94	MB		Changes to support SCpp/ASLM builds,
									#1150864.
		 <5>	  2/7/94	JA		Tiger Team Makeover!
		 <4>	 11/4/93	RR		Moved class FW_CPrivAssociation to .cp
		 <3>	10/28/93	RR		Removed ambiguous constructor
		 <1>	 8/13/93	RCR		first checked in

	To Do:
*/

#ifndef FWDICTLI_H
#define FWDICTLI_H

#ifndef FWSTDDEF_H
#include "FWStdDef.h"
#endif

#ifndef FWLIST_H
#include "FWList.h"
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

//==============================================================================
// Theory of Operation
//==============================================================================

// OrdereCollection is an ordered collection of elements of type void* (since
// we can't use templates)
// Duplicates are allowed.

//==============================================================================
// Constants
//==============================================================================

//==============================================================================
// Scalar Types
//==============================================================================

typedef void* FW_PrivKeyType;
typedef void* FW_PrivValueType;

//=====================================================================================
// Classes defined in this interface
//=====================================================================================

class FW_CPrivDictionaryList;	

//=====================================================================================
// Classes used by this interface
//=====================================================================================

//=====================================================================================
// Class FW_CPrivAssociation
//=====================================================================================

class FW_CLASS_ATTR FW_CPrivAssociation : public FW_CPrivLink
{
public:
	FW_CPrivAssociation(FW_PrivKeyType key, FW_PrivValueType value);		
	virtual ~FW_CPrivAssociation();
	FW_PrivKeyType	GetKey()	{ return fKey;}
	void		SetKey(FW_PrivKeyType key)	{ fKey = key;}
	FW_PrivValueType	GetValue()	{ return fValue;}
	void		SetValue(FW_PrivValueType value)	{ fValue = value;}

private:
	FW_PrivKeyType 		fKey;
	FW_PrivValueType 		fValue;
};
	
//=====================================================================================
// Global Variables
//=====================================================================================


//=====================================================================================
// Class FW_CPrivDictionaryList
//=====================================================================================

class FW_CLASS_ATTR FW_CPrivDictionaryList
{
public:

	FW_CPrivDictionaryList();
	virtual ~FW_CPrivDictionaryList();

	virtual void	AddPair(FW_PrivKeyType key, FW_PrivValueType value);
	
	virtual unsigned long Count();
	virtual FW_Boolean ContainsKey(FW_PrivKeyType key);
	virtual FW_PrivValueType ValueAtKey(FW_PrivKeyType key);

	virtual void	RemoveKey(FW_PrivKeyType key);
	virtual void	RemoveAll();			
	virtual void	DeleteKeys();
	virtual void	DeleteValues();
	
	FW_CPrivLinkedListIterator* CreateIterator()
		{ return new FW_CPrivLinkedListIterator(&fImplementation); }
		// This feature added by JEL for FWArchiv support.
	
	void Merge(FW_CPrivDictionaryList &other);
		// This feature added by JEL for FWArchiv support.
	
protected:

	 virtual FW_CPrivAssociation* CreateNewAssociation(FW_PrivKeyType key, FW_PrivValueType value) const;
	 virtual FW_Boolean	KeysMatch(FW_PrivKeyType key1,FW_PrivKeyType key2) const;
	 
	 	// Does a straight == comparison by default 

private:

	FW_CPrivLinkedList		fImplementation;
};

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export off
#endif

#endif // FWDICTLI_H
