//========================================================================================
//
//	File:				FWExcTas.cpp
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	(c) 1993, 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "FWFound.hpp"

#ifndef   FWEXCTAS_H
#include "FWExcTas.h"
#endif

#ifndef   FWDELSTA_H
#include "FWDelSta.h"
#endif

#ifndef   FWEXCRUN_H
#include "FWExcRun.h"
#endif

#ifndef   FWPOSTEX_H
#include "FWPostEx.h"
#endif

#ifndef   FWTRYBLO_H
#include "FWTryBlo.h"
#endif

#ifndef   FWNEWHEL_H
#include "FWNewHel.h"
#endif

#ifndef   FWPRIMEM_H
#include "FWPriMem.h"
#endif

#ifndef   FWTHRDGD_H
#include "FWThrdGd.h"
#endif

#if defined(FW_BUILD_MAC) && !defined(__GESTALTEQU__)
#include <GestaltEqu.h>
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

#ifdef FW_BUILD_MAC
#pragma segment BEL
#endif

//----------------------------------------------------------------------------------------
// FW_CExceptionTaskGlobals::gGlobals
//----------------------------------------------------------------------------------------

static FW_SPrivExceptionGlobals defaultGlobals;
FW_SPrivExceptionGlobals *FW_CExceptionTaskGlobals::gGlobals = &defaultGlobals;

//----------------------------------------------------------------------------------------
//	FW_CExceptionTaskGlobals::GetExceptionGlobals
//----------------------------------------------------------------------------------------

FW_SPrivExceptionGlobals& FW_CExceptionTaskGlobals::GetExceptionGlobals()
{
	FW_SPrivExceptionGlobals *globals = gGlobals;
	if (globals->gInitialized == 0)
		Initialize(*globals);
	return *globals;
}

//----------------------------------------------------------------------------------------
// FW_CExceptionTaskGlobals::Initialize()
//----------------------------------------------------------------------------------------

void FW_CExceptionTaskGlobals::Initialize(FW_SPrivExceptionGlobals& globals)
{
	globals.gInitialized = 1;

#ifndef FW_NATIVE_EXCEPTIONS
	FW_CPrivDeleteStack::Initialize(globals);
#endif
	
#ifdef FW_USE_NEW_HELPER
	FW_CPrivNewHelper::Initialize(globals);
#endif

#ifndef FW_NATIVE_EXCEPTIONS
	FW_CPrivExceptionRuntime::Initialize(globals);
#endif
}

//----------------------------------------------------------------------------------------
// FW_CExceptionTaskGlobals::Terminate()
//----------------------------------------------------------------------------------------
void FW_CExceptionTaskGlobals::Terminate()
{
#ifndef FW_NATIVE_EXCEPTIONS
	FW_CPrivExceptionRuntime::Terminate();
#endif
	
#ifdef FW_USE_NEW_HELPER
	FW_CPrivNewHelper::Terminate();
#endif

#ifndef FW_NATIVE_EXCEPTIONS
	FW_CPrivDeleteStack::Terminate();
#endif
}


//========================================================================================
// CLASS FW_CThreadGuardExceptionGlobals
//========================================================================================

class FW_CLASS_ATTR FW_CThreadGuardExceptionGlobals : public FW_CThreadGuard {
public:
	FW_CThreadGuardExceptionGlobals();

	virtual void Created(FW_ThreadID newlyCreatedThread);
	virtual void Terminating(FW_ThreadID threadBeingKilled);

#ifdef	FW_BUILD_MAC
	virtual void Switch(FW_ThreadID aThread, FW_Boolean switchingIn);

private:
	ThreadID fMainThread;
	FW_SPrivExceptionGlobals *fMainThreadGlobals;
#endif

private:
	static FW_CThreadGuardExceptionGlobals fgThreadGuard;
};


#ifdef	FW_BUILD_MAC
//----------------------------------------------------------------------------------------
// FW_CThreadGuardExceptionGlobals::fgThreadGuard
//	This instance is used strictly for the side-effect of its construction.  Because only 
//	the Mac has thread safety problems, only create it for the Mac.
//----------------------------------------------------------------------------------------

FW_CThreadGuardExceptionGlobals FW_CThreadGuardExceptionGlobals::fgThreadGuard;
#endif


//----------------------------------------------------------------------------------------
// FW_CThreadGuardExceptionGlobals::FW_CThreadGuardExceptionGlobals
//----------------------------------------------------------------------------------------

FW_CThreadGuardExceptionGlobals::FW_CThreadGuardExceptionGlobals() :
	FW_CThreadGuard(kSwitchInThreadEvent)
{
#ifdef	FW_BUILD_MAC
	fMainThreadGlobals = FW_CExceptionTaskGlobals::gGlobals;

	// If the thread manager is present, associate the default globals with the
	// ThreadID of the main thread.
	long response;
	OSErr anErr = Gestalt(gestaltThreadMgrAttr, &response);

	if (anErr == noErr && (response & (1 << gestaltThreadMgrPresent)))
	{
		// [jkp] - Check the address.  We will use weak links in the SOM future 
		// and want to be sure that the routine was loaded.
#ifndef __SC__
		if (&GetCurrentThread != 0)
		{
			anErr = GetCurrentThread(&fMainThread);
			if (anErr != noErr)
				DebugStr("\pGetCurrentThread failed");
		}
#endif
	}
#endif
}


//----------------------------------------------------------------------------------------
// FW_CThreadGuardExceptionGlobals::Created
//----------------------------------------------------------------------------------------

void FW_CThreadGuardExceptionGlobals::Created(FW_ThreadID newlyCreatedThread)
{
#ifdef	FW_BUILD_MAC
	char *exceptionGlobals = (char *)FW_PrimitiveAllocateBlock(sizeof(FW_SPrivExceptionGlobals));
	const char *limit = exceptionGlobals + sizeof(FW_SPrivExceptionGlobals);

	AddThreadInfo(newlyCreatedThread, exceptionGlobals);

	while (exceptionGlobals < limit)
		*exceptionGlobals++ = 0;
#endif
}


//----------------------------------------------------------------------------------------
// FW_CExceptionTaskGlobals::Terminating
//----------------------------------------------------------------------------------------

void FW_CThreadGuardExceptionGlobals::Terminating(FW_ThreadID threadBeingKilled)
{
#ifdef	FW_BUILD_MAC
	if (threadBeingKilled != fMainThread)
	{
		FW_SPrivExceptionGlobals *exceptionGlobals = (FW_SPrivExceptionGlobals *)GetThreadInfo(threadBeingKilled);

		FW_PrimitiveFreeBlock(exceptionGlobals);
		RemoveThreadInfo(threadBeingKilled);
	}
#endif
}


#ifdef	FW_BUILD_MAC
//----------------------------------------------------------------------------------------
// FW_CExceptionTaskGlobals::Switch
//----------------------------------------------------------------------------------------

void FW_CThreadGuardExceptionGlobals::Switch(FW_ThreadID aThread, FW_Boolean)
{
	if (aThread == fMainThread)
	{
		FW_CExceptionTaskGlobals::gGlobals = fMainThreadGlobals;
	}
	else
	{
		FW_SPrivExceptionGlobals *exceptionGlobals = (FW_SPrivExceptionGlobals *)GetThreadInfo(aThread);
	
		if (exceptionGlobals != 0)
			FW_CExceptionTaskGlobals::gGlobals = exceptionGlobals;
		else
			DebugStr("\pNull exception globals for thread");
	}
}
#endif

