//========================================================================================
//
//	File:				FWClaInf.h
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	(c) 1993, 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWCLAINF_H
#define FWCLAINF_H

#include <stddef.h>

#ifndef FWSTDDEF_H
#include "FWStdDef.h"
#endif

#ifdef FW_BUILD_WIN
#include <Windows.h>
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

//========================================================================================
// FW_CAN_EXPORT_DATA
//========================================================================================

#ifdef FW_BUILD_MAC
#define FW_CAN_EXPORT_DATA
#endif

#ifdef FW_BUILD_WIN
// #define FW_CAN_EXPORT_DATA // No shared data for Windows (define on command line)
#endif

//========================================================================================
// CLASS FW_CClassInfo
//========================================================================================

class FW_CLASS_ATTR FW_CClassInfo;
typedef const FW_CClassInfo *FW_ClassInfoPtr;

class FW_CLASS_ATTR FW_CClassInfo
{
public:

	~FW_CClassInfo();
	FW_CClassInfo(const char *const className,
					size_t instanceSize, 
					FW_ClassInfoPtr  const *ancestors,
					const size_t * const offsets);
	
	const char *const GetClassName() const;
		// Returns a char string containing name of the class

	const size_t GetInstanceSize() const;
		// Returns the size of instances of the class

	FW_ClassInfoPtr  const * GetAncestors() const;
		// Returns the list of ancestors of the class

	long GetHashKey() const;
		// Returns the hash key for the class name

	FW_Boolean IsBaseOf(FW_ClassInfoPtr aDerivedClass) const;
		// Return true if the class is or a base of the class represented by aDerivedClass
		// Note, usage is: this->IsBaseOf(aDerivedClass)

	FW_Boolean IsKindOf(FW_ClassInfoPtr aBaseClass) const;
		// Return true if the class is or inherits from class represented by aBaseClass
		// Note, usage is: this->IsKindOf(aBaseClass)
	
	void* DynamicCast(const void* p, FW_ClassInfoPtr pointersClass, FW_ClassInfoPtr aBaseClass) const;
		// Return's nonzero pointer if p!=0 and IsKindOf(aBaseClass), otherwise return 0.
	
	int operator==(const FW_CClassInfo&) const;
	int operator!=(const FW_CClassInfo&) const;

private:

	FW_Boolean OffsetToBase(FW_ClassInfoPtr aBaseClass, size_t &offset) const;

	const char *const fClassName;
		// Name of the class

	size_t fInstanceSize;
		// Size of instances of the class
		
	FW_ClassInfoPtr  const *fAncestors;
		// ClassInfo for ancestors of the class.
		// Pointer to array of const pointers to const FW_CClassInfo
	
	const size_t *const fAncestorOffsets;
	
	FW_CClassInfo(const FW_CClassInfo&);
	FW_CClassInfo& operator=(const FW_CClassInfo&);
		// ClassInfo objects cannot be copied
		
};

//----------------------------------------------------------------------------------------
//	FW_CClassInfo::operator==
//----------------------------------------------------------------------------------------

inline int FW_CClassInfo::operator==(const FW_CClassInfo& other) const
{
	return &other == this;
}
	
//----------------------------------------------------------------------------------------
//	FW_CClassInfo::operator!=
//----------------------------------------------------------------------------------------

inline int FW_CClassInfo::operator!=(const FW_CClassInfo& other) const
{
	return &other != this;
}

//----------------------------------------------------------------------------------------
//	FW_CClassInfo::GetClassName
//----------------------------------------------------------------------------------------

inline const char *const FW_CClassInfo::GetClassName() const
{
	return fClassName;
}

//----------------------------------------------------------------------------------------
//	FW_CClassInfo::GetInstanceSize
//----------------------------------------------------------------------------------------

inline const size_t FW_CClassInfo::GetInstanceSize() const
{
	return fInstanceSize;
}

//----------------------------------------------------------------------------------------
//	FW_CClassInfo::GetAncestors
//----------------------------------------------------------------------------------------

inline FW_ClassInfoPtr  const * FW_CClassInfo::GetAncestors() const
{
	return fAncestors;
}

//----------------------------------------------------------------------------------------
//	FW_CClassInfo::IsBaseOf
//----------------------------------------------------------------------------------------

inline FW_Boolean FW_CClassInfo::IsBaseOf(FW_ClassInfoPtr aDerivedClass) const
{
	return aDerivedClass->IsKindOf(this);
}

//========================================================================================
// Macros for generating and accessing class information
//========================================================================================

#ifndef   FWCLAPRV_H
#include "FWClaPrv.h"
#endif

// ----- The public macros for declaring and defining RTTI scaffolding

#ifdef FW_CAN_EXPORT_DATA
	#define FW_DECLARE_CLASS															\
		virtual FW_ClassInfoPtr PrivVirtualGetClassInfo(void) const;					\
		FW_ClassInfoPtr PrivNonVirtualGetClassInfo(void) const							\
    		{ return &gClassInfo; }														\
		static const FW_CClassInfo FW_SHARED_MEMBER gClassInfo;							\
		static const FW_ClassInfoPtr FW_SHARED_MEMBER gAncestors[];						\
		static const size_t FW_SHARED_MEMBER gAncestorOffsets[];
#else
	#define FW_DECLARE_CLASS															\
		virtual FW_ClassInfoPtr PrivVirtualGetClassInfo(void) const;					\
		FW_ClassInfoPtr PrivNonVirtualGetClassInfo(void) const;							\
		static const FW_CClassInfo& PrivStaticGetClassInfo();							\
		static const FW_ClassInfoPtr* PrivGetAncestors();								\
		static const size_t* PrivGetAncestorOffsets();									\
private:																				\
		static const FW_CClassInfo gClassInfo;											\
		static const FW_ClassInfoPtr gAncestors[];										\
		static const size_t gAncestorOffsets[];											\
public:
#endif
    
#define FW_DEFINE_CLASS_M0(name)														\
	_FW_CLASS_ANCESTORS_IMPLEMENT_M0(name)												\
	_FW_DEFINE_CLASSINFO_OBJECT(name)													\
	_FW_CLASSINFO_ACCESSOR_MEMBERS(name)

#define FW_DEFINE_CLASS_M1(name, ancestor)												\
	_FW_CLASS_ANCESTORS_IMPLEMENT_M1(name, ancestor)									\
	_FW_DEFINE_CLASSINFO_OBJECT(name)													\
	_FW_CLASSINFO_ACCESSOR_MEMBERS(name)

#define FW_DEFINE_CLASS_M2(name, ancestor1, ancestor2)									\
	_FW_CLASS_ANCESTORS_IMPLEMENT_M2(name, ancestor1, ancestor2)						\
	_FW_DEFINE_CLASSINFO_OBJECT(name)													\
	_FW_CLASSINFO_ACCESSOR_MEMBERS(name)

#define FW_DEFINE_CLASS_M3(name, ancestor1, ancestor2, ancestor3)						\
	_FW_CLASS_ANCESTORS_IMPLEMENT_M3(name, ancestor1, ancestor2, ancestor3)				\
	_FW_DEFINE_CLASSINFO_OBJECT(name)													\
	_FW_CLASSINFO_ACCESSOR_MEMBERS(name)

#define FW_DEFINE_CLASS_M4(name, ancestor1, ancestor2, ancestor3, ancestor4)			\
	_FW_CLASS_ANCESTORS_IMPLEMENT_M4(name, ancestor1, ancestor2, ancestor3, ancestor4)	\
	_FW_DEFINE_CLASSINFO_OBJECT(name)													\
	_FW_CLASSINFO_ACCESSOR_MEMBERS(name)

//========================================================================================
// RTTI Portability Macros
//
//	These macros provide a transition path until all compilers support RTTI.
//========================================================================================

//----------------------------------------------------------------------------------------
// Dynamic Cast Macro
//----------------------------------------------------------------------------------------

#ifdef FW_COMPILER_SUPPORTS_RTTI
	#define FW_DYNAMIC_CAST(T, p) dynamic_cast<T*>(p)
#else
	#define FW_DYNAMIC_CAST(T, p) \
		( (p) ? (T*)(FW_TYPEID_FROM_POINTER(p).DynamicCast(p, (p)->PrivNonVirtualGetClassInfo(), FW_CLASS_INFO_PTR(T))) : (T*) 0)
#endif

//----------------------------------------------------------------------------------------
// typeid macros
//----------------------------------------------------------------------------------------

// Note, C++ typeid's are TypeInfo *references*.
// ODF TYPEIDs are FW_ClassInfo *references*
// Internally, ODF usually uses FW_ClassInfoPtrs, which are are FW_ClassInfo *pointers*.

#ifdef FW_COMPILER_SUPPORTS_RTTI
	#define FW_TYPEID_FROM_TYPE(type) (typeid(type))
	#define FW_TYPEID_FROM_POINTER(p) (typeid(*(p)))
#else
	#define FW_TYPEID_FROM_TYPE(type) (*FW_CLASS_INFO_PTR(type))
	#define FW_TYPEID_FROM_POINTER(p) (*(p)->PrivVirtualGetClassInfo())
#endif

//----------------------------------------------------------------------------------------
// Class name macros
//----------------------------------------------------------------------------------------

#ifdef FW_COMPILER_SUPPORTS_RTTI
	#define FW_CLASSNAME_FROM_TYPE(type) (typeid(type).name())
	#define FW_CLASSNAME_FROM_POINTER(p) (typeid(*(p)).name())
#else
	#define FW_CLASSNAME_FROM_TYPE(type) (FW_TYPEID_FROM_TYPE(type).GetClassName())
	#define FW_CLASSNAME_FROM_POINTER(p) (FW_TYPEID_FROM_POINTER(p).GetClassName())
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export off
#endif

#endif

