//========================================================================================
//
//	File:				FWBndStr.cpp
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	(c) 1993, 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "FWFound.hpp"

#ifndef FWBNDSTR_H
#include "FWBndStr.h"
#endif

#ifdef FW_BUILD_MAC
#include <Types.h>
#endif

#ifndef FWEXCLIB_H
#include "FWExcLib.h"
#endif

#ifndef FWPRIDEB_H
#include "FWPriDeb.h"
#endif

#ifndef FWSTRTOO_H
#include "FWStrToo.h"
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

#ifdef FW_BUILD_MAC
#pragma segment Strings
#endif

//========================================================================================
//	CLASS FW_CBoundedString
//========================================================================================

//----------------------------------------------------------------------------------------
//	FW_CBoundedString::~FW_CBoundedString
//----------------------------------------------------------------------------------------

FW_CBoundedString::~FW_CBoundedString()
{
}

//----------------------------------------------------------------------------------------
//	FW_CBoundedString::FW_CBoundedString
//----------------------------------------------------------------------------------------

FW_CBoundedString::FW_CBoundedString(FW_ByteCount stringSize, FW_Byte* storagePointer) :
	FW_CString(),
	fStringSize(stringSize)
{
	fRepresentation = storagePointer;
	fCapacity = stringSize * sizeof(FW_Char);
	PrivSetLength(0, 0);
}

//----------------------------------------------------------------------------------------
//	FW_CBoundedString::FW_CBoundedString
//----------------------------------------------------------------------------------------

FW_CBoundedString::FW_CBoundedString(FW_ByteCount charWidth, FW_ByteCount stringSize, FW_Byte* storagePointer) :
	FW_CString(charWidth),
	fStringSize(stringSize)
{
	fRepresentation = storagePointer;
	fCapacity = stringSize * sizeof(FW_Char);
	PrivSetLength(0, 0);
}

//----------------------------------------------------------------------------------------
//	FW_CBoundedString::GrowCapacity
//----------------------------------------------------------------------------------------

FW_ByteCount FW_CBoundedString::GrowCapacity(FW_ByteCount capacityNeeded)
{
	FW_UNUSED(capacityNeeded);

	return fStringSize * sizeof(FW_Char);
}

//========================================================================================
//	CLASS FW_CString32
//========================================================================================

//----------------------------------------------------------------------------------------
//	FW_CString32::FW_CString32
//----------------------------------------------------------------------------------------

FW_CString32::FW_CString32() :
	FW_CBoundedString(32, fStorage)
{
}

//----------------------------------------------------------------------------------------
//	FW_CString32::FW_CString32
//----------------------------------------------------------------------------------------

FW_CString32::FW_CString32(FW_ByteCount charWidth) :
	FW_CBoundedString(charWidth, 32, fStorage)
{
}

//----------------------------------------------------------------------------------------
//	FW_CString32::FW_CString32
//----------------------------------------------------------------------------------------

FW_CString32::FW_CString32(const FW_CString32 &string) :
	FW_CBoundedString(string.GetCharWidth(), 32, fStorage)
{
	Append(string);
}

//----------------------------------------------------------------------------------------
//	FW_CString32::FW_CString32
//----------------------------------------------------------------------------------------

FW_CString32::FW_CString32(const FW_CString &string) :
	FW_CBoundedString(string.GetCharWidth(), 32, fStorage)
{
	Append(string);
}

//----------------------------------------------------------------------------------------
//	FW_CString32::FW_CString32
//----------------------------------------------------------------------------------------

FW_CString32::FW_CString32(const FW_Char *items, FW_CharacterCount numberItems) :
	FW_CBoundedString(32, fStorage)
{
	Append(items, numberItems);
}

//----------------------------------------------------------------------------------------
//	FW_CString32::FW_CString32
//----------------------------------------------------------------------------------------

FW_CString32::FW_CString32(const FW_Char *items) :
	FW_CBoundedString(32, fStorage)
{
	Append(items, FW_StringLength(items));
}

//----------------------------------------------------------------------------------------
//	FW_CString32::~FW_CString32
//----------------------------------------------------------------------------------------

FW_CString32::~FW_CString32()
{
}

//========================================================================================
//	CLASS FW_CString255
//========================================================================================

//----------------------------------------------------------------------------------------
//	FW_CString255::FW_CString255
//----------------------------------------------------------------------------------------

FW_CString255::FW_CString255() :
	FW_CBoundedString(255, fStorage)
{
}

//----------------------------------------------------------------------------------------
//	FW_CString255::FW_CString255
//----------------------------------------------------------------------------------------

FW_CString255::FW_CString255(FW_ByteCount charWidth) :
	FW_CBoundedString(charWidth, 255, fStorage)
{
}

//----------------------------------------------------------------------------------------
//	FW_CString255::FW_CString255
//----------------------------------------------------------------------------------------

FW_CString255::FW_CString255(const FW_CString255 &string) :
	FW_CBoundedString(string.GetCharWidth(), 255, fStorage)
{
	Append(string);
}

//----------------------------------------------------------------------------------------
//	FW_CString255::FW_CString255
//----------------------------------------------------------------------------------------

FW_CString255::FW_CString255(const FW_CString &string) :
	FW_CBoundedString(string.GetCharWidth(), 255, fStorage)
{
	Append(string);
}

//----------------------------------------------------------------------------------------
//	FW_CString255::FW_CString255
//----------------------------------------------------------------------------------------

FW_CString255::FW_CString255(const FW_Char *items, FW_CharacterCount numberItems) :
	FW_CBoundedString(255, fStorage)
{
	Append(items, numberItems);
}

//----------------------------------------------------------------------------------------
//	FW_CString255::FW_CString255
//----------------------------------------------------------------------------------------

FW_CString255::FW_CString255(const FW_Char *items) :
	FW_CBoundedString(255, fStorage)
{
	Append(items, FW_StringLength(items));
}

//----------------------------------------------------------------------------------------
//	FW_CString255::~FW_CString255
//----------------------------------------------------------------------------------------

FW_CString255::~FW_CString255()
{
}

