//========================================================================================
//
//	File:				FWContrH.h
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#if	defined(FW_BUILD_MAC) && !defined(FWCONTRH_H)
#define FWCONTRH_H

// ----- OS Layer -----

#ifndef FWGRDEF_H
#include "FWGrDef.h"
#endif

#ifndef FWRECT_H
#include "FWRect.h"
#endif

// ----- Macintosh Includes -----

#if defined(FW_BUILD_MAC) && !defined(__CONTROLS__)
#include <Controls.h>
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

//========================================================================================
// Forward declarations
//========================================================================================

class FW_CLASS_ATTR FW_CGadget;
class FW_CLASS_ATTR FW_CScrollBar;
class FW_CLASS_ATTR FW_CPushButton;
class FW_CLASS_ATTR FW_CRadioButton;
class FW_CLASS_ATTR FW_CString;
class FW_CLASS_ATTR FW_SPrivControlOwner;

//========================================================================================
//	CLASS FW_CPrivMacControlHelper
//========================================================================================

class FW_CPrivMacControlHelper
{
public:
	FW_DECLARE_CLASS

// ----- Initialization/destruction
public:
							FW_CPrivMacControlHelper(Environment* ev,
													const FW_CClassInfo& classInfo,
													FW_CGadget* gadget);
	virtual				~FW_CPrivMacControlHelper();
	
	void					CheckForInitialize(Environment* ev, const FW_CClassInfo& classInfo);
	virtual void			Initialize(Environment* ev);

public:
	static FW_CPrivMacControlHelper *ControlHandleToControlMgrHelper(ControlHandle handle);
	
	ControlHandle			GetControlHandle() const;
									
	void					SetControlMargin(const FW_SPlatformRect& margin);
	
	void					SetBounds(const FW_SPlatformRect& controlBounds);
	void					GetBounds(FW_SPlatformRect& controlBounds) const;
		
	void					SetValue(WindowPtr window,
									 const FW_SPlatformPoint& location,
									 const FW_SPlatformPoint& size,
									 short value);
	void					SetValue(short value, FW_Boolean drawUsingCurrentPort = FALSE);
	short					GetValue() const;
	void					SetMin(short min);
	short					GetMin() const;
	void					SetMax(short max);
	short					GetMax() const;
	void					SetText(const FW_CString& text);
	void					GetText(FW_CString& text) const;

	virtual void			Draw(WindowPtr window, 
									const FW_SPlatformPoint& location,
									const FW_SPlatformPoint& size);
	virtual void 			Show(WindowPtr window, 
									const FW_SPlatformPoint& location,
									const FW_SPlatformPoint& size);
	virtual void 			Hide(WindowPtr window, 
									const FW_SPlatformPoint& location,
								 	const FW_SPlatformPoint& size);
	virtual Boolean			MouseDown(WindowPtr window,
									const FW_SPlatformPoint& whereMouse,
									const FW_SPlatformPoint& controlLocation,
									const FW_SPlatformPoint& controlSize);
	FW_CGadget*				GetGadget() const;

protected:
	virtual ControlHandle	CreateControlHandle(WindowPtr window,
												const FW_SPlatformPoint& location,
												const FW_SPlatformPoint& size) = 0;	
	void					SetupControl(WindowPtr window,
										 const FW_SPlatformPoint& location,
										 const FW_SPlatformPoint& size);
	void					RestoreControl();

protected:
	FW_CGadget*				fGadget;
	ControlHandle			fControlHandle;
	FW_SPrivControlOwner*	fControlOwnerHead;
	FW_SPlatformRect		fControlMargin;

private:
	static WindowPtr		gScratchWindow;

};

//----------------------------------------------------------------------------------------
// FW_CPrivMacControlHelper::GetControlHandle
//----------------------------------------------------------------------------------------

inline ControlHandle FW_CPrivMacControlHelper::GetControlHandle() const
{
	return fControlHandle;
}

//----------------------------------------------------------------------------------------
// FW_CPrivMacControlHelper::GetBounds
//----------------------------------------------------------------------------------------

inline void FW_CPrivMacControlHelper::GetBounds(FW_SPlatformRect& controlBounds) const
{
	controlBounds = (*fControlHandle)->contrlRect;
}

//----------------------------------------------------------------------------------------
// FW_CPrivMacControlHelper::GetValue
//----------------------------------------------------------------------------------------

inline short FW_CPrivMacControlHelper::GetValue() const
{
	return ::GetControlValue(fControlHandle);
}

//----------------------------------------------------------------------------------------
// FW_CPrivMacControlHelper::GetMin
//----------------------------------------------------------------------------------------

inline short FW_CPrivMacControlHelper::GetMin() const
{
	return ::GetControlMinimum(fControlHandle);
}

//----------------------------------------------------------------------------------------
// FW_CPrivMacControlHelper::GetMax
//----------------------------------------------------------------------------------------

inline short FW_CPrivMacControlHelper::GetMax() const
{
	return ::GetControlMaximum(fControlHandle);
}

//----------------------------------------------------------------------------------------
// FW_CPrivMacControlHelper::GetGadget
//----------------------------------------------------------------------------------------

inline FW_CGadget* FW_CPrivMacControlHelper::GetGadget() const
{
	return fGadget;
}

//========================================================================================
// CLASS FW_CPrivMacScrollBarHelper
//========================================================================================

class FW_CPrivMacScrollBarHelper : public FW_CPrivMacControlHelper
{
public:
	FW_DECLARE_CLASS

public:
							FW_CPrivMacScrollBarHelper(Environment* ev,
														const FW_CClassInfo& classInfo,
													   	FW_CScrollBar* scrollBar);
	virtual 				~FW_CPrivMacScrollBarHelper();

	virtual Boolean		MouseDown(WindowPtr window,
									  const FW_SPlatformPoint& whereMouse,
									  const FW_SPlatformPoint& controlLocation,
									  const FW_SPlatformPoint& controlSize);

protected:
	virtual ControlHandle	CreateControlHandle(WindowPtr window,
												const FW_SPlatformPoint& location,
												const FW_SPlatformPoint& size);	
private:
	static pascal void	CallActionProc(ControlHandle controlHandle,
										   short partCode);
	virtual void			ActionProc(short partCode);	
};

//========================================================================================
// CLASS FW_CPrivMacPushButtonHelper
//========================================================================================

class FW_CPrivMacPushButtonHelper : public FW_CPrivMacControlHelper
{
public:
	FW_DECLARE_CLASS

public:
							FW_CPrivMacPushButtonHelper(Environment* ev,
														const FW_CClassInfo& classInfo,
														FW_CPushButton* button);
	virtual 				~FW_CPrivMacPushButtonHelper();

	virtual Boolean		MouseDown(WindowPtr window,
									  const FW_SPlatformPoint& whereMouse,
									  const FW_SPlatformPoint& controlLocation,
									  const FW_SPlatformPoint& controlSize);

	void					MakeDefaultButton(FW_Boolean isDefault);

protected:
	virtual ControlHandle	CreateControlHandle(WindowPtr window,
												const FW_SPlatformPoint& location,
												const FW_SPlatformPoint& size);	
};

//========================================================================================
// CLASS FW_CPrivMacRadioButtonHelper
//========================================================================================

class FW_CPrivMacRadioButtonHelper : public FW_CPrivMacControlHelper
{
public:
	FW_DECLARE_CLASS

public:
							FW_CPrivMacRadioButtonHelper(Environment* ev,
														const FW_CClassInfo& classInfo,
														FW_CRadioButton* button);
	virtual 				~FW_CPrivMacRadioButtonHelper();

	virtual Boolean		MouseDown(WindowPtr window,
									  const FW_SPlatformPoint& whereMouse,
									  const FW_SPlatformPoint& controlLocation,
									  const FW_SPlatformPoint& controlSize);

	void					MakeDefaultButton(FW_Boolean isDefault);

protected:
	virtual ControlHandle	CreateControlHandle(WindowPtr window,
												const FW_SPlatformPoint& location,
												const FW_SPlatformPoint& size);	
};

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export off
#endif

#endif
