//========================================================================================
//
//	File:				FWGadget.h
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWGADGET_H
#define FWGADGET_H

// ----- Foundation Layer -----

#ifndef FWEXCLIB_H
#include "FWExcLib.h"
#endif

#ifndef FWUTIL_H
#include "FWUtil.h"
#endif

// ----- OS Layer -----

#ifndef FWPOINT_H
#include "FWPoint.h"
#endif

#ifndef FWRECT_H
#include "FWRect.h"
#endif

// ----- Framewrk Layer -----

#ifndef FWVIEW_H
#include "FWView.h"
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

//========================================================================================
// Forward declarations
//========================================================================================

class FW_CLASS_ATTR FW_CFrame;
class FW_CLASS_ATTR FW_CGadgetIterator;
class FW_CLASS_ATTR ODShape;
class FW_CLASS_ATTR ODFacet;

//========================================================================================
// CLASS FW_CGadget
//
//		gadget "any small mechanical contrivance or device", Webster's Dictionary
//========================================================================================

class FW_CLASS_ATTR FW_CGadget :  public FW_CView
{
public:
	FW_DECLARE_CLASS

// ----- Initialization/destruction

public:
	FW_CGadget(Environment* ev, 
			 FW_CView* container, ODID id,
			 const FW_CRect& bounds,
			 int contentSpace = 0);
	FW_CGadget(Environment* ev, 
			 FW_CReadableStream& archive);
	virtual ~ FW_CGadget();

	// ----- Archiving -----
	virtual void				Flatten(FW_CWritableStream& archive) const;
	static void 				Write(FW_CWritableStream& archive, const void* Gadget);

private:
	enum { kFrameObjectRegistryId = -1 };
	
	FW_Boolean					fConstructed;
};

//========================================================================================
// CLASS FW_CGadgetInitializer
//========================================================================================
// *LSD: not used anymore
#if 0

// Note: I don't really want to have a global variable that maintains a stack of the
// instantiated Gadget initializers, but there is no way to pass in the Gadget initializer
// when creating Gadgets from an archive. I would like to have a refcon parameter passed
// into the read and write functions for when archiving.

class FW_CLASS_ATTR FW_CGadgetInitializer FW_AUTO_DESTRUCT_OBJECT
{
// ----- Initialization/destruction
public:
	FW_CGadgetInitializer(Environment* ev);
	virtual ~FW_CGadgetInitializer();
	
	virtual void AddGadget(FW_CGadget* GadgetToInitialize);
	
	static FW_CGadgetInitializer& GetGadgetInitializer();

private:
	static FW_CGadgetInitializer* fInitializerStack;

	FW_CPrivOrderedCollection*		fGadgetsToInitialize;
	FW_CGadgetInitializer*	fNextInitializer;
};
#endif // FW_CGadgetInitializer

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export off
#endif

#endif
