//========================================================================================
//
//	File:				FWPushBu.h
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWPUSHBU_H
#define FWPUSHBU_H

#ifndef FWBUTTON_H
#include "FWButton.h"
#endif

#ifndef FWRUNTYP_H
#include "FWRunTyp.h"
#endif

#ifndef FWNOTIFR_H
#include "FWNotifr.h"
#endif

#if defined(FW_BUILD_WIN) && !defined(FWCNTRHW_H)
#include "FWCntrHW.h"
#endif

#if defined(FW_BUILD_MAC) && !defined(FWCONTRH_H)
#include "FWContrH.h"
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

//========================================================================================
//	Foward class declarations
//========================================================================================

class FW_CLASS_ATTR FW_CMouseEvent;
class FW_CLASS_ATTR FW_CPrivMacPushButtonHelper;
class FW_CLASS_ATTR FW_CPrivWinPushButtonHelper;
class FW_CLASS_ATTR FW_CString;

//========================================================================================
//	CLASS FW_CPushButton
//========================================================================================

class FW_CLASS_ATTR FW_CPushButton : public FW_CButton
{
// ----- Initialization/destruction
public:
	FW_DECLARE_CLASS

	FW_CPushButton(Environment* ev, 
				   FW_CView* container, ODID id,
				   const FW_CRect& bounds,
				   const FW_CString& label);
	FW_CPushButton(Environment* ev, FW_CReadableStream& archive);
	virtual ~FW_CPushButton();

// ----- Inherited geometry API

	virtual void		LocationChanged(Environment* ev, const FW_CPoint& oldLocation);
	virtual void		SizeChanged(Environment* ev, const FW_CPoint& oldSize);

// ----- Mouse events
public:
	virtual FW_Boolean	DoMouseDown(Environment* ev, const FW_CMouseEvent& theMouseEvent);

// ----- Update
public:
	virtual void 		Draw(Environment* ev, ODFacet* facet, ODShape* invalidShape);

// ----- Button methods
	virtual void		SetDefault(Environment* ev, FW_Boolean isDefault);
	FW_Boolean			IsDefault(Environment* ev) const;
	
	// ----- Archiving -----
	virtual void		Flatten(FW_CWritableStream& archive) const;
	static void* 		Read(FW_CReadableStream& archive);

private:
	void				ButtonPressed(Environment* ev);

private:
	FW_Boolean			fIsDefault;

#ifdef FW_BUILD_MAC
	FW_CPrivMacPushButtonHelper*	fButtonHelper;
#endif

#ifdef FW_BUILD_WIN
	FW_CPrivWinPushButtonHelper*	fButtonHelper;
#endif

	friend FW_CLASS_ATTR FW_CPrivMacPushButtonHelper;
	friend FW_CLASS_ATTR FW_CPrivWinPushButtonHelper;
};

//----------------------------------------------------------------------------------------
// FW_CPushButton::IsDefault
//----------------------------------------------------------------------------------------

inline FW_Boolean FW_CPushButton::IsDefault(Environment* ev) const
{
	return fIsDefault;
}

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export off
#endif

#endif
