//========================================================================================
//
//	File:				FWRadioB.h
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWRADIOB_H
#define FWRADIOB_H

#ifndef FWBUTTON_H
#include "FWButton.h"
#endif

#ifndef FWRUNTYP_H
#include "FWRunTyp.h"
#endif

#ifndef FWNOTIFR_H
#include "FWNotifr.h"
#endif

#ifndef FWNOTIFN_H
#include "FWNotifn.h"
#endif

#if defined(FW_BUILD_WIN) && !defined(FWCNTRHW_H)
#include "FWCntrHW.h"
#endif

#if defined(FW_BUILD_MAC) && !defined(FWCONTRH_H)
#include "FWContrH.h"
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

//========================================================================================
//	Foward class declarations
//========================================================================================

class FW_CLASS_ATTR FW_CMouseEvent;
class FW_CLASS_ATTR FW_CPrivMacRadioButtonHelper;
class FW_CLASS_ATTR FW_CPrivWinRadioButtonHelper;
class FW_CLASS_ATTR FW_CString;

//========================================================================================
//	CLASS FW_CRadioButton
//========================================================================================

class FW_CLASS_ATTR FW_CRadioButton : public FW_CButton
{
// ----- Initialization/destruction
public:
	FW_DECLARE_CLASS

	FW_CRadioButton(Environment* ev, 
					FW_CView* container, ODID id,
					const FW_CRect& bounds,
					const FW_CString& label);
	virtual ~FW_CRadioButton();

// ----- Inherited geometry API

	virtual void		LocationChanged(Environment* ev, const FW_CPoint& oldLocation);
	virtual void		SizeChanged(Environment* ev, const FW_CPoint& oldSize);

// ----- Update
public:
	virtual void 		Draw(Environment* ev, ODFacet* facet, ODShape* invalidShape);

// ----- Mouse events
public:
	virtual FW_Boolean	DoMouseDown(Environment* ev, const FW_CMouseEvent& theMouseEvent);

// ----- Button methods
	virtual void		SetState(Environment* ev, FW_CViewContext& gc, FW_Boolean on);
	virtual void		SetState(Environment* ev, FW_Boolean on);
	FW_Boolean			GetState(Environment* ev) const;
	
private:
	void				ButtonPressed(Environment* ev);

#ifdef FW_BUILD_MAC
	FW_CPrivMacRadioButtonHelper*	fButtonHelper;
#endif

#ifdef FW_BUILD_WIN
	FW_CPrivWinRadioButtonHelper*	fButtonHelper;
#endif
	
	friend FW_CLASS_ATTR FW_CPrivMacRadioButtonHelper;
	friend FW_CLASS_ATTR FW_CPrivWinRadioButtonHelper;
};

//----------------------------------------------------------------------------------------
// FW_CRadioButton::GetState
//----------------------------------------------------------------------------------------

inline FW_Boolean FW_CRadioButton::GetState(Environment* ev) const
{
#ifdef FW_BUILD_MAC
	return fButtonHelper->GetValue() != 0;
#endif

#ifdef FW_BUILD_WIN
	return (fButtonHelper->GetState() & 0x0003) != 0;
#endif
}

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export off
#endif

#endif
