//========================================================================================
//
//	File:				FWSclBar.h
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWSCLBAR_H
#define FWSCLBAR_H

#ifndef FWGADGET_H
#include "FWGadget.h"
#endif

#ifndef FWRUNTYP_H
#include "FWRunTyp.h"
#endif

#ifndef FWNOTIFR_H
#include "FWNotifr.h"
#endif

#ifndef FWNOTIFN_H
#include "FWNotifn.h"
#endif

#if defined(FW_BUILD_WIN) && !defined(FWCNTRHW_H)
#include "FWCntrHW.h"
#endif

#if defined(FW_BUILD_MAC) && !defined(FWCONTRH_H)
#include "FWContrH.h"
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

//========================================================================================
//	Foward class declarations
//========================================================================================

class FW_CLASS_ATTR FW_CMouseEvent;
class FW_CLASS_ATTR FW_CPrivMacScrollBarHelper;
class FW_CLASS_ATTR FW_CPrivWinScrollBarHelper;
class FW_CLASS_ATTR FW_CActivateEvent;

//========================================================================================
//	CLASS FW_CScrollBar
//========================================================================================

class FW_CLASS_ATTR FW_CScrollBar : public FW_CGadget, public FW_MNotifier
{
// ----- Constants
public:
	enum { kDefaultScrollMin =  0, kDefaultScrollMax = 100, kDefaultScrollPos = 0 };
	
// ----- Initialization/destruction
	FW_DECLARE_CLASS

	FW_CScrollBar(Environment* ev, 
				  FW_CView* container, ODID id,
				  const FW_CRect& bounds);
	virtual ~FW_CScrollBar();

// ----- Inherited geometry API
	virtual void		LocationChanged(Environment *ev, const FW_CPoint& oldLocation);
	virtual void		SizeChanged(Environment *ev, const FW_CPoint& oldSize);

	void				ScrollPositionChanged(Environment *ev, FW_CFixed absolute, FW_CFixed delta, FW_Boolean shoudlScroll = TRUE);

// ----- System information
	static FW_CPoint	GetDefaultScrollBarSize();
	
// ----- Update
	virtual void 		Draw(Environment *ev, ODFacet* facet, ODShape* invalidShape);

// ----- Events
	virtual FW_Boolean	DoMouseDown(Environment *ev, const FW_CMouseEvent& theMouseEvent);
	virtual FW_Boolean	DoActivateEvent(Environment *ev, const FW_CActivateEvent& theActivateEvent);

// ----- Accessors
	virtual void		SetMinorScrollUnits(Environment *ev, FW_CFixed units);
	FW_CFixed			GetMinorScrollUnits(Environment *ev) const;
	
	virtual void		SetMajorScrollUnits(Environment *ev, FW_CFixed units);
	FW_CFixed			GetMajorScrollUnits(Environment *ev) const;

	virtual void		SetScrollMax(Environment *ev, FW_CFixed units);
	FW_CFixed			GetScrollMax(Environment *ev) const;
	
	virtual void		SetScrollMin(Environment *ev, FW_CFixed units);
	FW_CFixed			GetScrollMin(Environment *ev) const;
	
	virtual void		SetScrollPos(Environment *ev, FW_CFixed units);
	virtual void		SetScrollPos(Environment *ev, FW_CFixed range, FW_CFixed pos);
	FW_CFixed			GetScrollPos(Environment *ev) const;
	
private:
#ifdef FW_BUILD_MAC
	FW_CPrivMacScrollBarHelper*	fScrollBarHelper;
#endif

#ifdef FW_BUILD_WIN
	FW_CPrivWinScrollBarHelper*	fScrollBarHelper;
#endif
	
	FW_CFixed				fMinorScrollUnits;
	FW_CFixed				fMajorScrollUnits;
	
//	FW_CFixed				fScrollMax;
//	FW_CFixed				fScrollMin;
	
	ODTypeToken				fScrollNotificationToken;

	friend FW_CLASS_ATTR FW_CPrivMacScrollBarHelper;
	friend FW_CLASS_ATTR FW_CPrivWinScrollBarHelper;
};

//----------------------------------------------------------------------------------------
// FW_CScrollBar::GetMinorScrollUnits
//----------------------------------------------------------------------------------------

inline FW_CFixed FW_CScrollBar::GetMinorScrollUnits(Environment *ev) const
{
	return fMinorScrollUnits;
}

//----------------------------------------------------------------------------------------
// FW_CScrollBar::GetMajorScrollUnits
//----------------------------------------------------------------------------------------

inline FW_CFixed FW_CScrollBar::GetMajorScrollUnits(Environment *ev) const
{
	return fMajorScrollUnits;
}

//----------------------------------------------------------------------------------------
// FW_CScrollBar::GetScrollMax
//----------------------------------------------------------------------------------------

inline FW_CFixed FW_CScrollBar::GetScrollMax(Environment *ev) const
{
	FW_ASSERT(fScrollBarHelper);
	
#ifdef FW_BUILD_MAC
	return FW_IntToFixed(fScrollBarHelper->GetMax());
#endif

#ifdef FW_BUILD_WIN
	int min, max;
	fScrollBarHelper->GetScrollRange(min, max);
	return FW_IntToFixed(max);
#endif
}

//----------------------------------------------------------------------------------------
// FW_CScrollBar::GetScrollMin
//----------------------------------------------------------------------------------------

inline FW_CFixed FW_CScrollBar::GetScrollMin(Environment *ev) const
{
	FW_ASSERT(fScrollBarHelper);
	
#ifdef FW_BUILD_MAC
	return FW_IntToFixed(fScrollBarHelper->GetMin());
#endif

#ifdef FW_BUILD_WIN
	int min, max;
	fScrollBarHelper->GetScrollRange(min, max);
	return FW_IntToFixed(min);
#endif
}

//----------------------------------------------------------------------------------------
// FW_CScrollBar::GetScrollPos
//----------------------------------------------------------------------------------------

inline FW_CFixed FW_CScrollBar::GetScrollPos(Environment *ev) const
{
	FW_ASSERT(fScrollBarHelper);
	
#ifdef FW_BUILD_MAC
	return FW_IntToFixed(fScrollBarHelper->GetValue());
#endif

#ifdef FW_BUILD_WIN
	return FW_IntToFixed(fScrollBarHelper->GetScrollPos());
#endif
}

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export off
#endif

#endif
