//========================================================================================
//
//	File:				FWGadgCn.h
//	Release Version:	$ 1.0d10 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWVIEW_H
#define FWVIEW_H

// ----- OpenDoc -----

#ifndef _ODTYPES_
#include <ODTypes.h>
#endif

// ----- Foundation Layer -----

#ifndef FWEXCLIB_H
#include "FWExcLib.h"
#endif

#ifndef FWCLAINF_H
#include "FWClaInf.h"
#endif

// ----- OS Layer -----

#ifndef FWPOINT_H
#include "FWPoint.h"
#endif

#ifndef FWRECT_H
#include "FWRect.h"
#endif

#ifndef FWGC_H
#include "FWGc.h"
#endif

// ----- ODUtil -----

#ifndef FWORDCOL_H
#include "FWOrdCol.h"
#endif

#ifndef FWEVENTH_H
#include "FWEventH.h"
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

//========================================================================================
// Forward declarations
//========================================================================================

class FW_CLASS_ATTR FW_CFrame;
class FW_CLASS_ATTR FW_CViewIterator;
class FW_CLASS_ATTR FW_CViewContext;
class FW_CLASS_ATTR FW_CPrivOrderedCollection;
class FW_CLASS_ATTR FW_CScroller;
class FW_CLASS_ATTR FW_CScrollBar;

struct FW_CLASS_ATTR Environment;

//========================================================================================
// CLASS FW_CView
//========================================================================================

class FW_CLASS_ATTR FW_CView : public FW_MEventHandler
{
	friend FW_CLASS_ATTR FW_CViewIterator;

public:
	FW_DECLARE_CLASS

	enum CoordinateSpaceEnum { kXaxis = 1, kYaxis = 2, kScale = 4 };

//----------------------------------------------------------------------------------------
//	Initialization/destruction
//
public:
	FW_CView(Environment* ev, 
				FW_CView* container,
				const FW_CRect& bounds, 
				const FW_CPoint& extent = FW_kZeroPoint, 
				FW_CScroller* scroller = NULL,
			 	int contentSpace = 0);
	FW_CView(Environment* ev, 
				FW_CView* container,
				ODID id, 
				FW_Boolean enabled, 
				FW_Priority thePriority, 
				const FW_CRect& bounds, 
				const FW_CPoint& extent = FW_kZeroPoint, 
				FW_CScroller* scroller = NULL,
			 	int contentSpace = 0);
	virtual ~FW_CView();
	
protected:
	FW_CView(Environment* ev);	// used only by FW_CFrame

//----------------------------------------------------------------------------------------
//	Inherited API
//
public:
	virtual FW_Boolean			HandleMouseUp(Environment* ev, 
									const FW_CMouseEvent& theMouseEvent);
	virtual FW_Boolean			HandleActivateEvent(Environment* ev, 
									const FW_CActivateEvent& theActivateEvent);
	virtual FW_Boolean 			HandleSuspendResumeEvent(Environment* ev, 
									const FW_CSuspendResumeEvent& theSuspendResumeEvent);

	virtual FW_Boolean			HandleMouseDown(Environment* ev, 
									const FW_CMouseEvent& theMouseEvent);	
	virtual FW_Boolean			HandleBGMouseDown(Environment* ev, 
									const FW_CMouseEvent& theMouseEvent);
	virtual FW_Boolean			HandleMouseDownInEmbeddedFrameBorder(Environment* ev, 
									const FW_CBorderMouseEvent& theBorderMouseEvent);
	virtual FW_Boolean			HandleMouseDownInEmbeddedFrame(Environment* ev, 
									const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent);
	virtual FW_Boolean			HandleBGMouseDownInEmbeddedFrame(Environment* ev, 
									const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent);
	
	virtual FW_Boolean			DoBGMouseDown(Environment *ev, const FW_CMouseEvent& theMouseEvent);

	virtual FW_Boolean 			Activate(Environment* ev);	
	virtual void	 			Deactivate(Environment* ev);

	virtual FW_MEventHandler*	PrivGetEventHandlerContaining(Environment* ev, 
									ODFacet* odFacet, 
									const FW_CPoint& theMousePoint);
		
//----------------------------------------------------------------------------------------
//	New API
//
public:
	// ----- Getters/Setters -----
	FW_CView*					GetParentView(Environment* ev) const;
	
	FW_CRect 					GetBounds(Environment* ev) const;			// in parent view content coordinate
	FW_CRect					GetVisibleBounds(Environment* ev) const;	// in parent view content coordinate
	
	FW_CPoint 					GetExtent(Environment* ev) const;
	FW_CScroller* 				GetScroller(Environment* ev) const;
	FW_Boolean					IsContentView(Environment* ev) const;

	void	 					SetExtent(Environment* ev, const FW_CPoint& extent);
	void 						MakeContentView(Environment* ev);
	
	FW_Boolean					IsVisible(Environment* ev) const;
	void						SetVisible(Environment* ev, FW_Boolean visible, 
										FW_Boolean propagate = TRUE);
	
	// ----- Invalidate/Validate -----
	//	always in view content coordinate
	void						Invalidate(Environment* ev, const FW_CRect& rect);
	void						Validate(Environment* ev, const FW_CRect& rect);
	void						Invalidate(Environment* ev, ODShape* shape = NULL);
	void						Validate(Environment* ev, ODShape* shape = NULL);

	// ----- Geometry utilities -----
	FW_CPoint	 				GetLocation(Environment* ev) const;
	FW_CPoint	 				GetSize(Environment* ev) const;
	
	void	 					SetLocation(Environment* ev, const FW_CPoint& loc); 
	void	 					SetSize(Environment* ev, const FW_CPoint& size); 
	virtual void				LocationChanged(Environment* ev, const FW_CPoint& oldLoc);
	virtual void				SizeChanged(Environment* ev, const FW_CPoint& oldSize);
	
	FW_Boolean					IsInContentView(Environment* ev) const;
	
	FW_CView*					GetViewContaining(Environment* ev, 
									ODFacet* odFacet, 
									const FW_CPoint& theMousePoint);

	FW_Boolean					IsMouseWithin(Environment* ev, 
									ODFacet* odFacet, 
									const FW_CPoint& theMousePoint) const;
								
	// ----- Coordinates Conversion -----
	ODTransform* 				AcquireViewToFrameTransform(Environment* ev) const;
	ODTransform* 				AcquireViewContentToFrameTransform(Environment* ev) const;
	ODTransform* 				AcquireViewContentToViewTransform(Environment* ev) const;

	void						FrameToView(Environment *ev, FW_CPoint& point) const;
	void						FrameToView(Environment *ev, FW_CRect& rect) const;
	void						FrameToView(Environment *ev, ODShape* shape) const;
	void						ViewToFrame(Environment *ev, FW_CPoint& point) const;	
	void						ViewToFrame(Environment *ev, FW_CRect& rect) const;	
	void						ViewToFrame(Environment *ev, ODShape* shape) const;	

	void						FrameToViewContent(Environment *ev, FW_CPoint& point) const;
	void						FrameToViewContent(Environment *ev, FW_CRect& rect) const;
	void						FrameToViewContent(Environment *ev, ODShape* shape) const;
	void						ViewContentToFrame(Environment *ev, FW_CPoint& point) const;	
	void						ViewContentToFrame(Environment *ev, FW_CRect& rect) const;	
	void						ViewContentToFrame(Environment *ev, ODShape* shape) const;	
	
	void						ViewToViewContent(Environment *ev, FW_CPoint& point) const;
	void						ViewToViewContent(Environment *ev, FW_CRect& rect) const;
	void						ViewToViewContent(Environment *ev, ODShape* shape) const;
	void						ViewContentToView(Environment *ev, FW_CPoint& point) const;	
	void						ViewContentToView(Environment *ev, FW_CRect& rect) const;	
	void						ViewContentToView(Environment *ev, ODShape* shape) const;	
	
	// ----- Methods on subviews -----
	virtual void				CreateSubViews(Environment* ev);
	virtual void				AdjustSubViews(Environment* ev);

	void						AddSubView(Environment* ev, const FW_CView* subview);
	virtual void				SubViewAdded(Environment* ev, const FW_CView* subview);
	
	void						RemoveSubView(Environment* ev, const FW_CView* subview);
	virtual void				SubViewRemoved(Environment* ev, const FW_CView* subview);
	
	unsigned long				CountSubViews(Environment* ev) const;
	FW_Boolean					HasSubViews(Environment* ev) const;
	FW_Boolean					IsSubViewOf(Environment* ev, const FW_CView &parentView) const;
	
	// ----- Methods on Frame -----
	virtual FW_CFrame*			GetFrame(Environment* ev) const;
	FW_Boolean 					IsFrame(Environment* ev) const;
	
	// ----- Drawing -----
	virtual void				HandleDraw(Environment *ev, ODFacet* odFacet, ODShape* invalidShape);
	virtual void				Draw(Environment *ev, ODFacet* facet, ODShape* invalidShape);

	// ----- Scrolling -----
	void 						AddScrollBarScroller(Environment* ev, 
											FW_CScrollBar* horzSB, FW_CScrollBar* vertSB);

	// ----- ContentSpaceSpecifier -----
	void						SetContentSpaceInX(Environment *ev, FW_Boolean bool);
	void						SetContentSpaceInY(Environment *ev, FW_Boolean bool);
	void						SetContentSpaceInScale(Environment *ev, FW_Boolean bool);
	FW_Boolean					UseContentSpaceInX(Environment *ev) const;
	FW_Boolean					UseContentSpaceInY(Environment *ev) const;
	FW_Boolean					UseContentSpaceInScale(Environment *ev) const;
	
	// ----- Cursor Adjustment -----
	virtual FW_Boolean			AdjustCursor(Environment *ev, ODFacet* odFacet, const FW_CPoint& theMousePoint);

	
//----------------------------------------------------------------------------------------
//	Internal API
//
protected:
	// Private methods for FW_CFrame
	void						PrivSetBounds(Environment *ev, const FW_CRect& bounds);
	void						PrivSetExtent(Environment *ev, const FW_CPoint& extent);
	void						PrivSetIsContentView(Environment *ev, FW_Boolean bool);

public:
	void 						PrivInvalidateCachedTransforms(Environment* ev);

//----------------------------------------------------------------------------------------
//	Data Members
//
private:
	FW_CView*					fParentView;	// container (NULL for FW_CFrame)
	FW_CPrivOrderedCollection*	fSubViews;		// list of subviews
	FW_CRect					fBounds;		// position inside ParentView
	FW_CPoint					fExtent;		// size of view's content
	FW_CScroller*				fScroller;		 
	FW_CPrivOrderedCollection*	fAdorners;		// not implemented yet
	FW_Boolean					fVisible;
	FW_Boolean					fIsContentView;
	 
	ODTransform*				fViewContentToFrameTransform;
	ODTransform*				fViewToFrameTransform;
	ODTransform*				fViewContentToViewTransform;
	
	unsigned int				fContentSpaceSpecifier;
};

//----------------------------------------------------------------------------------------
// FW_CView inline Getters/Setters
//----------------------------------------------------------------------------------------

inline FW_CView* FW_CView::GetParentView(Environment* ev) const 
{
	return fParentView;
}

inline FW_CRect FW_CView::GetBounds(Environment* ev) const 
{
	return fBounds;
}

inline FW_CPoint FW_CView::GetExtent(Environment* ev) const 
{	
	return fExtent;
}

inline FW_CScroller* FW_CView::GetScroller(Environment* ev) const 
{
	return fScroller;
}

inline FW_CPoint FW_CView::GetLocation(Environment* ev) const 
{
	return fBounds.TopLeft();
}

inline FW_CPoint FW_CView::GetSize(Environment* ev) const 
{
	return fBounds.Size();
}

inline FW_Boolean FW_CView::IsFrame(Environment* ev) const 
{
	return (fParentView == NULL);
}

inline FW_Boolean FW_CView::IsContentView(Environment* ev) const 
{
	return fIsContentView;
}

inline FW_Boolean FW_CView::HasSubViews(Environment* ev) const
{
	return fSubViews != NULL;
}

inline	FW_Boolean	FW_CView::UseContentSpaceInX(Environment *ev) const
{
	return fContentSpaceSpecifier & FW_CView::kXaxis;
}

inline	FW_Boolean	FW_CView::UseContentSpaceInY(Environment *ev) const
{
	return fContentSpaceSpecifier & FW_CView::kYaxis;
}

inline	FW_Boolean	FW_CView::UseContentSpaceInScale(Environment *ev) const
{
	return fContentSpaceSpecifier & FW_CView::kScale;
}

inline void FW_CView::PrivSetBounds(Environment* ev, const FW_CRect& bounds)  
{
	fBounds = bounds;
}

inline void FW_CView::PrivSetExtent(Environment* ev, const FW_CPoint& extent)  
{
	fExtent = extent;
}

inline void FW_CView::PrivSetIsContentView(Environment* ev, FW_Boolean bool)  
{
	fIsContentView = bool;
}

//========================================================================================
// CLASS FW_CViewContext
//========================================================================================

class FW_CLASS_ATTR FW_CViewContext : public FW_CGraphicContext
{
// ----- Initialization/destruction
public:
					FW_CViewContext(Environment* ev,
									FW_CView* view, 
									ODFacet* facet,
									ODShape* clipShape = NULL);
	virtual			~FW_CViewContext();
	
	FW_CView*		GetView() const;
	ODFacet*		GetFacet() const;

private:
	FW_CView*			fView;
	ODFacet*			fFacet;
	ODTransform*		fTransform;

	friend FW_CLASS_ATTR FW_CView;
};

//----------------------------------------------------------------------------------------
// FW_CViewContext::GetView
//----------------------------------------------------------------------------------------

inline FW_CView* FW_CViewContext::GetView() const
{
	return fView;
}

//----------------------------------------------------------------------------------------
// FW_CViewContext::GetFacet
//----------------------------------------------------------------------------------------

inline ODFacet* FW_CViewContext::GetFacet() const
{
	return fFacet;
}


//========================================================================================
// CLASS FW_CViewIterator
//========================================================================================

// Ideally we would just subclass OrdereCollectionIterator. But then we would
// be required to call its constructor from our constructor passing it a valid
// view list which can be NULL.

class FW_CLASS_ATTR FW_CViewIterator FW_AUTO_DESTRUCT_OBJECT
{
public:
	FW_CViewIterator(Environment* ev, const FW_CView *container); 
	~ FW_CViewIterator();

	FW_CView*		First(Environment* ev)
						{return fIterator ? (FW_CView*)fIterator->First() : NULL;}
	FW_CView*		Next(Environment* ev)
						{return fIterator ? (FW_CView*)fIterator->Next() : NULL;}
	FW_CView*		Last(Environment* ev)
						{return fIterator ? (FW_CView*)fIterator->Last() : NULL;}
	FW_CView*		Previous(Environment* ev)
						{return fIterator ? (FW_CView*)fIterator->Previous() : NULL;}
	
	FW_Boolean		IsNotComplete(Environment* ev)
						{return fIterator ? fIterator->IsNotComplete() : NULL;}
	
private:
	FW_COrderedCollectionIterator *fIterator;
};

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export off
#endif

#endif
