//========================================================================================
//
//	File:				FWDrgDrp.h
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================
#ifndef FWDRGDRP_H
#define FWDRGDRP_H

// ----- Foundation Includes -----

#ifndef FWSTDDEF_H
#include "FWStdDef.h"
#endif

#ifndef FWRUNTYP_H
#include "FWRunTyp.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _ODTYPES_
#include <ODTypes.h>
#endif

#ifndef SOM_ODFacet_xh
#include <Facet.xh>
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

//========================================================================================
//	Forward Declarations
//========================================================================================

class FW_CLASS_ATTR FW_CPart;
class FW_CLASS_ATTR FW_CView;
class FW_CLASS_ATTR FW_CFrame;
class FW_CLASS_ATTR FW_CMouseEvent;
class FW_CLASS_ATTR FW_CPoint;
class FW_CLASS_ATTR FW_CDragCommand;
class FW_CLASS_ATTR FW_CDropCommand;
class FW_CLASS_ATTR FW_CDropTracker;

//========================================================================================
//	 class FW_MDragDroppable
//========================================================================================

class FW_CLASS_ATTR FW_MDragDroppable
{
public:
	FW_DECLARE_CLASS

//----------------------------------------------------------------------------------------
//	Constructor/Destructors
//
protected:
	FW_MDragDroppable(Environment *ev, FW_CPart* thePart);

public:
	virtual ~ FW_MDragDroppable();
	
//----------------------------------------------------------------------------------------
//	New API
//
public:
	virtual FW_Boolean			Drag(Environment* ev, 
									const FW_CMouseEvent& theMouseEvent);
	virtual ODDropResult 		Drop(Environment *ev, 
									ODDragItemIterator* dropInfo,
									ODFacet* facet, const FW_CPoint& where);
	
	virtual ODDragResult		CanAcceptDrop(Environment* ev, 
										ODDragItemIterator* dragInfo);
	
	virtual ODDragResult		DragEnter(Environment* ev, 
										ODFacet* odFacet, 
										ODDragItemIterator* dragInfo, 
										const FW_CPoint& where);
										
	virtual ODDragResult		DragWithin(Environment* ev, 
										ODFacet* odFacet, 
										ODDragItemIterator* dragInfo, 
										const FW_CPoint& where);
										
	virtual void 				DragLeave(Environment* ev, 
										ODFacet* odFacet, 
										const FW_CPoint& where);
 
	// ----- Drag hilite -----
 	virtual ODShape* 			AcquireDragHiliteShape(Environment* ev, ODFacet* odFacet);
	
	// ----- Factory Methods -----
	virtual FW_CDragCommand* 	NewDragCommand(Environment *ev, 
									FW_CFrame* theFrame);
	
	virtual FW_CDropCommand* 	NewDropCommand(Environment *ev, 
									FW_CFrame* frame,
									ODDragItemIterator* dropInfo, 
									ODFacet* odFacet, 
									const FW_CPoint& dropPoint);

	virtual FW_CDropTracker*	NewDropTracker(Environment* ev, FW_CView* view, ODFacet* facet);
	
//----------------------------------------------------------------------------------------
//	Getters/Setters
//
public:
	FW_Boolean					GetCursorInContent(Environment* ev) const;
	void						SetCursorInContent(Environment* ev, FW_Boolean isInContent);
	
	FW_Boolean					GetCanAcceptDrop(Environment* ev) const;
	void						SetCanAcceptDrop(Environment* ev, FW_Boolean acceptDrop);
	
	FW_CDropTracker*			GetDropTracker(Environment* ev) const;
		
	virtual void 				SetDragging(Environment* ev, FW_Boolean dragging);
	virtual void				SetDroppable(Environment* ev, FW_Boolean droppable);
		
//----------------------------------------------------------------------------------------
//	Casting 
//
public:	
	static FW_MDragDroppable*	ODFrameToDragDroppable(Environment* ev, ODFrame* theFrame);
	
//----------------------------------------------------------------------------------------
//	Data Members 
//
private:
	FW_CPart*					fPart;
	FW_Boolean					fCursorInContent;
	FW_Boolean					fCanAcceptDrop;
	FW_CDropTracker*			fDropTracker;
};

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export off
#endif


#endif