//========================================================================================
//
//	File:				FWFctClp.h
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================
//
//	Use a FacetClipper to calculate the clip shape of embedded facets.
//
//	Create a FW_CFacetClipper (or a subclass) on the stack or as a pointer.
//
//	If you want to calculate the clip shape of all the embedded facets of a presentation
//	call:
//	
//			FW_CFacetClipper::Clip(Environment*, FW_CPresentation*, ODShape*);
//
//	If you want to calculate the clip shape of the embedded facets of one frame call:
//
//			FW_CFacetClipper::Clip(Environment*, FW_CEmbeddingFrame*, ODShape*);
//
//	The limitShape parameter can be null. If it is not null it should be in content 
//	coordinate and only embedded facets intersecting with it will be clipped.
//	
//	The default version of ClipEmbeddedFacets doesn't take into account the content 
//	of your part. If your content overlaps your embedded facets you need to provide 
//	your own ClipEmbeddedFacets method. If there is no interaction between your content 
//	and the embedded facets this default version is fine.
//	NOTE: Your version of ClipEmbeddedFacets SHOULD NOT CALL INHERITED::ClipEmbeddedFacets.
//
//	If you override ClipEmbeddedFacets, call ClipOneEmbeddedFacet from your own 
//	ClipEmbeddedFacets method. The workingClip parameter is the clip region to apply to 
//	this embedded facet. When returning from ClipOneEmbeddedFacet, the usedShape of the 
//	embedded facet will have been removed from workingClip. WorkingClip is in window
//	coordinates

#ifndef FWFCTCLP_H
#define FWFCTCLP_H

#ifndef FWAUTODE_H
#include "FWAutoDe.h"
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

//========================================================================================
//	Forward Declarations
//========================================================================================

class FW_CLASS_ATTR FW_CPart;
class FW_CLASS_ATTR ODShape;
class FW_CLASS_ATTR ODFacet;
class FW_CLASS_ATTR FW_CEmbeddingFrame;
class FW_CLASS_ATTR FW_CPresentation;
struct FW_CLASS_ATTR Environment;

//========================================================================================
//	class FW_CFacetClipper
//========================================================================================

class FW_CLASS_ATTR FW_CFacetClipper FW_AUTO_DESTRUCT_OBJECT
{
public:
	FW_DECLARE_CLASS

//---------------------------------------------------------------------------------------
//	Initialization/Destruction
//
public:
	FW_CFacetClipper(Environment *ev, FW_CPart *part);
	virtual ~FW_CFacetClipper();

//---------------------------------------------------------------------------------------
//	New API
//
public:
	virtual void 		Clip(Environment *ev, FW_CPresentation *presentation, ODShape *limitShape = NULL);
	virtual void 		Clip(Environment *ev, FW_CEmbeddingFrame *embeddingFrame, ODShape *limitShape = NULL);
	
	virtual void 		ClipEmbeddedFacets(Environment *ev, 
								FW_CEmbeddingFrame *embeddingFrame, 
								ODFacet *containingFacet,
								ODShape *limitShape);

	void				ClipOneEmbeddedFacet(Environment *ev, 
								ODFacet *embeddedFacet, 
								ODShape *workingClip);
											
//---------------------------------------------------------------------------------------
//	Data Members
//
private:
	FW_CPart			*fPart;
};

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export off
#endif

#endif
