//========================================================================================
//
//	File:				FWFrame.h
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWFRAME_H
#define FWFRAME_H

#ifndef FWVIEW_H
#include "FWView.h"
#endif

#ifndef FWDRGDRP_H
#include "FWDrgDrp.h"
#endif

#ifndef FWIDLE_H
#include "FWIdle.h"
#endif

#ifndef FWPRTDEF_H
#include "FWPrtDef.h"
#endif

// ----- OS Includes -----

#ifndef FWRECT_H
#include "FWRect.h"
#endif

#ifndef FWFCTINF_H
#include "FWFctInf.h"
#endif

#ifndef FWACQUIR_H
#include "FWAcquir.h"
#endif

#ifndef FWFCSSET_H
#include "FWFcsSet.h"
#endif

// ----- Foundation Includes -----

#ifndef FWRUNTYP_H
#include "FWRunTyp.h"
#endif

#ifndef FWSTDDEF_H
#include "FWStdDef.h"
#endif

#ifndef FWHFCONN_H
#include "FWHFConn.h"
#endif

#ifndef FWSTRING_H
#include "FWString.h"
#endif

// ----- OpenDoc Includes -----

#ifndef SOM_ODFrame_xh
#include <Frame.xh>
#endif

#ifndef _ODTYPES_
#include <ODTypes.h>
#endif

#ifndef SOM_ODStorageUnit_xh
#include <StorageU.xh>
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

//========================================================================================
//	Forward Declaration
//========================================================================================

class FW_CLASS_ATTR ODFacet;
class FW_CLASS_ATTR ODWindow;
class FW_CLASS_ATTR ODShape;
class FW_CLASS_ATTR ODFocusSet;
class FW_CLASS_ATTR ODStorageUnitView;

class FW_CLASS_ATTR FW_CPrivOrderedCollection;

class FW_CLASS_ATTR FW_CPart;
class FW_CLASS_ATTR FW_CMenuBar;
class FW_CLASS_ATTR FW_CMenuEvent;
class FW_CLASS_ATTR FW_CWindow;
class FW_CLASS_ATTR FW_CGraphicDevice;
class FW_CLASS_ATTR FW_CFacetContext;
class FW_CLASS_ATTR FW_CMouseEvent;
class FW_CLASS_ATTR FW_CActivateEvent;
class FW_CLASS_ATTR FW_CEditCommand;
class FW_CLASS_ATTR FW_CPresentation;
class FW_CLASS_ATTR FW_CResumeSuspendEvent;
class FW_CLASS_ATTR FW_CCloneInfo;
class FW_CLASS_ATTR FW_CViewAs;
class FW_CLASS_ATTR FW_CHandleFunctionConnection;
class FW_CLASS_ATTR FW_CInterest;
class FW_CLASS_ATTR FW_CPrintHandler;
class FW_CLASS_ATTR FW_CScrollingViewIterator;

//========================================================================================
//	class FW_CFrame
//========================================================================================

class FW_CLASS_ATTR FW_CFrame : public FW_CView, public FW_MDragDroppable, 
								public FW_MIdle, public FW_MReceiver
{
	friend FW_CLASS_ATTR FW_CScrollingViewIterator;

public:
	FW_DECLARE_CLASS

	friend FW_CLASS_ATTR FW_CPart;

//----------------------------------------------------------------------------------------
//	Constructors/Destructors
//
public:
	FW_CFrame(Environment *ev, 
			ODFrame* frame, 
			FW_CPresentation* presentation, 
			FW_CPart* part);
			
	virtual ~ FW_CFrame();

//----------------------------------------------------------------------------------------
//	API from ODFrame (inlines)
//
public:
	unsigned long				GetFrameGroup(Environment* ev) const;
	void						SetFrameGroup(Environment* ev, 
									unsigned long groupID);

	FW_Boolean					IsPersistent(Environment* ev) const;
						
	FW_Boolean					IsRoot(Environment* ev) const;

	FW_Boolean					IsSubframe(Environment* ev) const;
	void						SetSubframe(Environment* ev, 
									FW_Boolean isSubframe);

	FW_Boolean					IsOverlaid(Environment* ev) const;

	FW_Boolean					IsInLimbo(Environment* ev) const;
	
	FW_Boolean					IsFrozen(Environment* ev) const;
	void						SetFrozen(Environment* ev, 
									FW_Boolean isFrozen);

	FW_Boolean					DoesPropagateEvents(Environment* ev) const;
	void						SetPropagateEvents(Environment* ev, 
									FW_Boolean doesPropagateEvents);

	FW_CPresentation*			GetPresentation(Environment* ev) const;
	ODTypeToken					GetViewType(Environment* ev) const;

	void						ChangeUsedShape(Environment* ev, 
									ODShape* newUsedShape, 
									ODCanvas* biasCanvas = NULL);
	FW_Boolean					RequestFrameShape(Environment* ev, 
									ODShape* shape,
									ODCanvas* biasCanvas = NULL);

	ODShape*					AcquireUsedShape(Environment* ev, 
									ODCanvas* biasCanvas = NULL) const;	
	ODTransform*				AcquireInternalTransform(Environment* ev, 
									ODCanvas* biasCanvas = NULL) const;	
	ODShape*					AcquireFrameShape(Environment* ev, 
									ODCanvas* biasCanvas = NULL) const;

	void						ChangeInternalTransform(Environment* ev, 
									ODTransform* internalTransform,
									ODCanvas* biasCanvas = NULL);
									
	ODFrame*					AcquireContainingFrame(Environment* ev) const;
	
	ODWindow*					AcquireODWindow(Environment* ev) const;

	ODID						GetID(Environment* ev) const;
	
	void						ClonePartInfo(Environment* ev, 
									ODDraftKey key,
									ODInfoType partInfo,
									ODStorageUnitView* storageUnitView,
									ODFrame* scope);

//----------------------------------------------------------------------------------------
//	Inherited from FW_MEventHandler
//
public:
	// ----- Activation -----
	virtual FW_Boolean			Activate(Environment* ev);
	
	// ----- Events -----
	virtual FW_MEventHandler*	GetTarget(Environment* ev) const;
	virtual void				SetTarget(Environment* ev, FW_MEventHandler* theTarget);
	virtual FW_Boolean			WantsToBeTarget(Environment* ev);

	virtual FW_Boolean			HandleActivateEvent(Environment* ev, const FW_CActivateEvent& theActivateEvent);
	virtual FW_Boolean			HandleSuspendResumeEvent(Environment* ev, const FW_CSuspendResumeEvent& theResumeSuspendEvent);
#ifdef FW_BUILD_MAC
	virtual FW_Boolean			HandleWindowEvent(Environment* ev, const FW_CMacWindowEvent& windowEvent);
#endif
	
	// ----- Menus -----
	virtual FW_Boolean			HandleAdjustMenus(Environment* ev, FW_CMenuBar* menuBar, FW_Boolean hasMenuFocus, FW_Boolean isRoot);
	virtual FW_Boolean 			HandleMenu(Environment* ev, const FW_CMenuEvent& theMenuEvent);

	// ----- Mouse Tracking -----
	virtual FW_Boolean			HandleMouseEnter(Environment* ev, ODFacet* odFacet, const FW_CPoint& where);
	virtual FW_Boolean			HandleMouseLeave(Environment* ev, ODFacet* odFacet);
	virtual FW_Boolean			HandleMouseWithin(Environment* ev, ODFacet* odFacet, const FW_CPoint& theMousePoint);
	
//----------------------------------------------------------------------------------------
//	Inherited from FW_MDragDroppable
//
public:
	// ----- Drag and Drop -----
	virtual void 				SetDragging(Environment* ev, FW_Boolean dragging);
	virtual void				SetDroppable(Environment* ev, FW_Boolean droppable);
		
//----------------------------------------------------------------------------------------
//	New API
//
public:
	FW_Boolean					IsTopFrame(Environment* ev) const;
	
	static FW_CFrame* 			ODtoFWFrame(Environment* ev, ODFrame* odFrame);
	ODFrame*					GetODFrame(Environment *ev) const;

	FW_Boolean					IsSiblingFrameOf(Environment *ev, ODFrame* otherODFrame) const;

	// ----- Adjust Cursor -----
	virtual FW_Boolean			AdjustCursor(Environment* ev, ODFacet* odFacet, const FW_CPoint& theMousePoint);
	
	// ----- Facet Notification -----		
	virtual void 				FacetAdded(Environment* ev, ODFacet* odFacet);
	virtual void 				FacetRemoved(Environment* ev, ODFacet* odFacet);
	unsigned short				GetFacetCount(Environment* ev) const;
	
	// ----- Add/Remove frame notification -----
	virtual void				FrameAdded(Environment* ev, FW_Boolean fromStorage);
	virtual void				FrameRemoved(Environment* ev, FW_Boolean toStorage);
		
	// ----- Focus Set -----
	FW_CFocusSet				GetFocusSet(Environment* ev) const;
	void 						SetFocusSet(Environment* ev, const FW_CFocusSet& focusSet);	
	
	virtual void 				FocusStateChanged(Environment* ev, ODTypeToken focus, FW_Boolean newState, ODFrame* newOwner);
	
	virtual FW_Boolean			BeginRelinquishFocus(Environment *ev, ODTypeToken focus, ODFrame* proposedFrame);
	virtual void				AbortRelinquishFocus(Environment *ev, ODTypeToken focus, ODFrame* proposedFrame);
	virtual void				CommitRelinquishFocus(Environment *ev, ODTypeToken focus, ODFrame* proposedFrame);

	// ----- Activation -----
	virtual void 				ActivateFrame(Environment* ev, ODFacet* activatedFacet);

	// ----- Window Geometry -----
	virtual void				AdjustZoomedWindowSize(Environment *ev, FW_CPoint& proposedSize);
	virtual void				AdjustWindowGrowLimits(Environment *ev, FW_CPoint& minSize, FW_CPoint& maxSize);

	// ----- Menus -----
	virtual void				InstallMenus(Environment* ev);
	
	// ----- ViewType/Presentation/Sequence -----
	virtual void 				ViewTypeChanged(Environment* ev, ODTypeToken newViewType, ODTypeToken oldViewType);
	virtual void 				PresentationChanged(Environment* ev);
	virtual void 				SequenceChanged(Environment* ev);

	// ----- Frame Geometry -----
	virtual void 				FrameShapeChanged(Environment* ev);
	virtual void				GeometryChanged(Environment *ev,
											ODFacet* odFacet,
											FW_Boolean clipShapeChanged,
											FW_Boolean externalTransformChanged);
	// ----- Used and Update Shape -----
	void 						UpdateUsedAndActiveShapes(Environment* ev);
	void						UpdateUsedShape(Environment* ev);
	void						UpdateActiveShape(Environment* ev, ODFacet* facet);

	virtual ODShape* 			AdjustUsedShape(Environment* ev, ODShape* suggestedUsedShape);
	virtual ODShape* 			AdjustActiveShape(Environment* ev, ODFacet* facet, ODShape* suggestedActiveShape);

	// ----- Persistence -----
	ODStorageUnit*				GetODStorageUnit(Environment* ev) const;
	ODStorageUnitID				GetODStorageUnitID(Environment* ev) const;
	
	// ----- Window -----
	FW_CWindow*					GetWindow(Environment* ev) const;
	virtual FW_CWindow* 		NewPartWindow(Environment* ev, ODFacet* facet);
	
	// ----- Focus -----
	FW_Boolean					HasSelectionFocus(Environment* ev) const;
	
	// ----- Open Selection -----	
	virtual void				OpenSelection(Environment* ev);

	// ----- Facet Part Info Factory -----
	virtual FW_CFacetPartInfo*	FacetPartInfoFactory(Environment* ev, 
									ODFacet* facet);
	
	// ----- Getter/Setter -----
	FW_CPart* 					GetPart(Environment* ev) const;
	ODPart* 					GetODPart(Environment* ev) const;

	FW_CView*					GetContentView(Environment* ev) const;
	void 						SetContentView(Environment* ev, FW_CView* view);
	
	FW_Boolean					GetFrontClicks(Environment* ev) const;
	void						SetFrontClicks(Environment* ev, FW_Boolean getFrontClicks);

	ODFacet* 					GetActiveFacet(Environment* ev) const;
	virtual void				SetActiveFacet(Environment* ev, ODFacet* activeFacet);
		
	virtual FW_Boolean			CanBeActiveFrame(Environment* ev) const;
	void						SetCanBeActiveFrame(Environment* ev, FW_Boolean beActive);
	
	// ----- Clipboard -----
	FW_Boolean					HasPropertyOnClipboard(Environment* ev, 
									ODPropertyName propertyName, 
									ODValueType type);

	// ----- Rendering -----
	void						HandleDraw(Environment* ev, ODFacet* odFacet, ODShape* invalidShape);
	
	virtual void 				Draw(Environment* ev, ODFacet* odFacet, ODShape* invalidShape);
	
	// ----- Thumbnail -----
	virtual void				BuildThumbnail(Environment* ev, FW_CGraphicContext& gc, const FW_CRect& destRect);
	
	// ----- Content -----
	void						GetContentExtent(Environment* ev, FW_CPoint& extent) const;
	ODShape* 					AcquireContentShape(Environment *ev);
	
	// ----- Commands/Undo -----
	virtual FW_CEditCommand* 	NewEditCommand(Environment* ev, ODCommandID commandID);
		// Override to return a subclass of FW_CEditCommand

	// ----- SubViews -----
	void 						AddScrollingView(Environment* ev, const FW_CView *view);
	void 						RemoveScrollingView(Environment* ev, const FW_CView *view);
	ODShape* 					AcquireContentScrollShape(Environment* ev, const FW_CPoint& by);
	
	// ----- External/InternalizeFrame -----
	virtual void				ExternalizeFrame(Environment* ev, ODStorageUnitView* storageUnitView);
	virtual void				InternalizeFrame(Environment* ev, ODStorageUnitView* storageUnitView);

	// ----- Scroll notifications -----
	void						AddConnection(Environment*ev, FW_CInterest& interest);
	void 						HandleNotification(const FW_CNotification& notification);
	
	// ----- Printing ------
	virtual FW_CPrintHandler*	NewPrintHandler(Environment* ev);
	virtual	FW_Boolean			IsCurrentlyPrintable(Environment* ev) const;
	virtual void				GetPrintContentExtent(Environment* ev, FW_CPoint& extent) const;
	virtual void				GetPrintJobTitle(Environment* ev, FW_CString& jobTitle) const;

private:
#ifdef FW_BUILD_WIN
	void						CreateShadowWindow(Environment* ev);
	static LRESULT CALLBACK 	ShadowWindowProc(HWND hWnd, UINT msg,
											 WPARAM wParam, LPARAM lParam);
	void						MoveSizeShadowWindow(Environment* ev);
	
public:
	ODPlatformWindow			GetShadowWindow(Environment* ev) const;
#endif

//----------------------------------------------------------------------------------------
//	Internal API
//
protected:
	void						PrivGetFrameShapeBounds(Environment* ev, FW_CRect& rect) const;
		
	// ----- Relinquish/Request Focus Set -----
	void						PrivRelinquishFocusSet(Environment* ev);
	FW_Boolean					PrivRequestFocusSet(Environment* ev);

public:	
	virtual void				PrivAttachSourceFrame(Environment* ev, FW_CFrame* sourceFrame);
	virtual void				PageEmbeddedFrames(Environment* ev);

	FW_EInternalizeResult		PrivHandleInternalizeScrap(Environment* ev, ODStorageUnit* sourceSU, ODCloneKind cloneKind);
	void						PrivHandleExternalizeScrap(Environment* ev, FW_EStorageKinds storageKind, ODStorageUnit* destinationSU, ODCloneKind cloneKind);

	FW_Boolean					PrivActiveWindowOnMouseDown(Environment* ev, ODFacet* facet, FW_Boolean clickedInFrame);
	FW_CWindow*					PrivCreateWindow(Environment* ev, ODWindow* odWindow);

	ODTypeToken					PrivGetPreviousViewType(Environment *ev) const;
	void						PrivSetPreviousViewType(Environment *ev, ODTypeToken viewType);

	void						PrivReleaseODFrame(Environment *ev);

	void						PrivContentViewLocationChanged(Environment *ev);
	
//----------------------------------------------------------------------------------------
//	Data Members
//
private:
	ODFrame*					fODFrame;
	FW_CPart*					fPart;
	FW_MEventHandler*			fTarget;
  	FW_CView*					fContentView;  
	FW_CFocusSet				fFocusSet;
	FW_CFocusSet				fChangedFocusSet;
	FW_Boolean					fFocusSetChanged;
	FW_Boolean					fGetFrontClicks;
	FW_Boolean					fNeedsFoci;
	ODFacet*					fActiveFacet;
	FW_Boolean					fCanBeActiveFrame;
	FW_CPresentation*			fPresentation;
	FW_CWindow*					fWindow;
	unsigned short				fFacetCount;
	ODTypeToken					fPreviousViewType;
	FW_CViewAs*					fViewAs;
	FW_CHandleFunctionConnection*	fConnection;
	
	FW_CView*					fViewUnderCursor;
	FW_CPrivOrderedCollection*	fScrollingViews;	
	
	FW_CPoint					fContentViewLocation;	// location of the content view in frame coordinates
	
#ifdef FW_BUILD_WIN
	ODPlatformWindow			fShadowWindow;
#endif
};

//========================================================================================
//	FW_CFrame Inlines
//========================================================================================

//----------------------------------------------------------------------------------------
//	FW_CFrame::GetFacetCount()
//----------------------------------------------------------------------------------------
inline unsigned short FW_CFrame::GetFacetCount(Environment* ev) const
{
	return fFacetCount;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::GetFrontClicks()
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CFrame::GetFrontClicks(Environment* ev) const
{
	return fGetFrontClicks;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::SetFrontClicks()
//----------------------------------------------------------------------------------------
inline void FW_CFrame::SetFrontClicks(Environment* ev, FW_Boolean getFrontClicks)
{
	fGetFrontClicks = getFrontClicks;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::GetPart()
//----------------------------------------------------------------------------------------
inline FW_CPart* FW_CFrame::GetPart(Environment* ev) const
{
	return fPart;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::GetContentView()
//----------------------------------------------------------------------------------------
inline FW_CView* FW_CFrame::GetContentView(Environment* ev) const
{
	return (fContentView == NULL) ? (FW_CView*)this : fContentView;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::GetFocusSet()
//----------------------------------------------------------------------------------------
inline FW_CFocusSet FW_CFrame::GetFocusSet(Environment* ev) const
{
	return fFocusSet;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::GetODStorageUnit
//----------------------------------------------------------------------------------------

inline ODStorageUnit* FW_CFrame::GetODStorageUnit(Environment* ev) const
{
	return fODFrame->GetStorageUnit(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::GetActiveFacet
//----------------------------------------------------------------------------------------
inline ODFacet* FW_CFrame::GetActiveFacet(Environment* ev) const
{
	return fActiveFacet;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::SetCanBeActiveFrame
//----------------------------------------------------------------------------------------
inline void FW_CFrame::SetCanBeActiveFrame(Environment* ev, FW_Boolean beActive)
{
	fCanBeActiveFrame = beActive;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::ChangeUsedShape()
//----------------------------------------------------------------------------------------
inline void FW_CFrame::ChangeUsedShape(Environment* ev, ODShape* newUsedShape, ODCanvas* biasCanvas)
{
	fODFrame->ChangeUsedShape(ev, newUsedShape, biasCanvas);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::GetODFrame
//----------------------------------------------------------------------------------------
inline ODFrame* FW_CFrame::GetODFrame(Environment *ev) const
{
	return fODFrame;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::PrivGetPreviousViewType
//----------------------------------------------------------------------------------------
inline ODTypeToken FW_CFrame::PrivGetPreviousViewType(Environment *ev) const
{
	return fPreviousViewType;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::PrivSetPreviousViewType
//----------------------------------------------------------------------------------------
inline void FW_CFrame::PrivSetPreviousViewType(Environment *ev, ODTypeToken viewType)
{
	fPreviousViewType = viewType;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::GetFrameGroup
//----------------------------------------------------------------------------------------
inline unsigned long FW_CFrame::GetFrameGroup(Environment* ev) const
{
	return fODFrame->GetFrameGroup(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::SetFrameGroup
//----------------------------------------------------------------------------------------
inline void FW_CFrame::SetFrameGroup(Environment* ev, unsigned long groupID)
{
	fODFrame->SetFrameGroup(ev, groupID);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::IsRoot
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CFrame::IsRoot(Environment* ev) const
{
	return fODFrame->IsRoot(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::IsPersistent
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CFrame::IsPersistent(Environment* ev) const
{
	return fODFrame->GetStorageUnit(ev) != NULL;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::IsOverlaid
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CFrame::IsOverlaid(Environment* ev) const
{
	return fODFrame->IsOverlaid(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::IsOverlaid
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CFrame::IsInLimbo(Environment* ev) const
{
	return fODFrame->IsInLimbo(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::IsFrozen
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CFrame::IsFrozen(Environment* ev) const
{
	return fODFrame->IsFrozen(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::SetFrozen
//----------------------------------------------------------------------------------------
inline void FW_CFrame::SetFrozen(Environment* ev, FW_Boolean isFrozen)
{
	fODFrame->SetFrozen(ev, isFrozen);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::DoesPropagateEvents
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CFrame::DoesPropagateEvents(Environment* ev) const
{
	return fODFrame->DoesPropagateEvents(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::SetPropagateEvents
//----------------------------------------------------------------------------------------
inline void FW_CFrame::SetPropagateEvents(Environment* ev, FW_Boolean doesPropagateEvents)
{
	fODFrame->SetPropagateEvents(ev, doesPropagateEvents);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::GetViewType
//----------------------------------------------------------------------------------------
inline ODTypeToken FW_CFrame::GetViewType(Environment* ev) const
{
	return fODFrame->GetViewType(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::GetPresentation
//----------------------------------------------------------------------------------------
inline FW_CPresentation* FW_CFrame::GetPresentation(Environment* ev) const
{
	return fPresentation;
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::IsSubframe
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CFrame::IsSubframe(Environment* ev) const
{
	return fODFrame->IsSubframe(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::SetSubframe
//----------------------------------------------------------------------------------------
inline void FW_CFrame::SetSubframe(Environment* ev, FW_Boolean isSubframe)
{
	fODFrame->SetSubframe(ev, isSubframe);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::AcquireUsedShape
//----------------------------------------------------------------------------------------
inline ODShape* FW_CFrame::AcquireUsedShape(Environment* ev, ODCanvas* biasCanvas) const
{
	return fODFrame->AcquireUsedShape(ev, biasCanvas);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::AcquireFrameShape
//----------------------------------------------------------------------------------------
inline ODShape* FW_CFrame::AcquireFrameShape(Environment* ev, ODCanvas* biasCanvas) const
{
	return fODFrame->AcquireFrameShape(ev, biasCanvas);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::AcquireContainingFrame
//----------------------------------------------------------------------------------------
inline ODFrame* FW_CFrame::AcquireContainingFrame(Environment* ev) const
{
	return fODFrame->AcquireContainingFrame(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::AcquireInternalTransform
//----------------------------------------------------------------------------------------
inline ODTransform* FW_CFrame::AcquireInternalTransform(Environment* ev, ODCanvas* biasCanvas) const
{
	return fODFrame->AcquireInternalTransform(ev, biasCanvas);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::ChangeInternalTransform
//----------------------------------------------------------------------------------------
inline void FW_CFrame::ChangeInternalTransform(Environment* ev, ODTransform* internalTransform, ODCanvas* biasCanvas)
{
	fODFrame->ChangeInternalTransform(ev, internalTransform, biasCanvas);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::AcquireODWindow
//----------------------------------------------------------------------------------------
inline ODWindow* FW_CFrame::AcquireODWindow(Environment* ev) const
{
	return fODFrame->AcquireWindow(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CFrame::GetID
//----------------------------------------------------------------------------------------
inline ODID FW_CFrame::GetID(Environment* ev) const
{
	return fODFrame->GetID(ev);
}

#ifdef FW_BUILD_WIN
//----------------------------------------------------------------------------------------
//	FW_CFrame::GetShadowWindow()
//----------------------------------------------------------------------------------------
inline ODPlatformWindow FW_CFrame::GetShadowWindow(Environment* ev) const
{
	return fShadowWindow;
}
#endif

//========================================================================================
// CLASS FW_CScrollingViewIterator
//========================================================================================

// Ideally we would just subclass OrdereCollectionIterator. But then we would
// be required to call its constructor from our constructor passing it a valid
// view list which can be NULL.

class FW_CLASS_ATTR FW_CScrollingViewIterator FW_AUTO_DESTRUCT_OBJECT
{
public:
	FW_CScrollingViewIterator(Environment* ev, const FW_CFrame *container); 
	~ FW_CScrollingViewIterator();

	FW_CView*		First(Environment* ev)
						{return fIterator ? (FW_CView*)fIterator->First() : NULL;}
	FW_CView*		Next(Environment* ev)
						{return fIterator ? (FW_CView*)fIterator->Next() : NULL;}
	FW_CView*		Last(Environment* ev)
						{return fIterator ? (FW_CView*)fIterator->Last() : NULL;}
	FW_CView*		Previous(Environment* ev)
						{return fIterator ? (FW_CView*)fIterator->Previous() : NULL;}
	
	FW_Boolean		IsNotComplete(Environment* ev)
						{return fIterator ? fIterator->IsNotComplete() : NULL;}
	
private:
	FW_COrderedCollectionIterator *fIterator;
};

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export off
#endif

#endif

