//========================================================================================
//
//	File:				FWFrming.h
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWFRMING_H
#define FWFRMING_H

#ifndef FWFRAME_H
#include "FWFrame.h"
#endif

// ----- ODF Foundation Includes -----

#ifndef FWRUNTYP_H
#include "FWRunTyp.h"
#endif

#ifndef FWSTDDEF_H
#include "FWStdDef.h"
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

//========================================================================================
//	Forward Declarations
//========================================================================================

class FW_CLASS_ATTR ODFacet;
class FW_CLASS_ATTR ODShape;
class FW_CLASS_ATTR ODObjectSpec;
class FW_CLASS_ATTR ODDragItemIterator;

class FW_CLASS_ATTR FW_CPrivOrderedCollection;
class FW_CLASS_ATTR FW_CEmbeddingPart;
class FW_CLASS_ATTR FW_MProxy;
class FW_CLASS_ATTR FW_CBorderMouseEvent;
class FW_CLASS_ATTR FW_CEmbeddedMouseEvent;
class FW_CLASS_ATTR FW_CPoint;
class FW_CLASS_ATTR FW_CProxyFrame;

//========================================================================================
//	class FW_CEmbeddingFrame
//========================================================================================

class FW_CLASS_ATTR FW_CEmbeddingFrame : public FW_CFrame
{
public:
	FW_DECLARE_CLASS

	friend class FW_CLASS_ATTR FW_CFrameProxyFrameIterator;
	
//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
public:
	FW_CEmbeddingFrame(Environment* ev, 
				ODFrame* odFrame, 
				FW_CPresentation* presentation,
				FW_CEmbeddingPart* part);
	virtual ~ FW_CEmbeddingFrame();

//----------------------------------------------------------------------------------------
//	Inherited API
//
public:
	// ----- Adjust Cursor -----
	virtual FW_Boolean			AdjustCursor(Environment* ev, 
									ODFacet* odFacet, 
									const FW_CPoint& theMousePoint);
	// ----- ViewType -----
	virtual void				ViewTypeChanged(Environment* ev, 
									ODTypeToken newViewType, 
									ODTypeToken oldViewType);
	
	// ----- Facets -----
	virtual void 				FacetAdded(Environment* ev, 
									ODFacet* odFacet);
	virtual void 				FacetRemoved(Environment* ev, 
									ODFacet* odFacet);

	// ----- Notification -----	
	virtual void				FrameAdded(Environment* ev, FW_Boolean fromStorage);
	virtual void				FrameRemoved(Environment* ev, FW_Boolean toStorage);

	// ----- Menu Events -----
	virtual FW_Boolean			HandleAdjustMenus(Environment* ev, 
									FW_CMenuBar* menuBar,
									FW_Boolean hasMenuFocus, 
									FW_Boolean isRoot);
	virtual FW_Boolean			HandleMenu(Environment* ev, 
									const FW_CMenuEvent& theMenuEvent);

	// ----- [HLX] Temporary. To be removed (See FW_CFrame::FocusChanged()) -----
	virtual FW_Boolean			Activate(Environment* ev);
	virtual void				Deactivate(Environment* ev);

	FW_CEmbeddingPart*			GetPart(Environment* ev) const;
	
//----------------------------------------------------------------------------------------
//	New API
//
public:
	// ----- Embedded Facets -----
	virtual ODFacet*			CreateEmbeddedFacet(Environment* ev,
									ODFacet* embeddingFacet,
									FW_MProxy* proxy,
									ODFrame* embeddedFrame,
									ODShape* proposedClipShape) = 0;
									
	virtual void 				EmbeddedFacetAdded(Environment* ev,
									ODFacet* odFacet);
	virtual void 				EmbeddedFacetRemoved(Environment* ev, 
									ODFacet* odFacet);
									
	FW_Boolean					HasASelectedEmbeddedFacet(Environment* ev) const;

	// ----- Frame Spec -----
	virtual void 				EmbeddedFrameSpec(Environment *ev, 
									ODFrame* embeddedFrame, 
									ODObjectSpec* spec);

	// ----- Embedded frames -----
	unsigned long				CountProxyFrames(Environment* ev) const;
	
	virtual void				EmbedSingleFrame(Environment* ev,
									ODPart* odEmbeddedPart, 
									ODFrame* odEmbeddedFrame,
									ODShape* frameShape) = 0;

	virtual FW_Boolean 			RevealFrame(Environment *ev, 
									ODFrame* embeddedFrame, 
									ODShape* revealShape);

	virtual ODFrame* 			EmbeddedFrameRequested(Environment *ev,
									ODFrame* odBaseFrame,
									ODShape* frameShape,
									ODPart* embedPart,
									ODTypeToken viewType,
									ODTypeToken presentation,
									ODBoolean isOverlaid);
	
	FW_MProxy*					GetProxy(Environment *ev, 
									ODFrame* embeddedFrame) const;

	// ----- Frame Paging -----
	virtual void				PageEmbeddedFrames(Environment* ev);
	
	// ----- Facet Hierarchy -----
	ODFacet* 					GetEmbeddedFacet(Environment* ev, 
									ODFacet* odFacet, 
									FW_MProxy* proxy);
	
	// ----- Drag And Drop -----
	virtual ODDragResult		CanAcceptDrop(Environment* ev, 
									ODDragItemIterator* dragInfo);
	
	// ----- OD to FW conversion -----
	static FW_CEmbeddingFrame*	ODtoFWEmbeddingFrame(Environment* ev, 
									ODFrame* odFrame);

	// ----- OpenDoc UI -----
	virtual void				ShowPartInfo(Environment* ev);
	virtual void				OpenSelection(Environment* ev);

//----------------------------------------------------------------------------------------
//	Public but for Internal use only
//
public:
	virtual void				PrivAttachSourceFrame(Environment* ev, 
									FW_CFrame* sourceFrame);
									
	virtual void				PrivAttachOrphans(Environment* ev);

	// ----- Create/Remove Embedded Frames -----
	FW_CProxyFrame*				PrivCreateEmbeddedFrame(Environment* ev, 
									ODType frameType,
									ODPart *embeddedODPart,
									FW_MProxy* proxy,					// can be null
									ODShape* frameShape,
									ODCanvas* biasCanvas,				// can be null
									ODTypeToken viewType,
									ODTypeToken presentation,			// can be null
									ODID frameGroupID,
									FW_Boolean isOverlaid,
									FW_Boolean subFrame);
	
	void						PrivRemoveEmbeddedFrame(Environment* ev,
									FW_CProxyFrame* proxyFrame);	

	// ----- ProxyFrame management -----
	void						PrivAddProxyFrame(Environment* ev, 
									FW_CProxyFrame* proxyFrame);
	long						PrivRemoveProxyFrame(Environment* ev, 
									FW_CProxyFrame* proxyFrame);
									
	FW_CProxyFrame*				PrivAcquireProxyFrame(Environment* ev, 
									ODStorageUnitID frameID) const;
								

	// ----- Create/Remove Embedded Facets -----
	void						PrivCreateEmbeddedFacets(Environment* ev, 
									FW_MProxy* proxy, 
									FW_CProxyFrame* proxyFrame);									
	void						PrivCreateEmbeddedFacet(Environment* ev, 
									FW_MProxy* proxy, 
									FW_CProxyFrame* proxyFrame,
									ODFacet* embeddingFacet);									

	void						PrivRemoveEmbeddedFacets(Environment* ev, 
									FW_CProxyFrame* proxyFrame);									
	void						PrivRemoveEmbeddedFacet(Environment* ev, 
									FW_CProxyFrame* proxyFrame,
									ODFacet* embeddingFacet);									

//----------------------------------------------------------------------------------------
//	Data Members
//
private:
	FW_CPrivOrderedCollection*		fProxyFrames;
};

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export off
#endif

#endif

