//========================================================================================
//
//	File:				FWLnkMgr.h
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWLNKMGR_H
#define FWLNKMGR_H

#ifndef FWSTDDEF_H
#include "FWStdDef.h"
#endif

#ifndef FWRUNTYP_H
#include "FWRunTyp.h"
#endif

#ifndef _ODTYPES_
#include <ODTypes.h>
#endif

#ifndef FWLINK_H
#include "FWLink.h"
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

//========================================================================================
//	Forward Declarations
//========================================================================================

class FW_CLASS_ATTR FW_CPartPublishIterator;
class FW_CLASS_ATTR FW_CPartSubscribeIterator;
class FW_CLASS_ATTR ODLinkSource;
class FW_CLASS_ATTR ODLink;
class FW_CLASS_ATTR FW_CPresentation;
class FW_CLASS_ATTR FW_CPart;
class FW_CLASS_ATTR FW_CPrivOrderedCollection;

//========================================================================================
//	class FW_CLinkManager - abstract base class that manages Linking
//========================================================================================

class FW_CLASS_ATTR FW_CLinkManager FW_AUTO_DESTRUCT_OBJECT
{
	friend class FW_CLASS_ATTR FW_CPartPublishIterator;
	friend class FW_CLASS_ATTR FW_CPartSubscribeIterator;

  public:
	FW_DECLARE_CLASS

//----------------------------------------------------------------------------------------
//	Constructors/Destructors
//
  public:
	FW_CLinkManager(Environment* ev, FW_CPart* thePart);
	virtual ~FW_CLinkManager();

	void RemoveAllLinks(Environment *ev);

//----------------------------------------------------------------------------------------
//	Methods that must be overridden
//
  public:
	virtual FW_CPublishLink* 	NewPublishLink(Environment* ev, 
											   ODUpdateID updateID, 
											   FW_CPresentation* presentation) = 0;
	virtual FW_CSubscribeLink*	NewSubscribeLink(Environment* ev, 
												 ODLink* odLink,
												 ODLinkInfo* linkInfo,
												 FW_CPresentation* presentation) = 0;

//----------------------------------------------------------------------------------------
//	Methods that may be overridden
//
  public:
	virtual ODLinkSource*		CreateLink(Environment* ev, ODByteArray* data);

	virtual void				DoChangeLinkStatus(Environment* ev, ODFrame* odFrame);
								// formerly named LinkStatusChanged

	virtual void				DoUpdateLinks(Environment* ev,
											  ODFrame* odEmbeddedFrame,
											  ODUpdateID updateID);
								// formerly named EmbeddedFrameUpdated

	virtual FW_Boolean			EditInLinkAttempted(Environment* ev, ODFrame* odFrame);
	virtual void				RevealLink(Environment* ev, ODLinkSource* linkSource);

  protected:
	virtual void				MakeSubscribeLink(Environment* ev,
												  ODLink* odLink,
												  ODLinkInfo* linkInfo,
												  FW_CPresentation* presentation);

//----------------------------------------------------------------------------------------
//	Methods that cannot be overridden
//
  public:
	FW_Boolean					PasteWithLink(Environment* ev, ODStorageUnit* storageUnit, 
											  ODPasteAsResult& pasteAsResult,
											  FW_CPresentation* presentation);

	void						BreakPublishLink(Environment* ev, FW_CPublishLink* publishLink);
	void						BreakSubscribeLink(Environment* ev, FW_CSubscribeLink* subscribeLink);

	void						RestorePublishLink(Environment* ev, FW_CPublishLink* publishLink);
	void						RestoreSubscribeLink(Environment* ev, FW_CSubscribeLink* subscribeLink);

//----------------------------------------------------------------------------------------
//	Utility methods
//
  public:
	FW_CPublishLink*			GetPendingPublish(Environment* ev) const;
	void						SetPendingPublish(Environment* ev, FW_CPublishLink* publishLink);
	void						DeletePendingPublish(Environment* ev);

	void						AddToPublishLinkList(Environment* ev, FW_CPublishLink* publishLink);
	void						RemoveFromPublishLinkList(Environment* ev, FW_CPublishLink* publishLink);

	void						AddToSubscribeLinkList(Environment* ev, FW_CSubscribeLink* subscribeLink);
	void						RemoveFromSubscribeLinkList(Environment* ev, FW_CSubscribeLink* subscribeLink);

	unsigned long				CountPublishers(Environment* ev) const;
	unsigned long				CountSubscribers(Environment* ev) const;

	FW_CPublishLink*			ODtoPublishLink(Environment* ev, ODLinkSource* linkSource) const;
	FW_CSubscribeLink*			ODtoSubscribeLink(Environment* ev, ODLink* odLink) const;

	FW_CPublishLink*			GetPendingDropLink(Environment* ev) const;
	void						SetPendingDropLink(Environment* ev, FW_CPublishLink* publishLink);

//----------------------------------------------------------------------------------------
//	Data Members
//
  protected:
	FW_CPart*					fPart; 
	FW_CPublishLink*			fPendingPublishLink;

  private:
	FW_CPublishLink*			fPendingDropLink;
	FW_CPrivOrderedCollection*	fPublishLinkList;
	FW_CPrivOrderedCollection*	fSubscribeLinkList;
};

//========================================================================================
//	FW_CLinkManager Inlines
//========================================================================================

//----------------------------------------------------------------------------------------
//	FW_CLinkManager::GetPendingPublish
//----------------------------------------------------------------------------------------
inline FW_CPublishLink* FW_CLinkManager::GetPendingPublish(Environment *ev) const
{
	return fPendingPublishLink;
}

//----------------------------------------------------------------------------------------
//	FW_CLinkManager::SetPendingPublish
//----------------------------------------------------------------------------------------
inline void FW_CLinkManager::SetPendingPublish(Environment *ev, FW_CPublishLink* publishLink)
{
	fPendingPublishLink = publishLink;
}

//----------------------------------------------------------------------------------------
//	FW_CLinkManager::GetPendingDropLink
//----------------------------------------------------------------------------------------
inline FW_CPublishLink* FW_CLinkManager::GetPendingDropLink(Environment* ev) const
{
	return fPendingDropLink;
}

//----------------------------------------------------------------------------------------
//	FW_CLinkManager::SetPendingDropLink
//----------------------------------------------------------------------------------------
inline void FW_CLinkManager::SetPendingDropLink(Environment* ev, FW_CPublishLink* publishLink)
{
	fPendingDropLink = publishLink;
}

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export off
#endif

#endif
