//========================================================================================
//
//	File:				FWPart.h
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWPART_H
#define FWPART_H

#ifndef FWIDLE_H
#include "FWIdle.h"
#endif

#ifndef FWEVENTH_H
#include "FWEventH.h"
#endif

#ifndef FWPRTDEF_H
#include "FWPrtDef.h"
#endif

#ifndef FWWINDOW_H
#include "FWWindow.h"
#endif

// ----- OpenDoc Includes -----

#ifndef SOM_ODPart_xh
#include <Part.xh>
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

//========================================================================================
//	Forward Declarations
//========================================================================================

class FW_CLASS_ATTR ODFrame;
class FW_CLASS_ATTR ODWindow;
class FW_CLASS_ATTR ODSession;

class FW_CLASS_ATTR FW_CFrame;
class FW_CLASS_ATTR FW_CSelection;
class FW_CLASS_ATTR FW_CPoint;
class FW_CLASS_ATTR FW_CMenuBar;
class FW_CLASS_ATTR FW_CEventDispatcher;
class FW_CLASS_ATTR FW_CCommand;
class FW_CLASS_ATTR FW_CPrivOrderedCollection;
class FW_CLASS_ATTR FW_CCloneInfo;
class FW_CLASS_ATTR FW_CPromise;
class FW_CLASS_ATTR FW_CPullDownMenu;
class FW_CLASS_ATTR FW_CWindow;
class FW_CLASS_ATTR FW_CPresentation;
class FW_CLASS_ATTR FW_CExtensionManager;
class FW_CLASS_ATTR FW_CLinkManager;

class FW_CLASS_ATTR FW_CPrintInfo;

//========================================================================================
//	class FW_CPart
//========================================================================================

class FW_CLASS_ATTR FW_CPart : public FW_MEventHandler, public FW_MIdle
{
	friend class FW_CLASS_ATTR ODFModule_FW_SOMPart;
	friend class FW_CLASS_ATTR FW_CPartPresentationIterator;
	friend class FW_CLASS_ATTR FW_CPartFrameIterator;
	friend class FW_CLASS_ATTR FW_CPartPublishIterator;
	friend class FW_CLASS_ATTR FW_CPartSubscribeIterator;
	
public:
	FW_DECLARE_CLASS

//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
public:
	FW_CPart(ODPart* odPart, 
			 ODValueType partKind, 
			 ODValueType partUserName,
			 FW_Instance partInstance, 
			 FW_ResourceId iconID);
	
	virtual void 				Initialize(Environment *ev);
	virtual 					~FW_CPart();

//----------------------------------------------------------------------------------------
//	Inherited from FW_MEventHander
//
public:
	virtual FW_Boolean			HandleAdjustMenus(Environment *ev, 
									FW_CMenuBar* menuBar,
									FW_Boolean hasMenuFocus,
									FW_Boolean isRoot);
	virtual FW_Boolean			IsActive(Environment *ev) const;

//----------------------------------------------------------------------------------------
//	New API
//
public:	
	//--------------------------------------------------------------------------
	// method from ODPart
	ODStorageUnit*				GetStorageUnit(Environment *ev) const;

	virtual void				Release(Environment *ev);
	virtual void				ReleaseAll(Environment *ev);
	
	void						GetPartName(Environment *ev, FW_CString& partName);
	
	virtual ODSize				Purge(Environment *ev, ODSize size);
									
	//--------------------------------------------------------------------------
	// New API For Modal Dialogs

	enum DialogStyle { kModal, kMovableModal, kModeless };
	virtual void				LaunchDialog(Environment *ev,
									FW_CPresentation* presentation,
									FW_CWindow::WindowStyle style,
									const FW_CPoint& location, 
									const FW_CPoint& size);
	virtual void				EnableMenuBar(Environment *ev,
									FW_Boolean enable);
		// The parts derived from FW_CPart are responsible for overriding this
		// method to disable menu they added to the menu bar. Called when a
		// modal dialog box is shown. Call inherited to get the File and
		// Edit menus disabled.
	void						CloseDialogWindowDeferred(Environment *ev,
									FW_CWindow* closeDialog);
		// Close this window at the next idle event. By deferring closing the
		// dialog we give frames and views a chance to clean up before they
		// are deleted as a result of closing a dialog.
		
	//--------------------------------------------------------------------------
	// Promises

	FW_CPromise* 				GetPromise(Environment *ev, 
									FW_EStorageKinds storageKind) const;
	void 						SetPromise(Environment *ev, 
									FW_CPromise* promise);	
	void 						DeletePromise(Environment *ev, 
									FW_EStorageKinds storageKind);
	
	void						ResolvePromise(Environment* ev, 
									ODPropertyName propertyName = NULL, 	// then uses kODPropContents
									ODValueType valueType = NULL);			// then uses GetPartKind()
	void						ResolveAllPromises(Environment* ev);

	//--------------------------------------------------------------------------
	// Draft

	virtual FW_Boolean			IsReadOnly(Environment* ev) const;
	
	//--------------------------------------------------------------------------
	// Presentation

	FW_CPresentation*			RegisterPresentation(Environment *ev, 
										ODType presentationType, 
										FW_Boolean defaultPresentation,
										FW_CSelection* selection = NULL);
	virtual FW_CPresentation*	NewPresentation(Environment *ev, 
											FW_CSelection* selection,
											ODTypeToken presentationType);
											
	virtual FW_CPresentation*	ValidatePresentation(Environment *ev,
										FW_CPresentation* presentation,
										FW_Boolean isRootFrame) const;
	FW_CPresentation*			FindPresentation(Environment *ev, 
										ODTypeToken presentationType) const;

	//--------------------------------------------------------------------------
	// Properties

	virtual void 				AddProperties(Environment *ev, 
									ODStorageUnit* storageUnit);
	
	//--------------------------------------------------------------------------
	// Event Dispatcher

	virtual FW_CEventDispatcher*	CreateEventDispatcher(Environment *ev);
		
	//--------------------------------------------------------------------------
	// Document Window Creation

	virtual FW_CWindow* 			NewDocumentWindow(Environment* ev);
	
	//--------------------------------------------------------------------------
	// Frame 

	virtual FW_CFrame* 				NewFrame(Environment *ev,
										ODFrame* odFrame, 
										FW_CPresentation* presentation,
										FW_Boolean fromStorage) = 0;
	
	FW_Boolean						IsValidDisplayFrame(Environment *ev, 
										ODStorageUnitID displayFrameID) const;
	
	//--------------------------------------------------------------------------
	// Menus

	virtual void 					InstallMenus(Environment *ev,
										FW_CMenuBar* menuBar);
	
	//--------------------------------------------------------------------------
	// Getters/Setters

	ODPart* 						GetODPart(Environment *ev) const;
	
	FW_CFrame* 						GetActiveFrame(Environment *ev) const;
	virtual void					SetActiveFrame(Environment *ev, 
										FW_CFrame* activeFrame);
									
	ODFacet* 						GetActiveFacet(Environment *ev) const;
	
	FW_CMenuBar* 					GetMenuBar(Environment *ev) const;
	FW_CMenuBar* 					GetPartMenuBar(Environment *ev) const;
	
	unsigned long					CountDisplayFrame(Environment *ev) const;
	
	ODSession* 						GetSession(Environment *ev) const;
	
	ODValueType						GetPartKind(Environment *ev) const;
	
	FW_Instance						GetPartInstance(Environment *ev) const;
	FW_ResourceId					GetIconID(Environment *ev) const;

	FW_CEventDispatcher*			GetEventDispatcher(Environment *ev) const;
	
	void							SetPrintInfo(FW_CPrintInfo* printInfo);
	FW_CPrintInfo*					GetPrintInfo() const;

	//--------------------------------------------------------------------------
	// Windows

	void							HideShowFloatingWindows(Environment *ev, FW_Boolean state);
	
	//--------------------------------------------------------------------------
	// Change

	void							Changed(Environment *ev);
		
	virtual void					ContentUpdated(Environment* ev,
											   FW_CFrame* frame, 
											   ODUpdateID updateID = kODUnknownUpdate);
		
	//--------------------------------------------------------------------------
	// Linking

	virtual FW_CLinkManager* 		NewLinkManager(Environment* ev);
	FW_CLinkManager*				GetLinkManager(Environment* ev) const;

	virtual void					LinkStatusChanged(Environment *ev, ODFrame* odFrame);

	//--------------------------------------------------------------------------
	// Externalize/Internalize Content

	virtual void				ExternalizeContent(Environment *ev,
									ODStorageUnit* storageUnit,
									FW_CCloneInfo* cloneInfo);
									
	virtual void				InternalizeContent(Environment *ev,
									ODStorageUnit* storageUnit, 
									FW_CCloneInfo* cloneInfo);

	//--------------------------------------------------------------------------
	// Commands

	void						ExecuteCommand(Environment *ev,
									FW_CCommand* command);
	
	//--------------------------------------------------------------------------
	// Lazy Instantiation

	FW_Boolean					IsSupportingFramePaging(Environment *ev) const;
	void						SupportFramePaging(Environment *ev, FW_Boolean state);
	
	//--------------------------------------------------------------------------	
	// Extensions
	
	void						PrivSetExtensionManager(FW_CExtensionManager* manager);
	FW_CExtensionManager* 		GetExtensionManager() const;
	
//----------------------------------------------------------------------------------------
//	Internal API (you should never have to called it)
//
public:
	virtual void				Dragging(Environment *ev, 
									FW_CFrame* frame, 
									FW_Boolean dragging);

	FW_CPresentation*			PrivGetPresentation(Environment *ev, ODFrame* odFrame) const;
	
	// ----- Cloning -----
	virtual void				PrivCloneInto(Environment *ev,
									ODDraftKey key,
									ODStorageUnit* toSU,
									ODFrame* scope);

	// ---- Do nothing in FW_CPart -----					
	virtual void				PrivInternalizeProxyFrames(Environment *ev, 
									ODStorageUnit* storageUnit, 
									FW_CCloneInfo* cloneInfo);
	virtual void				PrivExternalizeProxyFrames(Environment *ev, 
									ODStorageUnit* storageUnit, 
									FW_CCloneInfo* cloneInfo);
	// ----- Idle Management -----
	void						PrivCountIdleRegistering(FW_Boolean registering);
	
	// ----- Session Globals -----
	void						PrivInitGlobals(Environment *ev);
	void						PrivReleaseGlobals();
	
//----------------------------------------------------------------------------------------
//	Data members
//
public:
	static unsigned short		gSessionGlobalsCount;
	
	static ODTypeToken			gSelectionFocusToken;
	static ODTypeToken			gMenuFocusToken;
	static ODTypeToken			gKeyFocusToken;
	static ODTypeToken			gModalFocusToken;
	static ODTypeToken			gClipboardFocusToken;
	static ODTypeToken			gScrollingFocusToken;
	static ODTypeToken			gViewAsFrameToken;
	static ODTypeToken			gViewAsSmallIconToken;
	static ODTypeToken			gViewAsLargeIconToken;
	static ODTypeToken			gViewAsThumbnailToken;
	
#ifdef FW_BUILD_MAC	
	static ODType				gMacPICTDataType;
	static ODType				gMacPICTFileType;
	
	static ODType				gMachfsDataType;
	
	static ODType				gMacTEXTDataType;
	static ODType				gMacTEXTFileType;

	static ODType				gMacMooVFileType;
#endif

private:
	ODPart*						fODPart;
	ODValueType					fPartKind;
	ODValueType					fPartUserName;			// user name
	
	short						fIdleCount;

	FW_CMenuBar*				fPartMenuBar;
	FW_CMenuBar*				fMenuBar;			// This is the current menubar 
	FW_CFrame* 					fActiveFrame;
	FW_CPrivOrderedCollection* 	fPresentations;	
	FW_CPresentation*			fDefaultPresentation;
	
	FW_Instance					fPartInstance;
	FW_ResourceId				fIconID;
	
private:
	FW_CLinkManager*			fLinkManager;		// NULL if Part doesn't create Links

	FW_CEventDispatcher*		fEventDispatcher;
	
	FW_Boolean					fFramePaging;
	
	FW_CWindow*					fDialogToClose;
	
	FW_CPromise*				fDragAndDropPromise;
	FW_CPromise*				fClipboardPromise;

	FW_CExtensionManager*		fExtensionManager;
	
	FW_CPrintInfo*				fPrintInfo;			// May be NULL
};

//========================================================================================
//	Inlines
//========================================================================================

//----------------------------------------------------------------------------------------
//	FW_CPart::GetEventDispatcher
//----------------------------------------------------------------------------------------
inline FW_CEventDispatcher* FW_CPart::GetEventDispatcher(Environment *ev) const
{
	return fEventDispatcher;
}

//----------------------------------------------------------------------------------------
//	FW_CPart::GetPartInstance
//----------------------------------------------------------------------------------------
inline FW_Instance FW_CPart::GetPartInstance(Environment *ev) const
{
	return fPartInstance;
}

//----------------------------------------------------------------------------------------
//	FW_CPart::GetIconID
//----------------------------------------------------------------------------------------
inline FW_ResourceId FW_CPart::GetIconID(Environment *ev) const
{
	return fIconID;
}

//----------------------------------------------------------------------------------------
//	FW_CPart::GetStorageUnit
//----------------------------------------------------------------------------------------
inline ODStorageUnit* FW_CPart::GetStorageUnit(Environment *ev) const
{
	return fODPart->GetStorageUnit(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CPart::GetActiveFrame
//----------------------------------------------------------------------------------------
inline FW_CFrame* FW_CPart::GetActiveFrame(Environment *ev) const
{
	return fActiveFrame;
}

//----------------------------------------------------------------------------------------
//	FW_CPart::GetMenuBar
//----------------------------------------------------------------------------------------
inline FW_CMenuBar* FW_CPart::GetMenuBar(Environment *ev) const
{
	return fMenuBar;
}

//----------------------------------------------------------------------------------------
//	FW_CPart::GetMenuBar
//----------------------------------------------------------------------------------------
inline FW_CMenuBar* FW_CPart::GetPartMenuBar(Environment *ev) const
{
	return fPartMenuBar;
}

//---------------------------------------------------------------------------------------
//	FW_CPart::GetPartKind
//---------------------------------------------------------------------------------------
inline ODValueType FW_CPart::GetPartKind(Environment *ev) const
{
FW_UNUSED(ev);
	return  fPartKind;
}

//----------------------------------------------------------------------------------------
//	FW_CPart::IsSupportingFramePaging
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CPart::IsSupportingFramePaging(Environment *ev) const
{
	return fFramePaging;
}

//----------------------------------------------------------------------------------------
//	FW_CPart::GetODPart
//----------------------------------------------------------------------------------------
inline ODPart* FW_CPart::GetODPart(Environment *ev) const
{
	return fODPart;
}

//----------------------------------------------------------------------------------------
//	FW_CPart::PrivSetExtensionManager
//----------------------------------------------------------------------------------------
inline void	FW_CPart::PrivSetExtensionManager(FW_CExtensionManager* manager)
{
	fExtensionManager = manager;
}

//----------------------------------------------------------------------------------------
//	FW_CPart::GetExtensionManager
//----------------------------------------------------------------------------------------
inline FW_CExtensionManager* FW_CPart::GetExtensionManager() const
{
	return fExtensionManager;
}

//---------------------------------------------------------------------------------------
//	FW_CPart::GetLinkManager
//----------------------------------------------------------------------------------------
inline FW_CLinkManager* FW_CPart::GetLinkManager(Environment* ev) const
{
	return fLinkManager;
}

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export off
#endif

#endif



