//========================================================================================
//
//	File:				FWPartng.h
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWPARTNG_H
#define FWPARTNG_H

#ifndef FWPART_H
#include "FWPart.h"
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

//========================================================================================
//	Forward Declarations
//========================================================================================

class FW_CLASS_ATTR FW_MProxy;
class FW_CLASS_ATTR FW_CPrivOrderedCollection;
class FW_CLASS_ATTR FW_CCloneInfo;
class FW_CLASS_ATTR FW_CProxyFrame;
class FW_CLASS_ATTR FW_CPartProxyFrameIterator;

//========================================================================================
//	class FW_CEmbeddingPart
//========================================================================================

class FW_CLASS_ATTR FW_CEmbeddingPart : public FW_CPart
{
	friend class FW_CLASS_ATTR FW_CPartProxyFrameIterator;

public:
	FW_DECLARE_CLASS

//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
public:
	FW_CEmbeddingPart(ODPart* odPart, 
			 ODValueType partKind, 
			 ODValueType partUserName,
			 FW_Instance partInstance, 
			 FW_ResourceId iconID);
	
	virtual void 				Initialize(Environment *ev);
	virtual 					~FW_CEmbeddingPart();

//----------------------------------------------------------------------------------------
//	Inherited API
//
public:											
	virtual FW_CPresentation*	NewPresentation(Environment *ev, 
											FW_CSelection* selection,
											ODTypeToken presentationType);

	virtual void				ReleaseAll(Environment *ev);
	virtual ODSize				Purge(Environment *ev, ODSize size);

	virtual void				LinkStatusChanged(Environment *ev, ODFrame* odFrame);

//----------------------------------------------------------------------------------------
//	New API
//
public:											
	FW_MProxy*					GetProxy(Environment *ev, ODFrame* embeddedFrame) const;
				
//----------------------------------------------------------------------------------------
//	Internal API (you should never have to called it)
//
public:
	// ----- Cloning -----
	virtual void				PrivCloneInto(Environment *ev,
									ODDraftKey key,
									ODStorageUnit* toSU,
									ODFrame* scope);

	// ----- ProxyFrame list management -----		
	void 						PrivAddProxyFrame(Environment* ev, FW_CProxyFrame* proxyFrame);
	long 						PrivRemoveProxyFrame(Environment* ev, FW_CProxyFrame* proxyFrame);
	
	FW_CProxyFrame*				PrivAcquireProxyFrame(Environment* ev, ODStorageUnitID frameID) const;
	
	virtual void				PrivInternalizeProxyFrames(Environment *ev, 
									ODStorageUnit* storageUnit, 
									FW_CCloneInfo* cloneInfo);
	virtual void				PrivExternalizeProxyFrames(Environment *ev, 
									ODStorageUnit* storageUnit, 
									FW_CCloneInfo* cloneInfo);
	
	virtual void				Dragging(Environment *ev, 
									FW_CFrame* frame, 
									FW_Boolean dragging);

#ifdef FW_BUILD_WIN
	void						WinAddEmbedMenu(Environment *ev, 
									FW_CMenuBar *menuBar);
	ODPart*						WinCreateEmbeddedPartFromCommand(Environment* ev, 
									ODCommandID commandID);

	void 						WinEnableEmbedMenu(Environment* ev,
									FW_CMenuBar* menuBar);
#endif

//----------------------------------------------------------------------------------------
//	Data members
//
private:	
	FW_CPrivOrderedCollection* 	fProxyFrames;	

#ifdef FW_BUILD_WIN
	// This is used by the Windows version to create the embed menu
	FW_CPullDownMenu*			fWinEmbedMenu;
	ODCommandID					fWinFirstPartID;
	ODCommandID					fWinLastPartID;
	char  						fTypeString[1024];		// [HLX] need to be dynamic
	char 						*fTypePtr [25];			// [HLX] need to be dynamic
#endif
};

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export off
#endif

#endif



