//========================================================================================
//
//	File:				FWPrHdlr.h
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWPRHDLR_H
#define FWPRHDLR_H

#ifndef FWEVENTH_H
#include "FWEventH.h"
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

//========================================================================================
//	Forward Declarations
//========================================================================================

struct Environment;

class FW_CLASS_ATTR	FW_CPrintInfo;
class FW_CLASS_ATTR	FW_CPrintEnvironment;
class FW_CLASS_ATTR	FW_CFrame;
class FW_CLASS_ATTR FW_CPart;
class FW_CLASS_ATTR FW_CString;

//========================================================================================
//	class FW_CPrintHandler
//========================================================================================

class FW_CLASS_ATTR FW_CPrintHandler : public FW_MEventHandler
{
public:
// ----- Construction
								FW_CPrintHandler(FW_CPart* part, FW_CFrame* frame);
	virtual						~FW_CPrintHandler();

// ----- Operations
	void						HandlePageSetup(Environment* ev);

	void						HandlePrint(Environment* ev);
									
// ----- Attributes
	FW_CPart*					GetPart() const;
	FW_CFrame*					GetFrame() const;
	FW_CFrame*					GetPrintFrame() const;
									
// ----- FW_MEventHandler protocol
public:
	virtual FW_Boolean 			DoAdjustMenus(Environment *ev, FW_CMenuBar* menuBar, FW_Boolean hasMenuFocus, FW_Boolean isRoot);
	virtual FW_Boolean 			DoMenu(Environment* ev, const FW_CMenuEvent& theMenuEvent);

// ----- Overridables - dialogs
public:
	virtual void				CreateProgressDialog(
									Environment*		ev,
									ODFrame*			frame,
									long				numPages,
									const FW_CString&	title);

	virtual void				DestroyProgressDialog(
									Environment*		ev,
									ODFrame*			frame);

	virtual void				UpdateProgressDialog(
									long				pageNumber);
protected:
	virtual FW_Boolean			DoPageSetupDialog(Environment* ev);
	virtual FW_Boolean			DoJobDialog(Environment* ev);
	virtual void				NotifyNoDefaultPrinter(Environment* ev);
	virtual void				NotifyPrintingCanceled(Environment* ev);

// ----- Overridables - printing
protected:
	virtual FW_CFrame*			CreatePrintFrame(Environment* ev);
	virtual void				ReleasePrintFrame(Environment* ev);

	virtual	void				PrintDocument(Environment* ev, FW_CPrintEnvironment* printEnv);
	virtual void				CountPrintPages(Environment* ev, long& xPageCount, long& yPageCount);
	virtual void				PrintPage(Environment* ev, ODFacet* facet, long xPage, long yPage);

// ----- Helpers
private:
	FW_Boolean					PreModalDialog(Environment* ev);									
	void						PostModalDialog(Environment* ev);
	FW_CPrintInfo*				GetPrintInfo() const;							

// ----- Data
private:
	FW_CPart*					fPart;
	FW_CFrame*					fFrame;
	FW_CFrame*					fPrintFrame;
};

#endif // FWPRHDLR_H

