//========================================================================================
//
//	File:				FWPrmise.h
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWPRMISE_H
#define FWPRMISE_H

#ifndef FWPRTDEF_H
#include "FWPrtDef.h"
#endif

// ----- Foundation Includes -----

#ifndef FWRUNTYP_H
#include "FWRunTyp.h"
#endif

#ifndef FWPRIDEB_H
#include "FWPriDeb.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _ODTYPES_
#include <ODTypes.h>
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

//========================================================================================
//	Forward Declarations
//========================================================================================

class FW_CLASS_ATTR ODStorageUnitView;
class FW_CLASS_ATTR ODStorageUnit;

class FW_CLASS_ATTR FW_CFrame;
class FW_CLASS_ATTR FW_CPart;
class FW_CLASS_ATTR FW_CSelection;
class FW_CLASS_ATTR FW_CCloneInfo;

//========================================================================================
//	class FW_CPromise
//========================================================================================

class FW_CLASS_ATTR FW_CPromise
{
public:
	FW_DECLARE_CLASS

//----------------------------------------------------------------------------------------
//	Constructor/Destructor
//
public:
	FW_CPromise(Environment*ev, 
				ODUpdateID updateID, 
				FW_EStorageKinds storageKind, 
				FW_CPart* part, 
				FW_CFrame* scopeFrame);
	virtual ~FW_CPromise();

//----------------------------------------------------------------------------------------
//	New API
//
public:
	virtual void		PromisePropertiesAndValues(Environment *ev, 
									ODStorageUnit* storageUnit);
	
	virtual void 		FulfillPromise(Environment *ev, 
									ODStorageUnitView *promiseSUView, 
									ODPropertyName propertyName,
									ODValueType valueType, 
									FW_CCloneInfo* cloneInfo) = 0;

	void				Promise(Environment *ev, 
									ODStorageUnit* storageUnit, 
									ODPropertyName propertyName, 
									ODValueType valueType);
	
	FW_CPart*			GetPart(Environment *ev) const;
	FW_CFrame*			GetScopeFrame(Environment *ev) const;
	
	FW_EStorageKinds	GetStorageKind(Environment *ev) const;
	
	ODUpdateID			GetUpdateID(Environment *ev) const;

//----------------------------------------------------------------------------------------
//	Internal only
//
public:
	unsigned short		PrivHandleFulfillPromise(Environment *ev, 
												ODStorageUnitView *promiseSUView);

//----------------------------------------------------------------------------------------
//	Data members
//
private:
	FW_CPart*			fPart;
	FW_CFrame*			fScopeFrame;
	unsigned short		fUnfulfillPromises;
	FW_EStorageKinds	fStorageKind;
	ODUpdateID			fUpdateID;			// Use for clipboard
};

//----------------------------------------------------------------------------------------
//	FW_CPromise::GetUpdateID
//----------------------------------------------------------------------------------------
inline ODUpdateID FW_CPromise::GetUpdateID(Environment *ev) const
{
	FW_ASSERT(fStorageKind == FW_kClipboardStorage);
	return fUpdateID;
}

//----------------------------------------------------------------------------------------
//	FW_CPromise::GetPart
//----------------------------------------------------------------------------------------
inline FW_CPart* FW_CPromise::GetPart(Environment *ev) const
{
	return fPart;
}

//----------------------------------------------------------------------------------------
//	FW_CPromise::GetScopeFrame
//----------------------------------------------------------------------------------------
inline FW_CFrame* FW_CPromise::GetScopeFrame(Environment *ev) const
{
	return fScopeFrame;
}

//----------------------------------------------------------------------------------------
//	FW_CPromise::GetStorageKind
//----------------------------------------------------------------------------------------
inline FW_EStorageKinds FW_CPromise::GetStorageKind(Environment *ev) const
{
	return fStorageKind;
}

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export off
#endif

#endif