//========================================================================================
//
//	File:				FWProxy.h
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWPROXY_H
#define FWPROXY_H

#ifndef FWSELECT_H
#include "FWSelect.h"
#endif

// ----- ODF OS Includes -----

#ifndef FWACQUIR_H
#include "FWAcquir.h"
#endif

#ifndef FWFXMATH_H
#include "FWFxMath.h"
#endif

// ----- ODF Foundation Includes -----

#ifndef FWSTDEF_H
#include "FWStdDef.h"
#endif

#ifndef FWCLAINF_H
#include "FWClaInf.h"
#endif

#ifndef FWITERS_H
#include "FWIters.h"
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

//========================================================================================
//	Forward Declarations
//========================================================================================

class FW_CLASS_ATTR ODFrame;
class FW_CLASS_ATTR ODShape;
class FW_CLASS_ATTR ODFacet;
class FW_CLASS_ATTR ODStorageUnitView;

class FW_CLASS_ATTR FW_CPrivOrderedCollection;

class FW_CLASS_ATTR FW_CFrame;
class FW_CLASS_ATTR FW_CEmbeddingFrame;
class FW_CLASS_ATTR FW_CEmbeddingPart;
class FW_CLASS_ATTR FW_CPoint;
class FW_CLASS_ATTR FW_CProxyFrame;
class FW_CLASS_ATTR FW_CPresentation;
class FW_CLASS_ATTR FW_CCloneInfo;

//========================================================================================
//	class FW_MProxy
//========================================================================================

class FW_CLASS_ATTR FW_MProxy
{
 public:
	FW_DECLARE_CLASS
	
	friend class FW_CLASS_ATTR FW_CProxyProxyFrameIterator;
	
//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
public:
	FW_MProxy(Environment* ev, FW_CEmbeddingPart* embeddingPart, FW_CPresentation* presentation);
	virtual ~FW_MProxy();
	
//----------------------------------------------------------------------------------------
//	New API
//
public:
	// ----- Embedded frames manipulation -----
	void						DetachEmbeddedFrames(Environment *ev);
	void						AttachEmbeddedFrames(Environment *ev);
	void						RemoveEmbeddedFrames(Environment *ev);

	// ----- Notification -----
	virtual void				UsedShapeChanged(Environment *ev,
												FW_CEmbeddingFrame* embeddingFrame,
												ODFrame* odEmbeddedFrame);

	virtual ODShape* 			FrameShapeRequested(Environment* ev, 
													ODFrame* odEmbeddedFrame, 
													ODShape* askedFrameShape);
										
	// ----- Query -----
	ODFacet*					GetEmbeddedFacetUnderMouse(Environment* ev, 
									ODFacet* embeddingFacet, 
									const FW_CPoint& mouse) const;	
									
	ODFrame*					GetEmbeddedFrame(Environment* ev, 
									FW_CEmbeddingFrame* embeddingFrame) const;
									
	FW_CProxyFrame*				AcquireProxyFrame(Environment* ev, 
									FW_CEmbeddingFrame* embeddingFrame) const;
	
	FW_Boolean					IsEmbeddedFrame(Environment* ev,
									ODFrame* odEmbeddedFrame) const;
									
	// ----- Active border -----
	virtual void				AdjustBorderShape(Environment* ev,
									FW_CEmbeddingFrame* embeddingFrame, 
									ODFacet* embeddedFacet, 
									ODShape *shape);
									
	// ----- Persistence -----
	void						Externalize(Environment* ev, 
									ODStorageUnitView* destinationSUView,
									FW_CCloneInfo *cloneInfo);
	void						Internalize(Environment* ev, 
									ODStorageUnitView* sourceSUView,
									FW_CCloneInfo *cloneInfo);
																		  
	// ----- Selection state -----
	virtual void				SetSelectState(Environment* ev,
									FW_Boolean state);
	FW_Boolean					GetSelectState(Environment* ev) const;
									
	// ----- Getters/Setters -----
	ODPart*						GetEmbeddedPart(Environment* ev) const;
	FW_CEmbeddingPart*			GetPart(Environment* ev) const;
	
	// ----- Presentation -----
	FW_CPresentation*			GetPresentation(Environment* ev) const;
	
	// ----- Set the View Type of this proxy frames -----
	void						SetEmbeddedFramesViewType(Environment *ev, 
									ODTypeToken viewType);
	
	// ----- Position all embedded frames of this proxy -----
	void 						ChangeExternalTransforms(Environment* ev, 
									ODTransform* transform);
	void 						ChangeExternalTransforms(Environment* ev, 
									FW_CFixed x,
									FW_CFixed y);
									
	// ----- Change the frame shape of all embedded frames of this proxy -----
	void 						ChangeFrameShapes(Environment* ev, 
									ODShape* newShape);
	void 						ChangeFrameShapes(Environment* ev, 
									FW_CFixed width,
									FW_CFixed height);

//----------------------------------------------------------------------------------------
//	Implementation API
//
public:
	void						PrivAddProxyFrame(Environment* ev, FW_CProxyFrame* proxyFrame);
	long						PrivRemoveProxyFrame(Environment* ev, FW_CProxyFrame* proxyFrame);

	FW_CProxyFrame*				PrivAcquireProxyFrame(Environment* ev, ODStorageUnitID frameID) const;
	
	void						PrivPostClone(Environment* ev,
									FW_CFrame* scopeFrame);
	
	void						PrivEmbeddedFacetAdded(Environment* ev);
	void						PrivEmbeddedFacetRemoved(Environment* ev);
															
//----------------------------------------------------------------------------------------
//	Data Members
//----------------------------------------------------------------------------------------
private:
	FW_CPrivOrderedCollection*	fProxyFrames;
	FW_CEmbeddingPart*			fPart;
	FW_CPresentation*			fPresentation;
	FW_Boolean					fSelected;
	ODStorageUnitID				fClonedFrameID;		// Used during cloning
	unsigned short				fEmbeddedFacetCount;
};

//========================================================================================
//	FW_MProxy Inlines
//========================================================================================

//----------------------------------------------------------------------------------------
//	FW_MProxy::GetPart
//----------------------------------------------------------------------------------------
inline FW_CEmbeddingPart* FW_MProxy::GetPart(Environment* ev) const
{
	return fPart;
}

//----------------------------------------------------------------------------------------
//	FW_MProxy::GetPresentation
//----------------------------------------------------------------------------------------
inline FW_CPresentation* FW_MProxy::GetPresentation(Environment* ev) const
{
	return fPresentation;
}

//----------------------------------------------------------------------------------------
//	FW_MProxy::GetSelectState
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_MProxy::GetSelectState(Environment* ev) const
{
	return fSelected;
}

//----------------------------------------------------------------------------------------
//	FW_MProxy::PrivEmbeddedFacetAdded
//----------------------------------------------------------------------------------------
inline void FW_MProxy::PrivEmbeddedFacetAdded(Environment* ev)
{
	fEmbeddedFacetCount++;
}

//----------------------------------------------------------------------------------------
//	FW_MProxy::PrivEmbeddedFacetRemoved
//----------------------------------------------------------------------------------------
inline void FW_MProxy::PrivEmbeddedFacetRemoved(Environment* ev)
{
	FW_ASSERT(fEmbeddedFacetCount != 0);
	fEmbeddedFacetCount--;
}

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export off
#endif

#endif

