//========================================================================================
//
//	File:				FWPrtIte.h
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================
//
//	This file contains the iterators on a part defined by ODF .
//
//		- FW_CPartFrameIterator
//			Iterates through all the display frames of your part for a specific
//			presentation. If kNoPresentation is used, all the display frames are returned.
//
//		- FW_CPartFacetIterator
//			Iterates through all the facets of your part for a specific
//			presentation. If kNoPresentation is used, all the facets are returned.
//
//		- FW_CPartEmbeddedFrameIterator
//			This an iterator requiered by OpenDoc. It iterates through all the embedded frames
//			of the specified display frames.
//

#ifndef FWPRTITE_H
#define FWPRTITE_H

#ifndef FWSTDDEF_H
#include "FWStdDef.h"
#endif

#ifndef FWAUTODE_H
#include "FWAutoDe.h"
#endif

#ifndef FWPARTNG_H
#include "FWPartng.h"
#endif

// ----- ODUtil -----

#ifndef FWORDCOL_H
#include "FWOrdCol.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _ODTYPES_
#include <ODTypes.h>
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

//========================================================================================
//	Forward Declarations
//========================================================================================

class FW_CLASS_ATTR FW_CPart;
class FW_CLASS_ATTR FW_CFrame;
class FW_CLASS_ATTR FW_CEmbeddingFrame;
class FW_CLASS_ATTR FW_MProxy;
class FW_CLASS_ATTR ODFrame;
class FW_CLASS_ATTR FW_CProxyODFramesIterator;
class FW_CLASS_ATTR FW_CPartFrameIterator;
class FW_CLASS_ATTR FW_COrderedCollectionIterator;
class FW_CLASS_ATTR FW_CPresentationFrameIterator;
class FW_CLASS_ATTR FW_CPartPresentationIterator;
class FW_CLASS_ATTR FW_CFrameEmbeddedFrameIterator;
class FW_CLASS_ATTR FW_CProxyFrame;
class FW_CLASS_ATTR FW_CEmbeddingPart;

//========================================================================================
//	class FW_CPartFrameIterator
//========================================================================================

class FW_CLASS_ATTR FW_CPartFrameIterator FW_AUTO_DESTRUCT_OBJECT
{
public:
	FW_CPartFrameIterator(const FW_CPart* part);
	~FW_CPartFrameIterator();

	FW_CFrame*			First();
	FW_CFrame*			Next();
	FW_CFrame*			Last();
	FW_CFrame*			Previous();
	FW_Boolean			IsNotComplete()
							{return fCurrentFrame != NULL;}

protected:
	void				PrivFirstFrame(FW_CPresentation* presentation);
	void				PrivLastFrame(FW_CPresentation* presentation);
		
private:
	FW_CFrame*						fCurrentFrame;
	FW_CPartPresentationIterator*	fPresentationIterator;
	FW_CPresentationFrameIterator*	fFrameIterator;
};

//========================================================================================
//	class FW_CPartPresentationIterator
//========================================================================================

class FW_CLASS_ATTR FW_CPartPresentationIterator FW_AUTO_DESTRUCT_OBJECT
{
public:
	FW_CPartPresentationIterator(const FW_CPart* part);
	~FW_CPartPresentationIterator();

	FW_CPresentation*	First()
							{return (FW_CPresentation*)fIterator->First();}
	FW_CPresentation*	Next()
							{return (FW_CPresentation*)fIterator->Next();}
	FW_CPresentation*	Last()
							{return (FW_CPresentation*)fIterator->Last();}
	FW_CPresentation*	Previous()
							{return (FW_CPresentation*)fIterator->Previous();}
	FW_Boolean			IsNotComplete()
							{return fIterator->IsNotComplete();}
							
private:
	FW_COrderedCollectionIterator*	fIterator;
};

//========================================================================================
//	class FW_CPartProxyFrameIterator
//========================================================================================

class FW_CLASS_ATTR FW_CPartProxyFrameIterator FW_AUTO_DESTRUCT_OBJECT
{
public:
	FW_CPartProxyFrameIterator(const FW_CEmbeddingPart* part);
	~FW_CPartProxyFrameIterator();

	FW_CProxyFrame*	First()
						{return (FW_CProxyFrame*)fIterator->First();}
	FW_CProxyFrame*	Next()
						{return (FW_CProxyFrame*)fIterator->Next();}
	FW_CProxyFrame*	Last()
						{return (FW_CProxyFrame*)fIterator->Last();}
	FW_CProxyFrame*	Previous()
						{return (FW_CProxyFrame*)fIterator->Previous();}
	FW_Boolean		IsNotComplete()
						{return fIterator->IsNotComplete();}
							
private:
	FW_COrderedCollectionIterator 		*fIterator;
};

//========================================================================================
//	class FW_CPartEmbeddedFrameIterator
//========================================================================================

class FW_CLASS_ATTR FW_CPartEmbeddedFrameIterator FW_AUTO_DESTRUCT_OBJECT
{
public:
	FW_CPartEmbeddedFrameIterator(Environment* ev, const FW_CEmbeddingPart* part, FW_CEmbeddingFrame* frame = NULL);
	~FW_CPartEmbeddedFrameIterator();

	ODFrame*		First(Environment* ev);
	ODFrame*		Next(Environment* ev);
	FW_Boolean		IsNotComplete(Environment* ev)
						{return fCurrentFrame != NULL;}

private:
	ODFrame*		PrivNext(Environment* ev, FW_CProxyFrame* proxyFrame);
								
private:
	FW_CPartProxyFrameIterator 		fIterator;
	ODFrame*						fCurrentFrame;
};

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export off
#endif

#endif