//========================================================================================
//
//	File:				FWPxyFrm.cpp
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWPXYFRM_H
#define FWPXYFRM_H

#ifndef FWREFCNT_H
#include "FWRefCnt.h"
#endif

#ifndef FWODMISS_H
#include "FWODMiss.h"
#endif

#ifndef _ODTYPES_
#include <ODTypes.h>
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

//========================================================================================
//	Forward Declarations
//========================================================================================

class FW_CLASS_ATTR ODFrame;
class FW_CLASS_ATTR ODCanvas;
class FW_CLASS_ATTR ODStorageUnitView;
class FW_CLASS_ATTR ODTransform;

class FW_CLASS_ATTR FW_CCloneInfo;
class FW_CLASS_ATTR FW_MProxy;
class FW_CLASS_ATTR FW_CEmbeddingFrame;
class FW_CLASS_ATTR FW_CEmbeddingPart;
class FW_CLASS_ATTR FW_CPrivOrderedCollection;
class FW_CLASS_ATTR FW_CRect;

//========================================================================================
//	class FW_CProxyFrame
//========================================================================================

class FW_CLASS_ATTR FW_CProxyFrame : public FW_MRefCount
{
	friend class FW_CLASS_ATTR FW_CEmbeddingPart;
	
//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
public:
	FW_CProxyFrame(Environment* ev, 
				FW_CEmbeddingPart* part);
	FW_CProxyFrame(Environment* ev, 
				FW_CEmbeddingPart* part, 
				FW_MProxy* proxy, 
				FW_CEmbeddingFrame* containingFrame, 
				ODFrame* embeddedFrame);
	FW_CProxyFrame(Environment* ev, 
				FW_CEmbeddingPart* part,
				FW_MProxy* proxy,
				ODID frameID);

protected:
	~FW_CProxyFrame();
	
//----------------------------------------------------------------------------------------
//	New API
//
public:
	FW_CEmbeddingFrame*	GetContainingFrame(Environment* ev) const;
	ODStorageUnitID		GetContainingFrameID(Environment* ev) const;
	
	void				SetContainingFrame(Environment* ev, FW_CEmbeddingFrame* containingFrame);
	
	FW_MProxy*			GetProxy(Environment* ev) const;
	void				SetProxy(Environment* ev, FW_MProxy* proxy);
	
	FW_Boolean			IsFrameInMemory(Environment* ev) const;

	ODFrame*			GetFrame(Environment* ev) const;
	ODStorageUnitID		GetFrameID(Environment* ev) const;

	FW_Boolean			InternalizeProxyFrame(Environment* ev, ODStorageUnitView* suView, FW_CCloneInfo* cloneInfo);
	FW_Boolean			ExternalizeProxyFrame(Environment* ev, ODStorageUnitView* suView, FW_CCloneInfo* cloneInfo);
	
	void				Close(Environment* ev);

	ODSize				Purge(Environment* ev);
				
	FW_Boolean			IsOrphan(Environment* ev) const;
	
	FW_Boolean			IsVisible(Environment* ev, ODFacet* containingFacet, const FW_CRect& bounds) const;
	
	void				SetPurgeable(Environment* ev, FW_Boolean purgeable);
	FW_Boolean			IsPurgeable(Environment* ev) const;
			
	FW_Boolean			HasASelectedFacet(Environment* ev) const;

//----------------------------------------------------------------------------------------
//	Internal API
//
public:	
	void					PrivAttach(Environment* ev);	
	void					PrivDetach(Environment* ev);	
	static FW_CProxyFrame*	PrivAcquireProxyFrame(Environment* ev, 
								FW_CPrivOrderedCollection* proxyFrames, 
								ODStorageUnitID frameID);

//----------------------------------------------------------------------------------------
//	Data Members
//
private:	
	FW_CEmbeddingPart*	fPart;
	
	ODStorageUnitID 	fFrameID;	
	ODStorageUnitID		fContainingFrameID;

	FW_MProxy*			fProxy;
	ODFrame* 			fEmbeddedFrame;
		
	FW_Boolean			fPurgeable;
	FW_Boolean			fAttached;				// Used by FW_MProxy::Attach/DetachEmbeddedFrames
};

//========================================================================================
// CLASS FW_CAcquiredProxyFrame
//========================================================================================

class FW_CLASS_ATTR FW_CAcquiredProxyFrame FW_AUTO_DESTRUCT_OBJECT
{
public:
	FW_CAcquiredProxyFrame();
	FW_CAcquiredProxyFrame(const FW_CAcquiredProxyFrame& other);
	FW_CAcquiredProxyFrame(FW_CProxyFrame* proxyFrame);
	~FW_CAcquiredProxyFrame();

	FW_CAcquiredProxyFrame& operator=(const FW_CAcquiredProxyFrame& other);
	FW_CAcquiredProxyFrame& operator=(FW_CProxyFrame* proxyFrame);

	FW_Boolean operator==(const FW_CAcquiredProxyFrame& other) const
		{return fProxyFrame == other.fProxyFrame;}
	FW_Boolean operator!=(const FW_CAcquiredProxyFrame& other) const
		{return fProxyFrame != other.fProxyFrame;}

	FW_Boolean operator==(FW_CProxyFrame* proxyFrame) const
		{return fProxyFrame == proxyFrame;}
	FW_Boolean operator!=(FW_CProxyFrame* proxyFrame) const
		{return fProxyFrame != proxyFrame;}

	operator FW_CProxyFrame*() const
		{return fProxyFrame;}
	operator const void*() const
		{return fProxyFrame;}		
	FW_CProxyFrame* operator->() const
		{return fProxyFrame;}
	
private:
	void				PrivRelease();

private:
	FW_CProxyFrame*		fProxyFrame;
};

//----------------------------------------------------------------------------------------
//	FW_CProxyFrame::IsPurgeable
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CProxyFrame::IsPurgeable(Environment* ev) const
{
	return fPurgeable;
}

//----------------------------------------------------------------------------------------
//	FW_CProxyFrame::IsFrameInMemory
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CProxyFrame::IsFrameInMemory(Environment* ev) const
{
	return fEmbeddedFrame != NULL;
}

//----------------------------------------------------------------------------------------
//	FW_CProxyFrame::GetFrameID
//----------------------------------------------------------------------------------------
inline ODStorageUnitID FW_CProxyFrame::GetFrameID(Environment* ev) const
{
	return fFrameID;
}

//----------------------------------------------------------------------------------------
//	FW_CProxyFrame::GetProxy
//----------------------------------------------------------------------------------------
inline FW_MProxy* FW_CProxyFrame::GetProxy(Environment* ev) const
{
	return fProxy;
}

//----------------------------------------------------------------------------------------
//	FW_CProxyFrame::GetContainingFrameID
//----------------------------------------------------------------------------------------
inline ODStorageUnitID FW_CProxyFrame::GetContainingFrameID(Environment* ev) const
{
	return fContainingFrameID;
}

//----------------------------------------------------------------------------------------
//	FW_CProxyFrame::IsOrphan
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CProxyFrame::IsOrphan(Environment* ev) const
{
	return fContainingFrameID == kODNULLID;
}

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export off
#endif


#endif
