//========================================================================================
//
//	File:				FWScrolr.h
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1993, 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWSCROLR_H
#define FWSCROLR_H

// ----- OS Layer -----

#ifndef FWPOINT_H
#include "FWPoint.h"
#endif

// ----- Foundation Layer -----

#ifndef FWHFCONN_H
#include "FWHFConn.h"
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

//========================================================================================
// Forward declarations
//========================================================================================

class FW_CLASS_ATTR FW_CPoint;
class FW_CLASS_ATTR FW_CView;
class FW_CLASS_ATTR FW_CScrollBar;
class FW_CLASS_ATTR ODFacet;

//========================================================================================
// CLASS FW_CScroller
//========================================================================================

class FW_CLASS_ATTR FW_CScroller : public FW_MReceiver
{
// ----- Constructors/Destructors
public:
	FW_CScroller(Environment* ev, FW_CView* view);
	virtual ~FW_CScroller();
	
	// ----- Notification
	
	virtual void			RegisterScrollNotifier(Environment* ev, FW_MNotifier* notifier);
	virtual void			UnregisterScrollNotifier(Environment* ev, FW_MNotifier* notifier);
	virtual void			HandleNotification(const FW_CNotification& notification);
	
	// ----- Scrolling
	
	virtual void			ScrollRelative(Environment* ev, const FW_CPoint& by);
	virtual void			UpdateScrollParameters(Environment* ev, FW_Boolean notify = TRUE);

private:
	void					ScrollDraw(Environment* ev, ODFacet* facet, const FW_CPoint& by);

protected:
	FW_CView*						fView;
	
	FW_CHandleFunctionConnection	fConnection;
	
	FW_CFixed						fAutoScrollIncrement;
	ODTypeToken						fScrollNotificationToken;
};

//========================================================================================
// CLASS FW_CScrollBarScroller
//========================================================================================

class FW_CLASS_ATTR FW_CScrollBarScroller : public FW_CScroller
{
public:
	enum Direction { kHorizontal, kVertical };

// ----- Constructors/Destructors
public:
	FW_CScrollBarScroller(Environment* ev,
						  FW_CView* view, 
						  FW_CScrollBar* horzSB, FW_CScrollBar* vertSB);
	virtual ~FW_CScrollBarScroller();

	FW_CScrollBar*			GetScrollBar(Environment* ev, Direction direction) const;
	
	virtual void			UpdateScrollParameters(Environment* ev, FW_Boolean notify = TRUE);

private:
	FW_CScrollBar*			fHorzSB;
	FW_CScrollBar*			fVertSB;
};

//----------------------------------------------------------------------------------------
// FW_CScrollBarScroller::FW_CScrollBarScroller
//----------------------------------------------------------------------------------------

inline FW_CScrollBar* FW_CScrollBarScroller::GetScrollBar(Environment* ev,
														  Direction direction) const
{
	if (direction == kVertical)
		return fVertSB;
	else
		return fHorzSB;
}

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export off
#endif

#endif
