//========================================================================================
//
//	File:				FWSelect.h
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWSELECT_H
#define FWSELECT_H

#ifndef FWPRTDEF_H
#include "FWPrtDef.h"
#endif

// ----- Foundation Includes -----

#ifndef FWSTDDEF_H
#include "FWStdDef.h"
#endif

#ifndef FWRUNTYP_H
#include "FWRunTyp.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _ODTYPES_
#include <ODTypes.h>
#endif

//========================================================================================
//	Runtime Informations
//========================================================================================

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

//========================================================================================
//	Forward Declarations
//========================================================================================

class FW_CLASS_ATTR ODStorageUnit;
class FW_CLASS_ATTR ODStorageUnitView;
class FW_CLASS_ATTR ODFacet;
class FW_CLASS_ATTR ODShape;
class FW_CLASS_ATTR ODSession;

class FW_CLASS_ATTR FW_CPart;
class FW_CLASS_ATTR FW_CFrame;
class FW_CLASS_ATTR FW_CPresentation;
class FW_CLASS_ATTR FW_CCloneInfo;
class FW_CLASS_ATTR FW_CPromise;
class FW_CLASS_ATTR FW_CPoint;
class FW_CLASS_ATTR FW_CMouseEvent;
class FW_CLASS_ATTR FW_CPublishLink;

//========================================================================================
//	constants
//========================================================================================

extern const ODPropertyName FW_kPropFrameInfo; 			// = "Framework:Property:FrameInfo";

//========================================================================================
//	class FW_CSelection
//========================================================================================

class FW_CLASS_ATTR FW_CSelection
{
public:
	FW_DECLARE_CLASS

//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
public:
	FW_CSelection(Environment* ev, FW_Boolean allowPublish, FW_Boolean allowSubscribe);
	virtual	~FW_CSelection();

//----------------------------------------------------------------------------------------
//	New API
//
public:
	virtual FW_Boolean				IsEmpty(Environment* ev) const = 0;	
	virtual void					CloseSelection(Environment* ev) = 0;
	virtual void					SelectAll(Environment* ev) = 0;

	virtual FW_Boolean				ClearSelection(Environment* ev);
	virtual ODShape*				CreateSelectionFrameShape(Environment* ev) const;
	
	// ----- Linking -----	
	virtual FW_Boolean				IsSelectionPublishable(Environment* ev);
	virtual FW_Boolean				CanSubscribe(Environment* ev, ODPasteAsMergeSetting& setting);

	virtual FW_CPublishLink*		DoFindPublisher(Environment* ev);

	// ----- Selection region and hit testing -----	
	virtual FW_Boolean				IsMouseInDraggableItem(Environment* ev, 
											FW_CFrame* frame,
											const FW_CMouseEvent& theMouseEvent, 
											FW_Boolean inBackground) const;							
	virtual ODShape*				CreateSelectionOutline(Environment* ev, 
											ODFacet* facet, 
											FW_CFrame* frame) const;
	virtual ODShape*				CreateSelectionShape(Environment* ev, 
											ODFacet* facet, 
											FW_CFrame* frame) const;

	virtual void					UpdateSelectionOnMouseDown(Environment* ev, 
											const FW_CMouseEvent& mouseEvent,
											ODFacet* embeddedFacet,
											FW_Boolean inEmbeddedFrameBorder,
											FW_Boolean inBackground);
											
	// ----- Promises -----
	virtual FW_CPromise*			NewPromise(Environment* ev, 
											ODUpdateID updateID,
											FW_EStorageKinds storageKind, 
											FW_CFrame* scopeFrame, 
											ODCloneKind cloneKind);
	

	// ----- Externalize/Internalize Selection -----
	virtual FW_EInternalizeResult 	InternalizeData(Environment* ev, 
											FW_CFrame* scopeFrame,
											ODStorageUnit* sourceSU, 
											ODCloneKind cloneKind);
	virtual void					ExternalizeData(Environment* ev, 
											FW_CFrame* scopeFrame,
											FW_EStorageKinds storageKind, 
											ODStorageUnit* destinationSU, 
											ODCloneKind cloneKind);
	
	
	virtual void					DoExternalizeSelection(Environment* ev, 
										ODStorageUnit* destinationSU, 
										FW_CCloneInfo* cloneInfo);
	virtual FW_Boolean				DoInternalizeSelection(Environment* ev, 
										ODStorageUnit* sourceSU, 
										FW_CCloneInfo* cloneInfo) = 0;
	
	// ----- Getters/Setters -----
	FW_CPresentation*				GetPresentation(Environment* ev) const;
	ODSession*						GetSession(Environment* ev) const;
	FW_CPart*						GetPart(Environment* ev) const;

//----------------------------------------------------------------------------------------
//	For Internal Use Only
//
public:	
	void							PrivSetPresentation(FW_CPresentation* presentation);
	
	virtual void					PrivPostInternalizeSelection(Environment* ev, 
										FW_CCloneInfo* cloneInfo,
										FW_EInternalizeResult result);
							
	virtual void					PrivHandleExternalizeSelection(Environment* ev, 
										FW_EStorageKinds storageKind, 
										ODStorageUnit* destinationSU, 
										FW_CCloneInfo* cloneInfo);
	virtual FW_EInternalizeResult	PrivHandleInternalizeSelection(Environment* ev, 
										ODStorageUnit* sourceSU, 
										FW_CCloneInfo* cloneInfo);

//----------------------------------------------------------------------------------------
//	Data Members
//
protected:
	FW_CPresentation*		fPresentation;
	FW_Boolean				fAllowPublish;
	FW_Boolean				fAllowSubscribe;
};

//========================================================================================
//	FW_CSelection Inlines
//========================================================================================

//----------------------------------------------------------------------------------------
//	FW_CSelection::GetPresentation
//----------------------------------------------------------------------------------------
inline FW_CPresentation* FW_CSelection::GetPresentation(Environment*) const
{
	return fPresentation;
}

//----------------------------------------------------------------------------------------
//	FW_CSelection::PrivSetPresentation
//----------------------------------------------------------------------------------------
inline void FW_CSelection::PrivSetPresentation(FW_CPresentation*  presentation)
{
	fPresentation = presentation;
}

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export off
#endif

#endif
