//========================================================================================
//
//	File:				FWTrackr.h
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================
//
//	Three Classes:
//			FW_CTracker
//			FW_CThreeStepTracker
//			FW_COneStepTracker
//
//

#ifndef FWTRACKR_H
#define FWTRACKR_H

#ifndef FWPOINT_H
#include "FWPoint.h"
#endif

#ifndef FWAUTODE_H
#include "FWAutoDe.h"
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

//========================================================================================
//	Forward Declaration
//========================================================================================

class FW_CLASS_ATTR ODFacet;
class FW_CLASS_ATTR ODSession;
class FW_CLASS_ATTR FW_CView;
class FW_CLASS_ATTR FW_CMouseEvent;

//========================================================================================
//	class FW_CBaseTracker
//========================================================================================

class FW_CLASS_ATTR FW_CBaseTracker FW_AUTO_DESTRUCT_OBJECT
{
//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
public:
	FW_CBaseTracker(Environment* ev, FW_CView* view, ODFacet* facet);
	virtual ~FW_CBaseTracker();
	
//----------------------------------------------------------------------------------------
//	New API
//
public:	
	// ----- Tracking -----
	virtual FW_CPoint 	BeginTracking(Environment* ev, 
							const FW_CPoint& anchorPoint) = 0;
	virtual FW_CPoint 	ContinueTracking(Environment* ev,
							const FW_CPoint& anchorPoint, 
							const FW_CPoint& previousPoint, 
							const FW_CPoint& currentPoint) = 0;
	virtual FW_Boolean 	EndTracking(Environment* ev,
							const FW_CPoint& anchorPoint, 
							const FW_CPoint& lastPoint) = 0;
	
	ODFacet*			GetFacet(Environment* ev) const
							{return fFacet;}
	FW_CView*			GetView(Environment* ev) const
							{return fView;}
							
//----------------------------------------------------------------------------------------
//	Data Members
//
protected:
	FW_CPoint			fAnchorPoint;
	FW_CPoint			fPreviousPoint;
	
private:
	ODFacet*			fFacet;	
	FW_CView*			fView;	
};

//========================================================================================
//	class FW_CDropTracker
//========================================================================================

class FW_CLASS_ATTR FW_CDropTracker : public FW_CBaseTracker
{
//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
public:
	FW_CDropTracker(Environment* ev, FW_CView* view, ODFacet* facet);
	virtual ~FW_CDropTracker();
	
//----------------------------------------------------------------------------------------
//	New API
//
public:
	// ----- Dispatch tracking -----
	void 				HandleBeginTracking(Environment *ev,
							const FW_CPoint& where,
							unsigned long dragAttributes);
	void				HandleContinueTracking(Environment* ev,
							const FW_CPoint& where);
	FW_Boolean 			HandleEndTracking(Environment* ev,
							const FW_CPoint& where);

	// ----- Tracking -----
	virtual FW_CPoint 	BeginTracking(Environment* ev, 
							const FW_CPoint& anchorPoint);
	virtual FW_CPoint 	ContinueTracking(Environment* ev,
							const FW_CPoint& anchorPoint, 
							const FW_CPoint& previousPoint, 
							const FW_CPoint& currentPoint);
	virtual FW_Boolean 	EndTracking(Environment* ev,
							const FW_CPoint& anchorPoint, 
							const FW_CPoint& lastPoint);

	// ----- Drag Hilite -----
	virtual void		ShowDragHilite(Environment* ev, ODShape* hiliteShape, FW_Boolean showInside);
	virtual void		HideDragHilite(Environment* ev);

	FW_Boolean			IsInSourceFrame(Environment* ev) const
							{return (fDragAttributes & kODDragIsInSourceFrame);}
	unsigned long		GetDragAttributes(Environment* ev) const;

//----------------------------------------------------------------------------------------
//	Data Members
//
private:
	unsigned long 		fDragAttributes;
	ODSession*			fSession;
	
#ifdef FW_BUILD_MAC
	FW_Boolean			fMacHiliteOn;
#endif
};

//========================================================================================
//	class FW_CTracker
//========================================================================================

class FW_CLASS_ATTR FW_CTracker : public FW_CBaseTracker
{
//----------------------------------------------------------------------------------------
//	Initialization/Destruction
//
public:
	FW_CTracker(Environment* ev, FW_CView* view, ODFacet* facet, FW_Boolean waitMouseMoved = TRUE);
	virtual ~FW_CTracker();
	
//----------------------------------------------------------------------------------------
//	New API
//
public:
	virtual FW_Boolean	Track(Environment* ev, const FW_CMouseEvent& theMouseEvent);

//----------------------------------------------------------------------------------------
//	Data Members
//
private:
	FW_Boolean			fWaitMouseMoved;
};

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export off
#endif

#endif