//========================================================================================
//
//	File:				FWUtil.h
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWUTIL_H
#define FWUTIL_H

#ifndef _STORUTIL_
#include <StorUtil.h>
#endif

// ----- Graphic Includes -----

#ifndef FWGC_H
#include "FWGC.h"
#endif

#ifndef FWGRUTIL_H
#include "FWGrUtil.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _ODTYPES_
#include <ODTypes.h>
#endif

#ifndef SOM_ODRefCntObject_xh
#include <RefCtObj.xh>
#endif

// ----- Macintosh Includes -----

#if defined(FW_BUILD_MAC) && !defined(__QUICKDRAW__)
#include <QuickDraw.h>
#endif

#if defined(FW_BUILD_MAC) && !defined(__TEXTEDIT__)
#include <TextEdit.h>
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

//========================================================================================
//	Forward Declarations
//========================================================================================

class FW_CLASS_ATTR ODFacet;
class FW_CLASS_ATTR ODShape;
class FW_CLASS_ATTR ODWindow;
class FW_CLASS_ATTR ODStorageUnit;

class FW_CLASS_ATTR FW_CFacetContext;
class FW_CLASS_ATTR FW_CGraphicDevice;
class FW_CLASS_ATTR FW_CPoint;
class FW_CLASS_ATTR FW_CRect;

//========================================================================================
//	Global Utilities
//========================================================================================

FW_FUNC_ATTR void 			CenterRectOnScreen(FW_SPlatformRect& aRect, FW_Boolean horizontally, FW_Boolean vertically, FW_Boolean forDialog);

FW_FUNC_ATTR ODShape* 		XorShape(Environment* ev, ODShape* a, ODShape* d);

FW_FUNC_ATTR void 			FW_Beep();

void FW_FUNC_ATTR 			FW_GetMainScreenBounds(FW_CRect& bounds);

//========================================================================================
//	Storage Utilities
//========================================================================================

FW_Boolean FW_FUNC_ATTR FW_SUExistsThenFocus(Environment* ev, 
								ODStorageUnit* su, 
								ODPropertyName prop,
								ODValueType val);
void FW_FUNC_ATTR FW_SUForceFocus(Environment* ev, 
								ODStorageUnit* su, 
								ODPropertyName prop, 
								ODValueType val);
void FW_FUNC_ATTR FW_SUAddPropValue(Environment* ev, 
								ODStorageUnit* su, 
								ODPropertyName prop, 
								ODValueType val);
void FW_FUNC_ATTR FW_SURemoveProperty(Environment* ev, 
								ODStorageUnit* su, 
								ODPropertyName prop);

//========================================================================================
//	class FW_CFacetContext
//========================================================================================

class FW_CLASS_ATTR FW_CFacetContext : public FW_CGraphicContext
{
public:
	FW_CFacetContext(Environment* ev, ODFacet *facet, ODShape* invalidShape = NULL);
	virtual ~ FW_CFacetContext();

	ODFacet*	GetFacet() const
					{return fFacet;}				
private:
	ODFacet*				fFacet;
};

//========================================================================================
//	class FW_CWindowContext
//========================================================================================

class FW_CLASS_ATTR FW_CWindowContext : public FW_CGraphicContext
{
public:
	FW_CWindowContext(Environment* ev, ODWindow *odWindow);	
	FW_CWindowContext(Environment* ev, ODFacet *odFacet);
	virtual	~ FW_CWindowContext();

protected:
	void		InitWindowContext(Environment* ev, ODWindow *odWindow);
	
private:
	ODWindow*			fODWindow;
	FW_CGraphicDevice*	fTempDevice;	// 	We need a temporary device in the case where the root
										//	facet is not one of ours
};

//========================================================================================
//	class FW_CPlatformGraphicContext
//========================================================================================

class FW_CLASS_ATTR FW_CPlatformGraphicContext : public FW_CGraphicContext
{
public:
	FW_CPlatformGraphicContext(Environment* ev, ODFacet *facet, ODShape* invalidShape = NULL);
	virtual ~ FW_CPlatformGraphicContext();

	ODFacet*	GetFacet() const
					{return fFacet;}
private:
	ODFacet*	fFacet;
#ifdef FW_BUILD_MAC
	GrafPtr		fMacPort;
	Point		fMacOrigin;
#endif
};

//========================================================================================
//	Global Inline Methods
//========================================================================================

//----------------------------------------------------------------------------------------
// FW_ReleaseODObject
//----------------------------------------------------------------------------------------
inline void FW_ReleaseODObject(Environment* ev, ODRefCntObject* odObject)
{
	if (odObject)
		odObject->Release(ev);
}

//----------------------------------------------------------------------------------------
// FW_SUForceFocus
//----------------------------------------------------------------------------------------
inline void FW_FUNC_ATTR FW_SUForceFocus(Environment* ev, 
											ODStorageUnit* su, 
											ODPropertyName prop, 
											ODValueType val)
{
	ODSUForceFocus(ev, su, prop, val);
}

//----------------------------------------------------------------------------------------
// FW_SUExistsThenFocus
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_FUNC_ATTR FW_SUExistsThenFocus(Environment* ev, 
													ODStorageUnit* su, 
													ODPropertyName prop, 
													ODValueType val)
{
	return ODSUExistsThenFocus(ev, su, prop, val);
}

//----------------------------------------------------------------------------------------
// FW_SURemoveProperty
//----------------------------------------------------------------------------------------
inline void FW_FUNC_ATTR FW_SURemoveProperty(Environment* ev, 
											ODStorageUnit* su, 
											ODPropertyName prop)
{
	ODSURemoveProperty(ev, su, prop);
}



#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export off
#endif

#endif
