//========================================================================================
//
//	File:				FWViewAs.h
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWVIEWAS_H
#define FWVIEWAS_H

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

#ifndef FWREFCNT_H
#include "FWRefCnt.h"
#endif

//========================================================================================
// Forward class declarations
//========================================================================================

class FW_CLASS_ATTR ODFacet;
class FW_CLASS_ATTR ODShape;
struct FW_CLASS_ATTR Environment;

class FW_CLASS_ATTR FW_CFrame;
class FW_CLASS_ATTR FW_CShape;
class FW_CLASS_ATTR FW_CPart;
class FW_CLASS_ATTR FW_CPresentation;
class FW_CLASS_ATTR FW_CGraphicContext;

//========================================================================================
// CLASS FW_CViewAs
//========================================================================================

class FW_CLASS_ATTR FW_CViewAs : public FW_MRefCount
{	
//----------------------------------------------------------------------------------------
//	Constructor(s)/Destructor
//
public:
	FW_CViewAs(Environment *ev, FW_CPart* part);
	virtual ~FW_CViewAs();
	
//----------------------------------------------------------------------------------------
//	New API
//
public:
	virtual void		Load(Environment* ev, FW_CFrame* frame) = 0;
	virtual ODShape*	CalcUsedShape(Environment* ev) const = 0;
	
	virtual void		Draw(Environment* ev, 
							ODFacet* facet, 
							ODShape* invalidShape);

//----------------------------------------------------------------------------------------
//	Data Members
//
protected:
	FW_CShape*			fShape;
	FW_CPart*			fPart;
};

//========================================================================================
// CLASS FW_CViewAsThumbnail
//========================================================================================

class FW_CLASS_ATTR FW_CViewAsThumbnail : public FW_CViewAs
{
//----------------------------------------------------------------------------------------
//	Constructor(s)/Destructor
//
public:
	FW_CViewAsThumbnail(Environment *ev, FW_CPart* part);
	virtual ~FW_CViewAsThumbnail();
	
//----------------------------------------------------------------------------------------
//	Inherited API
//
public:
	virtual void		Load(Environment* ev, FW_CFrame* frame);
	virtual ODShape*	CalcUsedShape(Environment* ev) const;
								
//----------------------------------------------------------------------------------------
//	New API
//
protected:
	void				RenderThumbnailFrame(Environment* ev, 
							FW_CGraphicContext& gc);		
};

//========================================================================================
// CLASS FW_CViewAsIcon
//========================================================================================

class FW_CLASS_ATTR FW_CViewAsIcon : public FW_CViewAs
{
//----------------------------------------------------------------------------------------
//	Constructor(s)/Destructor
//
public:
	FW_CViewAsIcon(Environment *ev, FW_CPart* part, short iconSize);
	virtual ~FW_CViewAsIcon();
	
//----------------------------------------------------------------------------------------
//	Inherited API
//
public:
	virtual void		Load(Environment* ev, FW_CFrame* frame);
	virtual ODShape*	CalcUsedShape(Environment* ev) const;
	
//----------------------------------------------------------------------------------------
//	Data Members
//
protected:
	short				fIconSize;
};


//========================================================================================
// CLASS FW_CViewAsSmallIcon
//========================================================================================

class FW_CLASS_ATTR FW_CViewAsSmallIcon : public FW_CViewAsIcon
{
//----------------------------------------------------------------------------------------
//	Constructor(s)/Destructor
//
public:
	FW_CViewAsSmallIcon(Environment *ev, FW_CPart* part);
	virtual ~FW_CViewAsSmallIcon();
};

//========================================================================================
// CLASS FW_CViewAsLargeIcon
//========================================================================================

class FW_CLASS_ATTR FW_CViewAsLargeIcon : public FW_CViewAsIcon
{
//----------------------------------------------------------------------------------------
//	Constructor(s)/Destructor
//
public:
	FW_CViewAsLargeIcon(Environment *ev, FW_CPart* part);
	virtual ~FW_CViewAsLargeIcon();	
};


#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export off
#endif

#endif