//========================================================================================
//
//	File:				FWClnInf.cpp
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "FWFrameW.hpp"

#ifndef FWCLNINF_H
#include "FWClnInf.h"
#endif

#ifndef FWFRAME_H
#include "FWFrame.h"
#endif

// ----- OpenDoc Includes -----

#ifndef SOM_ODDraft_xh
#include <Draft.xh>
#endif

#ifndef SOM_ODShape_xh
#include <Shape.xh>
#endif

//========================================================================================
//	Runtime Info
//========================================================================================

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

#ifdef FW_BUILD_MAC
#pragma segment fw_embedding
#endif

//========================================================================================
//	struct FW_CCloneInfo
//========================================================================================

//----------------------------------------------------------------------------------------
//	FW_CCloneInfo::FW_CCloneInfo
//----------------------------------------------------------------------------------------

FW_CCloneInfo::FW_CCloneInfo(Environment* ev, ODDraft* fromDraft, FW_CFrame* scopeFrame, ODCloneKind cloneKind):
	fKey(0),
	fFromDraft(fromDraft),
	fScopeFrame(scopeFrame),
	fCloning(FALSE),
	fBeganClone(FALSE),
	fCloneKind(cloneKind),
	fClonedProxyList(NULL),
	fClonedPartID(kODNULLID),
	fClonedFrameID(kODNULLID),
	fFrameShape(NULL)
{
	FW_END_CONSTRUCTOR
}

//----------------------------------------------------------------------------------------
//	FW_CCloneInfo::FW_CCloneInfo
//----------------------------------------------------------------------------------------

FW_CCloneInfo::FW_CCloneInfo(Environment* ev, ODDraftKey key, ODDraft* fromDraft, FW_CFrame* scopeFrame, ODCloneKind cloneKind):
	fKey(key),
	fFromDraft(fromDraft),
	fScopeFrame(scopeFrame),
	fCloning(TRUE),
	fBeganClone(FALSE),
	fCloneKind(cloneKind),
	fClonedProxyList(NULL),
	fClonedPartID(kODNULLID),
	fClonedFrameID(kODNULLID),
	fFrameShape(NULL)
{
	FW_END_CONSTRUCTOR
}

//----------------------------------------------------------------------------------------
//	FW_CCloneInfo::~FW_CCloneInfo
//----------------------------------------------------------------------------------------

FW_CCloneInfo::~FW_CCloneInfo()
{
	FW_START_DESTRUCTOR
	
	Environment* ev = somGetGlobalEnvironment();
	
	if (fCloning && fBeganClone)
		fFromDraft->AbortClone(ev, fKey);

	if (fFrameShape)
		fFrameShape->Release(ev);
		
	if (fClonedProxyList)
	{
		delete fClonedProxyList;	// will call remove all
		fClonedProxyList = NULL;
	}	
}

//----------------------------------------------------------------------------------------
//	FW_CCloneInfo::BeginClone
//----------------------------------------------------------------------------------------

void FW_CCloneInfo::BeginClone(Environment* ev, ODDraft* destinationDraft)
{
	FW_ASSERT(!fCloning);
	fKey = fFromDraft->BeginClone(ev, destinationDraft, fScopeFrame->GetODFrame(ev), fCloneKind);
	fCloning = TRUE;
	fBeganClone = TRUE;
}

//----------------------------------------------------------------------------------------
//	FW_CCloneInfo::EndClone
//----------------------------------------------------------------------------------------

void FW_CCloneInfo::EndClone(Environment* ev)
{
	FW_ASSERT(fCloning);
	FW_ASSERT(fBeganClone);
	
	fFromDraft->EndClone(ev, fKey);
	fCloning = FALSE;
}

//----------------------------------------------------------------------------------------
//	FW_CCloneInfo::AbortClone
//----------------------------------------------------------------------------------------

void FW_CCloneInfo::AbortClone(Environment* ev)
{
	FW_ASSERT(fCloning);
	FW_ASSERT(fBeganClone);

	fFromDraft->AbortClone(ev, fKey);
	fCloning = FALSE;
}

//----------------------------------------------------------------------------------------
//	FW_CCloneInfo::Clone
//----------------------------------------------------------------------------------------

ODID FW_CCloneInfo::Clone(Environment* ev, ODID fromObjectID, ODID toObjectID, ODID scope)
{
	FW_ASSERT(fCloning);

	return fFromDraft->Clone(ev, fKey, fromObjectID, toObjectID, scope);
}

