//========================================================================================
//
//	File:				FWDrgDrp.cpp
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "FWFrameW.hpp"

#ifndef FWDRGDRP_H
#include "FWDrgDrp.h"
#endif

#ifndef FWPART_H
#include "FWPart.h"
#endif

#ifndef FWFRAME_H
#include "FWFrame.h"
#endif

#ifndef FWSELECT_H
#include "FWSelect.h"
#endif

#ifndef FWDRCMD_H
#include "FWDrCmd.h"
#endif

#ifndef FWTRACKR_H
#include "FWTrackr.h"
#endif

#ifndef FWUTIL_H
#include "FWUtil.h"
#endif

#ifndef FWFRMINFL_H
#include "FWFrmInf.h"
#endif

#ifndef FWPRESEN_H
#include "FWPresen.h"
#endif

// ----- OS Includes -----

#ifndef FWEVENT_H
#include "FWEvent.h"
#endif

#ifndef FWBARRAY_H
#include "FWBArray.h"
#endif

#ifndef FWACQUIR_H
#include "FWAcquir.h"
#endif

// ----- Foundation Includes -----

#ifndef FWMEMMGR_H
#include "FWMemMgr.h"
#endif

#ifndef FWDEBUG_H
#include "FWDebug.h"
#endif

// ----- OpenDoc Includes -----

#ifndef SOM_Module_OpenDoc_StdDefs_defined
#include <StdDefs.xh>
#endif

#ifndef SOM_ODDragAndDrop_xh
#include <DragDrp.xh>
#endif

#ifndef SOM_ODDragItemIterator_xh
#include <DgItmIt.xh>
#endif

#ifndef SOM_ODSession_xh
#include <ODSessn.xh>
#endif

#ifndef SOM_ODTranslation_xh
#include <Translt.xh>
#endif

#ifndef SOM_ODTransform_xh
#include <Trnsform.xh>
#endif

#ifndef SOM_Module_OpenDoc_StdProps_defined
#include <StdProps.xh>
#endif

#ifndef SOM_Module_OpenDoc_StdTypes_defined
#include <StdTypes.xh>
#endif

// ----- Macintosh Includes -----

#if defined(FW_BUILD_MAC) && !defined(__DRAG__)
#include <Drag.h>
#endif

//========================================================================================
//	RunTime Info
//========================================================================================

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

#ifdef FW_BUILD_MAC
#pragma segment FWFrameworkCommands
#endif

FW_DEFINE_CLASS_M0(FW_MDragDroppable)

//========================================================================================
//	Statics
//========================================================================================

//----------------------------------------------------------------------------------------
//	FW_MDragDroppable::ODFrameToDragDroppable
//----------------------------------------------------------------------------------------

FW_MDragDroppable* FW_MDragDroppable::ODFrameToDragDroppable(Environment* ev, ODFrame* odFrame)
{
	FW_MDragDroppable* dragDrop = NULL;
	
	if (odFrame)
	{
		// FW_MEventHandler* eventHandler = (FW_MEventHandler*)odFrame->GetPartInfo(ev);
		FW_SEventHandlerPartInfo* partInfo = (FW_SEventHandlerPartInfo*)odFrame->GetPartInfo(ev);
		FW_MEventHandler* eventHandler = partInfo ? partInfo->fEventHandler : NULL;
		dragDrop = FW_DYNAMIC_CAST(FW_MDragDroppable, eventHandler);
	}
	
	return dragDrop;
}

//========================================================================================
//	class FW_MDragDroppable
//========================================================================================

//----------------------------------------------------------------------------------------
//	FW_MDragDroppable::FW_MDragDroppable
//----------------------------------------------------------------------------------------

FW_MDragDroppable::FW_MDragDroppable(Environment *ev, FW_CPart* thePart) :
	fPart(thePart),
	fCursorInContent(FALSE),
	fCanAcceptDrop(FALSE),
	fDropTracker(NULL)
{
}

//----------------------------------------------------------------------------------------
//	FW_MDragDroppable::~FW_MDragDroppable
//----------------------------------------------------------------------------------------

FW_MDragDroppable::~FW_MDragDroppable()
{
	delete	fDropTracker;
	fDropTracker = NULL;
}

//----------------------------------------------------------------------------------------
//	FW_MDragDroppable::GetCursorInContent()
//----------------------------------------------------------------------------------------

FW_Boolean FW_MDragDroppable::GetCursorInContent(Environment* ev) const
{
	return fCursorInContent;
}

//----------------------------------------------------------------------------------------
//	FW_MDragDroppable::SetCursorInContent()
//----------------------------------------------------------------------------------------

void FW_MDragDroppable::SetCursorInContent(Environment* ev, FW_Boolean isInContent)
{
	fCursorInContent = isInContent;
}

//----------------------------------------------------------------------------------------
//	FW_MDragDroppable::GetCanAcceptDrop()
//----------------------------------------------------------------------------------------

FW_Boolean FW_MDragDroppable::GetCanAcceptDrop(Environment* ev) const
{
	return fCanAcceptDrop;
}

//----------------------------------------------------------------------------------------
//	FW_MDragDroppable::SetCanAcceptDrop()
//----------------------------------------------------------------------------------------

void FW_MDragDroppable::SetCanAcceptDrop(Environment* ev, FW_Boolean acceptDrop)
{
	fCanAcceptDrop = acceptDrop;
}

//----------------------------------------------------------------------------------------
//	FW_MDragDroppable::GetDropTracker()
//----------------------------------------------------------------------------------------

FW_CDropTracker* FW_MDragDroppable::GetDropTracker(Environment* ev) const
{
	return fDropTracker;
}

//----------------------------------------------------------------------------------------
//	FW_MDragDroppable::SetDragging
//----------------------------------------------------------------------------------------

void FW_MDragDroppable::SetDragging(Environment* ev, FW_Boolean dragging)
{
}

//----------------------------------------------------------------------------------------
//	FW_MDragDroppable::SetDroppable
//----------------------------------------------------------------------------------------

void FW_MDragDroppable::SetDroppable(Environment* ev, FW_Boolean droppable)
{
}

//----------------------------------------------------------------------------------------
//	FW_MDragDroppable::Drag
//----------------------------------------------------------------------------------------

FW_Boolean FW_MDragDroppable::Drag(Environment *ev, const FW_CMouseEvent& theMouseEvent)
{
	if (!theMouseEvent.WaitUntilMouseMoved(ev))
		return FALSE;
	
	FW_CFrame *frame = FW_CFrame::ODtoFWFrame(ev, theMouseEvent.GetFacet(ev)->GetFrame(ev));
	FW_ASSERT(frame != NULL);
	
	FW_CDragCommand* dragCommand = this->NewDragCommand(ev, frame);
	if (dragCommand)
	{
		dragCommand->BeginDrag(ev, theMouseEvent);
		fPart->ExecuteCommand(ev, dragCommand);
		return TRUE;
	}

	return FALSE;
}

//----------------------------------------------------------------------------------------
//	FW_MDragDroppable::CanAcceptDrop
//----------------------------------------------------------------------------------------

ODDragResult FW_MDragDroppable::CanAcceptDrop(Environment *ev, ODDragItemIterator* dragInfo)
{
	ODDragResult canAcceptDrop = FALSE;
	
	ODStorageUnit *dragSU;
	ODTranslation *translate = fPart->GetSession(ev)->GetTranslation(ev);
	ODType applehfs = translate->GetISOTypeFromPlatformType(ev, 0x68667320UL, kODPlatformDataType); // 'hfs '

	for (dragSU = dragInfo->First(ev); dragSU; dragSU = dragInfo->Next(ev))
		if (dragSU->Exists(ev, kODPropContents, fPart->GetPartKind(ev), 0))
			canAcceptDrop = TRUE;
		else if (dragSU->Exists(ev, kODPropContents, applehfs, 0))
		{
#ifdef FW_BUILD_MAC
			HFSFlavor *valueData;
			long valueSize;
			
			dragSU->Focus(ev, kODPropContents, kODPosUndefined, applehfs, 0, kODPosUndefined);		
			valueSize = dragSU->GetSize(ev);
			valueData = (HFSFlavor*)FW_CMemoryManager::AllocateBlock(valueSize);
			if (valueData)
			{
				FW_CByteArray byteArray;	
				dragSU->GetValue(ev, valueSize, byteArray);
				byteArray.CopyBuffer(valueData, valueSize);
				
				if (valueData->fileType == 'bnto')
					canAcceptDrop = TRUE;
			}
			FW_CMemoryManager::FreeBlock(valueData);
#endif
		}
	
	return canAcceptDrop;
}

//----------------------------------------------------------------------------------------
//	FW_MDragDroppable::DragEnter
//----------------------------------------------------------------------------------------
// 'where' is in frame coordinate

ODDragResult FW_MDragDroppable::DragEnter(Environment *ev, 
											ODFacet* odFacet, 
											ODDragItemIterator* dragInfo, 
											const FW_CPoint& where)
{	
	FW_CFrame *frame = FW_CFrame::ODtoFWFrame(ev, odFacet->GetFrame(ev));
	FW_ASSERT(frame != NULL);

	FW_CView* contentView = frame->GetContentView(ev);
	FW_ASSERT(contentView != NULL);
	
	fCanAcceptDrop = fPart->IsReadOnly(ev) ? FALSE : CanAcceptDrop(ev, dragInfo);
	
	fDropTracker = NewDropTracker(ev, contentView, odFacet);
	
	fCursorInContent = FALSE;
		
	return fCanAcceptDrop;
}

//----------------------------------------------------------------------------------------
//	FW_MDragDroppable::DragWithin
//----------------------------------------------------------------------------------------
// 'where' is in frame coordinate

ODDragResult FW_MDragDroppable::DragWithin(Environment* ev, 
											ODFacet* odFacet, 
											ODDragItemIterator* dragInfo, 
											const FW_CPoint& where)
{
	FW_UNUSED(dragInfo);

	if (!fCanAcceptDrop)
		return FALSE;
		
// [HLX] OpenDoc Bug Begin
//	OpenDoc sometimes call DragWithin without calling DragEnter
	if (fDropTracker == NULL)
		DragEnter(ev, odFacet, dragInfo, where);
// 	[HLX] OpenDoc Bug End	

	ODSession* session = fPart->GetSession(ev);
	unsigned long attributes = session->GetDragAndDrop(ev)->GetDragAttributes(ev);

	FW_CFrame *frame = FW_CFrame::ODtoFWFrame(ev, odFacet->GetFrame(ev));
	FW_ASSERT(frame != NULL);

	FW_CView* contentView = frame->GetContentView(ev);
	
	FW_Boolean cursorInContent = contentView->IsMouseWithin(ev, odFacet, where);
	
	if (cursorInContent)
	{
		if (!fCursorInContent)
			fDropTracker->HandleBeginTracking(ev, where, attributes);
	}
	else
	{
		if (fCursorInContent)
			fDropTracker->HandleEndTracking(ev, where);
	}
	
	fCursorInContent = cursorInContent;
	
	if (fCursorInContent)
		fDropTracker->HandleContinueTracking(ev, where);
		
	return TRUE;
}

//----------------------------------------------------------------------------------------
//	FW_MDragDroppable::DragLeave
//----------------------------------------------------------------------------------------
// 'where' is in frame coordinate

void FW_MDragDroppable::DragLeave(Environment *ev, ODFacet* odFacet, const FW_CPoint& where)
{
	if (!fCanAcceptDrop)
		return;
	
	FW_ASSERT(fDropTracker);
	
	if (fCursorInContent)	
		fDropTracker->HandleEndTracking(ev, where);
	
	fCursorInContent = FALSE;
	
	// If there was a tracker delete it
	delete fDropTracker;		
	fDropTracker = NULL;
}

//----------------------------------------------------------------------------------------
//	FW_MDragDroppable::AcquireDragHiliteShape
//----------------------------------------------------------------------------------------

ODShape* FW_MDragDroppable::AcquireDragHiliteShape(Environment* ev, ODFacet* odFacet)
{
	FW_CFrame *frame = FW_CFrame::ODtoFWFrame(ev, odFacet->GetFrame(ev));
	return frame->AcquireContentShape(ev);
}

//----------------------------------------------------------------------------------------
//	FW_MDragDroppable::Drop	
//----------------------------------------------------------------------------------------

FW_DECLARE_THROW_POINT (FW_MDragDroppable_Drop_BeforePerform);

ODDropResult FW_MDragDroppable::Drop(Environment *ev, ODDragItemIterator* dropInfo,
									ODFacet* odFacet, const FW_CPoint& where)	// in frame coordinate
{
	ODDropResult dropResult = kODDropFail;

	if (fCursorInContent)
	{
		FW_ASSERT(fDropTracker);
		fDropTracker->HandleEndTracking(ev, where);
	}
	
	delete fDropTracker;	
	fDropTracker = NULL;

	if (fPart->IsReadOnly(ev))
		return dropResult;

	FW_CFrame *frame = FW_CFrame::ODtoFWFrame(ev, odFacet->GetFrame(ev));
	FW_ASSERT(frame != NULL);
			
	// ----- 'where' is in frame coordinate. Put it in content coordinate
	FW_CPoint dropPoint;
	{
		FW_CAcquiredODTransform aqInternalTransform = frame->AcquireInternalTransform(ev, NULL);
		dropPoint = where.InverseTransformCopy(ev, aqInternalTransform);
	}
	
	FW_CDropCommand* dropCommand = this->NewDropCommand(ev, frame, dropInfo, odFacet, dropPoint);
	
	if (dropCommand)
	{
		FW_TRY {
			FW_CHECK_THROW_POINT (FW_MDragDroppable_Drop_BeforePerform);
			dropCommand->PerformCommand(ev);
		}
		FW_CATCH_BEGIN
		FW_CATCH_EVERYTHING () {
			delete dropCommand;
			FW_THROW_SAME ();
		}
		FW_CATCH_END
		
		dropResult = dropCommand->GetDropResult(ev);
		if (dropResult == kODDropFail)
			delete dropCommand;
		else if (!dropCommand->HasAddedAction(ev))	// OpenDoc never saw this command
		{
			dropCommand->CommitDone(ev);
			delete dropCommand;
		}
	}

	return dropResult;
}

//----------------------------------------------------------------------------------------
//	FW_MDragDroppable::NewDragCommand
//----------------------------------------------------------------------------------------

FW_CDragCommand* FW_MDragDroppable::NewDragCommand(Environment *ev, FW_CFrame* theFrame)
{
	return FW_NEW(FW_CDragCommand, (ev, theFrame, FW_kCantUndo));
}

//----------------------------------------------------------------------------------------
//	FW_MDragDroppable::NewDropCommand
//----------------------------------------------------------------------------------------

FW_CDropCommand* FW_MDragDroppable::NewDropCommand(Environment *ev, FW_CFrame* frame,
										ODDragItemIterator* dropInfo, ODFacet* odFacet,
										const FW_CPoint& dropPoint)
{
	return FW_NEW(FW_CDropCommand, (ev, fPart, frame, dropInfo, odFacet, dropPoint, FW_kCantUndo));
}

//----------------------------------------------------------------------------------------
//	FW_MDragDroppable::NewDropTracker
//----------------------------------------------------------------------------------------

FW_CDropTracker* FW_MDragDroppable::NewDropTracker(Environment *ev, FW_CView* view, ODFacet* facet)
{
	return FW_NEW(FW_CDropTracker, (ev, view, facet));
}


