//========================================================================================
//
//	File:				FWPart.cpp
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "FWFrameW.hpp"

#ifndef FWPART_H
#include "FWPart.h"
#endif

#ifndef FWFRAME_H
#include "FWFrame.h"
#endif

#ifndef FWUTIL_H
#include "FWUtil.h"
#endif

#ifndef FWPRESEN_H
#include "FWPresen.h"
#endif

#ifndef FWSELECT_H
#include "FWSelect.h"
#endif

#ifndef FWPRTITE_H
#include "FWPrtIte.h"
#endif

#ifndef FWLNKMGR_H
#include "FWLnkMgr.h"
#endif

#ifndef FWCMD_H
#include "FWCmd.h"
#endif

#ifndef FWPRMISE_H
#include "FWPrmise.h"
#endif

#ifndef FWMNUBAR_H
#include "FWMnuBar.h"
#endif

#ifndef FWMEMMGR_H
#include "FWMemMgr.h"
#endif

#ifndef FWEVENTD_H
#include "FWEventD.h"
#endif

// ----- OpenDoc Utilites -----

#ifndef _INFOUTIL_
#include "InfoUtil.h"
#endif

// ----- OpenDoc Includes -----

#ifndef SOM_ODSession_xh
#include <ODSessn.xh>
#endif

#ifndef SOM_Module_OpenDoc_Foci_defined
#include <Foci.xh>
#endif

#ifndef SOM_Module_OpenDoc_StdProps_defined
#include <StdProps.xh>
#endif

#ifndef SOM_ODArbitrator_xh
#include <Arbitrat.xh>
#endif

#ifndef SOM_ODClipboard_xh
#include <Clipbd.xh>
#endif

#ifndef SOM_Module_OpenDoc_StdTypes_defined
#include <StdTypes.xh>
#endif

#ifndef SOM_Module_OpenDoc_Commands_defined
#include <CmdDefs.xh>
#endif

#ifndef SOM_ODTranslation_xh
#include <Translt.xh>
#endif

//========================================================================================
// Globals
//========================================================================================

unsigned short FW_CPart::gSessionGlobalsCount = 0;

ODTypeToken FW_CPart::gSelectionFocusToken = 0;
ODTypeToken FW_CPart::gMenuFocusToken = 0;
ODTypeToken FW_CPart::gKeyFocusToken = 0;
ODTypeToken FW_CPart::gModalFocusToken = 0;
ODTypeToken FW_CPart::gClipboardFocusToken = 0;
ODTypeToken FW_CPart::gScrollingFocusToken = 0;
ODTypeToken FW_CPart::gViewAsFrameToken = 0;
ODTypeToken FW_CPart::gViewAsSmallIconToken = 0;
ODTypeToken FW_CPart::gViewAsLargeIconToken = 0;
ODTypeToken FW_CPart::gViewAsThumbnailToken = 0;

#ifdef FW_BUILD_MAC
ODType FW_CPart::gMacPICTDataType = NULL;
ODType FW_CPart::gMacPICTFileType = NULL;
ODType FW_CPart::gMachfsDataType = NULL;
ODType FW_CPart::gMacTEXTFileType = NULL;
ODType FW_CPart::gMacTEXTDataType = NULL;
ODType FW_CPart::gMacMooVFileType = NULL;
#endif

//========================================================================================
// RunTime Info
//========================================================================================

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

#ifdef FW_BUILD_MAC
#pragma segment fwpart2
#endif

FW_DEFINE_CLASS_M2(FW_CPart, FW_MEventHandler, FW_MIdle)

//========================================================================================
//	class FW_CPart
//========================================================================================

//----------------------------------------------------------------------------------------
//	FW_CPart::FW_CPart	
//----------------------------------------------------------------------------------------

FW_CPart::FW_CPart(ODPart* odPart, 
					ODValueType partKind, 
					ODValueType partUserName,
					FW_Instance partInstance, 
					FW_ResourceId iconID) :
	FW_MEventHandler(),
	FW_MIdle(this, NULL),
	fODPart(odPart),
	fPartKind(partKind),
	fPartUserName(partUserName),
	fPartMenuBar(NULL),
	fMenuBar(NULL),
	fEventDispatcher(NULL),
	fActiveFrame(NULL),
	fPresentations(NULL),
	fDefaultPresentation(NULL),
	fDialogToClose(NULL),
	fFramePaging(FALSE),
	fIdleCount(0),
	fDragAndDropPromise(NULL),
	fClipboardPromise(NULL),
	fExtensionManager(NULL),
	fPartInstance(partInstance),
	fIconID(iconID),
	fLinkManager(NULL),
	fPrintInfo(NULL)

{
}

//----------------------------------------------------------------------------------------
//	FW_CPart::~FW_CPart	
//----------------------------------------------------------------------------------------

FW_CPart::~FW_CPart()
{	
	FW_ASSERT(fIdleCount == 0);
	
	FW_ASSERT(fDragAndDropPromise == NULL);	// Should be NULL by now
	FW_ASSERT(fClipboardPromise == NULL);	// Should be NULL by now
	
	delete fPresentations;
	fPresentations = NULL;

	delete fPartMenuBar;
	fPartMenuBar = NULL;
	
	delete fEventDispatcher;
	fEventDispatcher = NULL;

	delete fLinkManager;
	fLinkManager = NULL;
	
	PrivReleaseGlobals();
}

//---------------------------------------------------------------------------------------
//	FW_CPart::Initialize
//---------------------------------------------------------------------------------------
// Call inherited first when overriding

void FW_CPart::Initialize(Environment *ev)
{
	PrivInitGlobals(ev);
	
	// ----- menu bar -----
	fPartMenuBar = new FW_CMenuBar(ev, fODPart);
	fMenuBar = fPartMenuBar;
	
	// ----- Create Event Dispatcher -----
	fEventDispatcher = this->CreateEventDispatcher(ev);
	
	// ----- Presentation collection -----
	fPresentations = new FW_CPrivOrderedCollection;

	// ----- Linking Support -----
	fLinkManager = this->NewLinkManager(ev);
}

//---------------------------------------------------------------------------------------
//	FW_CPart::PrivInitGlobals
//---------------------------------------------------------------------------------------

void FW_CPart::PrivInitGlobals(Environment *ev)
{
	if (gSessionGlobalsCount == 0)
	{
		// ----- Get the session -----
		ODSession* session = GetSession(ev);

		FW_ASSERT(gSelectionFocusToken == 0);
		gSelectionFocusToken = session->Tokenize(ev, kODSelectionFocus);
		
		FW_ASSERT(gMenuFocusToken == 0);
		gMenuFocusToken = session->Tokenize(ev, kODMenuFocus);
		
		FW_ASSERT(gKeyFocusToken == 0);
		gKeyFocusToken = session->Tokenize(ev, kODKeyFocus);
		
		FW_ASSERT(gClipboardFocusToken == 0);
		gClipboardFocusToken = session->Tokenize(ev, kODClipboardFocus);
		
		FW_ASSERT(gModalFocusToken == 0);
		gModalFocusToken = session->Tokenize(ev, kODModalFocus);
		
		FW_ASSERT(gScrollingFocusToken == 0);
		gScrollingFocusToken = session->Tokenize(ev, kODScrollingFocus);
		
		FW_ASSERT(gViewAsFrameToken == 0);
		gViewAsFrameToken = session->Tokenize(ev, kODViewAsFrame);
		
		FW_ASSERT(gViewAsSmallIconToken == 0);
		gViewAsSmallIconToken = session->Tokenize(ev, kODViewAsSmallIcon);
		
		FW_ASSERT(gViewAsLargeIconToken == 0);
		gViewAsLargeIconToken = session->Tokenize(ev, kODViewAsLargeIcon);
		
		FW_ASSERT(gViewAsThumbnailToken == 0);
		gViewAsThumbnailToken = session->Tokenize(ev, kODViewAsThumbnail);

#ifdef FW_BUILD_MAC
		ODTranslation *translate = session->GetTranslation(ev);
		
		FW_ASSERT(gMacPICTDataType == NULL);
		gMacPICTDataType = translate->GetISOTypeFromPlatformType(ev, 'PICT', kODPlatformDataType);
		
		FW_ASSERT(gMacPICTFileType == NULL);
		gMacPICTFileType = translate->GetISOTypeFromPlatformType(ev, 'PICT', kODPlatformFileType);
		
		FW_ASSERT(gMachfsDataType == NULL);
		gMachfsDataType = translate->GetISOTypeFromPlatformType(ev, 'hfs ', kODPlatformDataType);

		FW_ASSERT(gMacTEXTDataType == NULL);
		gMacTEXTDataType = translate->GetISOTypeFromPlatformType(ev, 'TEXT', kODPlatformDataType);
		
		FW_ASSERT(gMacTEXTFileType == NULL);
		gMacTEXTFileType = translate->GetISOTypeFromPlatformType(ev, 'TEXT', kODPlatformFileType);

		FW_ASSERT(gMacMooVFileType == NULL);
		gMacMooVFileType = translate->GetISOTypeFromPlatformType(ev, 'MooV', kODPlatformFileType);
#endif
	}
	
	gSessionGlobalsCount++;
}	

//---------------------------------------------------------------------------------------
//	FW_CPart::PrivReleaseGlobals
//---------------------------------------------------------------------------------------

void FW_CPart::PrivReleaseGlobals()
{
	gSessionGlobalsCount--;
	FW_ASSERT(gSessionGlobalsCount >= 0);
	
	if (gSessionGlobalsCount == 0)
	{
		// I am resetting those globals to their default value
		//	because if the DLL is not unloaded they wont be reinitialized
		gSelectionFocusToken = 0;	
		gMenuFocusToken = 0;		
		gKeyFocusToken = 0;	
		gClipboardFocusToken = 0;		
		gModalFocusToken = 0;	
		gScrollingFocusToken = 0;	
		gViewAsFrameToken = 0;	
		gViewAsSmallIconToken = 0;	
		gViewAsLargeIconToken = 0;	
		gViewAsThumbnailToken = 0;

#ifdef FW_BUILD_MAC
		FW_ASSERT(gMacPICTDataType != NULL);
		FW_CMemoryManager::FreeBlock(gMacPICTDataType);
		gMacPICTDataType = NULL;
		
		FW_ASSERT(gMacPICTFileType != NULL);
		FW_CMemoryManager::FreeBlock(gMacPICTFileType);
		gMacPICTFileType = NULL;

		FW_ASSERT(gMacTEXTDataType != NULL);
		FW_CMemoryManager::FreeBlock(gMacTEXTDataType);
		gMacTEXTDataType = NULL;
		
		FW_ASSERT(gMacTEXTFileType != NULL);
		FW_CMemoryManager::FreeBlock(gMacTEXTFileType);
		gMacTEXTFileType = NULL;

		FW_ASSERT(gMacMooVFileType != NULL);
		FW_CMemoryManager::FreeBlock(gMacMooVFileType);
		gMacMooVFileType = NULL;

		FW_ASSERT(gMachfsDataType != NULL);
		FW_CMemoryManager::FreeBlock(gMachfsDataType);
		gMachfsDataType = NULL;
#endif
	}
}

//---------------------------------------------------------------------------------------
//	FW_CPart::AddProperties
//---------------------------------------------------------------------------------------
//	AddProperties can be called multiple time

void FW_CPart::AddProperties(Environment *ev, ODStorageUnit* storageUnit)
{
	//	FW_SUAddPropValue correctly test if the property and value exist before
	//	trying to create them
	FW_SUAddPropValue(ev, storageUnit, kODPropContents, this->GetPartKind(ev));
}

//---------------------------------------------------------------------------------------
//	FW_CPart::PrivCloneInto
//---------------------------------------------------------------------------------------

void FW_CPart::PrivCloneInto(Environment *ev,
							ODDraftKey key,
							ODStorageUnit* toSU,
							ODFrame* scope)
{		
FW_UNUSED(key);
FW_UNUSED(scope);

	// ----- Only think to do is to externalize my content -----
	ExternalizeContent(ev, toSU, NULL);
}

//---------------------------------------------------------------------------------------
//	FW_CPart::InternalizeContent
//---------------------------------------------------------------------------------------

void FW_CPart::InternalizeContent(Environment *ev, ODStorageUnit* storageUnit, FW_CCloneInfo* cloneInfo)
{
FW_UNUSED(storageUnit);
FW_UNUSED(ev);
FW_UNUSED(cloneInfo);
}

//---------------------------------------------------------------------------------------
//	FW_CPart::ExternalizeContent
//---------------------------------------------------------------------------------------

void FW_CPart::ExternalizeContent(Environment *ev, ODStorageUnit* storageUnit, FW_CCloneInfo* cloneInfo)
{
FW_UNUSED(storageUnit);
FW_UNUSED(ev);
FW_UNUSED(cloneInfo);
}

//---------------------------------------------------------------------------------------
//	FW_CPart::CreateEventDispatcher
//---------------------------------------------------------------------------------------

FW_CEventDispatcher* FW_CPart::CreateEventDispatcher(Environment *ev)
{
FW_UNUSED(ev);

	return new FW_CEventDispatcher(this, fMenuBar);
}

//----------------------------------------------------------------------------------------
//	FW_CPart::NewPresentation	
//----------------------------------------------------------------------------------------

FW_CPresentation* FW_CPart::NewPresentation(Environment *ev, 
											FW_CSelection* selection,
											ODTypeToken presentationType)
{
	return new FW_CPresentation(ev, this, selection, presentationType);
}

//----------------------------------------------------------------------------------------
//	FW_CPart::RegisterPresentation	
//----------------------------------------------------------------------------------------
//	the selection object will be owned by the presentation

FW_CPresentation* FW_CPart::RegisterPresentation(Environment *ev, 
												ODType presentationType,
												FW_Boolean defaultPresentation,
												FW_CSelection* selection)
{
	FW_CPresentation* presentation = NULL;
	FW_VOLATILE(presentation);
	
	FW_TRY
	{
		ODTypeToken token = this->GetSession(ev)->Tokenize(ev, presentationType);
	
		// ----- Test that we don't register a presentation twice -----
		FW_ASSERT(FindPresentation(ev, token) == NULL);
	
		presentation = NewPresentation(ev, selection, token);
		
		if (selection)
			selection->PrivSetPresentation(presentation);
		
		fPresentations->AddLast(presentation);
		
		// ----- if flagged as default or first one -----
		if (defaultPresentation || fDefaultPresentation == NULL)
			fDefaultPresentation = presentation;		
	}
	FW_CATCH_BEGIN
	FW_CATCH_EVERYTHING()
	{
		if (presentation == NULL)
			delete selection;		// Otherwise will be deleted by next line
				
		delete presentation;
		
		FW_THROW_SAME();
	}
	FW_CATCH_END

	return presentation;
}

//----------------------------------------------------------------------------------------
//	FW_CPart::FindPresentation	
//----------------------------------------------------------------------------------------

FW_CPresentation* FW_CPart::FindPresentation(Environment *ev, ODTypeToken presentationType) const
{
	FW_CPartPresentationIterator ite(this);
	for (FW_CPresentation* presentation = ite.First(); ite.IsNotComplete(); presentation = ite.Next())
	{
		if (presentation->GetPresentationType(ev) == presentationType)
			return presentation;
	}
	
	return NULL;
}

//----------------------------------------------------------------------------------------
//	FW_CPart::CountDisplayFrame
//----------------------------------------------------------------------------------------

unsigned long FW_CPart::CountDisplayFrame(Environment *ev) const
{
	unsigned long count = 0;
	
	FW_CPartPresentationIterator ite(this);
	for (FW_CPresentation* presentation = ite.First(); ite.IsNotComplete(); presentation = ite.Next())
	{
		count += presentation->CountFrame(ev);
	}
	
	return count;
}

//----------------------------------------------------------------------------------------
//	FW_CPart::ContentUpdated
//------------------------------------------------------------------------
void FW_CPart::ContentUpdated(Environment* ev, FW_CFrame* frame, ODUpdateID updateID)
{
	// When ContentUpdated is called because of updating a link, a change ID is supplied.
	// Otherwise, we need to create a unique change ID -
	if (updateID == kODUnknownUpdate)
		updateID = this->GetSession(ev)->UniqueUpdateID(ev);

	FW_ASSERT(frame);
	frame->GetODFrame(ev)->ContentUpdated(ev, updateID);
}

//---------------------------------------------------------------------------------------
//	FW_CPart::NewLinkManager
//---------------------------------------------------------------------------------------

FW_CLinkManager* FW_CPart::NewLinkManager(Environment *ev)
{
	// Must be overridden if part creates Links
	return NULL;
}

//---------------------------------------------------------------------------------------
//	FW_CPart::LaunchDialog
//---------------------------------------------------------------------------------------

void FW_CPart::LaunchDialog(Environment *ev, 
							FW_CPresentation* presentation,
							 FW_CWindow::WindowStyle style,
							 const FW_CPoint& location, const FW_CPoint& size)
{
/*
	FW_ASSERT(style & kModelessDialog || style & kModalDialog || style & kSystemDialog);
	
	// ----- We wait for FacetAdded to create the FW_CWindow
	ODWindow* window = FW_CWindow::CreateODWindow(ev, 
													fODPart, 
													NULL, 
													FALSE,	// Non Persistent Frame
													FW_CPart::gViewAsFrameToken,
													presentation->GetPresentationType(ev), "",
													size, location,
													style);

	if (style & kModalDialog || style & kSystemDialog)
	{
		ODFrame* odRootFrame = window->GetRootFrame(ev);
		if (GetSession(ev)->GetArbitrator(ev)->RequestFocus(ev, fModalFocusToken, odRootFrame))
		{
			// it would be tempting to call FocusAcquired but the recipe says no
			FW_CFrame* rootFrame = FW_CFrame::ODtoFWFrame(ev, odRootFrame);
			FW_ASSERT(rootFrame != NULL);

			rootFrame->FocusStateChanged(ev, fModalFocusToken, TRUE);
			this->HideShowFloatingWindows(ev, TRUE);
			
			EnableMenuBar(ev, FALSE);
		}
		else
		{
			delete window;
			window = NULL;
		}
	}

	if (window)
	{
		window->Open(ev);
		window->Show(ev);
		window->Select(ev);
	}
*/
}

//---------------------------------------------------------------------------------------
//	FW_CPart::HideShowFloatingWindows
//---------------------------------------------------------------------------------------

void FW_CPart::HideShowFloatingWindows(Environment *ev, FW_Boolean state)
{
	FW_CPartFrameIterator ite(this);
	for (FW_CFrame* frame = ite.First(); ite.IsNotComplete(); frame = ite.Next())
	{
		FW_CWindow* window = frame->GetWindow(ev);
		FW_ASSERT(window);
		window->HideShowOnActivate(ev, state);		// won't do anything if not a floating window
	}
}

//---------------------------------------------------------------------------------------
//	FW_CPart::GetPartName
//---------------------------------------------------------------------------------------

void FW_CPart::GetPartName(Environment *ev, FW_CString& partName)
{
	ODName* windowName = NULL;

	windowName = ODGetPOName(ev, fODPart, NULL);

	if (windowName != NULL && GetITextStringLength(windowName) != 0)
		partName = GetCStringFromIText(windowName);
	else
		partName = fPartUserName;
		
	if (windowName)
		DisposeIText(windowName);
}

//---------------------------------------------------------------------------------------
//	FW_CPart::EnableMenuBar
//---------------------------------------------------------------------------------------

void FW_CPart::EnableMenuBar(Environment *ev, FW_Boolean enable)
{
#ifdef FW_BUILD_MAC
	GetMenuBar(ev)->MacEnableMenuBar(ev, enable);
#endif

#ifdef FW_BUILD_WIN
	// Windows does this automatically!
#endif
}

//---------------------------------------------------------------------------------------
//	FW_CPart::CloseDialogDeferred
//---------------------------------------------------------------------------------------

void FW_CPart::CloseDialogWindowDeferred(Environment *ev, FW_CWindow* closeDialog)
{
	fDialogToClose = closeDialog;
}
														  
//---------------------------------------------------------------------------------------
//	FW_CPart::InstallMenus
//---------------------------------------------------------------------------------------

void FW_CPart::InstallMenus(Environment *ev, FW_CMenuBar* menuBar)
{
	if (menuBar == NULL)
		menuBar = GetPartMenuBar(ev);
		
	if (menuBar)
		menuBar->PrivDisplay(ev);
}

//---------------------------------------------------------------------------------------
//	FW_CPart::Changed
//---------------------------------------------------------------------------------------
//	ODF API

void FW_CPart::Changed(Environment *ev)
{
	this->GetStorageUnit(ev)->GetDraft(ev)->SetChangedFromPrev(ev);
}

//---------------------------------------------------------------------------------------
//	FW_CPart::PrivGetPresentation
//---------------------------------------------------------------------------------------

FW_CPresentation* FW_CPart::PrivGetPresentation(Environment *ev, ODFrame* odFrame) const
{
	ODTypeToken	presDefault = GetSession(ev)->Tokenize(ev, kODPresDefault);
	ODTypeToken token = presDefault;
	FW_Boolean isRoot = TRUE;
	
	FW_CPresentation* presentation = NULL;

	if (odFrame != NULL)
	{
		token = odFrame->GetPresentation(ev);
		isRoot = odFrame->IsRoot(ev);
	}
	
	if (token == presDefault)
		presentation = fDefaultPresentation;
	else
	{
		presentation = FindPresentation(ev, token);
		
		if (presentation == NULL)
			presentation = fDefaultPresentation;
	}
	
	return ValidatePresentation(ev, presentation, isRoot);
}

//---------------------------------------------------------------------------------------
//	FW_CPart::ValidatePresentation
//---------------------------------------------------------------------------------------

FW_CPresentation* FW_CPart::ValidatePresentation(Environment *ev, FW_CPresentation *presentation, FW_Boolean isRootFrame) const
{
FW_UNUSED(ev);
FW_UNUSED(isRootFrame);

	return presentation;
}

//----------------------------------------------------------------------------------------
//	FW_CPart::SetActiveFrame
//----------------------------------------------------------------------------------------

void FW_CPart::SetActiveFrame(Environment *ev, FW_CFrame*  activeFrame)
{
FW_UNUSED(ev);

	fActiveFrame = activeFrame;
	
	// ----- Activate/Deactivate the eventHandler -----
	if (fActiveFrame != NULL)
		Activate(ev);
	else
		Deactivate(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CPart::IsActive
//----------------------------------------------------------------------------------------

FW_Boolean FW_CPart::IsActive(Environment *ev) const
{
	if (FW_MEventHandler::IsActive(ev))
	{
		ODArbitrator* arbitrator = this->GetSession(ev)->GetArbitrator(ev);
		FW_CAcquiredODFrame aqODFrame = arbitrator->AcquireFocusOwner(ev, FW_CPart::gSelectionFocusToken);
		return aqODFrame == fActiveFrame->GetODFrame(ev);
	}
	else
		return FALSE;
}

//----------------------------------------------------------------------------------------
//	FW_CPart::GetActiveFacet
//----------------------------------------------------------------------------------------

ODFacet* FW_CPart::GetActiveFacet(Environment *ev) const
{
	return (fActiveFrame != NULL) ? fActiveFrame->GetActiveFacet(ev) : NULL;
}

//----------------------------------------------------------------------------------------
//	FW_CPart::GetSession
//----------------------------------------------------------------------------------------

ODSession* FW_CPart::GetSession(Environment *ev) const
{
	return this->GetStorageUnit(ev)->GetSession(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CPart::Release
//----------------------------------------------------------------------------------------
//	Call inherited first when overrided

void FW_CPart::Release(Environment *ev)
{
	unsigned long refcount = fODPart->GetRefCount(ev);

	// ------ The last refcount is caused by the register idle -----
	if (refcount == 1 && IsRegisteredForIdle(ev))
	{
		FW_ASSERT(fIdleCount == 1);	// We should only have the part register by now
		UnregisterIdle(ev);
	}
	
	if (refcount == 0)
	{
		GetStorageUnit(ev)->GetDraft(ev)->ReleasePart(ev, fODPart);
	}
}

//----------------------------------------------------------------------------------------
//	FW_CPart::ReleaseAll
//----------------------------------------------------------------------------------------
//	Call inherited first when overrided

void FW_CPart::ReleaseAll(Environment *ev)
{
	// 	Will resolve or just delete my promises
	ResolveAllPromises(ev);
	
	// ----- Delete any and all links
	if (fLinkManager)
		fLinkManager->RemoveAllLinks(ev);
	
	// ----- Delete all the presentations
	FW_CPresentation* presentation;
	while ((presentation = (FW_CPresentation*)fPresentations->First()) != NULL)
	{
		fPresentations->Remove(presentation);
		delete presentation;
	}		
}

//---------------------------------------------------------------------------------------
//	FW_CPart::ExecuteCommand
//---------------------------------------------------------------------------------------

void FW_CPart::ExecuteCommand(Environment *ev, FW_CCommand* command)
{
	command->PerformCommand(ev);		// execute the command

	if (!command->HasAddedAction(ev))	// OpenDoc never saw this command
	{
		command->CommitDone(ev);
		delete command;
	}
}

//----------------------------------------------------------------------------------------
//	FW_CPart::IsReadOnly
//----------------------------------------------------------------------------------------

FW_Boolean FW_CPart::IsReadOnly(Environment *ev) const
{
	return fODPart->GetStorageUnit(ev)->GetDraft(ev)->GetPermissions(ev) < kODDPSharedWrite;
}

//----------------------------------------------------------------------------------------
//	FW_CPart::HandleAdjustMenus
//----------------------------------------------------------------------------------------
//	The part should have the last word in case it is readonly

FW_Boolean FW_CPart::HandleAdjustMenus(Environment *ev, FW_CMenuBar* menuBar, FW_Boolean hasMenuFocus, FW_Boolean isRoot)
{	
	FW_Boolean result = FW_MEventHandler::HandleAdjustMenus(ev, menuBar, hasMenuFocus, isRoot);

	// ------ Disable items we know have to be disabled ----- 
	if (hasMenuFocus && IsReadOnly(ev))
	{
		menuBar->EnableCommand(ev, kODCommandPaste, FALSE);
		menuBar->EnableCommand(ev, kODCommandCut, FALSE);
		menuBar->EnableCommand(ev, kODCommandClear, FALSE);
		menuBar->EnableCommand(ev, kODCommandPasteAs, FALSE);
	}
	
	return result;
}

//----------------------------------------------------------------------------------------
//	FW_CPart::NewDocumentWindow
//----------------------------------------------------------------------------------------

FW_CWindow* FW_CPart::NewDocumentWindow(Environment* ev)
{
	FW_CRect windRect;	
	::FW_GetMainScreenBounds(windRect);	

	FW_CPresentation* presentation = PrivGetPresentation(ev, NULL);	// NULL means root frame

	return new FW_CWindow(ev,
						GetODPart(ev),
					 	FW_CPart::gViewAsFrameToken,
						presentation->GetPresentationType(ev),
						windRect.Size(),			// I know it is too big but FW_CWindow::__ct will adjust it, 
						windRect.TopLeft(),
						FW_kDocumentWindow);
}

//----------------------------------------------------------------------------------------
//	FW_CPart::PrivCountIdleRegistering
//----------------------------------------------------------------------------------------

void FW_CPart::PrivCountIdleRegistering(FW_Boolean registering)
{
	if (registering)
		fIdleCount++;
	else
	{
		FW_ASSERT(fIdleCount >= 1);
		fIdleCount--;
	}
}

//---------------------------------------------------------------------------------------
//	FW_CPart::IsValidDisplayFrame
//---------------------------------------------------------------------------------------

FW_Boolean FW_CPart::IsValidDisplayFrame(Environment* ev, ODStorageUnitID displayFrameID) const
{
	FW_CPartFrameIterator ite(this);
	for (FW_CFrame* frame = ite.First(); ite.IsNotComplete(); frame = ite.Next())
	{
		if (frame->GetID(ev) == displayFrameID)
			return TRUE;
	}
	
	return FALSE;
}

//---------------------------------------------------------------------------------------
//	FW_CPart::PrivInternalizeProxyFrames
//---------------------------------------------------------------------------------------

void FW_CPart::PrivInternalizeProxyFrames(Environment *ev, ODStorageUnit* storageUnit, FW_CCloneInfo* cloneInfo)
{	
}

//---------------------------------------------------------------------------------------
//	FW_CPart::PrivExternalizeProxyFrames
//---------------------------------------------------------------------------------------

void FW_CPart::PrivExternalizeProxyFrames(Environment *ev, ODStorageUnit* storageUnit, FW_CCloneInfo* cloneInfo)
{
}

//----------------------------------------------------------------------------------------
// FW_CPart::ResolvePromise
//----------------------------------------------------------------------------------------
//	ResolvePromise only works on the clipboard. Doesn't make sens to force on drag/drop

void FW_CPart::ResolvePromise(Environment* ev, 
							ODPropertyName propertyName,
							ODValueType valueType)
{
	FW_ASSERT(fClipboardPromise);
	ODClipboard* clipboard = this->GetSession(ev)->GetClipboard(ev);
	ODStorageUnit* clipboardSU = clipboard->GetContentStorageUnit(ev);
	
	if (clipboard->GetUpdateID(ev) == fClipboardPromise->GetUpdateID(ev))
	{
		if (propertyName == NULL)
			propertyName = kODPropContents;
		if (valueType == NULL)
			valueType = GetPartKind(ev);
			
		FW_ASSERT(clipboardSU->Exists(ev, propertyName, valueType, 0));
		clipboardSU->Focus(ev, propertyName, kODPosUndefined, valueType, 0, kODPosUndefined);
		
		FW_ASSERT(clipboardSU->IsPromiseValue(ev));
		clipboardSU->GetSize(ev);	// will force a FulFillPromise
	}
	else
		DeletePromise(ev, FW_kClipboardStorage);
	
	// ----- Because we should never have to force fulfill this one
	DeletePromise(ev, FW_kDragAndDropStorage);
}

//----------------------------------------------------------------------------------------
// FW_CPart::ResolveAllPromises
//----------------------------------------------------------------------------------------

void FW_CPart::ResolveAllPromises(Environment* ev)
{
	// ----- Clipboard -----
	if (fClipboardPromise != NULL)
	{
		ODClipboard* clipboard = this->GetSession(ev)->GetClipboard(ev);
		ODStorageUnit* clipboardSU = clipboard->GetContentStorageUnit(ev);
		if (clipboard->GetUpdateID(ev) == fClipboardPromise->GetUpdateID(ev))
			clipboardSU->ResolveAllPromises(ev);
		else
			DeletePromise(ev, FW_kClipboardStorage);
	}	
	FW_ASSERT(fClipboardPromise == NULL);		// should be null by now

	// ----- I never resolve the drag and drop promise -----
	DeletePromise(ev, FW_kDragAndDropStorage);
}

//---------------------------------------------------------------------------------------
//	FW_CPart::DeletePromise
//---------------------------------------------------------------------------------------

void FW_CPart::DeletePromise(Environment* ev, FW_EStorageKinds storageKind)
{
	if ((storageKind & FW_kDragAndDropStorage) != 0)
	{
		delete fDragAndDropPromise;
		fDragAndDropPromise = NULL;
	}
	
	if ((storageKind & FW_kClipboardStorage) != 0)
	{
		delete fClipboardPromise;
		fClipboardPromise = NULL;
	}
}

//----------------------------------------------------------------------------------------
//	FW_CPart::SetPromise
//----------------------------------------------------------------------------------------

void FW_CPart::SetPromise(Environment *ev, FW_CPromise* promise)
{
	FW_EStorageKinds storageKind = promise->GetStorageKind(ev);
	
	if (storageKind == FW_kDragAndDropStorage)
	{
		FW_ASSERT(fDragAndDropPromise == NULL);
		fDragAndDropPromise = promise;
	}
	else if (storageKind == FW_kClipboardStorage)
	{
		FW_ASSERT(fClipboardPromise == NULL);
		fClipboardPromise = promise;
	}
	else if (storageKind == FW_kLinkStorage)
	{
		// part doesn't manage this kind of promise, each source link does.
	}
	else
		FW_DEBUG_MESSAGE("Unknown storage kind");
}

//----------------------------------------------------------------------------------------
//	FW_CPart::GetPromise
//----------------------------------------------------------------------------------------

FW_CPromise* FW_CPart::GetPromise(Environment *ev, FW_EStorageKinds storageKind) const
{
	if (storageKind == FW_kDragAndDropStorage)
		return fDragAndDropPromise;
	else
		return fClipboardPromise;
}

//---------------------------------------------------------------------------------------
//	FW_CPart::SupportFramePaging
//---------------------------------------------------------------------------------------

void FW_CPart::SupportFramePaging(Environment* ev, FW_Boolean state)
{
	fFramePaging = state;	
	
	// [HLX] I should bring in all the embedded frames If I don't support frame paging anymore
}

//---------------------------------------------------------------------------------------
//	FW_CPart::Dragging
//---------------------------------------------------------------------------------------

void FW_CPart::Dragging(Environment *ev, FW_CFrame* frame, FW_Boolean state)
{
	// Nothing to do. See FW_CEmbeddingPart::Dragging
}

//---------------------------------------------------------------------------------------
//	FW_CPart::LinkStatusChanged
//---------------------------------------------------------------------------------------

void FW_CPart::LinkStatusChanged(Environment *ev, ODFrame* odFrame)
{
	// Nothing to do. See FW_CEmbeddingPart::LinkStatusChanged
}

//---------------------------------------------------------------------------------------
//	FW_CPart::Purge
//---------------------------------------------------------------------------------------

ODSize FW_CPart::Purge(Environment *ev, ODSize size)
{
	return 0;
}

//---------------------------------------------------------------------------------------
// FW_CPart::SetPrintInfo
//---------------------------------------------------------------------------------------

void FW_CPart::SetPrintInfo(FW_CPrintInfo* printInfo)
{
	fPrintInfo = printInfo;
}

//---------------------------------------------------------------------------------------
// FW_CPart::GetPrintInfo
//---------------------------------------------------------------------------------------

FW_CPrintInfo* FW_CPart::GetPrintInfo() const
{
	return fPrintInfo;
}

