//========================================================================================
//
//	File:				FWPrHdlr.cpp
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "FWFrameW.hpp"

#ifndef FWPRHDLR_H
#include "FWPrHdlr.h"
#endif

#ifndef FWPART_H
#include "FWPart.h"
#endif

#ifndef FWFRAME_H
#include "FWFrame.h"
#endif

#ifndef FWPRINFO_H
#include "FWPrInfo.h"
#endif

#ifndef FWCFMRES_H
#include "FWCFMRes.h"
#endif

#ifndef FWPRINT_K
#include "FWPrint.k"
#endif

#ifndef FWALERT_H
#include "FWAlert.h"
#endif

#ifndef FWGRUTIL_H
#include "FWGrUtil.h"
#endif

#ifndef FWMNUBAR_H
#include "FWMnuBar.h"
#endif

#ifndef FWEVENT_H
#include "FWEvent.h"
#endif

#ifndef   FWRESACC_H
#include "FWResAcc.h"
#endif

#ifndef FWRESTYP_H
#include "FWResTyp.h"
#endif

#ifndef FWODGEOM_H
#include "FWODGeom.h"
#endif

// ----- OpenDoc includes

#ifndef SOM_ODSession_xh
#include <ODSessn.xh>
#endif

#ifndef SOM_ODArbitrator_xh
#include <Arbitrat.xh>
#endif

#ifndef SOM_ODFoci_xh
#include <Foci.xh>
#endif

#ifndef SOM_ODFrame_xh
#include <Frame.xh>
#endif

#ifndef SOM_ODWindowState_xh
#include <WinStat.xh>
#endif

#ifndef SOM_Module_OpenDoc_Commands_defined
#include <CmdDefs.xh>
#endif

//========================================================================================
// RunTime Info
//========================================================================================

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

#ifdef FW_BUILD_MAC
#pragma segment FW_PrintStrategy
#endif

//========================================================================================
// Local helpers
//========================================================================================

#ifdef FW_BUILD_WIN

static FW_Boolean	gWinAbortPrinting = FALSE;
static HWND 		gWinProgressDlg = NULL;

static HWND	WinGetFrameWindow(
			Environment*	ev,
			ODFrame*		frame);
		

static void	WinShowControl(
			HWND		hWnd,
			int			nID,
			BOOL		bShow);

static int CALLBACK WinPrintProgessDlgProc(
			HWND		hWnd,
			UINT		message,
			WPARAM		wParam,
			LPARAM		lParam);

static BOOL CALLBACK WinAbortProc(
			HDC		hDC,
			int		nCode);

#endif

#ifdef FW_BUILD_MAC

static DialogPtr	gMacProgressDlg = NULL;
static PrIdleUPP	gMacIdleProcUPP = NULL;
static FW_Boolean	gMacPrintAborted = FALSE;

static pascal void MacIdleProc();

#endif

//----------------------------------------------------------------------------------------
//	FW_CPrintHandler::FW_CPrintHandler
//----------------------------------------------------------------------------------------

FW_CPrintHandler::FW_CPrintHandler(FW_CPart* part, FW_CFrame* frame) :
	fPart(part),
	fFrame(frame),
	fPrintFrame(NULL)
{
}

//----------------------------------------------------------------------------------------
//	FW_CPrintHandler::~FW_CPrintHandler
//----------------------------------------------------------------------------------------

FW_CPrintHandler::~FW_CPrintHandler()
{
}

//----------------------------------------------------------------------------------------
//	FW_CPrintHandler::GetPart
//----------------------------------------------------------------------------------------

FW_CPart* FW_CPrintHandler::GetPart() const
{
	return fPart;
}

//----------------------------------------------------------------------------------------
//	FW_CPrintHandler::GetFrame
//----------------------------------------------------------------------------------------

FW_CFrame* FW_CPrintHandler::GetFrame() const
{
	return fFrame;
}

//----------------------------------------------------------------------------------------
//	FW_CPrintHandler::GetPrintFrame
//----------------------------------------------------------------------------------------

FW_CFrame* FW_CPrintHandler::GetPrintFrame() const
{
	return fPrintFrame;
}

//----------------------------------------------------------------------------------------
//	FW_CPrintHandler::DoAdjustMenus
//----------------------------------------------------------------------------------------

FW_Boolean FW_CPrintHandler::DoAdjustMenus(Environment *ev, FW_CMenuBar* menuBar, FW_Boolean hasMenuFocus, FW_Boolean isRoot)
{
	if (isRoot)
	{
		menuBar->EnableCommand(ev, kODCommandPageSetup, TRUE);
		menuBar->EnableCommand(ev, kODCommandPrint, fFrame->IsCurrentlyPrintable(ev));
	}

	return FALSE;
}

//----------------------------------------------------------------------------------------
//	FW_CPrintHandler::DoMenu
//----------------------------------------------------------------------------------------

FW_Boolean FW_CPrintHandler::DoMenu(Environment* ev, const FW_CMenuEvent& theMenuEvent)
{
	FW_Boolean result = TRUE;
	
	switch (theMenuEvent.GetCommandID(ev))
	{
		case kODCommandPageSetup:
			HandlePageSetup(ev);
			break;

		case kODCommandPrint:
			HandlePrint(ev);
			break;

		default:
			result = FALSE;
	}
	
	return result;
}

//----------------------------------------------------------------------------------------
//	FW_CPrintHandler::DoPageSetupDialog
//----------------------------------------------------------------------------------------

FW_Boolean FW_CPrintHandler::DoPageSetupDialog(Environment* ev)
{
	FW_Boolean result = FALSE;

	if (PreModalDialog(ev))
	{
		FW_CPrintInfo* printInfo = GetPrintInfo();

#ifdef FW_SUPPORT_GX
		gxJob job = printInfo->GetGXJob();
		if (job != NULL)
		{
			result = ::GXJobDefaultFormatDialog(job, NULL);
		}
		else
		{
#endif
#ifdef FW_BUILD_WIN
			LPPRINTDLG lppd = printInfo->WinGetPrintDlg()
			lppd->Flags	= PD_PRINTSETUP;
			lppd->hwndOwner = WinGetFrameWindow(ev, fFrame->GetODFrame(ev));
			result = ::PrintDlg(lppd);
#endif
#ifdef FW_BUILD_MAC
			THPrint thPrint = printInfo->MacGetTHPrint();
			result = ::PrStlDialog(thPrint);
#endif
#ifdef FW_SUPPORT_GX
		}
#endif
		PostModalDialog(ev);
	}
	
	return result;
}

//----------------------------------------------------------------------------------------
//	FW_CPrintHandler::DoJobDialog
//----------------------------------------------------------------------------------------

FW_Boolean FW_CPrintHandler::DoJobDialog(Environment* ev)
{
	FW_Boolean result = FALSE;

	if (PreModalDialog(ev))
	{
		FW_CPrintInfo* printInfo = GetPrintInfo();

#ifdef FW_SUPPORT_GX
		gxJob job = printInfo->GetGXJob();
		if (job != NULL)
		{
			result = ::GXJobPrintDialog(job, NULL);
		}
		else
		{
#endif
#ifdef FW_BUILD_WIN
			LPPRINTDLG lppd = printInfo->WinGetPrintDlg()
			lppd->Flags	= PD_RETURNDC;
			lppd->hwndOwner = WinGetFrameWindow(ev, fFrame->GetODFrame(ev));
			if (::PrintDlg(lppd))
			{
				printInfo->WinUpdatePageRect();
				result = TRUE;
			}
#endif
#ifdef FW_BUILD_MAC
			THPrint thPrint = printInfo->MacGetTHPrint();
			(*thPrint)->prJob.iFstPage = 1;
			(*thPrint)->prJob.iLstPage = 9999;
	
			result = ::PrJobDialog(thPrint);
#endif
#ifdef FW_SUPPORT_GX
		}
#endif
		PostModalDialog(ev);
	}

	return result;
}

//----------------------------------------------------------------------------------------
//	FW_CPrintHandler::NotifyNoDefaultPrinter
//----------------------------------------------------------------------------------------

void FW_CPrintHandler::NotifyNoDefaultPrinter(Environment* ev)
{
	FW_CDynamicString partName;
	fPart->GetPartName(ev, partName);
	
	FW_CSharedLibraryResourceFile resFile;
	FW_CDynamicString message;
	::FW_LoadStringByID(resFile, FW_kPrintingStrings, MULTISTRINGRES, FW_kNoDefaultPrinterMsg, message);
	
	FW_CAlert::DoNote(partName, message);
}

//----------------------------------------------------------------------------------------
//	FW_CPrintHandler::NotifyPrintingCanceled
//----------------------------------------------------------------------------------------

void FW_CPrintHandler::NotifyPrintingCanceled(Environment* ev)
{
	FW_CDynamicString partName;
	fPart->GetPartName(ev, partName);
	
	FW_CSharedLibraryResourceFile resFile;
	FW_CDynamicString message;
	::FW_LoadStringByID(resFile, FW_kPrintingStrings, MULTISTRINGRES, FW_kPrintingCanceledMsg, message);
	
	FW_CAlert::DoNote(partName, message);
}

//----------------------------------------------------------------------------------------
//	FW_CPrintHandler::PreModalDialog
//----------------------------------------------------------------------------------------

FW_Boolean FW_CPrintHandler::PreModalDialog(Environment* ev)
{
	ODFrame* odFrame = fFrame->GetODFrame(ev);
	ODSession* session = fPart->GetSession(ev);
	ODToken modalFocus = session->Tokenize(ev, kODModalFocus);

	return session->GetArbitrator(ev)->RequestFocus(ev, modalFocus, odFrame);
}

//----------------------------------------------------------------------------------------
//	FW_CPrintHandler::PostModalDialog
//----------------------------------------------------------------------------------------

void FW_CPrintHandler::PostModalDialog(Environment* ev)
{
	ODFrame* odFrame = fFrame->GetODFrame(ev);
	ODSession* session = fPart->GetSession(ev);
	ODToken modalFocus = session->Tokenize(ev, kODModalFocus);

	session->GetArbitrator(ev)->RelinquishFocus(ev, modalFocus, odFrame);
}

//----------------------------------------------------------------------------------------
// FW_CPrintHandler::GetPrintInfo
//----------------------------------------------------------------------------------------

FW_CPrintInfo* FW_CPrintHandler::GetPrintInfo() const
{
	FW_CPrintInfo* printInfo = fPart->GetPrintInfo();

#if 0
	if (printInfo == NULL)
	{
		printInfo = fPart->NewPrintInfo();
		fPart->SetPrintInfo(printInfo);
	}
#endif
	
	return printInfo;
}

//----------------------------------------------------------------------------------------
// FW_CPrintHandler::CreateProgressDialog
//----------------------------------------------------------------------------------------

void FW_CPrintHandler::CreateProgressDialog(
		Environment*		ev,
		ODFrame*			frame,
		long				numPages,
		const FW_CString&	title)
{
	FW_CPrintInfo* printInfo = GetPrintInfo();

#ifdef FW_SUPPORT_GX
	gxJob job = printInfo->GetGXJob();
	if (job != NULL)
		return;
#endif
#ifdef FW_BUILD_WIN
	FW_ASSERT(gWinProgressDlg == NULL);

	HWND hWndParent = WinGetFrameWindow(ev, frame);
	gWinProgressDlg = ::CreateDialog(FW_gInstance,
		FW_kWinPrintDialog, hWndParent, WinPrintProgessDlgProc);
	::EnableWindow(hWndParent, FALSE);

	// Initialize some fields
	::SetDlgItemText(gWinProgressDlg, FW_kFileNameID, title);
	::SetDlgItemInt(gWinProgressDlg, FW_kTotalPagesNumID, numPages, TRUE);
	::ShowWindow(gWinProgressDlg, SW_NORMAL);
	::UpdateWindow(gWinProgressDlg);

	// Set up the abort proc
	gWinAbortPrinting = FALSE;
	::SetAbortProc(lppd->hDC, WinAbortProc);
#endif
#ifdef FW_BUILD_MAC
	FW_ASSERT(gMacProgressDlg == NULL);
	
	THPrint thPrint = printInfo->MacGetTHPrint();
	gMacIdleProcUPP = NewPrIdleProc(MacIdleProc);
	(*thPrint)->prJob.pIdleProc = gMacIdleProcUPP;
	
	gMacPrintAborted = FALSE;

	FW_CSharedLibraryResourceFile resourceFile;
	gMacProgressDlg = ::GetNewDialog(FW_kMacPrintDialog, NULL, (WindowPtr) -1L);
	
	::HideDialogItem(gMacProgressDlg, FW_kCurrentPageStrID);
	::HideDialogItem(gMacProgressDlg, FW_kCurrentPageNumID);

	::HideDialogItem(gMacProgressDlg, FW_kCancelingJobID);

	short iType;
	Handle iHndl;
	Rect iRect;
	Str255 pasString;
	
	::GetDialogItem(gMacProgressDlg, FW_kFileNameID, &iType, &iHndl, &iRect);
	title.ExportPascal(pasString);
	::SetDialogItemText(iHndl, pasString);
	::SetWTitle(gMacProgressDlg, pasString);

	::GetDialogItem(gMacProgressDlg, FW_kTotalPagesNumID, &iType, &iHndl, &iRect);
	::NumToString(numPages, pasString);
	::SetDialogItemText(iHndl, pasString);

	::ShowWindow(gMacProgressDlg);
	::SelectWindow(gMacProgressDlg);
	::UpdateControls(gMacProgressDlg, gMacProgressDlg->visRgn);
	::UpdateDialog(gMacProgressDlg, gMacProgressDlg->visRgn);
#endif
}

//----------------------------------------------------------------------------------------
// FW_CPrintHandler::DestroyProgressDialog
//----------------------------------------------------------------------------------------

void FW_CPrintHandler::DestroyProgressDialog(
		Environment*		ev,
		ODFrame*			frame)
{
	FW_CPrintInfo* printInfo = GetPrintInfo();

#ifdef FW_SUPPORT_GX
	gxJob job = printInfo->GetGXJob();
	if (job != NULL)
		return;
#endif
#ifdef FW_BUILD_WIN
	FW_ASSERT(gWinProgressDlg != NULL);

	::SetAbortProc(lppd->hDC, NULL);

	HWND hWndParent = WinGetFrameWindow(ev, frame);
	::EnableWindow(hWndParent, TRUE);
	::EndDialog(gWinProgressDlg, 0);
	gWinProgressDlg = NULL;
#endif
#ifdef FW_BUILD_MAC
	FW_ASSERT(gMacProgressDlg != NULL);
	
	::DisposeDialog(gMacProgressDlg);
	gMacProgressDlg = NULL;
	
	DisposeRoutineDescriptor(gMacIdleProcUPP);
	gMacIdleProcUPP = NULL;
#endif
}

//----------------------------------------------------------------------------------------
// FW_CPrintHandler::UpdateProgressDialog
//----------------------------------------------------------------------------------------

void FW_CPrintHandler::UpdateProgressDialog(
		long				pageNumber)
{
	FW_CPrintInfo* printInfo = GetPrintInfo();

#ifdef FW_SUPPORT_GX
	gxJob job = printInfo->GetGXJob();
	if (job != NULL)
		return;
#endif
#ifdef FW_BUILD_WIN
	::WinShowControl(gWinProgressDlg, FW_kStartingJobID,	FALSE);
	
	::WinShowControl(gWinProgressDlg, FW_kTotalPagesStrID,	TRUE);
	::WinShowControl(gWinProgressDlg, FW_kTotalPagesNumID,	TRUE);
	
	::WinShowControl(gWinProgressDlg, FW_kCurrentPageStrID,	TRUE);
	::WinShowControl(gWinProgressDlg, FW_kCurrentPageNumID,	TRUE);
	
	::SetDlgItemInt(gWinProgressDlg, FW_kCurrentPageNumID, pageNumber, FALSE);
	
	::UpdateWindow(gWinProgressDlg);
#endif
#ifdef FW_BUILD_MAC
	FW_ASSERT(gMacProgressDlg != NULL);
	
	::HideDialogItem(gMacProgressDlg, FW_kStartingJobID);
	
	::ShowDialogItem(gMacProgressDlg, FW_kTotalPagesStrID);
	::ShowDialogItem(gMacProgressDlg, FW_kTotalPagesNumID);
	
	::ShowDialogItem(gMacProgressDlg, FW_kCurrentPageStrID);
	::ShowDialogItem(gMacProgressDlg, FW_kCurrentPageNumID);

	short iType;
	Handle iHndl;
	Rect iRect;
	Str255 pasString;
	
	::GetDialogItem(gMacProgressDlg, FW_kCurrentPageNumID, &iType, &iHndl, &iRect);
	::NumToString(pageNumber, pasString);
	::SetDialogItemText(iHndl, pasString);

	::UpdateControls(gMacProgressDlg, gMacProgressDlg->visRgn);
	::UpdateDialog(gMacProgressDlg, gMacProgressDlg->visRgn);
#endif
}

//----------------------------------------------------------------------------------------
//	FW_CPrintHandler::HandlePageSetup
//----------------------------------------------------------------------------------------

void FW_CPrintHandler::HandlePageSetup(Environment* ev)
{
	FW_TRY
	{
		FW_CPrintEnvironment printEnv;
	
		FW_CPrintInfo* printInfo = GetPrintInfo();
		if (printInfo == NULL)
		{
			ODStorageUnit*	storageUnit = fPart->GetStorageUnit(ev);
			printInfo = FW_NEW(FW_CPrintInfo, (ev, storageUnit, &printEnv));
			fPart->SetPrintInfo(printInfo);
		}
	
		if(DoPageSetupDialog(ev))
		{
			ODStorageUnit* storageUnit = fPart->GetStorageUnit(ev);
			printInfo->SaveToStorageUnit(ev, storageUnit);
		}
	}
	FW_CATCH_BEGIN
	FW_CATCH_REFERENCE(FW_XException, pr)
	{
		if (pr.GetPlatformError() == FW_xNoDefaultPrinter)
			NotifyNoDefaultPrinter(ev);
		else
			FW_THROW_SAME();
	}
	FW_CATCH_EVERYTHING()
	{
		FW_THROW_SAME();
	}
	FW_CATCH_END
}

//----------------------------------------------------------------------------------------
//	FW_CPrintHandler::HandlePrint
//----------------------------------------------------------------------------------------

void FW_CPrintHandler::HandlePrint(Environment* ev)
{
	FW_CFrame* printFrame = NULL;

	FW_TRY
	{
		FW_CPrintEnvironment printEnv;

		FW_CPrintInfo* printInfo = GetPrintInfo();
		if (printInfo == NULL)
		{
			ODStorageUnit*	storageUnit = fPart->GetStorageUnit(ev);
			printInfo = FW_NEW(FW_CPrintInfo, (ev, storageUnit, &printEnv));
			fPart->SetPrintInfo(printInfo);
		}

		if (DoJobDialog(ev))
		{
			fPrintFrame = CreatePrintFrame(ev);
			PrintDocument(ev, &printEnv);
		}
	}
	FW_CATCH_BEGIN
	FW_CATCH_REFERENCE(FW_XException, pr)
	{
		switch(pr.GetPlatformError())
		{
			case FW_xNoDefaultPrinter:
			{
				NotifyNoDefaultPrinter(ev);
				break;
			}
			case FW_xPrintingCanceled:
			{
				NotifyPrintingCanceled(ev);
				break;
			}
			default:
			{
				if (fPrintFrame != NULL)
				{
					ReleasePrintFrame(ev);
					fPrintFrame = NULL;
				}
				FW_THROW_SAME();
			}
		}
	}
	FW_CATCH_END
	
	if (fPrintFrame != NULL)
	{
		ReleasePrintFrame(ev);
		fPrintFrame = NULL;
	}
}

//----------------------------------------------------------------------------------------
//	FW_CPrintHandler::CreatePrintFrame
//----------------------------------------------------------------------------------------

FW_CFrame* FW_CPrintHandler::CreatePrintFrame(Environment* ev)
{
	return fFrame;
}

//----------------------------------------------------------------------------------------
//	FW_CPrintHandler::ReleasePrintFrame
//----------------------------------------------------------------------------------------

void FW_CPrintHandler::ReleasePrintFrame(Environment* ev)
{
	// Does nothing
}

//----------------------------------------------------------------------------------------
//	FW_CPrintHandler::PrintDocument
//----------------------------------------------------------------------------------------

void FW_CPrintHandler::PrintDocument(Environment* ev, FW_CPrintEnvironment* printEnv)
{
	long xPageCount, yPageCount;
	CountPrintPages(ev, xPageCount, yPageCount);

	FW_ASSERT(xPageCount > 0);
	FW_ASSERT(yPageCount > 0);

	FW_CPrintInfo* printInfo = GetPrintInfo();
	FW_ASSERT(printInfo != NULL);

	short numCopies;
	FW_Boolean collate;
	printInfo->GetCopyCount(numCopies, collate);

	long firstPage, lastPage;
	printInfo->GetPages(firstPage, lastPage);

	FW_ASSERT(firstPage > 0);
	FW_ASSERT(lastPage > 0);

	long numPages = xPageCount * yPageCount;

	if (lastPage > numPages)
	{
		// prevent printing past last page
		lastPage = numPages;
	}
	
	// Get some OD objects that we will need
	ODSession* session = fPart->GetSession(ev);
	ODFrame* odFrame = fPrintFrame->GetODFrame(ev);

	FW_CPrintJob printJob(printInfo, this, printEnv);

	// Print the requested number of copies
	short numCopiesPerPrintJob	= collate ? 1 : numCopies;
	short numCopiesPerPage		= collate ? numCopies : 1;

	for (short copiesOfJob = 1; copiesOfJob <= numCopiesPerPrintJob; ++ copiesOfJob)
	{
		FW_CDynamicString jobTitle;
		fPrintFrame->GetPrintJobTitle(ev, jobTitle);

		ODFacet* facet = printJob.OpenDocument(ev, odFrame, session, numPages, jobTitle);

		FW_TRY
		{
			for (long pageNumber = firstPage; pageNumber <= lastPage; ++ pageNumber)
			{
				for (short copiesOfPage = 1; copiesOfPage <= numCopiesPerPage; ++ copiesOfPage)
				{
					printJob.OpenPage(pageNumber);

					FW_TRY
					{
						long xPage = (pageNumber - 1) % xPageCount;
						long yPage = (pageNumber - 1) / xPageCount;

						PrintPage(ev, facet, xPage, yPage);
					}
					FW_CATCH_BEGIN
					FW_CATCH_EVERYTHING()
					{
						printJob.ClosePageOnError();
						FW_THROW_SAME();
					}
					FW_CATCH_END

					printJob.ClosePage();
				}
			}
		}
		FW_CATCH_BEGIN
		FW_CATCH_EVERYTHING()
		{
			printJob.CloseDocumentOnError(ev, facet);
			FW_THROW_SAME();
		}
		FW_CATCH_END

		printJob.CloseDocument(ev, facet);
	}

	printJob.FinishPrinting();
}

//----------------------------------------------------------------------------------------
//	FW_CPrintHandler::CountPrintPages
//----------------------------------------------------------------------------------------

void FW_CPrintHandler::CountPrintPages(Environment* ev, long& xPageCount, long& yPageCount)
{
	FW_CPrintInfo* printInfo = GetPrintInfo();
	FW_ASSERT(printInfo != NULL);

	FW_CRect 		pageBounds  = printInfo->GetPageBounds();
	FW_CFixed		pageWidth	= pageBounds.Width();
	FW_CFixed		pageHeight	= pageBounds.Height();

	FW_CPoint		content;
	fPrintFrame->GetPrintContentExtent(ev, content);
	
	FW_ASSERT(content.x && content.y);

	xPageCount = ((content.x  + pageWidth.Half()  - FW_kFixedPos1) / pageWidth ).AsInt();
	yPageCount = ((content.y + pageHeight.Half() - FW_kFixedPos1) / pageHeight).AsInt();
}

//----------------------------------------------------------------------------------------
//	FW_CPrintHandler::PrintPage
//----------------------------------------------------------------------------------------

void FW_CPrintHandler::PrintPage(Environment* ev, ODFacet* facet, long xPage, long yPage)
{
    FW_ASSERT(xPage >= 0);
    FW_ASSERT(yPage >= 0);

	FW_CPrintInfo*	printInfo = GetPrintInfo();

	FW_CRect 		pageBounds  = printInfo->GetPageBounds();
	FW_CFixed		pageWidth	= pageBounds.Width();
	FW_CFixed		pageHeight	= pageBounds.Height();

	FW_CPoint		pageOffset(
				pageWidth.MultipliedByInt(xPage),
				pageHeight.MultipliedByInt(yPage));

	// Construct an appropriate transform and shape for this page
	FW_CAcquiredODTransform aqTransform = facet->CreateTransform(ev);
	aqTransform->SetOffset(ev, (ODPoint*) &pageOffset);

	FW_CAcquiredODShape	aqClipShape = facet->CreateShape(ev);
	aqClipShape->SetRectangle(ev, (ODRect*) &pageBounds);
	aqClipShape->Transform(ev, aqTransform);

	// Intersect the clip shape with the print frame's extent
	FW_CPoint			content;
	fPrintFrame->GetPrintContentExtent(ev, content);

	FW_CRect			contentRect(FW_kFixed0, FW_kFixed0, content.x, content.y);

	FW_CAcquiredODShape aqExtentShape = facet->CreateShape(ev);
	aqExtentShape->SetRectangle(ev, (ODRect*) &contentRect);
	aqClipShape->Intersect(ev, aqExtentShape);

	// Make a copy of the clip shape for use as invalid shape
	FW_CAcquiredODShape	aqInvalShape = aqClipShape->Copy(ev);

	aqTransform->Invert(ev);

	// Set the facet to use this transform and shape
	facet->ChangeGeometry(ev, aqClipShape, aqTransform, NULL);
	
	// Now redraw the facet and its embedded facets
	facet->Update(ev, aqInvalShape, NULL);
}

#ifdef FW_BUILD_WIN
//----------------------------------------------------------------------------------------
// WinGetFrameWindow
//----------------------------------------------------------------------------------------

HWND WinGetFrameWindow(
		Environment*	ev,
		ODFrame*		frame)
{
	ODFrameFacetIterator* ite = frame->->CreateFacetIterator(ev);
	ODFacet* facet = ite->First(ev);
	delete ite;
	
	return facet->GetWindow(ev)->GetPlatformWindow(ev);
}

//----------------------------------------------------------------------------------------
// WinShowControl
//----------------------------------------------------------------------------------------

void WinShowControl(
		HWND		hWnd,
		int			nID,
		BOOL		bShow)
{
	HWND hWndControl = ::GetDlgItem(hWnd, nID);
	if (hWndControl != NULL)
		::ShowWindow(hWndControl, bShow ? SW_SHOW : SW_HIDE);
}

//----------------------------------------------------------------------------------------
// WinPrintProgessDlgProc
//----------------------------------------------------------------------------------------

int CALLBACK WinPrintProgessDlgProc(
		HWND		hWnd,
		UINT		message,
		WPARAM		wParam,
		LPARAM		lParam)
{
	switch (message)
	{
	case WM_INITDIALOG:
		return TRUE;

	case WM_COMMAND:
		gWinAbortPrinting = TRUE;
		
		::WinShowControl(gWinProgressDlg, FW_kStartingJobID,		FALSE);

		::WinShowControl(gWinProgressDlg, FW_kTotalPagesStrID,	FALSE);
		::WinShowControl(gWinProgressDlg, FW_kTotalPagesNumID,	FALSE);

		::WinShowControl(gWinProgressDlg, FW_kCurrentPageStrID,	FALSE);
		::WinShowControl(gWinProgressDlg, FW_kCurrentPageNumID,	FALSE);
		
		::WinShowControl(gWinProgressDlg, FW_kCancelingJobID,	TRUE);
		
		::UpdateWindow(gWinProgressDlg);
		
		return TRUE;
	
	default:
		return FALSE;
	}
}

//----------------------------------------------------------------------------------------
// WinAbortProc
//----------------------------------------------------------------------------------------

BOOL CALLBACK WinAbortProc(
			HDC		hDC,
			int		nCode)
{
	MSG msg;
	
	while (::PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
	{
		if (gWinProgressDlg == NULL || !::IsDialogMessage(gWinProgressDlg, &msg))
		{
			::TranslateMessage(&msg);
			::DispatchMessage(&msg);
		}
	}
	
	return !gWinAbortPrinting;
}
#endif

#ifdef FW_BUILD_MAC

//----------------------------------------------------------------------------------------
// MacIdleProc
//----------------------------------------------------------------------------------------

pascal void MacIdleProc()
{
#if 0
	short resFile = ::CurResFile();

	if (!gMacPrintAborted)
	{
		FW_Boolean bCancel = FALSE;

		FW_CMacTempPort tempPort = gMacProgressDlg;

		EventRecord event;
		RgnHandle rgnCursor = NULL;
		if (::WaitNextEvent(everyEvent, &event, 0, rgnCursor))
		{
			if (event.what == keyDown || event.what == autoKey)
			{
				if ((event.message & charCodeMask) == '.')
				{
					if ((event.modifiers & cmdKey) != 0)
					{
						short 			itemType;
						ControlHandle 	hItem = NULL;
						Rect			box;

						::GetDialogItem(gMacProgressDlg, FW_kCancelButtonID, &itemType, (Handle*) &hItem, &box);
						if (hItem != NULL)
						{
							long theTick;
							::HiliteControl(hItem, 1);
							::Delay(6,&theTick);
							::HiliteControl(hItem, 0);
						}

						bCancel = TRUE;
					}
				}
			}
			else if (::IsDialogEvent(&event))
			{
				short nItemHit = 0;
				DialogPtr dialog = NULL;
				if (::DialogSelect(&event, &dialog, &nItemHit) &&
					dialog == gMacProgressDlg &&
					nItemHit == FW_kCancelButtonID)
				{
					bCancel = TRUE;
				}
			}
		}

		if (bCancel)
		{
			// Update the dialog
			::HideDialogItem(gMacProgressDlg, FW_kStartingJobID);
	
			::HideDialogItem(gMacProgressDlg, FW_kTotalPagesStrID);
			::HideDialogItem(gMacProgressDlg, FW_kTotalPagesNumID);
	
			::HideDialogItem(gMacProgressDlg, FW_kCurrentPageStrID);
			::HideDialogItem(gMacProgressDlg, FW_kCurrentPageNumID);
	
			::ShowDialogItem(gMacProgressDlg, FW_kCancelingJobID);
	
			::UpdateControls(gMacProgressDlg, gMacProgressDlg->visRgn);
			::UpdateDialog(gMacProgressDlg, gMacProgressDlg->visRgn);
			
			// Tell the print manager that the job should be aborted
			::PrSetError(iPrAbort);
			
			gMacPrintAborted = TRUE;
		}
	}

	::UseResFile(resFile);
#endif
}

#endif

