//========================================================================================
//
//	File:				FWPrmise.cpp
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "FWFrameW.hpp"

#ifndef FWPRMISE_H
#include "FWPrmise.h"
#endif

#ifndef FWPART_H
#include "FWPart.h"
#endif

#ifndef FWFRAME_H
#include "FWFrame.h"
#endif

#ifndef FWCLNINF_H
#include "FWClnInf.h"
#endif

// ----- OS Layer Includes -----

#ifndef FWBARRAY_H
#include "FWBArray.h"
#endif

// ----- Foundation Layer Includes -----

#ifndef FWMEMORY_H
#include "FWMemory.h"
#endif

// ----- OpenDoc Includes -----

#ifndef SOM_ODStorageUnitView_xh
#include <SUView.xh>
#endif

#ifndef SOM_Module_OpenDoc_StdProps_defined
#include <StdProps.xh>
#endif

#ifndef SOM_ODDraft_xh
#include <Draft.xh>
#endif

//========================================================================================
// RunTime Info
//========================================================================================

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

#ifdef FW_BUILD_MAC
#pragma segment fwpart2
#endif

FW_DEFINE_CLASS_M0(FW_CPromise)

//========================================================================================
//	class FW_CPromise
//========================================================================================

//----------------------------------------------------------------------------------------
//	FW_CPromise::FW_CPromise
//----------------------------------------------------------------------------------------

FW_CPromise::FW_CPromise(Environment*ev, ODUpdateID updateID, FW_EStorageKinds storageKind, FW_CPart* part, FW_CFrame* scopeFrame) :
	fScopeFrame(scopeFrame),
	fPart(part),
	fUnfulfillPromises(0),
	fStorageKind(storageKind),
	fUpdateID(updateID)
{
}

//----------------------------------------------------------------------------------------
//	FW_CPromise::~FW_CPromise
//----------------------------------------------------------------------------------------

FW_CPromise::~FW_CPromise()
{
}

//----------------------------------------------------------------------------------------
//	FW_CPromise::PromisePropertiesAndValues
//----------------------------------------------------------------------------------------
// 	By default only add the kODPropContents with the part kind value
//
//	Call inherited::PromisePropertiesAndValues first when overriding

void FW_CPromise::PromisePropertiesAndValues(Environment *ev, ODStorageUnit* storageUnit)
{
	Promise(ev, storageUnit, kODPropContents, fPart->GetPartKind(ev));
}

//----------------------------------------------------------------------------------------
//	FW_CPromise::Promise
//----------------------------------------------------------------------------------------

void FW_CPromise::Promise(Environment *ev, ODStorageUnit* storageUnit, ODPropertyName propertyName, ODValueType valueType)
{
	FW_ASSERT(propertyName != NULL);
	FW_ASSERT(valueType != NULL);

	storageUnit->Focus(ev, propertyName, kODPosUndefined, NULL, 0, kODPosUndefined);

	FW_CPromise* buffer = this;
	FW_CByteArray bArray(&buffer, sizeof(FW_CPromise*));
	storageUnit->SetPromiseValue(ev,
								valueType,
								0,
								bArray,
								fPart->GetODPart(ev));
	fUnfulfillPromises++;
}

//----------------------------------------------------------------------------------------
//	FW_CPromise::PrivHandleFulfillPromise
//----------------------------------------------------------------------------------------

unsigned short FW_CPromise::PrivHandleFulfillPromise(Environment *ev, ODStorageUnitView *promiseSUView)
{
	FW_ASSERT(fUnfulfillPromises != 0);
	
	ODDraft* fromDraft = fPart->GetStorageUnit(ev)->GetDraft(ev);
	ODDraft* dstDraft = promiseSUView->GetStorageUnit(ev)->GetDraft(ev);

	FW_CAcquireODPropertyName propertyName(ev, promiseSUView);
		
	ODCloneKind cloneKind = (fStorageKind == FW_kLinkStorage) ? kODCloneToLink : kODCloneCopy;

	ODValueType valueType = promiseSUView->GetType(ev);	

	FW_TRY
	{
		FW_CCloneInfo cloneInfo(ev, fromDraft, fScopeFrame, cloneKind);	
		cloneInfo.BeginClone(ev, dstDraft);

		FulfillPromise(ev, promiseSUView, propertyName, valueType, &cloneInfo);
	
		cloneInfo.EndClone(ev);
	}
	FW_CATCH_BEGIN
	FW_CATCH_EVERYTHING()
	{
		FW_CMemoryManager::FreeBlock(valueType);
	
		FW_THROW_SAME();
	}
	FW_CATCH_END

	FW_CMemoryManager::FreeBlock(valueType);

	return --fUnfulfillPromises;
}
