//========================================================================================
//
//	File:				FWTrackr.cpp
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "FWFrameW.hpp"

#ifndef FWTRACKR_H
#include "FWTrackr.h"
#endif

#ifndef FWUTIL_H
#include "FWUtil.h"
#endif

#ifndef FWVIEW_H
#include "FWView.h"
#endif

#ifndef FWFRAME_H
#include "FWFrame.h"
#endif

#ifndef FWPART_H
#include "FWPart.h"
#endif

// ----- OS Includes -----

#ifndef FWRECT_H
#include "FWRect.h"
#endif

#ifndef FWEVENT_H
#include "FWEvent.h"
#endif

#ifndef FWEVENTU_H
#include "FWEventU.h"
#endif

#ifndef FWACQUIR_H
#include "FWAcquir.h"
#endif

// ----- Foundation Includes -----

#ifndef FWDEBUG_H
#include "FWDebug.h"
#endif

// ----- OpenDoc Includes -----

#ifndef SOM_ODTransform_xh
#include <Trnsform.xh>
#endif

#ifndef SOM_ODFacet_xh
#include <Facet.xh>
#endif

#ifndef SOM_ODFrame_xh
#include <Frame.xh>
#endif

#ifndef SOM_ODSession_xh
#include <ODSessn.xh>
#endif

#ifndef SOM_ODDragAndDrop_xh
#include <DragDrp.xh>
#endif

//========================================================================================
//	Runtime Info
//========================================================================================

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

#ifdef FW_BUILD_MAC
#pragma segment framework
#endif

//========================================================================================
//	class FW_CBaseTracker
//========================================================================================

//----------------------------------------------------------------------------------------
//	FW_CBaseTracker::FW_CBaseTracker
//----------------------------------------------------------------------------------------

FW_CBaseTracker::FW_CBaseTracker(Environment *ev, FW_CView* view, ODFacet* facet) :
	fFacet(facet),
	fView(view)
{
}

//----------------------------------------------------------------------------------------
//	FW_CBaseTracker::~FW_CBaseTracker
//----------------------------------------------------------------------------------------

FW_CBaseTracker::~FW_CBaseTracker()
{
}

//========================================================================================
//	class FW_CDropTracker
//========================================================================================

//----------------------------------------------------------------------------------------
//	FW_CDropTracker::FW_CDropTracker
//----------------------------------------------------------------------------------------

FW_CDropTracker::FW_CDropTracker(Environment *ev, FW_CView* view, ODFacet* facet) :
	FW_CBaseTracker(ev, view, facet)
#ifdef FW_BUILD_MAC
	,fMacHiliteOn(FALSE)
#endif
{
	fSession = view->GetFrame(ev)->GetPart(ev)->GetSession(ev);
}

//----------------------------------------------------------------------------------------
//	FW_CDropTracker::~FW_CDropTracker
//----------------------------------------------------------------------------------------

FW_CDropTracker::~FW_CDropTracker()
{
}

//----------------------------------------------------------------------------------------
//	FW_CDropTracker::HandleBeginTracking
//----------------------------------------------------------------------------------------
//	where is in frame coordinate

void FW_CDropTracker::HandleBeginTracking(Environment *ev, const FW_CPoint& where, unsigned long dragAttributes)
{		
	fDragAttributes = dragAttributes;
	
	FW_CAcquiredODTransform aqInternalTransform = GetFacet(ev)->GetFrame(ev)->AcquireInternalTransform(ev, NULL);
	FW_CPoint anchorPoint = where.InverseTransformCopy(ev, aqInternalTransform);
	fAnchorPoint = this->BeginTracking(ev, anchorPoint);
}

//----------------------------------------------------------------------------------------
//	FW_CDropTracker::HandleContinueTracking
//----------------------------------------------------------------------------------------
//	where is in frame coordinate

void FW_CDropTracker::HandleContinueTracking(Environment *ev, const FW_CPoint& where)
{
	FW_CAcquiredODTransform aqInternalTransform = GetFacet(ev)->GetFrame(ev)->AcquireInternalTransform(ev, NULL);
	FW_CPoint currentPoint = where.InverseTransformCopy(ev, aqInternalTransform);
	fPreviousPoint = this->ContinueTracking(ev, fAnchorPoint, fPreviousPoint, currentPoint);
}

//----------------------------------------------------------------------------------------
//	FW_CDropTracker::HandleEndTracking
//----------------------------------------------------------------------------------------
//	where is in frame coordinate

FW_Boolean FW_CDropTracker::HandleEndTracking(Environment *ev, const FW_CPoint& where)
{
	FW_CAcquiredODTransform aqInternalTransform = GetFacet(ev)->GetFrame(ev)->AcquireInternalTransform(ev, NULL);
	FW_CPoint currentPoint = where.InverseTransformCopy(ev, aqInternalTransform);
	return this->EndTracking(ev, fAnchorPoint, currentPoint);
}

//----------------------------------------------------------------------------------------
//	FW_CDropTracker::BeginTracking
//----------------------------------------------------------------------------------------

FW_CPoint FW_CDropTracker::BeginTracking(Environment* ev, const FW_CPoint& anchorPoint)
{
	if (!IsInSourceFrame(ev))
	{
		FW_CAcquiredODShape aqHiliteShape = GetView(ev)->GetFrame(ev)->AcquireDragHiliteShape(ev, GetFacet(ev));
		this->ShowDragHilite(ev, aqHiliteShape, TRUE);
	}
	
	return anchorPoint;
}

//----------------------------------------------------------------------------------------
//	FW_CDropTracker::BeginTracking
//----------------------------------------------------------------------------------------

FW_CPoint FW_CDropTracker::ContinueTracking(Environment* ev,
											const FW_CPoint& anchorPoint, 
											const FW_CPoint& previousPoint, 
											const FW_CPoint& currentPoint)
{
	return currentPoint;
}

//----------------------------------------------------------------------------------------
//	FW_CDropTracker::EndTracking
//----------------------------------------------------------------------------------------

FW_Boolean FW_CDropTracker::EndTracking(Environment* ev,
										const FW_CPoint& anchorPoint, 
										const FW_CPoint& lastPoint)
{
	this->HideDragHilite(ev);	
	return anchorPoint != lastPoint;
}

//----------------------------------------------------------------------------------------
//	FW_CDropTracker::ShowDragHilite
//----------------------------------------------------------------------------------------
// hiliteShape is in frame coordinate

void FW_CDropTracker::ShowDragHilite(Environment* ev, ODShape* hiliteShape, FW_Boolean showInside)
{	
#ifdef FW_BUILD_MAC
	FW_ASSERT(fMacHiliteOn == FALSE);
	FW_ASSERT(hiliteShape);
	
	ODFacet* facet = GetFacet(ev);
	FW_CFrame* frame = GetView(ev)->GetFrame(ev);
	FW_ASSERT(facet->GetFrame(ev) == frame->GetODFrame(ev));	//just to be sure

	FW_CWindowContext gc(ev, facet);		// focus drawing to the window

	FW_CAcquiredODShape aqHiliteShape = hiliteShape->Copy(ev);
	FW_CAcquiredODTransform aqWindowFrameXForm = GetFacet(ev)->AcquireWindowFrameTransform(ev, NULL);
	aqHiliteShape->Transform(ev, aqWindowFrameXForm);

	::ShowDragHilite(fSession->GetDragAndDrop(ev)->GetDragReference(ev), 
					 aqHiliteShape->GetQDRegion(ev), 
					 showInside);
	fMacHiliteOn = TRUE;
#endif
}

//----------------------------------------------------------------------------------------
//	FW_CDropTracker::HideDragHilite
//----------------------------------------------------------------------------------------

void FW_CDropTracker::HideDragHilite(Environment* ev)
{
#ifdef FW_BUILD_MAC
	if (fMacHiliteOn)
	{
		FW_CWindowContext gc(ev, GetFacet(ev));		// focus drawing to the window
		::HideDragHilite(fSession->GetDragAndDrop(ev)->GetDragReference(ev));
		
		fMacHiliteOn = FALSE;
	}
#endif
}

//========================================================================================
//	class FW_CTracker
//========================================================================================

//----------------------------------------------------------------------------------------
//	FW_CTracker::FW_CTracker
//----------------------------------------------------------------------------------------

FW_CTracker::FW_CTracker(Environment *ev, FW_CView* view, ODFacet* facet, FW_Boolean waitMouseMoved) :
	FW_CBaseTracker(ev, view, facet),
	fWaitMouseMoved(waitMouseMoved)
{
}

//----------------------------------------------------------------------------------------
//	FW_CTracker::~FW_CTracker
//----------------------------------------------------------------------------------------

FW_CTracker::~FW_CTracker()
{
}

//----------------------------------------------------------------------------------------
// 	FW_CTracker::Track
//----------------------------------------------------------------------------------------

FW_Boolean FW_CTracker::Track(Environment *ev, const FW_CMouseEvent& theMouseEvent)
{
	FW_ASSERT(GetFacet(ev) != NULL);
	FW_ASSERT(GetView(ev) != NULL);

	// ----- Flush update event pending for this window
	FW_CAcquiredODWindow aqWindow = GetFacet(ev)->GetFrame(ev)->AcquireWindow(ev);
	aqWindow->Update(ev);
	
	if (fWaitMouseMoved && !theMouseEvent.WaitUntilMouseMoved(ev))
		return FALSE;
	
	FW_CPoint where = theMouseEvent.GetMousePosition(ev, FW_CMouseEvent::kFrame);
	FW_CFrame* frame = FW_CFrame::ODtoFWFrame(ev, GetFacet(ev)->GetFrame(ev));
	frame->GetContentView(ev)->FrameToViewContent(ev, where);

	fAnchorPoint = BeginTracking(ev, where);
	
	FW_CPoint currentLoc;
	fPreviousPoint = fAnchorPoint;
	
	do  
	{
		FW_CAcquiredODWindow aqODWindow = GetFacet(ev)->GetFrame(ev)->AcquireWindow(ev);
		
		{
			FW_CViewContext vc(ev,GetView(ev), GetFacet(ev));
			currentLoc = ::FW_GetMouseLocation(aqODWindow, vc);
		}
		
		fPreviousPoint = ContinueTracking(ev, fAnchorPoint, fPreviousPoint, currentLoc);
	} 
	while (FW_WaitMouseUp());
	
	return EndTracking(ev, fAnchorPoint, currentLoc);
}

