//========================================================================================
//
//	File:				FWEventH.cpp
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "FWOS.hpp"

#ifndef FWEVENT_H
#include "FWEvent.h"
#endif

#ifndef FWEVENTH_H
#include "FWEventH.h"
#endif

#ifndef FWMNUBAR_H
#include "FWMnuBar.h"
#endif

#ifndef FWORDCOL_H
#include "FWOrdCol.h"
#endif

#ifndef FWCURSOR_H
#include "FWCursor.h"
#endif

#ifndef SOM_ODFrame_xh
#include <Frame.xh>
#endif

#ifndef SOM_ODFacet_xh
#include <Facet.xh>
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

#pragma segment fwevents

//========================================================================================
//	RunTime Info
//========================================================================================

FW_DEFINE_CLASS_M0(FW_MEventHandler)

//========================================================================================
// class FW_MEventHandler
//========================================================================================

//----------------------------------------------------------------------------------------
// FW_MEventHandler::FW_MEventHandler
//----------------------------------------------------------------------------------------

FW_MEventHandler::FW_MEventHandler()
{
	fIdentifier = kNoIdentifier;
	fNextEventHandler = NULL;
	fActive = FALSE;
	fEnabled = TRUE;
	fPriority = kNoPriority;
	fEventHandlers = NULL;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::FW_MEventHandler
//----------------------------------------------------------------------------------------

FW_MEventHandler::FW_MEventHandler(Environment* ev, ODID theID, FW_MEventHandler* nextEventHandler, FW_Boolean enabled, FW_Priority thePriority)
{
	fIdentifier = theID;
	fNextEventHandler = nextEventHandler;
	fActive = FALSE;
	fEnabled = enabled;
	fPriority = thePriority;
	fEventHandlers = NULL;
}

//----------------------------------------------------------------------------------------
//	FW_MEventHandler::~FW_MEventHandler
//----------------------------------------------------------------------------------------

FW_MEventHandler::~FW_MEventHandler()
{
	if (fEventHandlers)
	{
		FW_MEventHandler* eventHandler;
		while ((eventHandler = (FW_MEventHandler*)fEventHandlers->First()) != NULL)
		{
			fEventHandlers->Remove(eventHandler);
			delete eventHandler;
		}		

		delete fEventHandlers;
	}
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::SetIdentifier
//----------------------------------------------------------------------------------------

void FW_MEventHandler::SetIdentifier(Environment* ev, ODID indentifier)
{
	fIdentifier = indentifier;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::IsActive()
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::IsActive(Environment* ev) const
{
	return fActive;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::Activate
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::Activate(Environment* ev)
{
	fActive = TRUE;
	return TRUE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::Deactivate
//----------------------------------------------------------------------------------------

void FW_MEventHandler::Deactivate(Environment* ev)
{
	fActive = FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::IsEnabled
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::IsEnabled(Environment* ev) const
{
	return fEnabled;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::Enable
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::Enable(Environment* ev)
{
	fEnabled = TRUE;
	return TRUE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::Disable
//----------------------------------------------------------------------------------------

void FW_MEventHandler::Disable(Environment* ev)
{
	fEnabled = FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::GetPriority
//----------------------------------------------------------------------------------------

FW_Priority FW_MEventHandler::GetPriority(Environment* ev) const
{
	return fPriority;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::SetPriority
//----------------------------------------------------------------------------------------

void FW_MEventHandler::SetPriority(Environment* ev, FW_Priority thePriority)
{
	fPriority = thePriority;
	// [B1 Conversion] Changing the priority should change its position ????
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::CreateEventHandlers
//----------------------------------------------------------------------------------------

void FW_MEventHandler::CreateEventHandlers(Environment* ev)
{
	FW_CPrivOrderedCollection* eventHandlers = new FW_CPrivOrderedCollection;
	fEventHandlers = eventHandlers;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::Insert
//----------------------------------------------------------------------------------------

void FW_MEventHandler::PrivInsert(Environment* ev, FW_MEventHandler* eventHandler)
{
	FW_MEventHandler* before = NULL;
	
	FW_COrderedCollectionIterator ite(fEventHandlers);
	for (FW_MEventHandler* handler = (FW_MEventHandler*)ite.First(); ite.IsNotComplete(); handler = (FW_MEventHandler*)ite.Next())
	{
		if (handler->GetPriority(ev) > eventHandler->GetPriority(ev))
		{
			before = handler;
			break;
		}
	}
	
	if (before)
		fEventHandlers->AddBefore(before, eventHandler);
	else
		fEventHandlers->AddLast(eventHandler);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::AdoptEventHandler
//----------------------------------------------------------------------------------------

void FW_MEventHandler::AdoptEventHandler(Environment* ev, FW_MEventHandler* eventHandler)
{
	if (!fEventHandlers)
		this->CreateEventHandlers(ev);

	PrivInsert(ev, eventHandler);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::RemoveEventHandler
//----------------------------------------------------------------------------------------

FW_MEventHandler* FW_MEventHandler::RemoveEventHandler(Environment* ev, ODID theID)
{
	FW_MEventHandler* theEventHandler = NULL;
	if (fEventHandlers)
	{
		FW_COrderedCollectionIterator ite(fEventHandlers);
		for (FW_MEventHandler* eventHandler = (FW_MEventHandler*)ite.First(); ite.IsNotComplete(); eventHandler = (FW_MEventHandler*)ite.Next())
		{
			if (eventHandler->GetIdentifier(ev) == theID)
			{
				fEventHandlers->Remove(eventHandler);
				theEventHandler = eventHandler;
				break;
			}
		}
	}
	return theEventHandler;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::RemoveEventHandler
//----------------------------------------------------------------------------------------

FW_MEventHandler* FW_MEventHandler::RemoveEventHandler(Environment* ev, FW_MEventHandler* eventHandler)
{
	FW_MEventHandler* theEventHandler = NULL;
	if (fEventHandlers)
	{
		fEventHandlers->Remove(eventHandler);
		theEventHandler = eventHandler;
	}
	return theEventHandler;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::PrivGetEventHandlerContaining
//----------------------------------------------------------------------------------------

FW_MEventHandler* FW_MEventHandler::PrivGetEventHandlerContaining(Environment* ev, ODFacet* facet, const FW_CPoint& whereInFrame)
{
	return this;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DispatchIdle
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DispatchIdle(Environment* ev, const FW_CNullEvent& theNullEvent)
{
	if (fEventHandlers)
	{
		FW_COrderedCollectionIterator ite(fEventHandlers);		
		for (FW_MEventHandler* eventHandler = (FW_MEventHandler*)ite.First(); ite.IsNotComplete(); eventHandler = (FW_MEventHandler*)ite.Next())
		{
			if (eventHandler->DispatchIdle(ev, theNullEvent))
				return TRUE;
		}
	}
	
	return IsEnabled(ev) ? HandleIdle(ev, theNullEvent) : FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::HandleIdle
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::HandleIdle(Environment* ev, const FW_CNullEvent& theNullEvent)
{	
	return DoIdle(ev, theNullEvent);	
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DoIdle
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DoIdle(Environment* ev, const FW_CNullEvent& theNullEvent)
{
	return FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DispatchMouseEnter
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DispatchMouseEnter(Environment* ev, ODFacet* odFacet, const FW_CPoint& theMousePoint)
{
	if (fEventHandlers)
	{
		FW_COrderedCollectionIterator ite(fEventHandlers);
		for (FW_MEventHandler* eventHandler = (FW_MEventHandler*)ite.First(); ite.IsNotComplete(); eventHandler = (FW_MEventHandler*)ite.Next())
		{
			if (eventHandler->DispatchMouseEnter(ev, odFacet, theMousePoint))
				return TRUE;
		}
	}
			
	return IsEnabled(ev) ? HandleMouseEnter(ev, odFacet, theMousePoint) : FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::HandleMouseEnter
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::HandleMouseEnter(Environment* ev, ODFacet* odFacet, const FW_CPoint& theMousePoint)
{
	return DoMouseEnter(ev, odFacet, theMousePoint);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DoMouseEnter
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DoMouseEnter(Environment* ev, ODFacet* odFacet, const FW_CPoint& theMousePoint)
{
	return FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DispatchMouseWithin
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DispatchMouseWithin(Environment* ev, ODFacet* odFacet, const FW_CPoint& theMousePoint)
{
	if (fEventHandlers)
	{
		FW_COrderedCollectionIterator ite(fEventHandlers);
		for (FW_MEventHandler* eventHandler = (FW_MEventHandler*)ite.First(); ite.IsNotComplete(); eventHandler = (FW_MEventHandler*)ite.Next())
		{
			if (eventHandler->DispatchMouseWithin(ev, odFacet, theMousePoint))
				return TRUE;
		}
	}
			
	return IsEnabled(ev) ? HandleMouseWithin(ev, odFacet, theMousePoint) : FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::HandleMouseWithin
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::HandleMouseWithin(Environment* ev, ODFacet* odFacet, const FW_CPoint& theMousePoint)
{
	return DoMouseWithin(ev, odFacet, theMousePoint);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DoMouseWithin
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DoMouseWithin(Environment* ev, ODFacet* odFacet, const FW_CPoint& theMousePoint)
{
	return FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DispatchMouseLeave
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DispatchMouseLeave(Environment* ev, ODFacet* odFacet)
{
	if (fEventHandlers)
	{
		FW_COrderedCollectionIterator ite(fEventHandlers);
		for (FW_MEventHandler* eventHandler = (FW_MEventHandler*)ite.First(); ite.IsNotComplete(); eventHandler = (FW_MEventHandler*)ite.Next())
		{
			if (eventHandler->DispatchMouseLeave(ev, odFacet))
				return TRUE;
		}
	}
	
	return IsEnabled(ev) ? HandleMouseLeave(ev, odFacet) : FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::HandleMouseLeave
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::HandleMouseLeave(Environment* ev, ODFacet* odFacet)
{
	return DoMouseLeave(ev, odFacet);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DoMouseLeave
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DoMouseLeave(Environment* ev, ODFacet* odFacet)
{
	return FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DispatchMouseDown
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DispatchMouseDown(Environment* ev, const FW_CMouseEvent& theMouseEvent)
{
	if (fEventHandlers)
	{
		FW_COrderedCollectionIterator ite(fEventHandlers);
		for (FW_MEventHandler* eventHandler = (FW_MEventHandler*)ite.First(); ite.IsNotComplete(); eventHandler = (FW_MEventHandler*)ite.Next())
		{
			if (eventHandler->DispatchMouseDown(ev, theMouseEvent))
				return TRUE;
		}
	}
	
	return HandleMouseDown(ev, theMouseEvent);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::HandleMouseDown
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::HandleMouseDown(Environment* ev, const FW_CMouseEvent& theMouseEvent)
{	
	return IsEnabled(ev) ? DoMouseDown(ev, theMouseEvent) : FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DoMouseDown
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DoMouseDown(Environment* ev, const FW_CMouseEvent& theMouseEvent)
{
	return FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DispatchBGMouseDown
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DispatchBGMouseDown(Environment* ev, const FW_CMouseEvent& theMouseEvent)
{
	if (fEventHandlers)
	{
		FW_COrderedCollectionIterator ite(fEventHandlers);
		for (FW_MEventHandler* eventHandler = (FW_MEventHandler*)ite.First(); ite.IsNotComplete(); eventHandler = (FW_MEventHandler*)ite.Next())
		{
			if (eventHandler->DispatchBGMouseDown(ev, theMouseEvent))
				return TRUE;
		}
	}
	
	return IsEnabled(ev) ? HandleBGMouseDown(ev, theMouseEvent) : FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::HandleBGMouseDown
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::HandleBGMouseDown(Environment* ev, const FW_CMouseEvent& theMouseEvent)
{
	return DoBGMouseDown(ev, theMouseEvent);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DoBGMouseDown
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DoBGMouseDown(Environment* ev, const FW_CMouseEvent& theMouseEvent)
{
	return FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DispatchMouseUp
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DispatchMouseUp(Environment* ev, const FW_CMouseEvent& theMouseEvent)
{
	if (fEventHandlers)
	{
		FW_COrderedCollectionIterator ite(fEventHandlers);
		for (FW_MEventHandler* eventHandler = (FW_MEventHandler*)ite.First(); ite.IsNotComplete(); eventHandler = (FW_MEventHandler*)ite.Next())
		{
			if (eventHandler->DispatchMouseUp(ev, theMouseEvent))
				return TRUE;
		}
	}
	
	return IsEnabled(ev) ? HandleMouseUp(ev, theMouseEvent) : FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::HandleMouseUp
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::HandleMouseUp(Environment* ev, const FW_CMouseEvent& theMouseEvent)
{
	return DoMouseUp(ev, theMouseEvent);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DoMouseUp
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DoMouseUp(Environment* ev, const FW_CMouseEvent& theMouseEvent)
{
	return FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DispatchMouseDownInEmbeddedFrame
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DispatchMouseDownInEmbeddedFrame(Environment* ev, 
															const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent)
{
	if (fEventHandlers)
	{
		FW_COrderedCollectionIterator ite(fEventHandlers);
		for (FW_MEventHandler* eventHandler = (FW_MEventHandler*)ite.First(); ite.IsNotComplete(); eventHandler = (FW_MEventHandler*)ite.Next())
		{
			if (eventHandler->DispatchMouseDownInEmbeddedFrame(ev, theEmbeddedMouseEvent))
				return TRUE;
		}
	}
			
	return IsEnabled(ev) ? HandleMouseDownInEmbeddedFrame(ev, theEmbeddedMouseEvent) : FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::HandleMouseDownInEmbeddedFrame
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::HandleMouseDownInEmbeddedFrame(Environment* ev, 
															const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent)
{
	return DoMouseDownInEmbeddedFrame(ev, theEmbeddedMouseEvent);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DoMouseDownInEmbeddedFrame
//----------------------------------------------------------------------------------------
//	By default just call DoMouseDown

FW_Boolean FW_MEventHandler::DoMouseDownInEmbeddedFrame(Environment* ev, const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent)
{
	return DoMouseDown(ev, theEmbeddedMouseEvent);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DispatchBGMouseDownInEmbeddedFrame
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DispatchBGMouseDownInEmbeddedFrame(Environment* ev, 
															const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent)
{
	if (fEventHandlers)
	{
		FW_COrderedCollectionIterator ite(fEventHandlers);
		for (FW_MEventHandler* eventHandler = (FW_MEventHandler*)ite.First(); ite.IsNotComplete(); eventHandler = (FW_MEventHandler*)ite.Next())
		{
			if (eventHandler->DispatchBGMouseDownInEmbeddedFrame(ev, theEmbeddedMouseEvent))
				return TRUE;
		}
	}
			
	return IsEnabled(ev) ? HandleBGMouseDownInEmbeddedFrame(ev, theEmbeddedMouseEvent) : FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::HandleBGMouseDownInEmbeddedFrame
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::HandleBGMouseDownInEmbeddedFrame(Environment* ev, 
															const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent)
{
	return DoBGMouseDownInEmbeddedFrame(ev, theEmbeddedMouseEvent);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DoBGMouseDownInEmbeddedFrame
//----------------------------------------------------------------------------------------
//	By default just call DoBGMouseDown

FW_Boolean FW_MEventHandler::DoBGMouseDownInEmbeddedFrame(Environment* ev, const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent)
{
	return DoBGMouseDown(ev, theEmbeddedMouseEvent);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DispatchMouseUpInEmbeddedFrame
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DispatchMouseUpInEmbeddedFrame(Environment* ev, const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent)
{
	if (fEventHandlers)
	{
		FW_COrderedCollectionIterator ite(fEventHandlers);
		for (FW_MEventHandler* eventHandler = (FW_MEventHandler*)ite.First(); ite.IsNotComplete(); eventHandler = (FW_MEventHandler*)ite.Next())
		{
			if (eventHandler->DispatchMouseUpInEmbeddedFrame(ev, theEmbeddedMouseEvent))
				return TRUE;
		}
	}
	
	return IsEnabled(ev) ? HandleMouseUpInEmbeddedFrame(ev, theEmbeddedMouseEvent) : FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::HandleMouseUpInEmbeddedFrame
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::HandleMouseUpInEmbeddedFrame(Environment* ev, const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent)
{
	return DoMouseUpInEmbeddedFrame(ev, theEmbeddedMouseEvent);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DoMouseUpInEmbeddedFrame
//----------------------------------------------------------------------------------------
// By Default just call DoMouseUp

FW_Boolean FW_MEventHandler::DoMouseUpInEmbeddedFrame(Environment* ev, const FW_CEmbeddedMouseEvent& theEmbeddedMouseEvent)
{
	return DoMouseUp(ev, theEmbeddedMouseEvent);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DispatchMouseDownInEmbeddedFrameBorder
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DispatchMouseDownInEmbeddedFrameBorder(Environment* ev, const FW_CBorderMouseEvent& theBorderMouseEvent)
{
	if (fEventHandlers)
	{
		FW_COrderedCollectionIterator ite(fEventHandlers);
		for (FW_MEventHandler* eventHandler = (FW_MEventHandler*)ite.First(); ite.IsNotComplete(); eventHandler = (FW_MEventHandler*)ite.Next())
		{
			if (eventHandler->DispatchMouseDownInEmbeddedFrameBorder(ev, theBorderMouseEvent))
				return TRUE;
		}
	}
	
	return IsEnabled(ev) ? HandleMouseDownInEmbeddedFrameBorder(ev, theBorderMouseEvent) : FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::HandleMouseDownInEmbeddedFrameBorder
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::HandleMouseDownInEmbeddedFrameBorder(Environment* ev, const FW_CBorderMouseEvent& theBorderMouseEvent)
{
	return DoMouseDownInEmbeddedFrameBorder(ev, theBorderMouseEvent);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DoMouseDownInEmbeddedFrameBorder
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DoMouseDownInEmbeddedFrameBorder(Environment* ev, const FW_CBorderMouseEvent& theBorderMouseEvent)
{
	return FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DisaptchVirtualKey
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DispatchVirtualKey(Environment* ev, const FW_CVirtualKeyEvent& theVirtualKeyEvent)
{
	if (fEventHandlers)
	{
		FW_COrderedCollectionIterator ite(fEventHandlers);
		for (FW_MEventHandler* eventHandler = (FW_MEventHandler*)ite.First(); ite.IsNotComplete(); eventHandler = (FW_MEventHandler*)ite.Next())
		{
			if (eventHandler->DispatchVirtualKey(ev, theVirtualKeyEvent))
				return TRUE;
		}
	}
	
	return IsEnabled(ev) ? HandleVirtualKey(ev, theVirtualKeyEvent) : FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::HandleVirtualKey
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::HandleVirtualKey(Environment* ev, const FW_CVirtualKeyEvent& theVirtualKeyEvent)
{
	return DoVirtualKey(ev, theVirtualKeyEvent);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DoVirtualKey
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DoVirtualKey(Environment* ev, const FW_CVirtualKeyEvent& theVirtualKeyEvent)
{
	return FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DispatchCharKey
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DispatchCharKey(Environment* ev, const FW_CCharKeyEvent& theCharKeyEvent)
{
	if (fEventHandlers)
	{
		FW_COrderedCollectionIterator ite(fEventHandlers);
		for (FW_MEventHandler* eventHandler = (FW_MEventHandler*)ite.First(); ite.IsNotComplete(); eventHandler = (FW_MEventHandler*)ite.Next())
		{
			if (eventHandler->DispatchCharKey(ev, theCharKeyEvent))
				return TRUE;
		}
	}
	
	return IsEnabled(ev) ? HandleCharKey(ev, theCharKeyEvent) : FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::HandleCharKey
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::HandleCharKey(Environment* ev, const FW_CCharKeyEvent& theCharKeyEvent)
{
	return DoCharKey(ev, theCharKeyEvent);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DoCharKey
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DoCharKey(Environment* ev, const FW_CCharKeyEvent& theCharKeyEvent)
{
	return FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DispatchSuspendResumeEvent
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DispatchSuspendResumeEvent(Environment* ev, const FW_CSuspendResumeEvent& theSuspendResumeEvent)
{
	if (fEventHandlers)
	{
		FW_COrderedCollectionIterator ite(fEventHandlers);
		for (FW_MEventHandler* eventHandler = (FW_MEventHandler*)ite.First(); ite.IsNotComplete(); eventHandler = (FW_MEventHandler*)ite.Next())
		{
			if (eventHandler->DispatchSuspendResumeEvent(ev, theSuspendResumeEvent))
				return TRUE;
		}
	}
	
	return IsEnabled(ev) ? HandleSuspendResumeEvent(ev, theSuspendResumeEvent) : FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::HandleSuspendResumeEvent
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::HandleSuspendResumeEvent(Environment* ev, const FW_CSuspendResumeEvent& theSuspendResumeEvent)
{
	return this->DoSuspendResumeEvent(ev, theSuspendResumeEvent);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DoSuspendResumeEvent
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DoSuspendResumeEvent(Environment* ev, const FW_CSuspendResumeEvent& theSuspendResumeEvent)
{
	return FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DispatchActivateEvent
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DispatchActivateEvent(Environment* ev, const FW_CActivateEvent& theActivateEvent)
{
	if (fEventHandlers)
	{
		FW_COrderedCollectionIterator ite(fEventHandlers);
		for (FW_MEventHandler* eventHandler = (FW_MEventHandler*)ite.First(); ite.IsNotComplete(); eventHandler = (FW_MEventHandler*)ite.Next())
		{
			if (eventHandler->DispatchActivateEvent(ev, theActivateEvent))
				return TRUE;
		}
	}
	
	return IsEnabled(ev) ? HandleActivateEvent(ev, theActivateEvent) : FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::HandleActivateEvent
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::HandleActivateEvent(Environment* ev, const FW_CActivateEvent& theActivateEvent)
{
	return this->DoActivateEvent(ev, theActivateEvent);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DoActivateEvent
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DoActivateEvent(Environment* ev, const FW_CActivateEvent& theActivateEvent)
{
	return FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DispatchMenuEvent
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DispatchMenuEvent(Environment* ev, const FW_CMenuEvent& theMenuEvent)
{
	if (fEventHandlers)
	{
		FW_COrderedCollectionIterator ite(fEventHandlers);
		for (FW_MEventHandler* eventHandler = (FW_MEventHandler*)ite.First(); ite.IsNotComplete(); eventHandler = (FW_MEventHandler*)ite.Next())
		{
			if (eventHandler->DispatchMenuEvent(ev, theMenuEvent))
				return TRUE;
		}
	}
	
	return IsEnabled(ev) ? HandleMenu(ev, theMenuEvent) : FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::HandleMenu
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::HandleMenu(Environment* ev, const FW_CMenuEvent& theMenuEvent)
{
	return this->DoMenu(ev, theMenuEvent);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DoMenu
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DoMenu(Environment* ev, const FW_CMenuEvent& theMenuEvent)
{
	return FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DispatchAdjustMenus
//----------------------------------------------------------------------------------------
//	Attention: behaviors for adjust menus are done AFTER the main handlers. This allows
//	someone to 'override' a handler

FW_Boolean FW_MEventHandler::DispatchAdjustMenus(Environment *ev, FW_CMenuBar* menuBar, 
												FW_Boolean hasMenuFocus, FW_Boolean isRoot)
{
	FW_Boolean handled = IsEnabled(ev) ? HandleAdjustMenus(ev, menuBar, hasMenuFocus, isRoot) : FALSE;

	if (!handled && fEventHandlers)
	{
		FW_COrderedCollectionIterator ite(fEventHandlers);
		for (FW_MEventHandler* eventHandler = (FW_MEventHandler*)ite.First(); ite.IsNotComplete(); eventHandler = (FW_MEventHandler*)ite.Next())
		{
			if ((handled = eventHandler->DispatchAdjustMenus(ev, menuBar, hasMenuFocus, isRoot)) == TRUE)
				break;
		}
	}
	
	return handled;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::HandleAdjustMenus
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::HandleAdjustMenus(Environment *ev, 
												FW_CMenuBar* menuBar, 
												FW_Boolean hasMenuFocus, 
												FW_Boolean isRoot)
{
	return this->DoAdjustMenus(ev, menuBar, hasMenuFocus, isRoot);;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::DoAdjustMenus
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DoAdjustMenus(Environment *ev, FW_CMenuBar* menuBar, 
											FW_Boolean hasMenuFocus, FW_Boolean isRoot)
{
FW_UNUSED(hasMenuFocus);
FW_UNUSED(isRoot);

	return FALSE;	// not done handling
}

#ifdef FW_BUILD_MAC
//----------------------------------------------------------------------------------------
// FW_MEventHandler::DispatchWindowEvent
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DispatchWindowEvent(Environment* ev, const FW_CMacWindowEvent& windowEvent)
{
	if (fEventHandlers)
	{
		FW_COrderedCollectionIterator ite(fEventHandlers);
		for (FW_MEventHandler* eventHandler = (FW_MEventHandler*)ite.First(); ite.IsNotComplete(); eventHandler = (FW_MEventHandler*)ite.Next())
		{
			if (eventHandler->DispatchWindowEvent(ev, windowEvent))
				return TRUE;
		}
	}
	
	return IsEnabled(ev) ? HandleWindowEvent(ev, windowEvent) : FALSE;
}
#endif

#ifdef FW_BUILD_MAC
//----------------------------------------------------------------------------------------
// FW_MEventHandler::HandleWindowEvent
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::HandleWindowEvent(Environment* ev, const FW_CMacWindowEvent& windowEvent)
{
	return this->DoWindowEvent(ev, windowEvent);
}
#endif

#ifdef FW_BUILD_MAC
//----------------------------------------------------------------------------------------
// FW_MEventHandler::DoWindowEvent
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::DoWindowEvent(Environment* ev, const FW_CMacWindowEvent& windowEvent)
{
	return FALSE;	// not done handling
}
#endif

//----------------------------------------------------------------------------------------
// FW_MEventHandler::GetTarget
//----------------------------------------------------------------------------------------

FW_MEventHandler* FW_MEventHandler::GetTarget(Environment* ev) const
{
	if (fNextEventHandler)
		return fNextEventHandler->GetTarget(ev);
	else
		return NULL;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::SetTarget
//----------------------------------------------------------------------------------------

void FW_MEventHandler::SetTarget(Environment* ev, FW_MEventHandler* theTarget)
{
	if (fNextEventHandler)
		fNextEventHandler->SetTarget(ev, theTarget);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::IsTarget: 
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::IsTarget(Environment* ev)
{
	return this == this->GetTarget(ev);
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::BecomeTarget: 
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::BecomeTarget(Environment* ev)
{
	FW_MEventHandler* currentTarget = this->GetTarget(ev);
	if (this != currentTarget)
	{
		if (currentTarget->ResignTarget(ev))
		{
			this->SetTarget(ev, this);
			return TRUE;
		}
		else
			return FALSE;
	}
	return TRUE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::ResignTarget: 
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::ResignTarget(Environment* ev)
{
	FW_MEventHandler* currentTarget = this->GetTarget(ev);
	long reason = currentTarget->WillingToResignTarget(ev);
			
	if (reason == 0)
	{
		currentTarget->TargetValidationSucceeded(ev);

		this->SetTarget(ev, fNextEventHandler);		// ??? What do we do in this case with no gApplication ???
		return TRUE;
	}
	else
	{
		currentTarget->TargetValidationFailed(ev, reason);
		return FALSE;
	}
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::WantsToBeTarget: 
//----------------------------------------------------------------------------------------

FW_Boolean FW_MEventHandler::WantsToBeTarget(Environment* ev)
{
	return FALSE;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::WillingToResignTarget: 
//----------------------------------------------------------------------------------------

long FW_MEventHandler::WillingToResignTarget(Environment* ev)
{
	return 0;
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::TargetValidationSucceeded: 
//----------------------------------------------------------------------------------------

void FW_MEventHandler::TargetValidationSucceeded(Environment* ev)
{
}

//----------------------------------------------------------------------------------------
// FW_MEventHandler::TargetValidationFailed: 
//----------------------------------------------------------------------------------------

void FW_MEventHandler::TargetValidationFailed(Environment* ev, long /* reason */)
{
}

//----------------------------------------------------------------------------------------
//	FW_MEventHandler::ODFrameToEventHandler
//----------------------------------------------------------------------------------------

FW_MEventHandler* FW_MEventHandler::ODFrameToEventHandler(Environment* ev, ODFrame* odFrame)
{
	FW_MEventHandler* eventHandler = NULL;
	if (odFrame)
	{
		FW_SEventHandlerPartInfo* partInfo = (FW_SEventHandlerPartInfo*)odFrame->GetPartInfo(ev);
		eventHandler = partInfo ? partInfo->fEventHandler : NULL;
	
	}
	return eventHandler;
}

