//========================================================================================
//
//	File:		FWFileSy.cpp
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "FWOS.hpp"

#ifndef FWFILESY_H
#include "FWFileSy.h"
#endif

#ifndef FWFILESP_H
#include "FWFileSp.h"
#endif

#ifndef FWPRIDEB_H
#include "FWPriDeb.h"
#endif

#ifndef FWEXCDEF_H
#include "FWExcDef.h"
#endif

#ifndef FWSTRS_H
#include "FWStrs.h"
#endif

#ifndef FWBNDSTR_H
#include "FWBndStr.h"
#endif

#ifndef FWFILPAR_H
#include "FWFilPar.h"
#endif

#if defined(FW_BUILD_MAC) && !defined(__SCRIPT__)
#include <Script.h>
#endif

#if defined(FW_BUILD_MAC) && !defined(__FILES__)
#include <Files.h>
#endif

#if defined(FW_BUILD_MAC) && !defined(__ERRORS__)
#include <Errors.h>
#endif

#if defined(FW_BUILD_WIN) && !defined(FWMEMMGR_H)
#include "FWMemMgr.h"
#endif

#if defined(FW_BUILD_WIN) && !defined(__DIRECT_H)
#include <direct.h>
#endif

#if defined(FW_BUILD_WIN) && !defined(__DOS_H)
#include <dos.h>
#endif

#if defined(FW_BUILD_WIN) && !defined(__IO_H)
#include <io.h>
#endif

#if defined(FW_BUILD_WIN) && !defined(__CTYPE_H)
#include <ctype.h>
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

#ifdef FW_BUILD_WIN16
extern "C" void FAR PASCAL DOS3Call();				// We don't do "int 21h" under Windows
#endif



//========================================================================================
// Constants
//========================================================================================
#ifdef FW_BUILD_MAC
#define FW_kMacVolumeIsSharedBitMask	0x00000200
#endif




//========================================================================================
//	Runtime type information
//========================================================================================

#ifdef FW_BUILD_MAC
#pragma segment FileSystem
#endif


//========================================================================================
//	CLASS FW_CAccessPermission
//========================================================================================

//----------------------------------------------------------------------------------------
//	FW_CAccessPermission::FW_CAccessPermission
//
//	Constructor sets the values for access.  Both parameters are optional.  The default
//    access is exclusive read/write.  The default deny mode is deny read/write.
//----------------------------------------------------------------------------------------

FW_CAccessPermission::FW_CAccessPermission(Access access,
										   Deny deny) :
	fAccess(access),
	fDeny(deny)
{
}


//----------------------------------------------------------------------------------------
//	FW_CAccessPermission::FW_CAccessPermission
//
//  Copy constructor
//----------------------------------------------------------------------------------------

FW_CAccessPermission::FW_CAccessPermission(const FW_CAccessPermission& permission) :
	fAccess(permission.fAccess),
	fDeny(permission.fDeny)
{
}


//----------------------------------------------------------------------------------------
//	FW_CAccessPermission::~FW_CAccessPermission
//----------------------------------------------------------------------------------------

FW_CAccessPermission::~FW_CAccessPermission()
{
}


//----------------------------------------------------------------------------------------
//	FW_CAccessPermission::GetPermission
//
//  Returns the access permissions in the parameters access and deny.
//----------------------------------------------------------------------------------------

void FW_CAccessPermission::GetPermission(Access& access,
										 Deny& deny) const
{
	access = fAccess;
	deny = fDeny;
}


//----------------------------------------------------------------------------------------
//	FW_CAccessPermission::operator=
//----------------------------------------------------------------------------------------

FW_CAccessPermission& FW_CAccessPermission::operator=(const FW_CAccessPermission& permission)
{
	fAccess = permission.fAccess;
	fDeny = permission.fDeny;
	return(*this);
}


//----------------------------------------------------------------------------------------
//	FW_CAccessPermission::operator==
//----------------------------------------------------------------------------------------

FW_Boolean FW_CAccessPermission::operator==(const FW_CAccessPermission& permission) const
{
	return ((fAccess == permission.fAccess) && (fDeny == permission.fDeny));
}


//----------------------------------------------------------------------------------------
//	FW_CAccessPermission::operator!=
//----------------------------------------------------------------------------------------

FW_Boolean FW_CAccessPermission::operator!=(const FW_CAccessPermission& permission) const
{
	return (!FW_CAccessPermission::operator==(permission));
}




//========================================================================================
//	CLASS FW_CFileSystem
//========================================================================================

//----------------------------------------------------------------------------------------
//	FW_CFileSystem::CreateFile
//
//  Create the file specified by fileSpec.
//  On the Macintosh, the file is created as a TeachText text file by default.  The user
//    can always change the file type using the FW_CAccessFileInfo helper class.  
//----------------------------------------------------------------------------------------

void FW_CFileSystem::CreateFile(const FW_CFileSpecification& fileSpec,
								FW_Boolean overWriteExisting)
{
	FW_CFileSystem::FileError theError = FW_CFileSystem::kNoError;

#ifdef FW_BUILD_WIN
	FW_FileAccessHandle fileHandle;
	FW_CDynamicString fileName;

	fileSpec.GetFullPath(fileName);
#endif

#ifdef FW_BUILD_WIN16
	OFSTRUCT Buffer;

	if (!overWriteExisting)
	{
		fileHandle = ::OpenFile((LPCSTR)fileName, &Buffer, OF_EXIST);

		if (fileHandle != FW_kInvalidAccessHandle)
			FW_Failure((FW_PlatformError)FW_CFileSystem::kFileExists);
	}
	
	fileHandle = ::OpenFile((LPCSTR)fileName, &Buffer, OF_CREATE);
	if (fileHandle == FW_kInvalidAccessHandle)
		theError = Buffer.nErrCode;
	else
		::_lclose(fileHandle);
	
#endif

#ifdef FW_BUILD_WIN32
	long createMode = (overWriteExisting ? TRUNCATE_EXISTING | CREATE_NEW : CREATE_NEW);
	
	fileHandle = ::CreateFile(fileName, 
							  GENERIC_READ | GENERIC_WRITE, 
							  0, 
							  NULL,
							  createMode, 
							  FILE_ATTRIBUTE_NORMAL, 
							  (HANDLE)NULL);
							   
	if (fileHandle == FW_kInvalidAccessHandle)
		theError = ::GetLastError();
#endif

#ifdef FW_BUILD_MAC
	short fileHandle;
	FSSpec theMacSpec;
	OSType aFileType;
	OSType aCreatorType;

	fileSpec.MacGetFSSpec(theMacSpec);
	fileSpec.MacGetTypeAndCreator(aFileType, aCreatorType);

	theError = ::FSpCreate(&theMacSpec, aCreatorType, aFileType, smSystemScript);

	if (overWriteExisting && (theError == FW_CFileSystem::kFileExists))
	{
		theError = ::FSpOpenDF(&theMacSpec, FW_CAccessPermission::kReadWrite, &fileHandle);
		if (theError == FW_CFileSystem::kNoError)
		{
			// Truncate the file.  If an error occurs record it for throwing and close the file.
			theError = ::SetEOF(fileHandle, 0);
			if (theError == FW_CFileSystem::kNoError)
				theError = ::FSClose(fileHandle);
			else
				::FSClose(fileHandle);
		}
	}
#endif

	FW_FailOnError(theError);
}


//----------------------------------------------------------------------------------------
//	FW_CFileSystem::DeleteFile
//
//  Delete the file specified by fileSpec.  If the file does not exist, throw a 
//    FW_XFileNotFound error containing a copy of the fileSpec.
//----------------------------------------------------------------------------------------

void FW_CFileSystem::DeleteFile(const FW_CFileSpecification& fileSpec)
{
	FW_CFileSystem::FileError theError = FW_CFileSystem::kNoError;

#ifdef FW_BUILD_WIN
	FW_CDynamicString fileName;

	fileSpec.GetFullPath(fileName);
#endif

#ifdef FW_BUILD_WIN16
	OFSTRUCT Buffer;
	if (::OpenFile((const FW_Char*)fileName, &Buffer, OF_DELETE) == FW_kInvalidAccessHandle)
		theError = Buffer.nErrCode;
#endif

#ifdef FW_BUILD_WIN32
	if(!::DeleteFile((const FW_Char*)fileName))
		theError = ::GetLastError();
#endif

#ifdef FW_BUILD_MAC
	FSSpec theMacSpec;

	fileSpec.MacGetFSSpec(theMacSpec);
	theError = ::FSpDelete(&theMacSpec);
#endif

	FW_FailOnError(theError);
}


#ifdef FW_BUILD_WIN16
//----------------------------------------------------------------------------------------
// PrivWinPrimitiveCreateDir
//----------------------------------------------------------------------------------------
FW_CFileSystem::FileError FW_CFileSystem::PrivWinPrimitiveCreateDir(const FW_Char* dirName)
{
	FW_CFileSystem::FileError theError = FW_CFileSystem::kNoError;
	
	__asm {
		push 	ds
		lds		dx, [dirName]
		mov     ah, 39h
	}
	
	DOS3Call();
		
	__asm {
		pop		ds
		jc		_error
		xor     ax, ax
	}
			
	_error:
	
	__asm {
		mov		theError, ax
	}
	
	return (theError);
}
#endif


//----------------------------------------------------------------------------------------
// FW_CFileSystem::CreateDirectory
//----------------------------------------------------------------------------------------

void FW_CFileSystem::CreateDirectory(const FW_CDirectorySpecification& directory)
{
	FW_CFileSystem::FileError theError = FW_CFileSystem::kNoError;
#ifdef FW_BUILD_WIN
	FW_CDynamicString directoryName;

	// Get full path name and remove trailing delimiter.
	//   This should always work since the directory can never be just the root path.
	directory.GetFullPath(directoryName);
	if (directoryName.GetLength() > 0)
		directoryName.Truncate(directoryName.GetLength()-1);
#endif

#ifdef FW_BUILD_WIN16
	const FW_Char* namePtr = (const FW_Char*)directoryName;
	theError = PrivWinPrimitiveCreateDir(namePtr);
#endif

#ifdef FW_BUILD_WIN32
	if(!::CreateDirectory((const FW_Char*)directoryName, NULL))
		theError = ::GetLastError();
#endif

#ifdef FW_BUILD_MAC
	long directoryID;
	FSSpec macSpec;
	
	directory.MacGetFSSpec(macSpec);
	
	theError = ::FSpDirCreate(&macSpec, smSystemScript, &directoryID);
#endif
	
	FW_FailOnError(theError);
}


#ifdef FW_BUILD_WIN16
//----------------------------------------------------------------------------------------
// PrivWinPrimitiveDeleteDir
//----------------------------------------------------------------------------------------
FW_CFileSystem::FileError FW_CFileSystem::PrivWinPrimitiveDeleteDir(const FW_Char* dirName)
{
	FW_CFileSystem::FileError theError = FW_CFileSystem::kNoError;
	
	__asm {
		push 	ds
		lds		dx, [dirName]
		mov     ah, 3Ah
	}
	
	DOS3Call();
		
	__asm {
		pop		ds
		jc		_error
		xor     ax, ax
	}
			
	_error:
	
	__asm {
		mov		theError, ax
	}
	
	return(theError);
}
#endif


//----------------------------------------------------------------------------------------
// FW_CFileSystem::DeleteDirectory
//----------------------------------------------------------------------------------------

void FW_CFileSystem::DeleteDirectory(const FW_CDirectorySpecification& directory)
{
	FW_CDirectorySpecification defaultDirectory;
	
	// if the directory we want to delete is the current directory, then we shouldn't
	//   delete it.
	if (defaultDirectory == directory)
		FW_Failure(FW_xCantDeleteWorkingDirectory);
		

	FW_CFileSystem::FileError theError = FW_CFileSystem::kNoError;
#ifdef FW_BUILD_WIN
	FW_CDynamicString directoryName;

	// Get full path name and remove trailing delimiter.
	//   This should always work since the directory can never be just the root path.
	directory.GetFullPath(directoryName);
	if (directoryName.GetLength() > 0)
		directoryName.Truncate(directoryName.GetLength()-1);
	
#endif

#ifdef FW_BUILD_WIN16
	const FW_Char* namePtr = (const FW_Char*)directoryName;
	theError = PrivWinPrimitiveDeleteDir(namePtr);			
#endif

#ifdef FW_BUILD_WIN32
	if(!::RemoveDirectory((const FW_Char*)directoryName))
		theError = ::GetLastError();
#endif

#ifdef FW_BUILD_MAC
	FSSpec macSpec;
	directory.MacGetFSSpec(macSpec);
	
	theError = ::FSpDelete(&macSpec);
#endif

	FW_FailOnError(theError);
}


//----------------------------------------------------------------------------------------
// FW_CFileSystem::GetCurrentDirectory
//
//  Returns the current working directory in directory.  
//----------------------------------------------------------------------------------------

void FW_CFileSystem::GetCurrentDirectory(FW_CDirectorySpecification& directory)
{
	FW_LOG_OR_TRIGGER_EXCEPTION(FW_XPathNotFound, FW_XPathNotFound(directory));
	FW_LOG_OR_TRIGGER_EXCEPTION(FW_XVolumeNotFound, FW_XVolumeNotFound(directory));

	FW_CFileSystem::FileError theError = FW_CFileSystem::kNoError;

#ifdef FW_BUILD_WIN
	FW_CDynamicString directoryName;
	theError = FW_CPrivFileSystemParser::PrivGetWorkingDirectory(directoryName);
#endif


#ifdef FW_BUILD_MAC
	FSSpec directoryName;
	
	// Workaround for MPW Shell 3.3 bug when passing 0, 0, NULL to FSMakeFSSpec.
	Str32 nullName = "\p";
	
	theError = ::FSMakeFSSpec(0, 0, nullName, &directoryName);
#endif

	// Have to create the directory from scratch, since we can't change all of the
	//   directory information once it's created.
	FW_CDirectorySpecification tempDirectory(directoryName);
	
	FW_FailOnError(theError);

	directory = tempDirectory;
}


#ifdef FW_BUILD_WIN16
//----------------------------------------------------------------------------------------
// PrivWinPrimitiveSetCurrentDir
//
//  driveNumber should be a number from 0 to 25 where drive A=0, B=1, C=2, etc.
//----------------------------------------------------------------------------------------
FW_CFileSystem::FileError FW_CFileSystem::PrivWinPrimitiveSetCurrentDir(const FW_Char* dirName, short driveNumber)
{
	FW_CFileSystem::FileError theError = FW_CFileSystem::kNoError;

	__asm {
		mov		dx, driveNumber
		mov 	ah, 0Eh
	}
	
	DOS3Call();
		
	// Now, set the directory.
	__asm {
		push 	ds
		lds		dx, [dirName]
		mov     ah, 3Bh
	}
	
	DOS3Call();
		
	__asm {
		pop		ds
		jc		_error
		xor     ax, ax
	}
			
	_error:
	
	__asm {
		mov		theError, ax
	}
	
	return (theError);
}
#endif


//----------------------------------------------------------------------------------------
// FW_CFileSystem::SetCurrentDirectory
//
// Sets the default working directory for the application.  This also sets the directory
//   that appears in the standard file package when an SFP dialog is displayed.
//----------------------------------------------------------------------------------------

void FW_CFileSystem::SetCurrentDirectory(const FW_CDirectorySpecification& directory)
{
	FW_CFileSystem::FileError theError = FW_CFileSystem::kNoError;
	
	if (!FW_CFileSystem::IsValidDirectory(directory))
		FW_Failure(FW_xInvalidDirectory);
		
#ifdef FW_BUILD_WIN

	FW_CDynamicString directoryName;
	
	// Get the pathname without the trailing delimiter character.
	directory.GetFullPath(directoryName);
	if (directoryName.GetLength() > 0)
		directoryName.Truncate(directoryName.GetLength()-1);
#endif

#ifdef FW_BUILD_WIN16
	const FW_Char* namePtr = (const FW_Char*)directoryName;
	
	// Set drive first
	short driveNumber = directory.WinGetDrive() - (FW_Char)('a');
	theError = PrivWinPrimitiveSetCurrentDir(namePtr, driveNumber);
#endif

#ifdef FW_BUILD_WIN32
	if(!::SetCurrentDirectory((const FW_Char*)directoryName))
		theError = ::GetLastError();
#endif

#ifdef FW_BUILD_MAC
	FSSpec macSpec;
	directory.MacGetFSSpec(macSpec);
	
	theError = 	::HSetVol(macSpec.name, macSpec.vRefNum, macSpec.parID);

#endif

	FW_FailOnError(theError);
}


//----------------------------------------------------------------------------------------
// FW_CFileSystem::IsValidDirectory
//
// Given a fully qualified or partial pathname specified in directoryName, IsValidDirectory
//   will return TRUE if the directory exists.  If the directory or drive doesn't exist, 
//   IsValidDirectory will return FALSE.
//----------------------------------------------------------------------------------------

FW_Boolean FW_CFileSystem::IsValidDirectory(const FW_CDirectorySpecification& directory)
{
	FW_Boolean result = FALSE;
	
#ifdef FW_BUILD_WIN
	FW_TRY
	{
		FW_CDynamicString pathName;
		
		directory.GetFullPath(pathName);
		if (pathName.GetLength() > 0)
			pathName.Truncate(pathName.GetLength()-1);
			
		result = WinPathExists(pathName);
	}
	FW_CATCH_BEGIN
	FW_CATCH_NO_INSTANCE(FW_XException)
	{
		result = FALSE;
	}
	FW_CATCH_EVERYTHING()
	{
		FW_THROW_SAME();
	}
	FW_CATCH_END
#endif


#ifdef FW_BUILD_MAC
	FSSpec theSpec;
	OSErr theError = kNoError;
	
	directory.MacGetFSSpec(theSpec);
	
	// Try and make an FSSpec out of the directory.  If any error is returned, the
	//   directory cannot exist.
	theError = ::FSMakeFSSpec(theSpec.vRefNum,
							  theSpec.parID, 
							  (StringPtr)&(theSpec.name), 
							  &theSpec);
	if (theError == kNoError)
		result = TRUE;

#endif

	return (result);
}


//----------------------------------------------------------------------------------------
// FW_CFileSystem::IsValidDrive
//
// Given a drive name specified in driveName, IsValidDrive will return TRUE if the drive
//   letter is used.  For removeable media drives, there may not be anything in the drive,
//   but the drive could still be valid.
//----------------------------------------------------------------------------------------

FW_Boolean FW_CFileSystem::IsValidDrive(const FW_CString& driveName)
{
	FW_Boolean result = FALSE;

#ifdef FW_BUILD_WIN
	FW_TRY
	{
		if (driveName.GetLength() != 0)
		{
			int driveType = 0;
			FW_CString32 strTemp(driveName);
			
			if (FW_CPrivFileSystemParser::WinGetDrivePath(driveName, strTemp))
			{
				strTemp.ToLower();
				if ((strTemp.GetLength() > 1) && (strTemp[(FW_CharacterPosition)1] == FW_kDriveDelimiter))
				{

#ifdef FW_BUILD_WIN16
					FW_Char dwDriveId = strTemp[0];
					driveType = ::GetDriveType((int)dwDriveId - (int)('a'));
#endif


#ifdef FW_BUILD_WIN32
					strTemp += FW_kPathDelimiter;
					driveType = ::GetDriveType((const FW_Char*)strTemp);
#endif

					// A result of 0 or 1 indicates the drive didn't exist.  Otherwise
					//   the return result is the type of the drive.  
					if (driveType > 1)
						result = TRUE;
				}
			}
		}
	}
	FW_CATCH_BEGIN
	FW_CATCH_NO_INSTANCE(FW_XException)
	{
		result = FALSE;
	}
	FW_CATCH_EVERYTHING()
	{
		FW_THROW_SAME();
	}
	FW_CATCH_END
	
#endif


#ifdef FW_BUILD_MAC
	short defaultVolumeRefNum = 0;

	FW_CString32 newDriveName = driveName;
	Str32 macDriveString;
	
	FW_CharacterPosition pathSeparator = 0;
	OSErr theError = kNoError;

	// The last character in the string should be a Mac path separator.
	if (newDriveName.FindCharacter(FW_kPathDelimiter, pathSeparator))
		newDriveName.Truncate(pathSeparator + 1);
	else
		newDriveName += FW_kPathDelimiter;
		
	newDriveName.ExportPascal((FW_PascalChar*)&macDriveString);

	// Save the default volume and then attempt to set the default volume to
	//   driveName.  Any errors indicates that the drive doesn't exist.
	theError = ::GetVol(NULL, &defaultVolumeRefNum);
	if (theError == kNoError)
	{
		// Use a non-negative, non-zero number for an invalid drive.
		theError = ::SetVol(macDriveString, 1);

		// Restore the default volume.
		::SetVol(NULL, defaultVolumeRefNum);

		if (theError == kNoError)
			result = TRUE;
	}
#endif

	return (result);
}


//----------------------------------------------------------------------------------------
//	FW_CFileSink::IsValidFile
//
//  Return TRUE if the file exists.
//----------------------------------------------------------------------------------------

FW_Boolean FW_CFileSystem::IsValidFile(const FW_CFileSpecification& theFile)
{
	FW_Boolean result = FALSE;

#ifdef FW_BUILD_WIN
	FW_CDynamicString fileName;
	FW_TRY
	{
		theFile.GetFullPath(fileName);

#ifdef FW_BUILD_WIN16
		FW_FileAccessHandle fileHandle;
		OFSTRUCT Buffer;

		fileHandle = ::OpenFile((LPCSTR)fileName, &Buffer, OF_EXIST);
		if (fileHandle != FW_kInvalidAccessHandle)
			result = TRUE;
#endif

#ifdef FW_BUILD_WIN32
		long fileAttributes = 0;
		
		fileAttributes = ::GetFileAttributes((const FW_Char*)fileName);
		if (fileAttributes != (-1))
			if (fileAttributes & FILE_ATTRIBUTE_DIRECTORY)
				result = TRUE;
#endif

	}
	FW_CATCH_BEGIN
	FW_CATCH_EVERYTHING()
	{
		result = FALSE;
	}
	FW_CATCH_END
#endif

#ifdef FW_BUILD_MAC
	FSSpec macSpec;
	OSErr theError;
	
	theFile.MacGetFSSpec(macSpec);
	theError = ::FSMakeFSSpec(macSpec.vRefNum, macSpec.parID, macSpec.name, &macSpec);
	if (theError == noErr)
		result = TRUE;
#endif

	return (result);
}


#ifdef FW_BUILD_WIN
//----------------------------------------------------------------------------------------
//	FW_CFileSink::WinPathExists
//
//  Return TRUE if the directory exists.
//----------------------------------------------------------------------------------------

FW_Boolean FW_CFileSystem::WinPathExists(const FW_CString& pathName)
{
	FW_Boolean result = FALSE;

#ifdef FW_BUILD_WIN16
	FW_CDynamicString saveDirectory;
	
	FW_CPrivFileSystemParser::PrivGetWorkingDirectory(saveDirectory);
	const FW_Char* namePtr = (const FW_Char*)pathName;
	short validDir = ::chdir((FW_Char*)namePtr);
	if (validDir == 0)
		result = TRUE;
	
	// Restore original directory.
	namePtr = (const FW_Char*)saveDirectory;
	::chdir((FW_Char*)namePtr);
#endif

#ifdef FW_BUILD_WIN32
	long fileAttributes = 0;
	
	fileAttributes = ::GetFileAttributes((const FW_Char*)pathName);
	if (fileAttributes != (-1))
		if (fileAttributes & FILE_ATTRIBUTE_DIRECTORY)
			result = TRUE;
#endif

	return (result);
}
#endif


#ifdef FW_BUILD_MAC
//----------------------------------------------------------------------------------------
//	FW_CFileSink::MacIsVolumeShared
//
//  Return TRUE if the volume specified by volumeRefNum is currently a shared volume.
//----------------------------------------------------------------------------------------

FW_Boolean FW_CFileSystem::MacIsVolumeShared(short volumeRefNum)
{
	FW_Boolean result = FALSE;
	HParamBlockRec paramBlock;
	GetVolParmsInfoBuffer volInfoBuffer;
	FW_CFileSystem::FileError theError = FW_CFileSystem::kNoError;
	
	paramBlock.ioParam.ioNamePtr = NULL;
	paramBlock.ioParam.ioVRefNum = volumeRefNum;
	paramBlock.ioParam.ioBuffer = (Ptr)&volInfoBuffer;
	paramBlock.ioParam.ioReqCount = sizeof(volInfoBuffer);
	
	theError = PBHGetVolParmsSync(&paramBlock);
	if (theError == FW_CFileSystem::kNoError) 
	{
		if (volInfoBuffer.vMAttrib & FW_kMacVolumeIsSharedBitMask)
			result = TRUE;
	}
		
	return (result);
}
#endif
