//========================================================================================
//
//	File:				FWBTxtSh.h
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1993, 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================
//
//	Class Definitions for
//			- FW_CBaseTextShape
//

#ifndef FWBTXTSH_H
#define FWBTXTSH_H

#ifndef FWSHAPE_H
#include "FWShape.h"
#endif

#ifndef FWGRGLOB_H
#include "FWGrGlob.h"
#endif

#include "FWStrs.h"

// ----- Macintosh Includes -----

#if defined(FW_BUILD_MAC) && !defined(__TEXTEDIT__)
#include <TextEdit.h>
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

//========================================================================================
//	Forward Declarations
//========================================================================================

class FW_CLASS_ATTR FW_CGraphicContext;
class FW_CLASS_ATTR FW_CTextReader;
class FW_CLASS_ATTR FW_CPrivTextBuffer;

//========================================================================================
//	class FW_CBaseTextShape
//========================================================================================

class FW_CLASS_ATTR FW_CBaseTextShape : public FW_CShape
{
public:
	FW_DECLARE_CLASS

//----------------------------------------------------------------------------------------
//	Constructors/Destructors
//
public:
	FW_CBaseTextShape();
	FW_CBaseTextShape(const FW_CString& string,
					  const FW_PInk& ink,
					  const FW_PFont& font);

	FW_CBaseTextShape(FW_CTextReader* textReader,
					  const FW_PInk& ink,
					  const FW_PFont& font);
	
	FW_CBaseTextShape(FW_CReadableStream& archive);
	FW_CBaseTextShape(const FW_CBaseTextShape& other);

	virtual ~ FW_CBaseTextShape();
	
//----------------------------------------------------------------------------------------
//	Operators
//
public:
	FW_CBaseTextShape& operator=(const FW_CBaseTextShape& other);
	
//----------------------------------------------------------------------------------------
//	Inherited API
//
public:
	// ----- Hit Testing -----
	virtual FW_Boolean HitTest(FW_CGraphicContext& gc,
								const FW_CPoint& test,
								FW_CFixed tolerance) const;

	// ----- Flatten -----
	virtual void		Flatten(FW_CWritableStream& archive) const;

//----------------------------------------------------------------------------------------
//	New API
//
public:
	// ----- Geometry -----
	void				GetString(FW_CString& string) const
							{string = fString;}
	void				SetString(const FW_CString& string);

	void				GetText(FW_CTextWriter* textWriter) const;
	void				SetText(FW_CTextReader* textReader);
	
//----------------------------------------------------------------------------------------
//	Data Members
//
protected:
	FW_CDynamicString		fString;
};

inline void FW_CBaseTextShape::SetString(const FW_CString& string)
{
	// MetroWerks bug workaroung [AMB]
	FW_CString& thisString = fString;
	thisString = string;
}

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export off
#endif

#endif
