//========================================================================================
//
//	File:				FWColor.h
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1993, 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWCOLOR_H
#define FWCOLOR_H

// ----- Foundation Includes -----

#ifndef FWSTDDEF_H
#include <FWStdDef.h>
#endif

// ----- Macintosh Includes -----

#if defined(FW_BUILD_MAC) && !defined(__QUICKDRAW__)
#include <QuickDraw.h>
#endif

// ----- Windows Includes -----

#if defined(FW_BUILD_WIN) && !defined(_INC_WINDOWS)
#include <windows.h>
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

//========================================================================================
//	Forward Class Declarations
//========================================================================================

class FW_CLASS_ATTR FW_CColor;
class FW_CLASS_ATTR FW_CWritableStream;
class FW_CLASS_ATTR FW_CReadableStream;

//========================================================================================
//	Globale operators << and >>
//========================================================================================

FW_FUNC_ATTR const FW_CWritableStream& operator<<(const FW_CWritableStream& stream, const FW_CColor& color);
FW_FUNC_ATTR const FW_CReadableStream& operator>>(const FW_CReadableStream& stream, FW_CColor& color);

//========================================================================================
//	struct FW_CColor
//========================================================================================

#ifdef FW_BUILD_WIN
#define FW_CColorBase
#endif

#ifdef FW_BUILD_MAC
#define FW_CColorBase : public RGBColor
#endif

class FW_CLASS_ATTR FW_CColor FW_CColorBase
{
//----------------------------------------------------------------------------------------
//	Constructors/Destructors
//
public:
	FW_CColor();
	FW_CColor(const FW_CColor& color);
	FW_CColor(unsigned short r, unsigned short g, unsigned short b);
	FW_CColor(const FW_CReadableStream& stream);

#ifdef FW_BUILD_WIN
	FW_CColor(COLORREF rgb);
#endif

#ifdef FW_BUILD_MAC
	FW_CColor(const RGBColor* rgb);
	FW_CColor(const RGBColor& rgb);
#endif

//----------------------------------------------------------------------------------------
//	Operators
//
public:
	// ----- Comparison operators -----
	FW_Boolean operator==(const FW_CColor& clr) const;
#ifdef FW_BUILD_WIN	
	FW_Boolean operator==(COLORREF rgb) const;
#endif
#ifdef FW_BUILD_MAC	
	FW_Boolean operator==(const RGBColor& rgb) const;
#endif
	
	FW_Boolean operator!=(const FW_CColor& clr) const;
#ifdef FW_BUILD_WIN	
	FW_Boolean operator!=(COLORREF rgb) const;
#endif
#ifdef FW_BUILD_MAC	
	FW_Boolean operator!=(const RGBColor& rgb) const;
#endif

	// ----- Assignment operator -----	
	FW_CColor& operator=(const FW_CColor& color);

#ifdef FW_BUILD_MAC
	FW_CColor& operator=(const RGBColor& rgb);
#endif
	
#ifdef FW_BUILD_WIN
	FW_CColor& operator=(COLORREF rgb);
	operator COLORREF() const;
	
	FW_CColor& operator=(RGBQUAD quad);
	operator RGBQUAD() const;

	FW_CColor& operator=(RGBTRIPLE triple);
	operator RGBTRIPLE() const;
#endif

	// ----- Arithmetic, by another color -----
	FW_CColor operator+(const FW_CColor& color) const;
	FW_CColor operator-(const FW_CColor& color) const;

	FW_CColor& operator+=(const FW_CColor& color);
	FW_CColor& operator-=(const FW_CColor& color);

	// ----- Arithmetic by a scalar -----

	FW_CColor operator+(unsigned short a) const;
	FW_CColor operator-(unsigned short a) const;
	FW_CColor operator*(unsigned short a) const;
	FW_CColor operator/(unsigned short a) const;

	FW_CColor& operator+=(unsigned short a);
	FW_CColor& operator-=(unsigned short a);
	FW_CColor& operator*=(unsigned short a);
	FW_CColor& operator/=(unsigned short a);

//----------------------------------------------------------------------------------------
//	New public API
//
public:
	// ----- Set/Get RGB Component -----
	void		SetRGB(unsigned short r, unsigned short g, unsigned short b);
	void		GetRGB(unsigned short& r, unsigned short& g, unsigned short& b) const;
	
	void 		MakeGrayer();

	FW_Boolean 	IsDarkerThan(const FW_CColor& color) const;
	FW_Boolean 	IsLighterThan(const FW_CColor& color) const;

	// ----- Persistence -----
	void		Read(const FW_CReadableStream& stream);
	void		Write(const FW_CWritableStream& stream) const;
	
//----------------------------------------------------------------------------------------
//	Private API
//
private:
	unsigned long 	Brightness() const;

	static unsigned short	AddComponents(unsigned short a, unsigned short b);
	static unsigned short	SubtractComponents(unsigned short a, unsigned short b);
	static unsigned short	MultiplyComponents(unsigned short a, unsigned short b);
	static unsigned short	DivideComponents(unsigned short a, unsigned short b);

#ifdef FW_BUILD_WIN
//----------------------------------------------------------------------------------------
//	Data Members
//
private:
	COLORREF	fRGBColor;
#endif
};

//========================================================================================
//	inlines
//========================================================================================

//----------------------------------------------------------------------------------------
//	FW_CColor::operator==
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CColor::operator==(const FW_CColor& color) const
{
#ifdef FW_BUILD_MAC
	return (red == color.red) && 
			(green == color.green) &&
			(blue == color.blue);
#endif
#ifdef FW_BUILD_WIN
	return fRGBColor == color.fRGBColor;
#endif
}

#ifdef FW_BUILD_WIN
//----------------------------------------------------------------------------------------
//	FW_CColor::operator==
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CColor::operator==(COLORREF rgb) const
{
	return fRGBColor == rgb;
}
#endif

#ifdef FW_BUILD_MAC
//----------------------------------------------------------------------------------------
//	FW_CColor::operator==
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CColor::operator==(const RGBColor& rgb) const
{
	return (red == rgb.red) && 
			(green == rgb.green) &&
			(blue == rgb.blue);
}
#endif

//----------------------------------------------------------------------------------------
//	FW_CColor::operator!=
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CColor::operator!=(const FW_CColor& color) const
{
	return !(*this == color);
}

#ifdef FW_BUILD_WIN
//----------------------------------------------------------------------------------------
//	FW_CColor::operator==
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CColor::operator!=(COLORREF rgb) const
{
	return !(*this == rgb);
}
#endif

#ifdef FW_BUILD_MAC
//----------------------------------------------------------------------------------------
//	FW_CColor::operator==
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CColor::operator!=(const RGBColor& rgb) const
{
	return !(*this == rgb);
}
#endif

#ifdef FW_BUILD_WIN
//----------------------------------------------------------------------------------------
//	FW_CColor::operator COLORREF
//----------------------------------------------------------------------------------------
inline FW_CColor::operator COLORREF() const
{
	return fRGBColor;
}
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export off
#endif

#endif
