//========================================================================================
//
//	File:				FWPicShp.h
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWPICSHP_H
#define FWPICSHP_H

#ifndef FWSHAPE_H
#include "FWShape.h"
#endif

#ifndef FWBNDSHP_H
#include "FWBndShp.h"
#endif

#ifndef FWPICTUR_H
#include "FWPictur.h"
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

//========================================================================================
//	Forward Declarations
//========================================================================================

class FW_CLASS_ATTR FW_CGraphicContext;
class FW_CLASS_ATTR FW_CBitmapShape;

//========================================================================================
//	class FW_CPictureShape
//========================================================================================

class FW_CLASS_ATTR FW_CPictureShape : public FW_CBoundedShape
{
public:
	FW_DECLARE_CLASS

//----------------------------------------------------------------------------------------
//	Constructors/Destructors
//
public:
	FW_CPictureShape(FW_PPicture picture, const FW_CRect& dstRect);

	FW_CPictureShape(const FW_CPictureShape& other);
	FW_CPictureShape(FW_CReadableStream& archive);

	virtual ~ FW_CPictureShape();
	
//----------------------------------------------------------------------------------------
//	Operators
//
public:
	FW_CPictureShape& operator=(const FW_CPictureShape& other);
	
//---------------------------------------------------------------------------------------
//	Inherited API
//
public:
	// ----- Rendering -----
	virtual void 				Render(FW_CGraphicContext& gc) const;
	
	// ----- Flatten -----
	virtual void				Flatten(FW_CWritableStream& archive) const;

//---------------------------------------------------------------------------------------
//	New API
//
public:
	// ----- Rendering -----
	static void				RenderPicture(FW_CGraphicContext& gc,
											FW_PPicture picture,
											const FW_CRect& dstRect);
	
	// ----- Copying -----
	virtual FW_CShape*			Copy() const;
	
	// ----- Archiving -----
	static void* 				Read(FW_CReadableStream& archive);

	// ----- Geometry -----
	void						SetGeometry(const FW_PPicture& picture, const FW_CRect& dstRect);
	void 						GetGeometry(FW_PPicture& picture, FW_CRect& dstRect) const;

	FW_PPicture					GetPicture() const
									{return fPicture;}
	void						SetPicture(const FW_PPicture& picture)
									{fPicture = picture;}
	
//---------------------------------------------------------------------------------------
//	Data Member
//	
private:
	FW_PPicture 	fPicture;
};
			
#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export off
#endif
	
#endif
