#ifndef FWSHLISH_H
#define FWSHLISH_H

//========================================================================================
//
//	File:				FWShpLst.h
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

// ----- Foundation Includes -----

#ifndef FWAUTODE_H
#include <FWAutoDe.h>
#endif

// ----- OS Includes

#ifndef FWSHAPE_H
#include "FWShape.h"
#endif

#ifndef FWSHPLST_H
#include "FWShpLst.h"
#endif

// ----- OpenDoc Includes -----

#ifdef _ODTYPES_
#include <ODTypes.h>
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

//========================================================================================
//	Forward Declarations
//========================================================================================

class FW_CLASS_ATTR FW_CGraphicContext;
class FW_CLASS_ATTR FW_CPoint;
class FW_CLASS_ATTR FW_CRect;
class FW_CLASS_ATTR FW_CShape;

//========================================================================================
//	class FW_CShapeListShape
//========================================================================================

class FW_CLASS_ATTR FW_CShapeListShape : public FW_CShape
{

public:
	FW_DECLARE_CLASS

//----------------------------------------------------------------------------------------
//	Constructors/Destructors
//
public:
								FW_CShapeListShape(const FW_PShapeList& list);
								FW_CShapeListShape(const FW_CShapeListShape& other);
								FW_CShapeListShape(FW_CReadableStream& archive);

	virtual					~FW_CShapeListShape();

//----------------------------------------------------------------------------------------
//	Operators
//
public:
	FW_CShapeListShape&			operator=(const FW_CShapeListShape& other);

//----------------------------------------------------------------------------------------
//	New API
//

	// ----- Shape list methods

	FW_PShapeList				GetShapeList() const;
	void						SetShapeList(const FW_PShapeList& list);

	
	FW_CShape*					HitTestList(FW_CGraphicContext& gc,
											 const FW_CPoint& test,
											 FW_CFixed tolerance) const;
	
//----------------------------------------------------------------------------------------
//	Shape API
//
public:
	// ----- Memory Management
	virtual void				Purge();

	// ----- Rendering
	virtual void 				Render(FW_CGraphicContext& gc) const;
	static  void				RenderShapeList(FW_CGraphicContext& gc,
												const FW_PShapeList& shapeList);
	
	// ----- Copying
	virtual FW_CShape*			Copy() const;
	
	// ----- Hit Testing
	virtual FW_Boolean			HitTest(FW_CGraphicContext& gc,
									const FW_CPoint& test,
									FW_CFixed tolerance) const;
	// ----- Transform
	virtual void				Transform(Environment *ev, ODTransform* transform);
	virtual void				InverseTransform(Environment *ev, ODTransform* transform);

	virtual void				MoveShape(FW_CFixed deltaX, FW_CFixed deltaY);
	virtual void				MoveShapeTo(FW_CFixed x, FW_CFixed y);
	
	virtual void				Inset(FW_CFixed h, FW_CFixed v);
	
	// ----- Bounds
	virtual void 				GetBounds(FW_CGraphicContext& gc, FW_CRect& rect) const;
	
	// ----- Archiving -----
	virtual void				Flatten(FW_CWritableStream& archive) const;
	static void* 				Read(FW_CReadableStream& archive);

//----------------------------------------------------------------------------------------
//	Implementation
//

private:
	FW_PShapeList				fShapeList;
};


#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export off
#endif

#endif // FWSHLISH_H
