//========================================================================================
//
//	File:				FWTxtBox.h
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1993, 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================
//
//	Class Definitions for
//			- FW_CTextBoxShape
//

#ifndef FWTXTBOX_H
#define FWTXTBOX_H

#ifndef FWBTXTSH_H
#include "FWBTxtSh.h"
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

//========================================================================================
//	class FW_CTextBoxShape
//========================================================================================

class FW_CLASS_ATTR FW_CTextBoxShape : public FW_CBaseTextShape
{
public:
	FW_DECLARE_CLASS

//----------------------------------------------------------------------------------------
//	Constructors/Destructors
//
public:

	enum
	{
		kDefaultJustification =
			FW_kTextBoxJustifyLeft |
			FW_kTextBoxJustifyTop |
			FW_kTextBoxWordWrap |
			FW_kTextBoxClipToBox
	};

	FW_CTextBoxShape();

	FW_CTextBoxShape(const FW_CString& string, 
					const FW_CRect& box,
					const FW_PFont& font = FW_kNormalFont,
					FW_TextBoxOptions options = kDefaultJustification,
					const FW_PInk& ink = FW_kNormalTextInk);

	FW_CTextBoxShape(FW_CTextReader* textReader, 
					const FW_CRect& box,
					const FW_PFont& font = FW_kNormalFont,
					FW_TextBoxOptions options = kDefaultJustification,
					const FW_PInk& ink = FW_kNormalTextInk);
					
	FW_CTextBoxShape(const FW_CTextBoxShape& other);
	FW_CTextBoxShape(FW_CReadableStream& archive);
	
	virtual ~ FW_CTextBoxShape();
	
//----------------------------------------------------------------------------------------
//	Operators
//
public:
	FW_CTextBoxShape& operator=(const FW_CTextBoxShape& other);
	
//----------------------------------------------------------------------------------------
//	Inherited API
//
public:
	// ----- Rendering -----
	virtual void 				Render(FW_CGraphicContext& gc) const;
							
	// ----- Transform -----
	virtual void				Transform(Environment* ev, ODTransform* odTransform);
	virtual void				InverseTransform(Environment* ev, ODTransform* odTransform);

	virtual void				MoveShape(FW_CFixed deltaX, FW_CFixed deltaY);
	virtual void				MoveShapeTo(FW_CFixed x, FW_CFixed y);

	virtual void				Inset(FW_CFixed x, FW_CFixed y);
	
	// ----- Copying -----
	virtual FW_CShape*			Copy() const;
	
	// ----- Geometry -----
	virtual void 				GetBounds(FW_CGraphicContext& gc, FW_CRect& rect) const;

	// ----- Flatten -----
	virtual void				Flatten(FW_CWritableStream& archive) const;

//----------------------------------------------------------------------------------------
//	New API
//
public:
	// ----- Rendering -----							
	static FW_CFixed			RenderTextBox(FW_CGraphicContext& gc, 
							  					FW_CTextReader* textReader, 
												const FW_CRect& box,
							  					const FW_PFont& font,
												FW_TextBoxOptions options = kDefaultJustification,
							  					const FW_PInk& ink = FW_kNormalTextInk);

	static FW_CFixed			RenderTextBox(FW_CGraphicContext& gc, 
							  					const FW_CString& string, 
												const FW_CRect& box,
							  					const FW_PFont& font,
												FW_TextBoxOptions options = kDefaultJustification,
							  					const FW_PInk& ink = FW_kNormalTextInk);

	// ----- Archiving -----
	static void* 				Read(FW_CReadableStream& archive);

	// ----- Measurement -----
	void						UpdateTextBox(FW_CGraphicContext& gc);
	
	static void				CalcTextBox(FW_CGraphicContext& gc,
							  				FW_CTextReader* textReader, 
											FW_TextBoxOptions options,
							  				const FW_PFont& font,
											FW_CRect& box);
	static void				CalcTextBox(FW_CGraphicContext& gc, 
							  				const FW_CString& string, 
											FW_TextBoxOptions options,
							  				const FW_PFont& font,
											FW_CRect& box);
	
	// ----- Getters/Setters -----
	void						GetTextBox(FW_CRect& textBox) const
									{textBox = fTextBox;}
	void						SetTextBox(const FW_CRect& textBox)
									{fTextBox = textBox;}

	FW_TextBoxOptions			GetOptions() const
									{return fOptions;}
	void						SetOptions(FW_TextBoxOptions options)
									{fOptions = options;}

//----------------------------------------------------------------------------------------
//	Data Members
//
protected:
	FW_CRect				fTextBox;
	FW_TextBoxOptions		fOptions;
};

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export off
#endif

#endif
