//========================================================================================
//
//	File:				FWIntStr.cpp
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "FWOS.hpp"

#ifndef FWINTSTR_H
#include "FWIntStr.h"
#endif

// --- Foundation includes ---

#ifndef FWEXCDEF_H
#include "FWExcDef.h"
#endif

// for FW_DYNAMIC_CAST
#ifndef FWCLAINF_H
#include "FWClaInf.h"
#endif

#ifndef FWMEMORY_H
#include "FWMemory.h"
#endif

// --- Macintosh includes ---

// for smJapanese, &c
#if defined(FW_BUILD_MAC) && !defined(__SCRIPT__)
#include <Script.h>
#endif

//========================================================================================
//	Runtime Information
//========================================================================================

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

#ifdef FW_BUILD_MAC
#pragma segment Strings
#endif

//========================================================================================
//	Global Functions
//========================================================================================
	
//----------------------------------------------------------------------------------------
// LookupCharWidth
//----------------------------------------------------------------------------------------
static FW_ByteCount LookupCharWidth(ODScriptCode scriptCode, ODLangCode langCode)
{
#ifdef FW_BUILD_MAC
	switch (scriptCode)
	{
		case smJapanese:
		case smTradChinese:
		case smSimpChinese:
			return 2;
	}
#endif

	// default is 1 byte per character
	return 1;
}

//========================================================================================
//	CLASS FW_CIntlString
//========================================================================================

FW_DEFINE_CLASS_M1(FW_CIntlString, FW_CString)

//----------------------------------------------------------------------------------------
// FW_CIntlString destructor
//----------------------------------------------------------------------------------------

FW_CIntlString::~FW_CIntlString()
{
	FW_START_DESTRUCTOR
	delete [] fRepresentation;
}

//----------------------------------------------------------------------------------------
// FW_CIntlString constructors
//----------------------------------------------------------------------------------------

FW_CIntlString::FW_CIntlString(ODScriptCode scriptCode, ODLangCode langCode) :
	FW_CString(),
	fScriptCode(scriptCode),
	fLanguageCode(langCode)
{
	AllocateRepresentation();
	PrivSetLength(0, 0, 1);
	FW_END_CONSTRUCTOR
}

//----------------------------------------------------------------------------------------
FW_CIntlString::FW_CIntlString(ODIText* iText) :
	FW_CString()
{
	fScriptCode = GetITextScriptCode(iText);
	fLanguageCode = GetITextLangCode(iText);

	FW_ByteCount numberBytes = GetITextStringLength(iText);
	FW_ByteCount charWidth = 1;

	AllocateRepresentation(numberBytes);
	
	FW_CHECK_ODITEXT_FORMAT(iText);
	
#if FW_OPENDOC_VERSION >= FW_OPENDOC_DR2
	ODTradITextData *data = (ODTradITextData*)iText->text._buffer;
	AppendBytes(data->theText, numberBytes);
#else
	AppendBytes((char*)iText->theText._buffer, numberBytes);
#endif

	PrivSetLength(numberBytes/charWidth, numberBytes, charWidth);
	FW_END_CONSTRUCTOR
}

//----------------------------------------------------------------------------------------
//	FW_CIntlString copy constructor
//----------------------------------------------------------------------------------------

FW_CIntlString::FW_CIntlString(const FW_CIntlString& string) :
	FW_CString()
{
	AllocateRepresentation(string.GetByteLength());
	PrivSetLength(0, 0, string.GetCharWidth());	// number of characters, number of bytes
	Append((const FW_Char*)string, string.GetByteLength());
	fScriptCode = string.GetScriptCode();
	fLanguageCode = string.GetLangCode();
	FW_END_CONSTRUCTOR
}

//----------------------------------------------------------------------------------------
FW_CIntlString::FW_CIntlString(const FW_CString& string, ODScriptCode scriptCode, ODLangCode langCode) :
	FW_CString()
{
	AllocateRepresentation(string.GetByteLength());
	PrivSetLength(0, 0, 1);
	Append(string);	// sets fScriptCode and fLanguageCode to those of string
	FW_END_CONSTRUCTOR
}

//----------------------------------------------------------------------------------------
FW_CIntlString::FW_CIntlString(const FW_Char* chars, FW_CharacterCount numberChars, 
							   ODScriptCode scriptCode, ODLangCode langCode) :
	FW_CString(),
	fScriptCode(scriptCode),
	fLanguageCode(langCode)
{
	AllocateRepresentation(numberChars);
	PrivSetLength(0, 0);
	Append(chars, numberChars);
	FW_END_CONSTRUCTOR
}

//----------------------------------------------------------------------------------------
FW_CIntlString::FW_CIntlString(const FW_Char *chars, 
							   ODScriptCode scriptCode, ODLangCode langCode) :
	FW_CString(),
	fScriptCode(scriptCode),
	fLanguageCode(langCode)
{
	FW_ByteCount charWidth = 1;
	FW_ByteCount numberBytes = FW_StringLength(chars);
	AllocateRepresentation(numberBytes);
	PrivSetLength(0, 0, charWidth);
	Append(chars, numberBytes / charWidth);
	FW_END_CONSTRUCTOR
}

//----------------------------------------------------------------------------------------
//	FW_CIntlString::operator=
//----------------------------------------------------------------------------------------

FW_CIntlString& FW_CIntlString::operator=(const FW_CIntlString& string)
{
	if (&string != this)
	{
		PrivSetLength(0, 0, string.GetCharWidth());
		fScriptCode = string.GetScriptCode();
		fLanguageCode = string.GetLangCode();
		Append(string);
	}
	return *this;
}

//----------------------------------------------------------------------------------------
FW_CIntlString& FW_CIntlString::operator=(const FW_CString& string)
{
	if (&string != this)
	{
		PrivSetLength(0, 0, string.GetCharWidth());
		fScriptCode = FW_kDefaultScriptCode;
		fLanguageCode = FW_kDefaultScriptCode;
		Append(string);
	}
	return *this;
}

//----------------------------------------------------------------------------------------
// FW_CIntlString::CreateIText
//----------------------------------------------------------------------------------------
ODIText* FW_CIntlString::CreateIText() const
{
	ODIText* result = ::CreateIText(this->GetByteLength());
	SetITextString(result, (char*)fRepresentation);
	SetITextScriptCode(result, fScriptCode);
	SetITextLangCode(result, fLanguageCode);
	
	return result;
}

//----------------------------------------------------------------------------------------
// FW_CIntlString::GrowCapacity
//----------------------------------------------------------------------------------------
FW_ByteCount FW_CIntlString::GrowCapacity(FW_ByteCount newCapacity)	// Override
{
	if (fCapacity < newCapacity)
	{
		FW_Byte *newRep = new FW_Byte[newCapacity+FW_kMedianCharacterSize];
		FW_BlockMove(fRepresentation, newRep, fCapacity+sizeof(FW_Char));
		delete [] fRepresentation;
		fRepresentation = newRep;
		fCapacity = newCapacity;
	}

	return fCapacity;
}

//----------------------------------------------------------------------------------------
// FW_CIntlString::AdjustCharWidth
//----------------------------------------------------------------------------------------
void FW_CIntlString::AdjustCharWidth()
{
	FW_ByteCount charWidth = LookupCharWidth(fScriptCode, fLanguageCode);
	if (charWidth != GetCharWidth())
	{
		FW_CharacterCount length = this->CalcLength();
		PrivSetLength(fByteLength, length, charWidth);
	}
}

//----------------------------------------------------------------------------------------
// FW_CIntlString::CalcLength
//----------------------------------------------------------------------------------------
FW_CharacterCount FW_CIntlString::CalcLength()
{
	FW_CharacterCount result = fByteLength / fCharWidth;

#ifdef FW_BUILD_MAC
	if (fScriptCode == smJapanese)
	{
		FW_CharacterCount count = 0;
		const FW_Char* textBuf = (const FW_Char*) this;
		short byteType;
		FW_ByteCount offset = 0;
		while (offset < fByteLength)
		{
			byteType = ::CharacterByteType((Ptr)textBuf, offset, fScriptCode);
			if (byteType == smFirstByte)
			{
				count++;
				offset++;	// skip 2nd byte of 2-byte character
			}
			else if (byteType == smSingleByte)
				count++;
			else			// (byteType == smLastByte) should never happen!
			{
				count = result;
				break;
			}
			offset++;
		}
		result = count;
	}
#endif

	return result;
}

