//========================================================================================
//
//	File:				FWSUSink.cpp
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "FWOS.hpp"

#ifndef FWSUSINK_H
#include "FWSUSink.h"
#endif

#ifndef FWSTDDEF_H
#include "FWStdDef.h"
#endif

#ifndef FWBARRAY_H
#include "FWBArray.h"
#endif

// ----- Platform Includes -----

#if defined(FW_BUILD_MAC) & !defined(SOMCorba_h)
#include <somcorba.h>
#endif

//========================================================================================
//	RunTime Info
//========================================================================================

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

#ifdef FW_BUILD_MAC
#pragma segment fwodutil
#endif

//========================================================================================
//	CLASS FW_CStorageUnitSink
//========================================================================================

//----------------------------------------------------------------------------------------
//	FW_CStorageUnitSink::FW_CStorageUnitSink
//----------------------------------------------------------------------------------------

FW_CStorageUnitSink::FW_CStorageUnitSink(ODStorageUnitView* storageUnitView) :
	fStorageUnitView(storageUnitView), fOwnsView(FALSE)
{
	PrivAcquire();

	FW_END_CONSTRUCTOR
}

//----------------------------------------------------------------------------------------
//	FW_CStorageUnitSink::FW_CStorageUnitSink
//----------------------------------------------------------------------------------------

FW_CStorageUnitSink::FW_CStorageUnitSink(ODStorageUnit* storageUnit, 
							ODPropertyName propertyName, ODValueType valueType) :
	fStorageUnitView(0), fOwnsView(TRUE)
{
	FW_ASSERT(propertyName != NULL);
	FW_ASSERT(valueType != NULL);
	
	Environment* ev = somGetGlobalEnvironment();
	storageUnit->Focus(ev, 
					propertyName, kODPosUndefined, 
					valueType, (ODValueIndex)0, kODPosUndefined);
	fStorageUnitView = storageUnit->CreateView(ev);

	PrivAcquire();

	FW_END_CONSTRUCTOR
}

//----------------------------------------------------------------------------------------
//	FW_CStorageUnitSink::~FW_CStorageUnitSink
//----------------------------------------------------------------------------------------

FW_CStorageUnitSink::~FW_CStorageUnitSink()
{
	FW_START_DESTRUCTOR

	this->Flush();
	
#if FW_OPENDOC_VERSION >= FW_OPENDOC_DR3
	if (fStorageUnitView)
	{
		Environment* ev = somGetGlobalEnvironment();
		fStorageUnitView->GetStorageUnit(ev)->Release(ev);
	}
#endif

	if (fOwnsView)
		delete fStorageUnitView;
}

//----------------------------------------------------------------------------------------
//	FW_CStorageUnitSink::PrivAcquire
//----------------------------------------------------------------------------------------

void FW_CStorageUnitSink::PrivAcquire()
{
#if FW_OPENDOC_VERSION >= FW_OPENDOC_DR3
	if (fStorageUnitView)
	{
		Environment* ev = somGetGlobalEnvironment();
		fStorageUnitView->GetStorageUnit(ev)->Acquire(ev);
	}
#endif
}

//----------------------------------------------------------------------------------------
//	FW_CStorageUnitSink::Read
//----------------------------------------------------------------------------------------

void FW_CStorageUnitSink::Read(void* destination, long count)
{
	this->Flush();
	this->PrivDoRead(destination, count);
}

//----------------------------------------------------------------------------------------
//	FW_CStorageUnitSink::PrivDoRead
//----------------------------------------------------------------------------------------

void FW_CStorageUnitSink::PrivDoRead(void* destination, long count)
{
#if FW_OPENDOC_VERSION >= FW_OPENDOC_DR3
	FW_CByteArray	byteArray;

	fStorageUnitView->GetValue(somGetGlobalEnvironment(), count, byteArray);
	byteArray.CopyBuffer(destination, count);
#elif FW_OPENDOC_VERSION == FW_OPENDOC_DR2
	FW_CByteArray	byteArray(destination, count);
	fStorageUnitView->GetValue(somGetGlobalEnvironment(), byteArray);
#else
	fStorageUnitView->GetValue(somGetGlobalEnvironment(), count, destination);
#endif
}

//----------------------------------------------------------------------------------------
//	FW_CStorageUnitSink::ReadPeek
//----------------------------------------------------------------------------------------

const void* FW_CStorageUnitSink::ReadPeek(long& availableReadBytes)
{
	char* source = (char*) fBuffer.ReadPeek(availableReadBytes);
	
	if (availableReadBytes == 0)
	{
		availableReadBytes = FW_Minimum(this->GetLength() - this->GetPosition(), fBuffer.GetCapacity());

		this->Flush();
		fBuffer.Initialize(FW_CStorageUnitBuffer::kReadPeek, availableReadBytes);

		// read data into the buffer
		source = fBuffer.GetAddress();
		long position = this->GetPosition();
		this->PrivDoRead(source, availableReadBytes);
		this->SetPosition(position);
	}
	
	return source;
}

//----------------------------------------------------------------------------------------
//	FW_CStorageUnitSink::ReadPeekAdvance
//----------------------------------------------------------------------------------------

void FW_CStorageUnitSink::ReadPeekAdvance(long bytesRead)
{
	fBuffer.ReadPeekAdvance(bytesRead);
	this->SetPosition(this->GetPosition() + bytesRead);
}

//----------------------------------------------------------------------------------------
//	FW_CStorageUnitSink::GetWritableBytes
//----------------------------------------------------------------------------------------

long FW_CStorageUnitSink::GetWritableBytes() const
{
	return 0x7FFFFFFFL;
}

//----------------------------------------------------------------------------------------
//	FW_CStorageUnitSink::Write
//----------------------------------------------------------------------------------------

void FW_CStorageUnitSink::Write(const void* source, long count)
{
	this->Flush();
	this->PrivDoWrite(source, count);
}

//----------------------------------------------------------------------------------------
//	FW_CStorageUnitSink::PrivDoWrite
//----------------------------------------------------------------------------------------

void FW_CStorageUnitSink::PrivDoWrite(const void* source, long count)
{
#if FW_OPENDOC_VERSION >= FW_OPENDOC_DR2
	FW_CByteArray	byteArray(source, count);
	
	fStorageUnitView->SetValue(somGetGlobalEnvironment(), byteArray);
#else
	fStorageUnitView->SetValue(somGetGlobalEnvironment(), count, (void*)source);
#endif
}

//----------------------------------------------------------------------------------------
//	FW_CStorageUnitSink::WritePeek
//----------------------------------------------------------------------------------------

void* FW_CStorageUnitSink::WritePeek(long& availableWriteBytes)
{
	char* source = (char*) fBuffer.WritePeek(availableWriteBytes);
	
	if (availableWriteBytes == 0)
	{
		this->Flush();
		
		availableWriteBytes = fBuffer.GetCapacity();
		fBuffer.Initialize(FW_CStorageUnitBuffer::kWritePeek, availableWriteBytes, this->GetPosition());
		source = fBuffer.GetAddress();
	}
	
	return source;
}

//----------------------------------------------------------------------------------------
//	FW_CStorageUnitSink::WritePeekAdvance
//----------------------------------------------------------------------------------------

void FW_CStorageUnitSink::WritePeekAdvance(long bytesWritten)
{
	fBuffer.WritePeekAdvance(bytesWritten);
	this->SetPosition(this->GetPosition() + bytesWritten);
}

//----------------------------------------------------------------------------------------
//	FW_CStorageUnitSink::Flush
//----------------------------------------------------------------------------------------

void FW_CStorageUnitSink::Flush()
{
	if (fBuffer.IsDirty())
	{
		long currentPosition = this->GetPosition();
		this->SetPosition(fBuffer.GetInitialPosition());
		this->PrivDoWrite(fBuffer.GetAddress(), fBuffer.GetPosition());
		this->SetPosition(currentPosition);
	}

	fBuffer.Initialize(FW_CStorageUnitBuffer::kInvalid);
}

//----------------------------------------------------------------------------------------
// 	FW_CStorageUnitSink::GetLength
//----------------------------------------------------------------------------------------

long FW_CStorageUnitSink::GetLength() const
{
	return fStorageUnitView->GetSize(somGetGlobalEnvironment());
}

//----------------------------------------------------------------------------------------
// 	FW_CStorageUnitSink::SetLength
//----------------------------------------------------------------------------------------
 
void FW_CStorageUnitSink::SetLength(long length)
{
	long prevLength = GetLength();
	if (length < prevLength)
	{
		SetPosition(length);
		fStorageUnitView->DeleteValue(somGetGlobalEnvironment(), prevLength-length);
	}
	else if (length > prevLength)
	{
		const long kSize = 128;
		char buffer[kSize] = {0};	// initialization is not required
		SetPosition(prevLength);
		length -= prevLength;
		while (length > 0)
		{
			long bytes = length < kSize ? length : kSize;
			Write(buffer, bytes);
			length -= bytes;
		}
	}
}

//----------------------------------------------------------------------------------------
// 	FW_CStorageUnitSink::GetPosition
//----------------------------------------------------------------------------------------

long FW_CStorageUnitSink::GetPosition() const
{
	return fStorageUnitView->GetOffset(somGetGlobalEnvironment());
}

//----------------------------------------------------------------------------------------
//	FW_CStorageUnitSink::SetPosition
//----------------------------------------------------------------------------------------

void FW_CStorageUnitSink::SetPosition(long position)
{
	fStorageUnitView->SetOffset(somGetGlobalEnvironment(), position);
}

//----------------------------------------------------------------------------------------
//	FW_CStorageUnitSink::GetStorageUnitView
//----------------------------------------------------------------------------------------

ODStorageUnitView*	FW_CStorageUnitSink::GetStorageUnitView() const
{
	return fStorageUnitView;
}
