//========================================================================================
//
//	File:				FWAlert.cpp
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1993, 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "FWOS.hpp"

#ifndef FWALERT_H
#include "FWAlert.h"
#endif

#ifndef FWRESOUR_H
#include "FWResour.h"
#endif

// ----- Macintosh Includes -----

#if defined(FW_BUILD_MAC) && !defined(__OSUTILS__)
#include <OSUtils.h>
#endif

#if defined(FW_BUILD_MAC) && !defined(__TEXTUTILS__)
#include <TextUtils.h>
#endif

#if defined(FW_BUILD_MAC) && !defined(__ICONS__)
#include <Icons.h>
#endif

#if defined(FW_BUILD_MAC) && !defined(__DIALOGS__)
#include <Dialogs.h>
#endif

//========================================================================================
//	Runtime Informations
//========================================================================================

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

#ifdef FW_BUILD_MAC	
#pragma segment fwosmisc
#endif

//========================================================================================
//	Globales
//========================================================================================

#ifdef FW_BUILD_MAC	
short FW_CAlert::gOkItemId = 0;
short FW_CAlert::gCancelItemId = 0;
#endif

#ifdef FW_BUILD_MAC	
//========================================================================================
//	MacAlertDialogFilter
//========================================================================================

pascal Boolean FW_CAlert::MacAlertDialogFilter(DialogPtr theDialog, 
											EventRecord * theEvent, 
											short *itemHit)
{
	Boolean result = FALSE;

	switch (theEvent->what)
	{
		case keyDown:
		case autoKey:
			{
				switch (theEvent->message & charCodeMask)
				{
					case '.':
						if (theEvent->modifiers & cmdKey && gCancelItemId != 0)
						{
							result = TRUE;
							*itemHit = gCancelItemId;
						}
						break;
					
					case 0x0D:
					case 0x03:
						result = TRUE;
						*itemHit = gOkItemId;
						break;
					
					case 0x1B:
						if (gCancelItemId != 0)
						{
							result = TRUE;
							*itemHit = gCancelItemId;
						}
						break;				
				}
				
				if (result)		// flash the button
				{
					long 	theTick;
					short 	itemType;
					Handle 	hItem;
					Rect	box;
					
					::GetDialogItem(theDialog, *itemHit, &itemType, &hItem, &box);
					::HiliteControl((ControlHandle)hItem, 1);
					::Delay(6,&theTick);
					::HiliteControl((ControlHandle)hItem, 0);			
				}
			}
			break;
		
		case updateEvt:
			if (theDialog == (DialogPtr)theEvent->message)
			{
				PenState 	ps;
				Rect		box;
				Handle		handle;
				short		type;
				
				::SetPort(theDialog);
				::GetPenState(&ps);
				::PenSize(3, 3);
				::GetDialogItem(theDialog, gOkItemId, &type, &handle, &box);
				::InsetRect(&box, -4, -4);
				::FrameRoundRect(&box, 16, 16);
				::SetPenState(&ps);
			}
			break;
	}

	return result;
}
#endif

#ifdef FW_BUILD_MAC
//----------------------------------------------------------------------------------------
//	MacSetButtonText
//----------------------------------------------------------------------------------------

static void MacSetButtonText(DialogPtr dialog, short buttonId, Str32 text, FW_Boolean moveButton)
{
	Rect 			rect;
	short 			type;
	Handle		 	handle;

	::GetDialogItem(dialog, buttonId, &type, &handle, &rect);
	if (text[0] != 0)
	{
		::SetControlTitle(ControlHandle(handle), text);
		if (moveButton)
		{
			::OffsetRect(&rect, 10, 0);
			::SetDialogItem(dialog, buttonId, type, handle, &rect);
			::MoveControl(ControlHandle(handle), rect.left, rect.top);
		}

	}
	else
	{
		::HideControl(ControlHandle(handle));
	}
}
#endif

//----------------------------------------------------------------------------------------
//	FW_CAlert::DoAlert
//----------------------------------------------------------------------------------------
FW_AlertResult FW_CAlert::DoAlert(const FW_CString& documentName,
								const FW_CString& message,
								FW_ButtonType buttonType,
								FW_IconType iconType,
								FW_DefaultButton defaultButton,
								FW_Boolean beep)
{		
#ifdef FW_BUILD_WIN
	UINT dialogFlags = buttonType | iconType | defaultButton | MB_TASKMODAL;
	dialogFlags ^= dialogFlags & MB_SYSTEMMODAL;
	
	// ::MessageBox() doesn't work if mouse capturing is active,
	//  so we release the capture (if any) and then restore it

	const HWND hwndCapture = ::GetCapture();

	if (hwndCapture != NULL)
		::ReleaseCapture();
	
	if (beep)
		::MessageBeep(-1);
	
	short result = ::MessageBox(NULL, message, documentName, dialogFlags);

	// Set mouse capture to the window that had it
	if (hwndCapture != 0)
		::SetCapture(hwndCapture);

	return result;
#endif

#ifdef FW_BUILD_MAC
#define kDialogID 500
// ----- dialog item ids
#define kButton1Id 	1
#define kButton2Id	2
#define kButton3Id 	3
#define	kMessageId	4
#define	kIconId		5

	// ----- Open create the dialog (can't do exception here because used to display exception messages)
	DialogPtr 		dlg;
	{
		FW_CAcquireCFMResourceAccess a;
		dlg = ::GetNewDialog(kDialogID, NULL, WindowPtr(-1));
	}
	Rect 			rect;
	short 			type;
	Handle		 	handle;
	
	::GetDialogItem(dlg, kMessageId, &type, &handle, &rect);
	Str255 str;
	message.ExportPascal(str);
	::SetDialogItemText(handle, str);

	Str32 but1, but2, but3;
	but1[0] = 0;
	but2[0] = 0;
	but3[0] = 0;
	short items[3];

	FW_Boolean moveButton = FALSE;
	
	switch (buttonType)
	{
		case FW_kOK:
			{
				FW_CAcquireCFMResourceAccess a;
				::GetIndString(but1, kDialogID, 1);		// OK
				gOkItemId = 1;
				gCancelItemId = 0;
				items[0] = FW_kOKButtonPressed;
				break;
			}
		case FW_kOKCancel:
			{
				FW_CAcquireCFMResourceAccess a;
				::GetIndString(but1, kDialogID, 1);		// OK
				::GetIndString(but2, kDialogID, 2);		// Cancel
				if (defaultButton == FW_kDefaultButton2)
				{
					gCancelItemId = 0;
					gOkItemId = 2;
				}
				else
				{
					gOkItemId = 1;
					gCancelItemId = 2;
				}
				items[0] = FW_kOKButtonPressed;
				items[1] = FW_kCancelButtonPressed;
				break;
			}
		case FW_kAbortRetryIgnore:
			{
				FW_CAcquireCFMResourceAccess a;
				::GetIndString(but1, kDialogID, 3);		// Abort
				::GetIndString(but2, kDialogID, 4);		// Retry
				::GetIndString(but3, kDialogID, 5);		// Ignore
				gOkItemId = defaultButton + 1;
				gCancelItemId = 0;
				items[0] = FW_kAbortButtonPressed;
				items[1] = FW_kRetryButtonPressed;
				items[2] = FW_kIgnoreButtonPressed;
				break;
			}
		case FW_kYesNoCancel:
			{
				FW_CAcquireCFMResourceAccess a;
				::GetIndString(but1, kDialogID, 6);		// Yes
				::GetIndString(but2, kDialogID, 7);		// No
				::GetIndString(but3, kDialogID, 2);		// Cancel
				if (defaultButton == FW_kDefaultButton1)
				{
					gOkItemId = 1;
					gCancelItemId = 3;
				}
				else if (defaultButton == FW_kDefaultButton2)
				{
					gOkItemId = 2;
					gCancelItemId = 3;
				}
				else
				{
					// I can't decide if Esc means Yes or No
					gOkItemId = 3;
					gCancelItemId = 0;
				}
				items[0] = FW_kYesButtonPressed;
				items[1] = FW_kNoButtonPressed;
				items[2] = FW_kCancelButtonPressed;
				moveButton = TRUE;
				break;
			}
		case FW_kYesNo:
			{
				FW_CAcquireCFMResourceAccess a;
				::GetIndString(but1, kDialogID, 6);		// Yes
				::GetIndString(but2, kDialogID, 7);		// No
				gCancelItemId = 0;
				if (defaultButton == FW_kDefaultButton2)
					gOkItemId = 2;
				else
					gOkItemId = 1;
				items[0] = FW_kYesButtonPressed;
				items[1] = FW_kNoButtonPressed;
				break;
			}
		case FW_kRetryCancel:
			{
				FW_CAcquireCFMResourceAccess a;
				::GetIndString(but1, kDialogID, 4);		// Retry
				::GetIndString(but2, kDialogID, 2);		// Cancel
				if (defaultButton == FW_kDefaultButton2)
				{
					gCancelItemId = 0;
					gOkItemId = 2;
				}
				else
				{
					gOkItemId = 1;
					gCancelItemId = 2;
				}
				items[0] = FW_kRetryButtonPressed;
				items[1] = FW_kCancelButtonPressed;
				break;
			}
		default:
			FW_ASSERT(0);							// unknown dialogFlags
			break;
	}

	MacSetButtonText(dlg, kButton1Id, but1, FALSE);
	MacSetButtonText(dlg, kButton2Id, but2, moveButton);
	MacSetButtonText(dlg, kButton3Id, but3, FALSE);
		
	// ----- Icons
	Handle hIcon = NULL;
	switch (iconType)
	{
		case FW_kStopAlert:
			hIcon = ::GetIcon(stopIcon);
			break;
		
		case FW_kCautionAlert:
			hIcon = ::GetIcon(cautionIcon);
			break;
		
		case FW_kNoteAlert:
			hIcon = ::GetIcon(noteIcon);
			break;
	};
	
	::GetDialogItem(dlg, kIconId, &type, &handle, &rect);
	if (hIcon)
	{
		::SetDialogItem(dlg, kIconId, type, hIcon, &rect);
	}
	else
	{
		::GetDialogItem(dlg, kIconId, &type, &handle, &rect);
		short left = rect.left;
		::OffsetRect(&rect, 1000, 0);
		::SetDialogItem(dlg, kIconId, type, hIcon, &rect);
		
		::GetDialogItem(dlg, kMessageId, &type, &handle, &rect);
		rect.left = left;
		::SetDialogItem(dlg, kMessageId, type, handle, &rect);		
	}
		
	if (beep)
		::SysBeep(16);

	::ShowWindow(dlg);
	
	short item;
	ModalFilterUPP filterProc = NewModalFilterProc(&MacAlertDialogFilter);
	do
	{
		::ModalDialog(filterProc, &item);
	} while (item <kButton1Id || item > kButton3Id);
	DisposeRoutineDescriptor(filterProc);
	
	::DisposeDialog(dlg);

	return (items[item - 1]);
#endif
}
