//========================================================================================
//
//	File:				FWFileOp.cpp
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "FWOS.hpp"

#ifndef FWFILEOP_H
#include "FWFileOp.h"
#endif

// ----- Foundation Includes -----

#ifndef FWBNDSTR_H
#include <FWBndStr.h>
#endif

#ifndef FWMEMMGR_H
#include <FWMemMgr.h>
#endif

//----- Windows includes -----

#if defined(FW_BUILD_WIN) && !defined(__STDLIB_H)
#include <stdlib.h>
#endif

#if defined(FW_BUILD_WIN) && !defined(__WINDOWS_H)
#include <windows.h>
#endif

#if defined(FW_BUILD_WIN) && !defined(__COMMDLG_H)
#include <commdlg.h>
#endif

#if defined(FW_BUILD_WIN) && !defined(IO_H)
#include "io.h"
#endif

//----- Macintosh includes -----

// for DisposeRoutineDescriptor
#if defined(FW_BUILD_MAC) && !defined(__MIXEDMODE__)
#include <MixedMode.h>
#endif

//========================================================================================
//	Runtime Informations
//========================================================================================

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

#ifdef FW_BUILD_MAC	
#pragma segment fwosmisc
#endif

//========================================================================================
//	CLASS FW_CChooseFile
//========================================================================================

//----------------------------------------------------------------------------------------
// FW_CChooseFile::ChooseFileToOpen
//----------------------------------------------------------------------------------------

FW_CFileSpecification* FW_CChooseFile::ChooseFileToOpen(const FW_COpenFileParameters& openFileParam)
{
	FW_CFileSpecification* fileSpec = NULL;

#ifdef FW_BUILD_MAC
	StandardFileReply reply;
// pascal void StandardGetFile(FileFilterUPP fileFilter, short numTypes, SFTypeList typeList, StandardFileReply *reply)
	FileFilterUPP fileFilter = NewFileFilterProc(openFileParam.MacGetFilterProc());
	::StandardGetFile(fileFilter,
					  openFileParam.MacCountTypes(), 
					  openFileParam.GetTypes(), 
					  &reply);
	DisposeRoutineDescriptor((UniversalProcPtr)fileFilter);

	if (reply.sfGood)
	{
		fileSpec = FW_NEW(FW_CFileSpecification, (reply.sfFile));
	}
#endif

#ifdef FW_BUILD_WIN
	OPENFILENAME 	ofn;
	char			fullFileName[_MAX_PATH];
		
	::memset(&ofn, 0, sizeof(OPENFILENAME));
	
	fullFileName[0] = 0;
	
	ofn.lStructSize		= sizeof(OPENFILENAME);
	ofn.hwndOwner		= openFileParam.WinGetOwnerWindow();
	ofn.lpstrFilter		= openFileParam.WinGetFilterString();
	ofn.lpstrFile		= fullFileName;
	ofn.nMaxFile		= _MAX_PATH;	// size of fullFileName buffer
	ofn.Flags			= OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;
	ofn.lpstrDefExt		= openFileParam.WinGetDefaultExt();
	
	if (::GetOpenFileName(&ofn))
	{
		fileSpec = FW_NEW(FW_CFileSpecification, (FW_CString255(fullFileName)));
	}
#endif

	return fileSpec;
}

//----------------------------------------------------------------------------------------
// FW_CChooseFile::ChooseFileToSave
//----------------------------------------------------------------------------------------

FW_CFileSpecification* FW_CChooseFile::ChooseFileToSave(const FW_CSaveFileParameters& saveFileParam)
{
	FW_CFileSpecification* fileSpec = NULL;

#ifdef FW_BUILD_MAC
// pascal void StandardPutFile(ConstStr255Param prompt, ConstStr255Param defaultName, StandardFileReply *reply)
	StandardFileReply reply;
	FW_CString255 str;
	Str255 defaultName, savePrompt;
	saveFileParam.MacGetDefaultName(str);
	str.ExportPascal(defaultName);
	saveFileParam.MacGetSavePrompt(str);
	str.ExportPascal(savePrompt);
	::StandardPutFile(savePrompt, defaultName, &reply);
	if (reply.sfGood)
	{
		fileSpec = FW_NEW(FW_CFileSpecification, (reply.sfFile));
	}
#endif

#ifdef FW_BUILD_WIN
	OPENFILENAME 	ofn;		
	::memset(&ofn, 0, sizeof(OPENFILENAME));
	
	ofn.lStructSize		= sizeof(OPENFILENAME);
	ofn.hwndOwner		= saveFileParam.WinGetOwnerWindow();
	ofn.lpstrFilter		= saveFileParam.WinGetFilterString();
	ofn.lpstrFile		= saveFileParam.WinGetDefaultName();
	ofn.nMaxFile		= _MAX_PATH;
	ofn.lpstrDefExt		= saveFileParam.WinGetDefaultExt();
	ofn.Flags			= OFN_OVERWRITEPROMPT;
	
	if (::GetSaveFileName(&ofn))
	{
		FW_CString255 fileName(saveFileParam.WinGetDefaultName());
		fileSpec = FW_NEW(FW_CFileSpecification, (fileName));
	}
#endif

	return fileSpec;
}

//========================================================================================
//	CLASS FW_COpenFileParameters
//========================================================================================

//----------------------------------------------------------------------------------------
//	FW_COpenFileParameters default constructor
//----------------------------------------------------------------------------------------

FW_COpenFileParameters::FW_COpenFileParameters()
{
#ifdef FW_BUILD_MAC
	fNumTypes = -1;
	fFileFilterProc = NULL;
#endif
#ifdef FW_BUILD_WIN
	fOwnerWindow = NULL;
	fFilterString = NULL;
	fDefaultExt[0] = 0;
#endif
}

//----------------------------------------------------------------------------------------
//	FW_COpenFileParameters::~FW_COpenFileParameters
//----------------------------------------------------------------------------------------

FW_COpenFileParameters::~FW_COpenFileParameters()
{
#ifdef FW_BUILD_WIN
	if (fFilterString)
		FW_CMemoryManager::FreeBlock(fFilterString);
#endif
}

#ifdef FW_BUILD_MAC
//----------------------------------------------------------------------------------------
//	FW_COpenFileParameters::FW_COpenFileParameters
//----------------------------------------------------------------------------------------

FW_COpenFileParameters::FW_COpenFileParameters(OSType* fileType, 
											   FileFilterProcPtr fileFilterProc)
{
	Set(1, fileType, fileFilterProc);
}

FW_COpenFileParameters::FW_COpenFileParameters(short numTypes,
											   OSType *fileType, 
											   FileFilterProcPtr fileFilterProc)
{
	Set(numTypes, fileType, fileFilterProc);
}
#endif

#ifdef FW_BUILD_MAC
//----------------------------------------------------------------------------------------
//	FW_COpenFileParameters::Set
//----------------------------------------------------------------------------------------

void FW_COpenFileParameters::Set(short numTypes, OSType *fileType, FileFilterProcPtr fileFilterProc)
{
	FW_ASSERT(numTypes>=0  && numTypes<5);
	fNumTypes = numTypes;
	
	for (short i = 0; i<numTypes; i++)
		fTypes[i] = fileType[i];
	
	fFileFilterProc = fileFilterProc;
}
#endif

#ifdef FW_BUILD_WIN
//----------------------------------------------------------------------------------------
//	FW_COpenFileParameters::FW_COpenFileParameters
//----------------------------------------------------------------------------------------

FW_COpenFileParameters::FW_COpenFileParameters(const FW_CString& filterString,
											   const FW_CString& defaultExt,
											   HWND ownerWindow)
{
	if (ownerWindow == NULL)
		fOwnerWindow = NULL;
	else
		fOwnerWindow = ownerWindow; // [KVV] Temporary code! ->GetXMPWindow()->GetPlatformWindow();

	Set(filterString, defaultExt);
}
#endif

#ifdef FW_BUILD_WIN
//----------------------------------------------------------------------------------------
//	FW_COpenFileParameters::Set
//----------------------------------------------------------------------------------------

void FW_COpenFileParameters::Set(const FW_CString& filterString,
								 const FW_CString& defaultExt)
{
	if (defaultExt.GetByteLength() < 4)
		defaultExt.Export(fDefaultExt);
	
	long length = filterString.GetByteLength();
	if (length == 0)
		return;

	// Allocate memory to hold the filter strings
	fFilterString = (char*)FW_CMemoryManager::AllocateBlock(length+1);	// add room for 0-terminator
	filterString.Export(fFilterString);
	
	char chReplace = fFilterString[length-1];	// final char should be "|" or other separator
	// replace separator characters with 0's, dividing filter into individual strings
	for (short i = 0; fFilterString[i] != '\0'; i++)
			if (fFilterString[i] == chReplace)
				fFilterString[i] = '\0';		
}
#endif

//========================================================================================
//	CLASS FW_CSaveFileParameters
//========================================================================================

//----------------------------------------------------------------------------------------
//	FW_CSaveFileParameters default constructor
//----------------------------------------------------------------------------------------
FW_CSaveFileParameters::FW_CSaveFileParameters()
{
#ifdef FW_BUILD_MAC
	fSavePrompt = "Save file as";
	fDefaultName = "";
#endif
#ifdef FW_BUILD_WIN
	fOwnerWindow = NULL;
	fFilterString = NULL;
	fDefaultExt[0] = 0;
	fDefaultName = NULL;
#endif
}

//----------------------------------------------------------------------------------------
//	FW_CSaveFileParameters destructor
//----------------------------------------------------------------------------------------
FW_CSaveFileParameters::~FW_CSaveFileParameters()
{
#ifdef FW_BUILD_WIN
	if (fFilterString)
		FW_CMemoryManager::FreeBlock(fFilterString);
	if (fDefaultName)
		FW_CMemoryManager::FreeBlock(fDefaultName);
#endif
}

#ifdef FW_BUILD_MAC
//----------------------------------------------------------------------------------------
//	FW_CSaveFileParameters constructor
//----------------------------------------------------------------------------------------
FW_CSaveFileParameters::FW_CSaveFileParameters(const FW_CString& savePrompt,
											   const FW_CString& defaultName)
{
	fSavePrompt = savePrompt;
	fDefaultName = defaultName;
}
#endif

#ifdef FW_BUILD_WIN
//----------------------------------------------------------------------------------------
//	FW_CSaveFileParameters constructor
//----------------------------------------------------------------------------------------
FW_CSaveFileParameters::FW_CSaveFileParameters(const FW_CString& filterString,
											   const FW_CString& defaultExt,
											   const FW_CString& defaultName,
							   				   HWND ownerWindow)
{
	if (ownerWindow == NULL)
		fOwnerWindow = NULL;
	else
		fOwnerWindow = ownerWindow; // [KVV] Temporary code! ->GetXMPWindow()->GetPlatformWindow();

	Set(filterString, defaultExt, defaultName);
}
#endif

#ifdef FW_BUILD_WIN
//----------------------------------------------------------------------------------------
//	FW_CSaveFileParameters::Set
//----------------------------------------------------------------------------------------
void FW_CSaveFileParameters::Set(const FW_CString& filterString,
								 const FW_CString& defaultExt,
								 const FW_CString& defaultName)
{
	if (defaultExt.GetByteLength() < 4)
		defaultExt.Export(fDefaultExt);

	// Allocate memory to hold the default name
	long length = defaultName.GetByteLength();
	fDefaultName = (char*)FW_CMemoryManager::AllocateBlock(length+1);	// add room for 0-terminator
	defaultName.Export(fDefaultName);

	// Allocate memory to hold the filter strings
	length = filterString.GetByteLength();
	fFilterString = (char*)FW_CMemoryManager::AllocateBlock(length+1);	// add room for 0-terminator
	filterString.Export(fFilterString);
	
	if (length == 0)
		return;

	// Divide filter string into individual strings
	char chReplace = fFilterString[length-1];			// final char should be "|" or other separator
	for (short i = 0; fFilterString[i] != '\0'; i++)	// replace separator characters with 0's
			if (fFilterString[i] == chReplace)
				fFilterString[i] = '\0';		
}
#endif

