//========================================================================================
//
//	File:				FWTime.cpp
//	Release Version:	$ 1.0d11 $
//
//	Copyright: 1993, 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "FWOS.hpp"

#ifndef FWTIME_H
#include "FWTime.h"
#endif

#ifndef FWSTRING_H
#include "FWString.h"
#endif

#if defined(FW_BUILD_MAC) && !defined(__TEXTUTILS__)
#include <TextUtils.h>
#endif

//========================================================================================
//	Runtime Informations
//========================================================================================

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

#ifdef FW_BUILD_MAC	
#pragma segment fwosmisc
#endif

//========================================================================================
// CLASS FW_CTime
//========================================================================================

//----------------------------------------------------------------------------------------
// FW_CTime constructor
//----------------------------------------------------------------------------------------

FW_CTime::FW_CTime()
{
	fTime = NULL;	// Invalid time
}

//----------------------------------------------------------------------------------------
// FW_CTime constructor
//----------------------------------------------------------------------------------------

FW_CTime::FW_CTime(time_t theTime)
{
	fTime = theTime;
}

//----------------------------------------------------------------------------------------
// FW_CTime constructor
//----------------------------------------------------------------------------------------

FW_CTime::FW_CTime(const FW_CTime& theTime)
{
	fTime = theTime.GetTime();
}

//----------------------------------------------------------------------------------------
// FW_CTime constructor
//----------------------------------------------------------------------------------------

FW_CTime::FW_CTime(short year, short month, short day, short hour, short minute, short second, short dayOfWeek)
{
#ifdef FW_BUILD_MAC
	DateTimeRec dateTime;
	
	dateTime.year = year;
	dateTime.month = month;
	dateTime.day = day;								// Day of month [1 to 31]
	dateTime.hour = hour;
	dateTime.minute = minute;
	dateTime.second = second;
	dateTime.dayOfWeek = dayOfWeek;
	
	::DateToSeconds(&dateTime, &fTime);
#endif


#ifdef FW_BUILD_WIN
	struct tm dateTime;

	dateTime.tm_year = year;
	dateTime.tm_mon = month - 1;			// Specified as 1-12, expects 0-11
	dateTime.tm_mday = day;
	dateTime.tm_hour = hour;
	dateTime.tm_min = minute;
	dateTime.tm_sec = second;
	dateTime.tm_wday = dayOfWeek - 1;	// Specified as 1-7, expects 0-6
	
	fTime = mktime(&dateTime);
#endif
}

//----------------------------------------------------------------------------------------
// FW_CTime destructor
//----------------------------------------------------------------------------------------

FW_CTime::~FW_CTime()
{
}

//----------------------------------------------------------------------------------------
// FW_CTime::GetCurrentTime
//----------------------------------------------------------------------------------------

FW_CTime FW_CTime::GetCurrentTime()
{
	time_t dateTime;

#ifdef FW_BUILD_MAC
    ::GetDateTime(&dateTime);
#endif

#ifdef FW_BUILD_WIN
    ::time(&dateTime);
#endif

	return FW_CTime(dateTime);
}

//----------------------------------------------------------------------------------------
// FW_CTime::GetTime
//----------------------------------------------------------------------------------------

time_t FW_CTime::GetTime() const
{
	return fTime;
}

//----------------------------------------------------------------------------------------
// FW_CTime::GetSecond
//----------------------------------------------------------------------------------------

short FW_CTime::GetSecond() const
{
#ifdef FW_BUILD_MAC
	DateTimeRec dateTime;
	
	::SecondsToDate(fTime, &dateTime);
	return dateTime.second;
#endif

#ifdef FW_BUILD_WIN
	struct tm* dateTime;
	
	dateTime = localtime(&fTime);
	return dateTime->tm_sec;

#endif
}

//----------------------------------------------------------------------------------------
// FW_CTime::GetMinute
//----------------------------------------------------------------------------------------

short FW_CTime::GetMinute() const
{
#ifdef FW_BUILD_MAC
	DateTimeRec dateTime;
	
	::SecondsToDate(fTime, &dateTime);
	return dateTime.minute;
#endif

#ifdef FW_BUILD_WIN
	struct tm* dateTime;
	
	dateTime = localtime(&fTime);
	return dateTime->tm_min;

#endif
}

//----------------------------------------------------------------------------------------
// FW_CTime::GetHour
//----------------------------------------------------------------------------------------

short FW_CTime::GetHour() const
{
#ifdef FW_BUILD_MAC
	DateTimeRec dateTime;
	
	::SecondsToDate(fTime, &dateTime);
	return dateTime.hour;
#endif

#ifdef FW_BUILD_WIN
	struct tm* dateTime;
	
	dateTime = localtime(&fTime);
	return dateTime->tm_hour;

#endif
}

//----------------------------------------------------------------------------------------
// FW_CTime::GetDayOfWeek
//----------------------------------------------------------------------------------------

short FW_CTime::GetDayOfWeek() const
{
#ifdef FW_BUILD_MAC
	DateTimeRec dateTime;
	
	::SecondsToDate(fTime, &dateTime);
	return dateTime.dayOfWeek;		// 1..7 for Sun thru Sat
#endif

#ifdef FW_BUILD_WIN
	struct tm* dateTime;
	
	dateTime = localtime(&fTime);
	return dateTime->tm_wday + 1;
#endif
}

//----------------------------------------------------------------------------------------
// FW_CTime::GetDayOfMonth
//----------------------------------------------------------------------------------------

short FW_CTime::GetDayOfMonth() const
{
#ifdef FW_BUILD_MAC
	DateTimeRec dateTime;
	
	::SecondsToDate(fTime, &dateTime);
	return dateTime.day;
#endif

#ifdef FW_BUILD_WIN
	struct tm* dateTime;
	
	dateTime = localtime(&fTime);
	return dateTime->tm_mday;
#endif
}

//----------------------------------------------------------------------------------------
// FW_CTime::GetDayOfYear
//----------------------------------------------------------------------------------------

short FW_CTime::GetDayOfYear() const
{
#ifdef FW_BUILD_MAC
	return 0;
#endif

#ifdef FW_BUILD_WIN
	struct tm* dateTime;
	
	dateTime = localtime(&fTime);
	return dateTime->tm_yday;		// 0 thru 365
#endif
}

//----------------------------------------------------------------------------------------
// FW_CTime::GetMonth
//----------------------------------------------------------------------------------------

short FW_CTime::GetMonth() const
{
#ifdef FW_BUILD_MAC
	DateTimeRec dateTime;
	
	::SecondsToDate(fTime, &dateTime);
	return dateTime.month;		// 1..12 Jan thru Dec
#endif

#ifdef FW_BUILD_WIN
	struct tm* dateTime;
	
	dateTime = localtime(&fTime);
	return dateTime->tm_mon + 1;
#endif
}

//----------------------------------------------------------------------------------------
// FW_CTime::GetYear
//----------------------------------------------------------------------------------------

short FW_CTime::GetYear() const
{
#ifdef FW_BUILD_MAC
	DateTimeRec dateTime;
	
	::SecondsToDate(fTime, &dateTime);
	return dateTime.year;
#endif

#ifdef FW_BUILD_WIN
	struct tm* dateTime;
	
	dateTime = localtime(&fTime);
	return dateTime->tm_year + 1900;
#endif
}

//----------------------------------------------------------------------------------------
// FW_CTime::IsDaylightSavings
//----------------------------------------------------------------------------------------

short FW_CTime::IsDaylightSavings() const
{
#ifdef FW_BUILD_MAC
	MachineLocation loc;
	
	::ReadLocation(&loc);
	// MetroWerks won't compiler this [AMB]
	// return loc.u.dlsDelta;
	return 0;
#endif

#ifdef FW_BUILD_WIN
	struct tm* dateTime;
	
	dateTime = localtime(&fTime);
	return dateTime->tm_isdst;
#endif
}

//----------------------------------------------------------------------------------------
// FW_CTime::GetTimeString
//----------------------------------------------------------------------------------------

void FW_CTime::GetTimeString(FW_CString& timeString, FW_Boolean displaySeconds) const
{
#ifdef FW_BUILD_MAC
	Str32 tempString;
	::IUTimeString(fTime, displaySeconds, tempString);	

	timeString.ReplaceAll(tempString);
#endif

#ifdef FW_BUILD_WIN
	struct tm* dateTime;
	FW_Char tempString[32];
	
	dateTime = localtime(&fTime);
	strftime(tempString, 32, "%X", dateTime);
	
	timeString.ReplaceAll(tempString);
	FW_UNUSED(displaySeconds);
#endif
}

//----------------------------------------------------------------------------------------
// FW_CTime::GetDateString
//----------------------------------------------------------------------------------------

void FW_CTime::GetDateString(FW_CString& dateString, DateForm form) const
{
#ifdef FW_BUILD_MAC
	Str255 tempString;
	
	if (form == kShortDate)
		::IUDateString(fTime, shortDate, tempString);	
	else if (form == kLongDate)
		::IUDateString(fTime, longDate, tempString);
	else
		::IUDateString(fTime, abbrevDate, tempString);

	dateString.ReplaceAll(tempString);
#endif

#ifdef FW_BUILD_WIN
	struct tm* dateTime;
	FW_Char tempString[255];
	
	dateTime = localtime(&fTime);
	strftime(tempString, 255, "%x", dateTime);
	
	dateString.ReplaceAll(tempString);
FW_UNUSED(form);
#endif
}

//----------------------------------------------------------------------------------------
//	FW_CTime::operator =
//----------------------------------------------------------------------------------------

const FW_CTime& FW_CTime::operator=(const FW_CTime& time)
{
	fTime = time.GetTime();
	return *this;
}

//----------------------------------------------------------------------------------------
//	FW_CTime::operator =
//----------------------------------------------------------------------------------------

const FW_CTime& FW_CTime::operator=(time_t time)
{
	fTime = time;
	return *this;
}

//----------------------------------------------------------------------------------------
//	FW_CTime::operator -
//----------------------------------------------------------------------------------------

FW_CTime FW_CTime::operator-(FW_CTimeSpan timeSpan) const
{
	return FW_CTime(fTime - timeSpan.GetTotalSeconds());
}

//----------------------------------------------------------------------------------------
//	FW_CTime::operator -
//----------------------------------------------------------------------------------------

FW_CTimeSpan FW_CTime::operator-(FW_CTime time) const
{
	return FW_CTimeSpan(fTime - time.GetTime());
}

//----------------------------------------------------------------------------------------
//	FW_CTime::operator +
//----------------------------------------------------------------------------------------

FW_CTime FW_CTime::operator+(FW_CTimeSpan timeSpan) const
{
	return FW_CTime(fTime + timeSpan.GetTotalSeconds());
}

//----------------------------------------------------------------------------------------
//	FW_CTime::operator -=
//----------------------------------------------------------------------------------------

const FW_CTime& FW_CTime::operator-=(FW_CTimeSpan timeSpan)
{
	fTime = fTime - timeSpan.GetTotalSeconds();
	return *this;
}

//----------------------------------------------------------------------------------------
//	FW_CTime::operator +=
//----------------------------------------------------------------------------------------

const FW_CTime& FW_CTime::operator+=(FW_CTimeSpan timeSpan)
{
	fTime = fTime + timeSpan.GetTotalSeconds();
	return *this;
}

//----------------------------------------------------------------------------------------
//	FW_CTime::operator ==
//----------------------------------------------------------------------------------------

FW_Boolean FW_CTime::operator==(FW_CTime time) const
{
	return fTime == time.GetTime();
}

//----------------------------------------------------------------------------------------
//	FW_CTime::operator !=
//----------------------------------------------------------------------------------------

FW_Boolean FW_CTime::operator!=(FW_CTime time) const
{
	return fTime != time.GetTime();
}

//----------------------------------------------------------------------------------------
//	FW_CTime::operator <
//----------------------------------------------------------------------------------------

FW_Boolean FW_CTime::operator<(FW_CTime time) const
{
	return fTime < time.GetTime();
}

//----------------------------------------------------------------------------------------
//	FW_CTime::operator >
//----------------------------------------------------------------------------------------

FW_Boolean FW_CTime::operator>(FW_CTime time) const
{
	return fTime > time.GetTime();
}

//----------------------------------------------------------------------------------------
//	FW_CTime::operator <=
//----------------------------------------------------------------------------------------

FW_Boolean FW_CTime::operator<=(FW_CTime time) const
{
	return fTime <= time.GetTime();
}

//----------------------------------------------------------------------------------------
//	FW_CTime::operator >=
//----------------------------------------------------------------------------------------

FW_Boolean FW_CTime::operator>=(FW_CTime time) const
{
	return fTime >= time.GetTime();
}

//========================================================================================
// CLASS FW_CTimeSpan
//========================================================================================

//----------------------------------------------------------------------------------------
// FW_CTimeSpan constructor
//----------------------------------------------------------------------------------------

FW_CTimeSpan::FW_CTimeSpan()
{
	fTotalSeconds = 0;
}

//----------------------------------------------------------------------------------------
// FW_CTimeSpan constructor
//----------------------------------------------------------------------------------------

FW_CTimeSpan::FW_CTimeSpan(time_t theTimeSpan)
{
	fTotalSeconds = theTimeSpan;
}

//----------------------------------------------------------------------------------------
// FW_CTimeSpan constructor
//----------------------------------------------------------------------------------------

FW_CTimeSpan::FW_CTimeSpan(const FW_CTimeSpan& theTimeSpan)
{
	fTotalSeconds = theTimeSpan.GetTotalSeconds();
}

//----------------------------------------------------------------------------------------
// FW_CTimeSpan constructor
//----------------------------------------------------------------------------------------

FW_CTimeSpan::FW_CTimeSpan(long days, short hours, short minutes, short seconds)
{
	fTotalSeconds = (days * 86400L) + (hours * 3600) + (minutes * 60) + seconds;
}

//----------------------------------------------------------------------------------------
// FW_CTimeSpan destructor
//----------------------------------------------------------------------------------------

FW_CTimeSpan::~FW_CTimeSpan()
{
}

//----------------------------------------------------------------------------------------
// FW_CTimeSpan::GetSeconds
//----------------------------------------------------------------------------------------

short FW_CTimeSpan::GetSeconds() const
{
	return fTotalSeconds % 60;
}

//----------------------------------------------------------------------------------------
// FW_CTimeSpan::GetMinutes
//----------------------------------------------------------------------------------------

short FW_CTimeSpan::GetMinutes() const
{
	return (fTotalSeconds % 3600) / 60;
}

//----------------------------------------------------------------------------------------
// FW_CTimeSpan::GetHours
//----------------------------------------------------------------------------------------

short FW_CTimeSpan::GetHours() const
{
	return (fTotalSeconds % 86400L) / 3600;
}

//----------------------------------------------------------------------------------------
// FW_CTimeSpan::GetDays
//----------------------------------------------------------------------------------------

long FW_CTimeSpan::GetDays() const
{
	return fTotalSeconds / 86400L;	// 60 seconds * 60 minutes * 24 hours
}


//----------------------------------------------------------------------------------------
// FW_CTimeSpan::GetTotalSeconds
//----------------------------------------------------------------------------------------

long FW_CTimeSpan::GetTotalSeconds() const
{
	return fTotalSeconds;
}

//----------------------------------------------------------------------------------------
// FW_CTimeSpan::GetTotalMinutes
//----------------------------------------------------------------------------------------

long FW_CTimeSpan::GetTotalMinutes() const
{
	return fTotalSeconds / 60;
}

//----------------------------------------------------------------------------------------
// FW_CTimeSpan::GetTotalHours
//----------------------------------------------------------------------------------------

long FW_CTimeSpan::GetTotalHours() const
{
	return fTotalSeconds / 3600;
}

//----------------------------------------------------------------------------------------
//	FW_CTimeSpan::operator =
//----------------------------------------------------------------------------------------

const FW_CTimeSpan& FW_CTimeSpan::operator=(const FW_CTimeSpan& source)
{
	fTotalSeconds = source.GetTotalSeconds();
	return *this;
}

//----------------------------------------------------------------------------------------
//	FW_CTimeSpan::operator -=
//----------------------------------------------------------------------------------------

const FW_CTimeSpan& FW_CTimeSpan::operator-=(FW_CTimeSpan source)
{
	fTotalSeconds = fTotalSeconds - source.GetTotalSeconds();
	return *this;
}

//----------------------------------------------------------------------------------------
//	FW_CTimeSpan::operator +=
//----------------------------------------------------------------------------------------

const FW_CTimeSpan& FW_CTimeSpan::operator+=(FW_CTimeSpan source)
{
	fTotalSeconds = fTotalSeconds + source.GetTotalSeconds();
	return *this;
}

//----------------------------------------------------------------------------------------
//	FW_CTimeSpan::operator -
//----------------------------------------------------------------------------------------

FW_CTimeSpan FW_CTimeSpan::operator-(FW_CTimeSpan timeSpan) const
{
	return FW_CTimeSpan((time_t)(GetTotalSeconds() - timeSpan.GetTotalSeconds()));
}

//----------------------------------------------------------------------------------------
//	FW_CTimeSpan::operator +
//----------------------------------------------------------------------------------------

FW_CTimeSpan FW_CTimeSpan::operator+(FW_CTimeSpan timeSpan) const
{
	return FW_CTimeSpan((time_t)(GetTotalSeconds() + timeSpan.GetTotalSeconds()));
}

//----------------------------------------------------------------------------------------
//	FW_CTimeSpan::operator ==
//----------------------------------------------------------------------------------------

FW_Boolean FW_CTimeSpan::operator==(FW_CTimeSpan timeSpan) const
{
	return fTotalSeconds == timeSpan.GetTotalSeconds();
}

//----------------------------------------------------------------------------------------
//	FW_CTimeSpan::operator !=
//----------------------------------------------------------------------------------------

FW_Boolean FW_CTimeSpan::operator!=(FW_CTimeSpan timeSpan) const
{
	return fTotalSeconds != timeSpan.GetTotalSeconds();
}

//----------------------------------------------------------------------------------------
//	FW_CTimeSpan::operator <
//----------------------------------------------------------------------------------------

FW_Boolean FW_CTimeSpan::operator<(FW_CTimeSpan timeSpan) const
{
	return fTotalSeconds < timeSpan.GetTotalSeconds();
}

//----------------------------------------------------------------------------------------
//	FW_CTimeSpan::operator >
//----------------------------------------------------------------------------------------

FW_Boolean FW_CTimeSpan::operator>(FW_CTimeSpan timeSpan) const
{
	return fTotalSeconds > timeSpan.GetTotalSeconds();
}

//----------------------------------------------------------------------------------------
//	FW_CTimeSpan::operator <=
//----------------------------------------------------------------------------------------

FW_Boolean FW_CTimeSpan::operator<=(FW_CTimeSpan timeSpan) const
{
	return fTotalSeconds <= timeSpan.GetTotalSeconds();
}

//----------------------------------------------------------------------------------------
//	FW_CTimeSpan::operator >=
//----------------------------------------------------------------------------------------

FW_Boolean FW_CTimeSpan::operator>=(FW_CTimeSpan timeSpan) const
{
	return fTotalSeconds >= timeSpan.GetTotalSeconds();
}

