//========================================================================================
//
//	File:				FWFloWin.h
//	Release Version:	$ 1.0d11 $
//
//	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================
//
//	FW_CFloatWindow: Wrapper for a floating ODWindow.
//	
//		The creation of a FW_CFloatingWindow works differently from FW_CWindow. Usually in your
//		part Initialize method your directly create the floating windows your need (don't forget
//		FW_CWindow and FW_CFloatingWindow are autodestruct objects)
//		
//		void CMyPart::Initialize()
//		{
//				...
//				fPaletteWindow = FW_NEW(FW_CFloatingWindow, (this, fPalettePresentation, paletteRect));
//				...
//		}
//
//		Floating windows are created hidden. You can call FW_CWindow::Show() or FW_CWindow::Hide() to 
//		show/Hide your floating windows. When your part is desactivated/activated, the visible floating windows 
//		will be automatically hidden/shown.
//	

#ifndef FWFLOWIN_H
#define FWFLOWIN_H

// ----- OS Layer -----

#ifndef FWWINDOW_H
#include "FWWindow.h"
#endif

// ----- OpenDoc Includes -----

#ifndef _ODTYPES_
#include <ODTypes.h>
#endif

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export on
#endif

//========================================================================================
//	Forward Declarations
//========================================================================================

class FW_CLASS_ATTR ODPart;

//========================================================================================
//	class FW_CFloatingWindow
//========================================================================================

class FW_CLASS_ATTR FW_CFloatingWindow : public FW_CWindow
{
public:
	FW_DECLARE_CLASS

//----------------------------------------------------------------------------------------
//	Constructors/Destructor
//
public:
	FW_CFloatingWindow(Environment* ev, 
						ODPart* thePart,
						ODTypeToken presentation,
						const FW_CString& windowTitle,
						const FW_CPoint& interiorSize,
						const FW_CPoint& position,
						FW_Boolean hasCloseBox);
	virtual ~FW_CFloatingWindow();
	
//----------------------------------------------------------------------------------------
//	Inherited API
//
public:
	virtual void		HideShowOnActivate(Environment* ev, FW_Boolean state);
		
//----------------------------------------------------------------------------------------
//	Data Members
//
private:
	FW_Boolean			fWasShown;			// Used when floating is hidden on deactivate
};

#if FW_LIB_EXPORT_PRAGMAS
#pragma lib_export off
#endif

#endif
